class
	P_OS_STATIC

inherit
	CE_UTILITIES
	S_SIGNAL_STATIC
	S_UNISTD_STATIC
	S_TIME_STATIC
	S_BITS_ERRNO_STATIC
	S_STRING_STATIC
	S_STDIO_STATIC

feature {ANY} -- public attributes

	reading: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	read_label: CE_STATIC [CE_EXCEPTION]
		once ("PROCESS")
		end

	consecutive_nulls: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

feature {ANY} -- public routines

	iread (a_fd: INTEGER_32; a_buf___1: CE_POINTER [INTEGER_8]; a_len: NATURAL_32): INTEGER_32
		local
			l_n: INTEGER_32
			l_tmp: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					read_label.item := create {CE_EXCEPTION}.make ($iread, l_pos, $l_tmp)
				when 1 then -- continue here on raising read_label:
					if l_tmp.to_boolean then
						reading.item := 0
						sigsetmask (0).do_nothing
						Result := (-2)
						l_return := True
					end
					if not l_return then
						(create {P_OUTPUT_STATIC}).flush
						reading.item := 1
						l_n := read (a_fd, create {CE_VOID_POINTER}.make_cast (a_buf___1), a_len)
						if not ((create {P_CH_STATIC}).ignore_eoi.item).to_boolean then
							if l_n = 0 then
								consecutive_nulls.item := consecutive_nulls.item + 1
							else
								consecutive_nulls.item := 0
							end
							if consecutive_nulls.item > 20 then
								(create {P_MAIN_STATIC}).quit (1)
							end
						end
						reading.item := 0
						if l_n < 0 then
							Result := (-1)
							l_return := True
						end
					end
					if not l_return then
						Result := l_n
					end
				when 2 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		rescue
			if attached {CE_EXCEPTION} (create {EXCEPTION_MANAGER}).last_exception as ex then
				if ex.routine = $iread then
					l_pos := ex.position + 1
					retry
				end
				ex.re_raise
			end
		end

	intread
		do
			read_label.item.raise (1)
		end

	get_time: INTEGER_32
		local
			l_t: CE_ARRAY [INTEGER_32]
		do
			create {CE_INTEGER_32_ARRAY} l_t.make (<<1>>)

			time (l_t).do_nothing
			Result := (l_t.item)
		end

	errno_message (a_filename: CE_POINTER [INTEGER_8]): CE_POINTER [INTEGER_8]
		require
			not a_filename.is_void
		local
			l_p: CE_POINTER [INTEGER_8]
			l_m: CE_POINTER [INTEGER_8]
			l_tmp___0: NATURAL_32
		do
			l_p := strerror ((errno_location).item)
			l_tmp___0 := strlen (a_filename)
			l_m := create {CE_INTEGER_8_POINTER}.make_cast (((create {P_MAIN_STATIC}).ecalloc (((l_tmp___0 + (strlen (l_p))) + {NATURAL_32} 3).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
			sprintf ([l_m, ce_string ("%%s: %%s"), a_filename, l_p]).do_nothing
			Result := l_m
		end

	percentage (a_num, a_den: INTEGER_32): INTEGER_32
		local
			l_num100: INTEGER_32
		do
			l_num100 := a_num * 100
			if l_num100 // 100 = a_num then
				Result := (l_num100 // a_den).to_integer_32
			else
				Result := (a_num // (a_den // 100)).to_integer_32
			end
		end

	percent_pos (a_pos: INTEGER_32; a_percent: INTEGER_32): INTEGER_32
		local
			l_result100: INTEGER_32
		do
			if a_percent = 0 then
				Result := (0).to_integer_32
			else
				l_result100 := a_pos * a_percent.to_integer_32
				if l_result100 // a_percent.to_integer_32 = a_pos then
					Result := (l_result100 // 100)
				else
					Result := (a_percent.to_integer_32 * (a_pos // 100))
				end
			end
		end

end
