class
	P_TAGS_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_STRING_STATIC
	S_FCNTL_STATIC
	S_UNISTD_STATIC
	S_STDIO_STATIC
	S_CTYPE_STATIC

feature {ANY} -- public attributes

	tags: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := ce_string ("tags")
		end

	total: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	curseq: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	taglist: CE_STATIC [CE_ARRAY [PD_TAGLIST]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_TAGLIST]}.make (<<1>>)
			Result.item[0] := create {PD_TAGLIST}.make_initialized ([create {CE_CLASS_POINTER [PD_TAG]}.make_cast (taglist.item), create {CE_CLASS_POINTER [PD_TAG]}.make_cast (taglist.item)])
		end

	curtag: CE_STATIC [CE_POINTER [PD_TAG]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [PD_TAG]}.make
		end

	circular: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

feature {ANY} -- public routines

	cleantags
		local
			l_tp: CE_POINTER [PD_TAG]
			l_break: BOOLEAN
		do
			from
			until
				(l_break)
			loop
				l_break := False
				l_tp := taglist.item.item.tl_first
				if l_tp.is_equal (taglist.item) then
					l_break := True
				end
				if not (l_break) then
					(l_tp.item.next).item.prev := l_tp.item.prev;
					(l_tp.item.prev).item.next := l_tp.item.next
					free (create {CE_VOID_POINTER}.make_cast (l_tp))
				end
			end
			l_break := False
			curtag.item := create {CE_CLASS_POINTER [PD_TAG]}.make_shared (default_pointer)
			curseq.item := 0
			total.item := curseq.item
		end

	maketagent (a_name, a_file: CE_POINTER [INTEGER_8]; a_linenum: INTEGER_32; a_pattern: CE_POINTER [INTEGER_8]; a_endline: INTEGER_32): CE_POINTER [PD_TAG]
		require
			not a_file.is_void
		local
			l_tp: CE_POINTER [PD_TAG]
		do
			l_tp := create {CE_CLASS_POINTER [PD_TAG]}.make_cast (((create {P_MAIN_STATIC}).ecalloc (((create {PD_TAG}.make).structure_size.to_natural_32).to_integer_32, {NATURAL_32} 1)))
			l_tp.item.tag_file := create {CE_INTEGER_8_POINTER}.make_cast (((create {P_MAIN_STATIC}).ecalloc (((strlen (a_file)) + {NATURAL_32} 1).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
			strcpy (l_tp.item.tag_file, a_file).do_nothing
			l_tp.item.tag_linenum := a_linenum
			l_tp.item.tag_endline := a_endline.to_integer_8
			if a_pattern.is_void then
				l_tp.item.tag_pattern := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			else
				l_tp.item.tag_pattern := create {CE_INTEGER_8_POINTER}.make_cast (((create {P_MAIN_STATIC}).ecalloc (((strlen (a_pattern)) + {NATURAL_32} 1).to_integer_32, {PLATFORM}.Integer_8_bytes.to_natural_32)))
				strcpy (l_tp.item.tag_pattern, a_pattern).do_nothing
			end
			Result := l_tp
		end

	gettagtype: INTEGER_32
		local
			l_f: INTEGER_32
			l_return: BOOLEAN
		do
			if (strcmp (tags.item, ce_string ("GTAGS"))) = 0 then
				Result := 2
				l_return := True
			end
			if not l_return then
				if (strcmp (tags.item, ce_string ("GRTAGS"))) = 0 then
					Result := 3
					l_return := True
				end
			end
			if not l_return then
				if (strcmp (tags.item, ce_string ("GSYMS"))) = 0 then
					Result := 4
					l_return := True
				end
			end
			if not l_return then
				if (strcmp (tags.item, ce_string ("GPATH"))) = 0 then
					Result := 5
					l_return := True
				end
			end
			if not l_return then
				if (strcmp (tags.item, ce_string ("-"))) = 0 then
					Result := 1
					l_return := True
				end
			end
			if not l_return then
				l_f := open ([tags.item, 0])
				if l_f >= 0 then
					close (l_f).do_nothing
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 2
			end
		end

	findtag (a_tag: CE_POINTER [INTEGER_8])
		local
			l_type: INTEGER_32
			l_result: NATURAL_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_type := (gettagtype)
					if l_type = 0 then
						l_result := findctag (a_tag)
					else
						l_result := findgtag (a_tag, l_type)
					end
					inspect l_result.to_integer_32 -- switch 1 head
					when 0 then l_pos := -1 + 1
					when 4 then l_pos := -1 + 1
					when 1 then l_pos := -1 + 2
					when 2 then l_pos := -1 + 3
					when 3 then l_pos := -1 + 4
					else l_pos := -1 + 5
					end
				when 1 then -- switch 1, case 0 | switch 1, case 4
					l_pos := -1 + 5 -- switch 1; break
				when 2 then -- switch 1, case 1;
					(create {P_OUTPUT_STATIC}).error (ce_string ("No tags file"), create {CE_CLASS_POINTER [PD_PARG]}.make_shared (default_pointer))
					l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 2;
					(create {P_OUTPUT_STATIC}).error (ce_string ("No such tag in tags file"), create {CE_CLASS_POINTER [PD_PARG]}.make_shared (default_pointer))
					l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, case 3;
					(create {P_OUTPUT_STATIC}).error (ce_string ("unknown tag type"), create {CE_CLASS_POINTER [PD_PARG]}.make_shared (default_pointer))
					l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	tagsearch: INTEGER_32
		local
			l_return: BOOLEAN
		do
			if curtag.item.is_void then
				Result := (-1).to_integer_32
				l_return := True
			end
			if not l_return then
				if curtag.item.item.tag_linenum /= 0 then
					Result := (gtagsearch)
				else
					Result := (ctagsearch)
				end
			end
		end

	nexttag (a_n: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_n: INTEGER_32
			l_tagfile: CE_POINTER [INTEGER_8]
			l_tmp: INTEGER_32
			l_break: BOOLEAN
		do
			l_n := a_n
			l_tagfile := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			from
			until
				(l_break)
			loop
				l_break := False
				l_tmp := l_n
				l_n := l_n - 1
				if not (l_tmp > 0) then
					l_break := True
				end
				if not (l_break) then
					l_tagfile := nextgtag
				end
			end
			l_break := False
			Result := l_tagfile
		end

	prevtag (a_n: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_n: INTEGER_32
			l_tagfile: CE_POINTER [INTEGER_8]
			l_tmp: INTEGER_32
			l_break: BOOLEAN
		do
			l_n := a_n
			l_tagfile := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
			from
			until
				(l_break)
			loop
				l_break := False
				l_tmp := l_n
				l_n := l_n - 1
				if not (l_tmp > 0) then
					l_break := True
				end
				if not (l_break) then
					l_tagfile := prevgtag
				end
			end
			l_break := False
			Result := l_tagfile
		end

	ntags: INTEGER_32
		do
			Result := total.item
		end

	curr_tag: INTEGER_32
		do
			Result := curseq.item
		end

	findctag (a_tag: CE_POINTER [INTEGER_8]): NATURAL_32
		local
			l_p: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_f: CE_POINTER [SD_IO_FILE]
			l_taglen: INTEGER_32
			l_taglinenum: INTEGER_32
			l_tagfile: CE_POINTER [INTEGER_8]
			l_tagpattern: CE_POINTER [INTEGER_8]
			l_tagendline: INTEGER_32
			l_search_char: INTEGER_32
			l_err: CE_ARRAY [INTEGER_32]
			l_tline: CE_ARRAY [INTEGER_8]
			l_tp: CE_POINTER [PD_TAG]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_POINTER_ARRAY [INTEGER_8]} l_p.make (<<1>>)
			create {CE_INTEGER_8_POINTER} l_tagfile.make
			create {CE_INTEGER_8_POINTER} l_tagpattern.make
			create {CE_INTEGER_32_ARRAY} l_err.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l_tline.make (<<512>>)

			l_p.item := (create {P_FILENAME_STATIC}).shell_unquote (tags.item)
			l_f := fopen (l_p.item, ce_string ("r"))
			free (create {CE_VOID_POINTER}.make_cast (l_p.item))
			if l_f.is_void then
				Result := ({PC_TAG_RESULT}.TAG_NOFILE)
				l_return := True
			end
			if not l_return then
				cleantags
				total.item := 0
				l_taglen := (strlen (a_tag)).to_integer_32
				from
				until
					(l_return or l_break)
				loop
					l_break := False
					l_continue := False
					if (fgets (l_tline, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (512).to_natural_32).to_integer_32, l_f)).is_void then
						l_break := True
					end
					if not (l_return or l_break or l_continue) and then ((l_tline[0]).to_integer_32 = 33) then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						if (strncmp (a_tag, l_tline, l_taglen.to_natural_32)) /= 0 then
							l_continue := True
						elseif (l_tline[l_taglen]).to_integer_32 /= 32 then
							if (l_tline[l_taglen]).to_integer_32 /= 9 then
								l_continue := True
							end
						end
					end
					if not (l_return or l_break or l_continue) then
						l_tagpattern := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						l_p.item := (create {P_MAIN_STATIC}).skipsp (l_tline + l_taglen)
						if (l_p.item[0]).to_integer_32 = 0 then
							l_continue := True
						end
					end
					if not (l_return or l_break or l_continue) then
						l_tagfile := l_p.item
						from
						until
							(l_break)
						loop
							l_break := False
							if (l_p.item[0]).to_integer_32 = 32 then
								l_break := True
							elseif (l_p.item[0]).to_integer_32 = 9 then
								l_break := True
							elseif (l_p.item[0]).to_integer_32 = 0 then
								l_break := True
							end
							if not (l_break) then
								l_p.item := l_p.item + 1
							end
						end
						l_break := False
						l_tmp___1 := l_p.item
						l_p.item := l_p.item + 1
						l_tmp___1[0] := (('%/0/').code).to_integer_8
						l_p.item := (create {P_MAIN_STATIC}).skipsp (l_p.item)
						if (l_p.item[0]).to_integer_32 = 0 then
							l_continue := True
						end
					end
					if not (l_return or l_break or l_continue) then
						l_tagendline := 0
						l_taglinenum := ((create {P_OPTION_STATIC}).getnum (l_p, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer), l_err)).to_integer_32
						if (l_err.item).to_boolean then
							l_taglinenum := (0).to_integer_32
							l_tmp___3 := l_p.item
							l_p.item := l_p.item + 1
							l_search_char := (l_tmp___3[0]).to_integer_32
							if (l_p.item[0]).to_integer_32 = 94 then
								l_p.item := l_p.item + 1
							end
							l_tagpattern := l_p.item
							from
							until
								(l_break)
							loop
								l_break := False
								if (l_p.item[0]).to_integer_32 /= l_search_char then
									if (l_p.item[0]).to_integer_32 = 0 then
										l_break := True
									end
								else
									l_break := True
								end
								if not (l_break) then
									if (l_p.item[0]).to_integer_32 = 92 then
										l_p.item := l_p.item + 1
									end
									l_p.item := l_p.item + 1
								end
							end
							l_break := False
							l_tagendline := ((l_p.item[-1]).to_integer_32 = 36).to_integer
							if l_tagendline.to_boolean then
								l_p.item := l_p.item - 1
							end
							l_p.item[0] := (('%/0/').code).to_integer_8
						end
						l_tp := maketagent (a_tag, l_tagfile, l_taglinenum, l_tagpattern, l_tagendline)
						l_tp.item.next := taglist.item.item.tl_first
						l_tp.item.prev := create {CE_CLASS_POINTER [PD_TAG]}.make_cast (taglist.item);
						(taglist.item.item.tl_first).item.prev := l_tp
						taglist.item.item.tl_first := l_tp
						total.item := total.item + 1
					end
				end
				l_break := False
				l_continue := False
				fclose (l_f).do_nothing
				if total.item = 0 then
					Result := ({PC_TAG_RESULT}.TAG_NOTAG)
					l_return := True
				end
			end
			if not l_return then
				curtag.item := taglist.item.item.tl_first
				curseq.item := 1
				Result := ({PC_TAG_RESULT}.TAG_FOUND)
			end
		end

	edit_tagfile: INTEGER_32
		local
			l_return: BOOLEAN
		do
			if curtag.item.is_void then
				Result := 1
				l_return := True
			end
			if not l_return then
				Result := ((create {P_EDIT_STATIC}).edit (curtag.item.item.tag_file))
			end
		end

	ctagsearch: INTEGER_32
		local
			l_pos2: INTEGER_32
			l_linepos: INTEGER_32
			l_linenum: INTEGER_32
			l_len: INTEGER_32
			l_line: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_return, l_break: BOOLEAN
		do
			create {CE_POINTER_ARRAY [INTEGER_8]} l_line.make (<<1>>)

			l_pos2 := (0).to_integer_32
			l_linenum := (create {P_LINENUM_STATIC}).find_linenum (l_pos2)
			from
			until
				(l_return or l_break)
			loop
				l_break := False
				if (((create {P_CH_STATIC}).sigs.item).bit_and (3)).to_boolean then
					Result := (-1).to_integer_32
					l_return := True
				end
				if not l_return then
					l_linepos := l_pos2
					l_pos2 := (create {P_LINE_STATIC}).forw_raw_line (l_pos2, l_line)
					if l_linenum /= 0 then
						l_linenum := l_linenum + 1
					end
					if l_pos2 = -1 then
						(create {P_OUTPUT_STATIC}).error (ce_string ("Tag not found"), create {CE_CLASS_POINTER [PD_PARG]}.make_shared (default_pointer))
						Result := (-1).to_integer_32
						l_return := True
					end
				end
				if not l_return then
					if ((create {P_LINE_STATIC}).linenums.item.item).to_boolean then
						(create {P_LINENUM_STATIC}).add_lnum (l_linenum, l_pos2)
					end
					l_len := (strlen (curtag.item.item.tag_pattern)).to_integer_32
					if (strncmp (curtag.item.item.tag_pattern, l_line.item, l_len.to_natural_32)) = 0 then
						if not (curtag.item.item.tag_endline).to_boolean then
							curtag.item.item.tag_linenum := (create {P_LINENUM_STATIC}).find_linenum (l_linepos)
							l_break := True
						elseif (l_line.item[l_len]).to_integer_32 = 0 then
							curtag.item.item.tag_linenum := (create {P_LINENUM_STATIC}).find_linenum (l_linepos)
							l_break := True
						elseif (l_line.item[l_len]).to_integer_32 = 13 then
							curtag.item.item.tag_linenum := (create {P_LINENUM_STATIC}).find_linenum (l_linepos)
							l_break := True
						end
					end
				end
			end
			if not l_return then
				l_break := False
				Result := l_linepos
			end
		end

	findgtag (a_tag: CE_POINTER [INTEGER_8]; a_type: INTEGER_32): NATURAL_32
		local
			l_buf___1: CE_ARRAY [INTEGER_8]
			l_fp: CE_POINTER [SD_IO_FILE]
			l_tp: CE_POINTER [PD_TAG]
			l_command: CE_ARRAY [INTEGER_8]
			l_flag: CE_POINTER [INTEGER_8]
			l_qtag: CE_POINTER [INTEGER_8]
			l_cmd: CE_POINTER [INTEGER_8]
			l_name: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_line: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_c: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_8_ARRAY} l_buf___1.make (<<256>>)
			create {CE_CLASS_POINTER [SD_IO_FILE]} l_fp.make
			create {CE_INTEGER_8_ARRAY} l_command.make (<<512>>)
			create {CE_INTEGER_8_POINTER} l_flag.make
			create {CE_POINTER_ARRAY [INTEGER_8]} l_name.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_file.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_line.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_type /= 1 then
						if a_tag.is_void then
							Result := ({PC_TAG_RESULT}.TAG_NOFILE)
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 12 -- skip if block
					end
				when 1 then -- if:
						cleantags
						total.item := 0
						if a_type /= 1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_fp := (create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.input.file_pointer))
						tags.item := ce_string ("tags")
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 12
				when 4 then -- else:
						l_cmd := ((create {P_DECODE_STATIC}).lgetenv (ce_string ("LESSGLOBALTAGS")))
						if l_cmd.is_void then
							Result := ({PC_TAG_RESULT}.TAG_NOFILE)
							l_return := True
						elseif (l_cmd[0]).to_integer_32 = 0 then
							Result := ({PC_TAG_RESULT}.TAG_NOFILE)
							l_return := True
						end
						if l_return then
							l_pos := -1 + 11 -- skip if block
						end
				when 5 then -- if:
						inspect a_type -- switch 1 head
						when 2 then l_pos := -1 + 6
						when 3 then l_pos := -1 + 7
						when 4 then l_pos := -1 + 8
						when 5 then l_pos := -1 + 9
						else l_pos := -1 + 10
						end
				when 6 then -- switch 1, case 2
						l_flag := ce_string ("")
						l_pos := -1 + 11 -- switch 1; break
				when 7 then -- switch 1, case 3
						l_flag := ce_string ("r")
						l_pos := -1 + 11 -- switch 1; break
				when 8 then -- switch 1, case 4
						l_flag := ce_string ("s")
						l_pos := -1 + 11 -- switch 1; break
				when 9 then -- switch 1, case 5
						l_flag := ce_string ("P")
						l_pos := -1 + 11 -- switch 1; break
				when 10 then -- switch 1, default:
						Result := ({PC_TAG_RESULT}.TAG_NOTYPE)
						l_return := True
				when 11 then -- switch 1, done: | if done
						if not l_return then
							l_qtag := (create {P_FILENAME_STATIC}).shell_quote (a_tag)
							if l_qtag.is_void then
								l_qtag := a_tag
							end
							sprintf ([l_command, ce_string ("%%s -x%%s %%s"), l_cmd, l_flag, l_qtag]).do_nothing
							if not l_qtag.is_equal (a_tag) then
								free (create {CE_VOID_POINTER}.make_cast (l_qtag))
							end
							l_fp := popen (l_command, ce_string ("r"))
						end
				when 12 then -- if done | if done
					if not l_return and then (not l_fp.is_void) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if (fgets (l_buf___1, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (256).to_natural_32).to_integer_32, l_fp)).is_void then
								l_break := True
							end
							if not (l_return or l_break) then
								if ((create {P_CH_STATIC}).sigs.item).to_boolean then
									if not l_fp.is_equal ((create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.input.file_pointer))) then
										pclose (l_fp).do_nothing
									end
									Result := ({PC_TAG_RESULT}.TAG_INTR)
									l_return := True
								end
								if not l_return then
									if (l_buf___1[((strlen (l_buf___1)) - {NATURAL_32} 1).to_integer_32]).to_integer_32 = 10 then
										l_buf___1[((strlen (l_buf___1)) - {NATURAL_32} 1).to_integer_32] := (0).to_integer_8
									else
										from
										until
											(l_break)
										loop
											l_break := False
											l_c := fgetc (l_fp)
											if l_c /= 10 then
												if l_c = -1 then
													l_break := True
												end
											else
												l_break := True
											end
										end
										l_break := False
									end
									if (getentry (l_buf___1, l_name, l_file, l_line)).to_boolean then
										l_break := True
									end
									if not (l_return or l_break) then
										l_tp := maketagent (l_name.item, l_file.item, (atoi (l_line.item)).to_integer_32, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer), 0)
										l_tp.item.next := taglist.item.item.tl_first
										l_tp.item.prev := create {CE_CLASS_POINTER [PD_TAG]}.make_cast (taglist.item);
										(taglist.item.item.tl_first).item.prev := l_tp
										taglist.item.item.tl_first := l_tp
										total.item := total.item + 1
									end
								end
							end
						end
						if not l_return then
							l_break := False
							if not l_fp.is_equal ((create {CE_CLASS_POINTER [SD_IO_FILE]}.make_shared (Io.input.file_pointer))) then
								if (pclose (l_fp)).to_boolean then
									curtag.item := create {CE_CLASS_POINTER [PD_TAG]}.make_shared (default_pointer)
									curseq.item := 0
									total.item := curseq.item
									Result := ({PC_TAG_RESULT}.TAG_NOFILE)
									l_return := True
								end
							end
						end
					end
					if not l_return then
						l_tp := taglist.item.item.tl_first
						if l_tp.is_equal (taglist.item) then
							Result := ({PC_TAG_RESULT}.TAG_NOTAG)
							l_return := True
						end
					end
					if not l_return then
						curtag.item := l_tp
						curseq.item := 1
						Result := ({PC_TAG_RESULT}.TAG_FOUND)
					end
				when 13 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	nextgtag: CE_POINTER [INTEGER_8]
		local
			l_tp: CE_POINTER [PD_TAG]
			l_return: BOOLEAN
		do
			if curtag.item.is_void then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_tp := curtag.item.item.next
				if l_tp.is_equal (taglist.item) then
					if not circular.item.to_boolean then
						Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						curtag.item := taglist.item.item.tl_first
						curseq.item := 1
					end
				else
					curtag.item := l_tp
					curseq.item := curseq.item + 1
				end
			end
			if not l_return then
				Result := (curtag.item.item.tag_file)
			end
		end

	prevgtag: CE_POINTER [INTEGER_8]
		local
			l_tp: CE_POINTER [PD_TAG]
			l_return: BOOLEAN
		do
			if curtag.item.is_void then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_tp := curtag.item.item.prev
				if l_tp.is_equal (taglist.item) then
					if not circular.item.to_boolean then
						Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						curtag.item := taglist.item.item.tl_last
						curseq.item := total.item
					end
				else
					curtag.item := l_tp
					curseq.item := curseq.item - 1
				end
			end
			if not l_return then
				Result := (curtag.item.item.tag_file)
			end
		end

	gtagsearch: INTEGER_32
		local
			l_return: BOOLEAN
		do
			if curtag.item.is_void then
				Result := (-1).to_integer_32
				l_return := True
			end
			if not l_return then
				Result := ((create {P_LINENUM_STATIC}).find_pos (curtag.item.item.tag_linenum))
			end
		end

	getentry (a_buf___1: CE_POINTER [INTEGER_8]; a_tag, a_file, a_line: CE_POINTER [CE_POINTER [INTEGER_8]]): INTEGER_32
		require
			not a_buf___1.is_void
			not a_tag.is_void
		local
			l_p: CE_POINTER [INTEGER_8]
			l_tmp___0: CE_POINTER [INTEGER_8]
			l_tmp___7: CE_POINTER [INTEGER_8]
			l_return, l_break: BOOLEAN
		do
			l_p := a_buf___1
			a_tag.item := l_p
			from
			until
				(l_break)
			loop
				l_break := False
				if (l_p[0]).to_boolean then
					if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
						l_break := True
					end
				else
					l_break := True
				end
				if not (l_break) then
					l_p := l_p + 1
				end
			end
			l_break := False
			if (l_p[0]).to_integer_32 = 0 then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_tmp___0 := l_p
				l_p := l_p + 1
				l_tmp___0[0] := (0).to_integer_8
				from
				until
					(l_break)
				loop
					l_break := False
					if (l_p[0]).to_boolean then
						if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_p := l_p + 1
					end
				end
				l_break := False
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (2_048)).to_boolean then
					from
					until
						(l_break)
					loop
						l_break := False
						if (l_p[0]).to_boolean then
							if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
								l_break := True
							end
						else
							l_break := True
						end
						if not (l_break) then
							l_p := l_p + 1
						end
					end
					l_break := False
					from
					until
						(l_break)
					loop
						l_break := False
						if (l_p[0]).to_boolean then
							if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
								l_break := True
							end
						else
							l_break := True
						end
						if not (l_break) then
							l_p := l_p + 1
						end
					end
					l_break := False
				end
				if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (2_048)).to_boolean then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				a_line.item := l_p
				a_line.item := l_p
				from
				until
					(l_break)
				loop
					l_break := False
					if (l_p[0]).to_boolean then
						if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_p := l_p + 1
					end
				end
				l_break := False
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				l_tmp___7 := l_p
				l_p := l_p + 1
				l_tmp___7[0] := (0).to_integer_8
				from
				until
					(l_break)
				loop
					l_break := False
					if (l_p[0]).to_boolean then
						if not ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_p := l_p + 1
					end
				end
				l_break := False
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				a_file.item := l_p
				a_file.item := l_p
				from
				until
					(l_break)
				loop
					l_break := False
					if (l_p[0]).to_boolean then
						if ((((ctype_b_loc).item[(l_p[0]).to_integer_32]).to_integer_32).bit_and (8_192)).to_boolean then
							l_break := True
						end
					else
						l_break := True
					end
					if not (l_break) then
						l_p := l_p + 1
					end
				end
				l_break := False
				if (l_p[0]).to_integer_32 = 0 then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				l_p[0] := (0).to_integer_8
				if (strlen (a_tag.item)).to_boolean then
					if (strlen (a_line.item)).to_boolean then
						if (strlen (a_file.item)).to_boolean then
							if (atoi (a_line.item)) > 0 then
								Result := 0
								l_return := True
							end
						end
					end
				end
			end
			if not l_return then
				Result := (-1)
			end
		end

end
