class
	P_PTIMER_STATIC

inherit
	CE_UTILITIES
	S_UNISTD_STATIC
	S_TIME_STATIC
	S_BITS_ERRNO_STATIC
	S_STRING_STATIC
	S_LIBINTL_STATIC
	S_STDLIB_STATIC

feature {ANY} -- public attributes

	posix_clock_id: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	posix_clock_resolution: CE_STATIC [REAL_64]
		once ("PROCESS")
		end

	clocks: CE_STATIC [CE_ARRAY [PD_CLOCKS]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_CLOCKS]}.make_initialized (<<
					create {PD_CLOCKS}.make_initialized ([1, 149]),
					create {PD_CLOCKS}.make_initialized ([0, -1])
				>>)
		end

	init_done: CE_STATIC [BOOLEAN]
		once ("PROCESS")
		end

feature {ANY} -- public routines

	posix_init
		local
			l_i: NATURAL_32
			l_r: CE_ARRAY [SD_TIMESPEC]
			l_tmp___2: CE_POINTER [INTEGER_8]
			l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [SD_TIMESPEC]} l_r.make (<<1>>)

			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < ((create {PD_CLOCKS}.make).structure_size.to_natural_32 * (2).to_natural_32) // (create {PD_CLOCKS}.make).structure_size.to_natural_32))
			loop
				l_break := False
				l_continue := False
				if clocks.item[l_i.to_integer_32].sysconf_name /= -1 then
					if (sysconf (clocks.item[l_i.to_integer_32].sysconf_name)) < 0 then
						l_continue := True
					end
				end
				if not (l_break or l_continue) then
					if (clock_getres (clocks.item[l_i.to_integer_32].id, l_r)) < 0 then
						l_continue := True
					end
				end
				if not (l_break or l_continue) then
					posix_clock_id.item := clocks.item[l_i.to_integer_32].id
					posix_clock_resolution.item := (l_r.item.tv_sec).to_double + (l_r.item.tv_nsec).to_double / 1.e9
					if posix_clock_resolution.item = (0).to_double then
						posix_clock_resolution.item := 1.e-3
					end
					l_break := True
				end
				if (not l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			l_continue := False
			if l_i = ((create {PD_CLOCKS}.make).structure_size.to_natural_32 * (2).to_natural_32) // (create {PD_CLOCKS}.make).structure_size.to_natural_32 then
				l_tmp___2 := strerror ((errno_location).item);
				(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("Cannot get REALTIME clock frequency: %%s%N"))), l_tmp___2])
				posix_clock_id.item := 0
				posix_clock_resolution.item := 1.e-3
			end
		end

	posix_resolution: REAL_64
		do
			Result := posix_clock_resolution.item
		end

	ptimer_new: CE_POINTER [PD_PTIMER]
		local
			l_pt: CE_POINTER [PD_PTIMER]
		do
			l_pt := create {CE_CLASS_POINTER [PD_PTIMER]}.make_cast (((create {P_XMALLOC_STATIC}).xcalloc ((1).to_natural_32, (create {PD_PTIMER}.make).structure_size.to_natural_32)))
			if not init_done.item then
				init_done.item := (1).to_boolean
				posix_init
			end
			l_pt.item.ptimer_reset
			Result := l_pt
		end

	ptimer_destroy (a_pt: CE_POINTER [PD_PTIMER])
		do
			free (create {CE_VOID_POINTER}.make_cast (a_pt))
		end

	ptimer_resolution: REAL_64
		do
			Result := (posix_resolution)
		end

end
