class
	P_RETR_STATIC

inherit
	CE_UTILITIES
	S_STRING_STATIC
	S_STDIO_STATIC
	S_ASSERT_STATIC
	S_LIBINTL_STATIC
	S_STDLIB_STATIC
	S_GCC_BUILTIN_STATIC
	S_UNISTD_STATIC
	S_BITS_ERRNO_STATIC

feature {ANY} -- public attributes

	total_downloaded_bytes: CE_STATIC [INTEGER_64]
		once ("PROCESS")
		end

	total_download_time: CE_STATIC [REAL_64]
		once ("PROCESS")
		end

	output_stream: CE_STATIC [CE_POINTER [SD_IO_FILE]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [SD_IO_FILE]}.make
		end

	output_stream_regular: CE_STATIC [BOOLEAN]
		once ("PROCESS")
		end

	res: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_ARRAY}.make (<<20>>)
		end

	rate_names: CE_STATIC [CE_ARRAY [CE_POINTER [INTEGER_8]]]
		once ("PROCESS")
			Result.item := create {CE_POINTER_ARRAY [INTEGER_8]}.make_initialized (<<
					ce_string ("B/s"),
					ce_string ("KB/s"),
					ce_string ("MB/s"),
					ce_string ("GB/s")
				>>)
		end

	first_retrieval: CE_STATIC [BOOLEAN]
		once ("PROCESS")
			Result.item := (1).to_boolean
		end

	rewritten_storage: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_ARRAY}.make (<<1_024>>)
		end

feature {ANY} -- public routines

	line_terminator (a_start, a_peeked: CE_POINTER [INTEGER_8]; a_peeklen: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_p___0: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
		do
			l_p___0 := create {CE_INTEGER_8_POINTER}.make_cast ((memchr (create {CE_VOID_POINTER}.make_cast (a_peeked), ('%N').code, a_peeklen.to_natural_32)))
			if (not l_p___0.is_void) then
				Result := (l_p___0 + 1)
				l_return := True
			end
			if not l_return then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
			end
		end

	retr_rate (a_bytes: INTEGER_64; a_secs: REAL_64): CE_POINTER [INTEGER_8]
		local
			l_units: CE_ARRAY [INTEGER_32]
			l_dlrate: REAL_64
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_units.make (<<1>>)

			l_dlrate := (calc_rate (a_bytes, a_secs, l_units))
			if l_dlrate >= 99.95 then
				l_tmp___1 := 0
			else
				if l_dlrate >= 9.995 then
					l_tmp___0 := 1
				else
					l_tmp___0 := 2
				end
				l_tmp___1 := l_tmp___0
			end
			sprintf ([res.item, ce_string ("%%.*f %%s"), l_tmp___1, l_dlrate, rate_names.item[l_units.item]]).do_nothing
			Result := res.item
		end

	calc_rate (a_bytes: INTEGER_64; a_secs: REAL_64; a_units: CE_POINTER [INTEGER_32]): REAL_64
		require
			not a_units.is_void
		local
			l_secs: REAL_64
			l_dlrate: REAL_64
		do
			l_secs := a_secs
			if not (l_secs >= (0).to_double) then
				assert_fail (ce_string ("secs >= 0"), ce_string ("retr.c"), {NATURAL_32} 562, ce_string ("calc_rate"))
			end
			if not (a_bytes >= {INTEGER_64} 0) then
				assert_fail (ce_string ("bytes >= 0"), ce_string ("retr.c"), {NATURAL_32} 563, ce_string ("calc_rate"))
			end
			if l_secs = (0).to_double then
				l_secs := ((create {P_PTIMER_STATIC}).ptimer_resolution) / 2.0
			end
			l_dlrate := a_bytes.to_double / l_secs
			if l_dlrate < 1024.0 then
				a_units.item := 0
			elseif l_dlrate < 1024.0 * 1024.0 then
				a_units.item := 1
				l_dlrate := l_dlrate / (1024.0)
			elseif l_dlrate < (1024.0 * 1024.0) * 1024.0 then
				a_units.item := 2
				l_dlrate := l_dlrate / (1024.0 * 1024.0)
			else
				a_units.item := 3
				l_dlrate := l_dlrate / ((1024.0 * 1024.0) * 1024.0)
			end
			Result := l_dlrate
		end

	retrieve_url (a_orig_parsed: CE_POINTER [PD_URL]; a_origurl: CE_POINTER [INTEGER_8]; a_file, a_newloc: CE_POINTER [CE_POINTER [INTEGER_8]]; a_refurl: CE_POINTER [INTEGER_8]; a_dt: CE_POINTER [INTEGER_32]; a_recursive: BOOLEAN; a_iri: CE_POINTER [PD_IRI]; a_register_status: BOOLEAN): NATURAL_32
		local
			l_refurl: CE_POINTER [INTEGER_8]
			l_dt: CE_POINTER [INTEGER_32]
			l_recursive: BOOLEAN
			l_result: NATURAL_32
			l_url: CE_POINTER [INTEGER_8]
			l_location_changed: BOOLEAN
			l_iri_fallbacked: BOOLEAN
			l_dummy: CE_ARRAY [INTEGER_32]
			l_mynewloc: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_proxy: CE_POINTER [INTEGER_8]
			l_u: CE_POINTER [PD_URL]
			l_proxy_url: CE_POINTER [PD_URL]
			l_up_error_code: CE_ARRAY [INTEGER_32]
			l_local_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_redirection_count: INTEGER_32
			l_post_data_suspended: BOOLEAN
			l_saved_post_data: CE_POINTER [INTEGER_8]
			l_saved_post_file_name: CE_POINTER [INTEGER_8]
			l_pi: CE_POINTER [PD_IRI]
			l_error___0: CE_POINTER [INTEGER_8]
			l_oldrec: BOOLEAN
			l_glob: BOOLEAN
			l_construced_newloc: CE_POINTER [INTEGER_8]
			l_newloc_parsed: CE_POINTER [PD_URL]
			l_error___1: CE_POINTER [INTEGER_8]
			l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_dummy.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_mynewloc.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_up_error_code.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_local_file.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_refurl := a_refurl
					l_dt := a_dt
					l_recursive := a_recursive
					l_iri_fallbacked := (0).to_boolean
					l_u := a_orig_parsed
					l_redirection_count := 0
					l_post_data_suspended := (0).to_boolean
					l_saved_post_data := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					l_saved_post_file_name := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					if l_dt.is_void then
						l_dt := l_dummy
						l_dummy.item := 0
					end
					l_url := (create {P_XMALLOC_STATIC}).xstrdup (a_origurl)
					if (not a_newloc.is_void) then
						a_newloc.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					end
					if (not a_file.is_void) then
						a_file.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					end
					if l_refurl.is_void then
						l_refurl := (create {P_OPTIONS_STATIC}).opt.item.item.referer
					end
				when 1 then -- redirected:
					l_result := {PC_UERR_T}.NOCONERROR
					l_mynewloc.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					l_local_file.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					l_proxy_url := create {CE_CLASS_POINTER [PD_URL]}.make_shared (default_pointer)
					l_proxy := getproxy (l_u)
					if l_proxy.is_void then
						l_pos := -1 + 4 -- skip if block
					end
				when 2 then -- if:
						l_pi := (create {P_IRI_STATIC}).dummy_iri.item
						l_pi.item.utf8_encode := (0).to_boolean
						l_proxy_url := (create {P_URL_STATIC}).url_parse (l_proxy, l_up_error_code, create {CE_CLASS_POINTER [PD_IRI]}.make_shared (default_pointer), (1).to_boolean)
						if l_proxy_url.is_void then
							l_error___0 := ((create {P_URL_STATIC}).url_error (l_proxy, l_up_error_code.item));
							(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("Error parsing proxy URL %%s: %%s.%N"))), l_proxy, l_error___0])
							free (create {CE_VOID_POINTER}.make_cast (l_url))
							free (create {CE_VOID_POINTER}.make_cast (l_error___0))
							from
							until
								(l_break)
							loop
								l_break := False
								if l_post_data_suspended then
									(create {P_OPTIONS_STATIC}).opt.item.item.post_data := l_saved_post_data;
									(create {P_OPTIONS_STATIC}).opt.item.item.post_file_name := l_saved_post_file_name
									l_post_data_suspended := (0).to_boolean
								end
								l_break := True
							end
							l_break := False
							l_result := {PC_UERR_T}.PROXERR
							l_pos := -1 + 24 -- goto bail
						end
				when 3 then
						if l_proxy_url.item.scheme /= {NATURAL_32} 0 then
							if l_proxy_url.item.scheme /= l_u.item.scheme then
								(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("Error in proxy URL %%s: Must be HTTP.%N"))), l_proxy])
								l_proxy_url.item.url_free
								free (create {CE_VOID_POINTER}.make_cast (l_url))
								from
								until
									(l_break)
								loop
									l_break := False
									if l_post_data_suspended then
										(create {P_OPTIONS_STATIC}).opt.item.item.post_data := l_saved_post_data;
										(create {P_OPTIONS_STATIC}).opt.item.item.post_file_name := l_saved_post_file_name
										l_post_data_suspended := (0).to_boolean
									end
									l_break := True
								end
								l_break := False
								l_result := {PC_UERR_T}.PROXERR
								l_pos := -1 + 24 -- goto bail
							end
						end
				when 4 then
					if l_u.item.scheme /= {NATURAL_32} 0 then
						l_pos := -1 + 7 -- skip if block
					end
				when 5 then -- if:
						l_result := (create {P_HTTP_STATIC}).http_loop (l_u, l_mynewloc, l_local_file, l_refurl, l_dt, l_proxy_url, a_iri)
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else:
						if l_proxy_url.is_void then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_proxy_url.item.scheme = {NATURAL_32} 0 then
							l_result := (create {P_HTTP_STATIC}).http_loop (l_u, l_mynewloc, l_local_file, l_refurl, l_dt, l_proxy_url, a_iri)
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_u.item.scheme = {NATURAL_32} 1 then
							l_oldrec := l_recursive
							l_glob := (create {P_OPTIONS_STATIC}).opt.item.item.ftp_glob
							if l_redirection_count.to_boolean then
								l_glob := (0).to_boolean
								l_oldrec := l_glob
							end
							l_result := l_u.item.ftp_loop (l_dt, l_proxy_url, l_recursive, l_glob)
							l_recursive := l_oldrec
							if l_redirection_count.to_boolean then
								if (not (l_local_file.item).is_void) then
									if l_u.item.scheme = {NATURAL_32} 1 then
										if ((create {P_UTILS_STATIC}).has_html_suffix_p (l_local_file.item)) then
											l_dt.item := (l_dt.item).bit_or ((1))
										end
									end
								end
							end
						end
				when 11 then -- if done
					if (not l_proxy_url.is_void) then
						l_proxy_url.item.url_free
						l_proxy_url := create {CE_CLASS_POINTER [PD_URL]}.make_shared (default_pointer)
					end
					l_location_changed := (l_result = {NATURAL_32} 6)
					if not l_location_changed then
						l_pos := -1 + 15 -- skip if block
					end
				when 12 then -- if:
						if (l_mynewloc.item).is_void then
							assert_fail (ce_string ("mynewloc != ((void *)0)"), ce_string ("retr.c"), {NATURAL_32} 729, ce_string ("retrieve_url"))
						end
						if (not (l_local_file.item).is_void) then
							free (create {CE_VOID_POINTER}.make_cast (l_local_file.item))
						end
						l_construced_newloc := (create {P_URL_STATIC}).uri_merge (l_url, l_mynewloc.item)
						free (create {CE_VOID_POINTER}.make_cast (l_mynewloc.item))
						l_mynewloc.item := l_construced_newloc
						a_iri.item.utf8_encode := (create {P_OPTIONS_STATIC}).opt.item.item.enable_iri
						if (not (a_iri.item.orig_url).is_void) then
							free (create {CE_VOID_POINTER}.make_cast (a_iri.item.orig_url))
						end
						l_newloc_parsed := (create {P_URL_STATIC}).url_parse (l_mynewloc.item, l_up_error_code, a_iri, (1).to_boolean)
						if l_newloc_parsed.is_void then
							l_error___1 := ((create {P_URL_STATIC}).url_error (l_mynewloc.item, l_up_error_code.item));
							(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, ce_string ("%%s: %%s.%N"), ((create {P_LOG_STATIC}).escnonprint_uri (l_mynewloc.item)), l_error___1])
							if not a_orig_parsed.is_equal (l_u) then
								l_u.item.url_free
							end
							free (create {CE_VOID_POINTER}.make_cast (l_url))
							free (create {CE_VOID_POINTER}.make_cast (l_mynewloc.item))
							free (create {CE_VOID_POINTER}.make_cast (l_error___1))
							from
							until
								(l_break)
							loop
								l_break := False
								if l_post_data_suspended then
									(create {P_OPTIONS_STATIC}).opt.item.item.post_data := l_saved_post_data;
									(create {P_OPTIONS_STATIC}).opt.item.item.post_file_name := l_saved_post_file_name
									l_post_data_suspended := (0).to_boolean
								end
								l_break := True
							end
							l_break := False
							l_pos := -1 + 24 -- goto bail
						end
				when 13 then
						free (create {CE_VOID_POINTER}.make_cast (l_mynewloc.item))
						l_mynewloc.item := (create {P_XMALLOC_STATIC}).xstrdup (l_newloc_parsed.item.url)
						l_redirection_count := l_redirection_count + 1
						if l_redirection_count > (create {P_OPTIONS_STATIC}).opt.item.item.max_redirect then
							(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("%%d redirections exceeded.%N"))), (create {P_OPTIONS_STATIC}).opt.item.item.max_redirect])
							l_newloc_parsed.item.url_free
							if not a_orig_parsed.is_equal (l_u) then
								l_u.item.url_free
							end
							free (create {CE_VOID_POINTER}.make_cast (l_url))
							free (create {CE_VOID_POINTER}.make_cast (l_mynewloc.item))
							from
							until
								(l_break)
							loop
								l_break := False
								if l_post_data_suspended then
									(create {P_OPTIONS_STATIC}).opt.item.item.post_data := l_saved_post_data;
									(create {P_OPTIONS_STATIC}).opt.item.item.post_file_name := l_saved_post_file_name
									l_post_data_suspended := (0).to_boolean
								end
								l_break := True
							end
							l_break := False
							l_result := {PC_UERR_T}.WRONGCODE
							l_pos := -1 + 24 -- goto bail
						end
				when 14 then
						free (create {CE_VOID_POINTER}.make_cast (l_url))
						l_url := l_mynewloc.item
						if not a_orig_parsed.is_equal (l_u) then
							l_u.item.url_free
						end
						l_u := l_newloc_parsed
						if not l_post_data_suspended then
							from
							until
								(l_break)
							loop
								l_break := False
								l_post_data_suspended := (1).to_boolean
								l_saved_post_data := (create {P_OPTIONS_STATIC}).opt.item.item.post_data
								l_saved_post_file_name := (create {P_OPTIONS_STATIC}).opt.item.item.post_file_name;
								(create {P_OPTIONS_STATIC}).opt.item.item.post_data := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer);
								(create {P_OPTIONS_STATIC}).opt.item.item.post_file_name := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
								l_break := True
							end
							l_break := False
						end
						l_pos := -1 + 1 -- goto redirected
				when 15 then
					if not ((l_dt.item).bit_and (2)).to_boolean then
						if a_iri.item.utf8_encode then
							a_iri.item.utf8_encode := (0).to_boolean
							if not a_orig_parsed.is_equal (l_u) then
								l_u.item.url_free
							end
							l_u := (create {P_URL_STATIC}).url_parse (a_origurl, create {CE_INTEGER_32_POINTER}.make_shared (default_pointer), a_iri, (1).to_boolean)
							if (not l_u.is_void) then
								from
								until
									(l_break)
								loop
									l_break := False
									if (builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)).to_boolean then
										(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("[IRI fallbacking to non-utf8 for %%s%N"), ((create {P_QUOTE_STATIC}).quote (l_url))])
									end
									l_break := True
								end
								l_break := False
								l_url := (create {P_XMALLOC_STATIC}).xstrdup (l_u.item.url)
								l_iri_fallbacked := (1).to_boolean
								l_pos := -1 + 1 -- goto redirected
							else
								from
								until
									(l_break)
								loop
									l_break := False
									if (builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)).to_boolean then
										(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("[Couldn%'t fallback to non-utf8 for %%s%N"), ((create {P_QUOTE_STATIC}).quote (l_url))])
									end
									l_break := True
								end
								l_break := False
							end
						end
					end
				when 16 then
					if (not (l_local_file.item).is_void) then
						if ((l_dt.item).bit_and (2)).to_boolean then
							(create {P_CONVERT_STATIC}).register_download (l_u.item.url, l_local_file.item)
							if l_redirection_count.to_boolean then
								if 0 /= (strcmp (a_origurl, l_u.item.url)) then
									(create {P_CONVERT_STATIC}).register_redirection (a_origurl, l_u.item.url)
								end
							end
							if ((l_dt.item).bit_and (1)).to_boolean then
								(create {P_CONVERT_STATIC}).register_html (l_u.item.url, l_local_file.item)
							end
							if ((l_dt.item).bit_and (2)).to_boolean then
								(create {P_CONVERT_STATIC}).register_download (l_u.item.url, l_local_file.item)
								if l_redirection_count.to_boolean then
									if 0 /= (strcmp (a_origurl, l_u.item.url)) then
										(create {P_CONVERT_STATIC}).register_redirection (a_origurl, l_u.item.url)
									end
								end
								if ((l_dt.item).bit_and (1)).to_boolean then
									(create {P_CONVERT_STATIC}).register_html (l_u.item.url, l_local_file.item)
								end
								if ((l_dt.item).bit_and (64)).to_boolean then
									(create {P_CONVERT_STATIC}).register_css (l_u.item.url, l_local_file.item)
								end
							end
						end
					end
					if (not a_file.is_void) then
						if (not (l_local_file.item).is_void) then
							a_file.item := l_local_file.item
						else
							a_file.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						end
					elseif (not (l_local_file.item).is_void) then
						free (create {CE_VOID_POINTER}.make_cast (l_local_file.item))
					end
					if not a_orig_parsed.is_equal (l_u) then
						l_u.item.url_free
					end
					if not (l_redirection_count.to_boolean) then
						l_pos := -1 + 19 -- skip if block
					end
				when 17 then -- if:
						l_pos := -1 + 20 -- goto _L___0
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 23
				when 19 then -- else:
						if not l_iri_fallbacked then
							l_pos := -1 + 22 -- skip if block
						end
				when 20 then -- if: | _L___0:
						if (not a_newloc.is_void) then
							a_newloc.item := l_url
						else
							free (create {CE_VOID_POINTER}.make_cast (l_url))
						end
				when 21 then -- if block done, skip else block:
						l_pos := -1 + 23
				when 22 then -- else:
						if (not a_newloc.is_void) then
							a_newloc.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						end
						free (create {CE_VOID_POINTER}.make_cast (l_url))
				when 23 then -- if done
					from
					until
						(l_break)
					loop
						l_break := False
						if l_post_data_suspended then
							(create {P_OPTIONS_STATIC}).opt.item.item.post_data := l_saved_post_data;
							(create {P_OPTIONS_STATIC}).opt.item.item.post_file_name := l_saved_post_file_name
							l_post_data_suspended := (0).to_boolean
						end
						l_break := True
					end
					l_break := False
				when 24 then -- bail:
					if a_register_status then
						(create {P_EXITS_STATIC}).inform_exit_status (l_result)
					end
					Result := l_result
				when 25 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	retrieve_from_file (a_file: CE_POINTER [INTEGER_8]; a_html: BOOLEAN; a_count: CE_POINTER [INTEGER_32]): NATURAL_32
		require
			not a_count.is_void
		local
			l_html: BOOLEAN
			l_status: NATURAL_32
			l_url_list: CE_POINTER [PD_URLPOS]
			l_cur_url: CE_POINTER [PD_URLPOS]
			l_iri: CE_POINTER [PD_IRI]
			l_input_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_url: CE_POINTER [INTEGER_8]
			l_dt: CE_ARRAY [INTEGER_32]
			l_url_err: CE_ARRAY [INTEGER_32]
			l_status___0: NATURAL_32
			l_url_parsed: CE_POINTER [PD_URL]
			l_error___0: CE_POINTER [INTEGER_8]
			l_filename: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_new_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_dt___0: CE_ARRAY [INTEGER_32]
			l_tmpiri: CE_POINTER [PD_IRI]
			l_parsed_url: CE_POINTER [PD_URL]
			l_old_follow_ftp: INTEGER_32
			l_tmp___4: CE_POINTER [PD_URL]
			l_tmp___5: CE_POINTER [PD_URL]
			l_return, l_break, l_continue: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_POINTER_ARRAY [INTEGER_8]} l_input_file.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_dt.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_url_err.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_filename.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_new_file.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_dt___0.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_html := a_html
					l_iri := (create {P_IRI_STATIC}).dummy_iri.item
					l_input_file.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					l_url := a_file
					l_status := {PC_UERR_T}.RETROK
					a_count.item := 0
					if ((create {P_URL_STATIC}).url_has_scheme (l_url)) then
						l_url_parsed := ((create {P_URL_STATIC}).url_parse (l_url, l_url_err, l_iri, (1).to_boolean))
						if l_url_parsed.is_void then
							l_error___0 := ((create {P_URL_STATIC}).url_error (l_url, l_url_err.item));
							(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, ce_string ("%%s: %%s.%N"), l_url, l_error___0])
							free (create {CE_VOID_POINTER}.make_cast (l_error___0))
							Result := ({PC_UERR_T}.URLERROR)
							l_return := True
						end
						if not l_return then
							if ((create {P_OPTIONS_STATIC}).opt.item.item.base_href).is_void then
								(create {P_OPTIONS_STATIC}).opt.item.item.base_href := (create {P_XMALLOC_STATIC}).xstrdup (l_url)
							end
							l_status___0 := retrieve_url (l_url_parsed, l_url, l_input_file, create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer), create {CE_INTEGER_8_POINTER}.make_shared (default_pointer), l_dt, (0).to_boolean, l_iri, (1).to_boolean)
							if l_status___0 /= {NATURAL_32} 31 then
								Result := l_status___0
								l_return := True
							end
						end
						if not l_return then
							if ((l_dt.item).bit_and (1)).to_boolean then
								l_html := (1).to_boolean
							end
							l_iri.item.utf8_encode := (create {P_OPTIONS_STATIC}).opt.item.item.enable_iri
							if (not (l_iri.item.orig_url).is_void) then
								free (create {CE_VOID_POINTER}.make_cast (l_iri.item.orig_url))
							end
							l_iri.item.orig_url := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						end
					else
						l_input_file.item := a_file
					end
					if l_return then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if l_html then
							l_url_list := ((create {P_HTML_URL_STATIC}).get_urls_html (l_input_file.item, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer), create {CE_BOOLEAN_POINTER}.make_shared (default_pointer), l_iri))
						else
							l_url_list := ((create {P_HTML_URL_STATIC}).get_urls_file (l_input_file.item))
						end
						l_cur_url := l_url_list
				when 2 then -- loop condition:
						if not (not (l_return or l_break) and then (not l_cur_url.is_void)) then
							l_pos := -1 + 19 -- skip loop
						end
				when 3 then -- loop body:
						l_break := False
						l_continue := False
						l_filename.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						l_new_file.item := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
						l_tmpiri := (create {P_IRI_STATIC}).dummy_iri.item
						l_parsed_url := create {CE_CLASS_POINTER [PD_URL]}.make_shared (default_pointer)
						if (l_cur_url.item.ignore_when_downloading).to_boolean then
							l_continue := True
						end
						if not (l_return or l_break or l_continue) and then (((create {P_OPTIONS_STATIC}).opt.item.item.quota).to_boolean) then
							if total_downloaded_bytes.item > (create {P_OPTIONS_STATIC}).opt.item.item.quota then
								l_status := {PC_UERR_T}.QUOTEXC
								l_break := True
							end
						end
						if (l_return or l_break or l_continue) then
							l_pos := -1 + 18 -- skip if block
						end
				when 4 then -- if:
						if (create {P_OPTIONS_STATIC}).opt.item.item.enable_iri then
							l_parsed_url := (create {P_URL_STATIC}).url_parse ((l_cur_url.item.url).item.url, create {CE_INTEGER_32_POINTER}.make_shared (default_pointer), l_tmpiri, (1).to_boolean)
						end
						if not ((create {P_OPTIONS_STATIC}).opt.item.item.recursive) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L___1
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 7 then -- else:
						if not ((create {P_OPTIONS_STATIC}).opt.item.item.page_requisites) then
							l_pos := -1 + 16 -- skip if block
						end
				when 8 then -- if: | _L___1:
						if (l_cur_url.item.url).item.scheme = {NATURAL_32} 1 then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						l_pos := -1 + 12 -- goto _L___0
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 11 then -- else:
						if (getproxy (l_cur_url.item.url)).is_void then
							l_pos := -1 + 14 -- skip if block
						end
				when 12 then -- if: | _L___0:
						l_old_follow_ftp := ((create {P_OPTIONS_STATIC}).opt.item.item.follow_ftp).to_integer
						if (l_cur_url.item.url).item.scheme = {NATURAL_32} 1 then
							(create {P_OPTIONS_STATIC}).opt.item.item.follow_ftp := (1).to_boolean
						end
						if (not l_parsed_url.is_void) then
							l_tmp___4 := l_parsed_url
						else
							l_tmp___4 := l_cur_url.item.url
						end
						l_status := l_tmp___4.item.retrieve_tree (l_tmpiri);
						(create {P_OPTIONS_STATIC}).opt.item.item.follow_ftp := l_old_follow_ftp.to_boolean
				when 13 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 14 then -- else:
						l_pos := -1 + 16 -- goto _L
				when 15 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 17
				when 16 then -- else: | _L:
						if (not l_parsed_url.is_void) then
							l_tmp___5 := l_parsed_url
						else
							l_tmp___5 := l_cur_url.item.url
						end
						l_status := retrieve_url (l_tmp___5, (l_cur_url.item.url).item.url, l_filename, l_new_file, create {CE_INTEGER_8_POINTER}.make_shared (default_pointer), l_dt___0, (create {P_OPTIONS_STATIC}).opt.item.item.recursive, l_tmpiri, (1).to_boolean)
				when 17 then -- if done
						if (not l_parsed_url.is_void) then
							l_parsed_url.item.url_free
						end
						if (not (l_filename.item).is_void) then
							if (create {P_OPTIONS_STATIC}).opt.item.item.delete_after then
								if ((create {P_UTILS_STATIC}).file_exists_p (l_filename.item)) then
									from
									until
										(l_break)
									loop
										l_break := False
										if (builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)).to_boolean then
											(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("Removing file due to --delete-after in retrieve_from_file():%N")])
										end
										l_break := True
									end
									l_break := False;
									(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, (gettext (ce_string ("Removing %%s.%N"))), l_filename.item])
									if (unlink (l_filename.item)).to_boolean then
										(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, ce_string ("unlink: %%s%N"), (strerror ((errno_location).item))])
									end
									l_dt___0.item := (l_dt___0.item).bit_and ((-3))
								end
							end
						end
						if (not (l_new_file.item).is_void) then
							free (create {CE_VOID_POINTER}.make_cast (l_new_file.item))
						end
						if (not (l_filename.item).is_void) then
							free (create {CE_VOID_POINTER}.make_cast (l_filename.item))
						end
				when 18 then -- if done
						if (not l_return and not l_break) then
							l_cur_url := l_cur_url.item.next
							a_count.item := (a_count.item) + 1
						end
						l_pos := -1 + 2 -- back to loop condition
				when 19 then -- loop done
						l_break := False
						l_continue := False
						free_urlpos (l_url_list)
						Result := l_status
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	printwhat (a_n1, a_n2: INTEGER_32)
		local
			l_tmp___1: CE_POINTER [INTEGER_8]
		do
			if a_n1 = a_n2 then
				l_tmp___1 := (gettext (ce_string ("Giving up.%N%N")))
			else
				l_tmp___1 := (gettext (ce_string ("Retrying.%N%N")))
			end
			(create {P_LOG_STATIC}).logputs ({PC_LOG_OPTIONS}.LOG_VERBOSE, l_tmp___1)
		end

	sleep_between_retrievals (a_count: INTEGER_32)
		local
			l_waitsecs: REAL_64
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if first_retrieval.item then
						first_retrieval.item := (0).to_boolean
						l_return := True
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if not (((create {P_OPTIONS_STATIC}).opt.item.item.waitretry).truncated_to_integer.to_boolean) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_count > 1 then
							if a_count.to_double <= (create {P_OPTIONS_STATIC}).opt.item.item.waitretry then
								(create {P_UTILS_STATIC}).xsleep ((a_count - 1).to_double)
							else
								(create {P_UTILS_STATIC}).xsleep ((create {P_OPTIONS_STATIC}).opt.item.item.waitretry)
							end
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if ((create {P_OPTIONS_STATIC}).opt.item.item.wait).truncated_to_integer.to_boolean then
							if not (create {P_OPTIONS_STATIC}).opt.item.item.random_wait then
								(create {P_UTILS_STATIC}).xsleep ((create {P_OPTIONS_STATIC}).opt.item.item.wait)
							elseif a_count > 1 then
								(create {P_UTILS_STATIC}).xsleep ((create {P_OPTIONS_STATIC}).opt.item.item.wait)
							else
								l_waitsecs := (0.5 + ((create {P_UTILS_STATIC}).random_float)) * (create {P_OPTIONS_STATIC}).opt.item.item.wait
								from
								until
									(l_break)
								loop
									l_break := False
									if (builtin_expect (((create {P_OPTIONS_STATIC}).opt.item.item.debug2).to_integer, 0)).to_boolean then
										(create {P_LOG_STATIC}).debug_logprintf ([ce_string ("sleep_between_retrievals: avg=%%f,sleep=%%f%N"), (create {P_OPTIONS_STATIC}).opt.item.item.wait, l_waitsecs])
									end
									l_break := True
								end
								l_break := False;
								(create {P_UTILS_STATIC}).xsleep (l_waitsecs)
							end
						end
				when 5 then -- if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	free_urlpos (a_l: CE_POINTER [PD_URLPOS])
		local
			l_l: CE_POINTER [PD_URLPOS]
			l_next: CE_POINTER [PD_URLPOS]
		do
			l_l := a_l
			from
			until
				l_l.is_void
			loop
				l_next := l_l.item.next
				if (not (l_l.item.url).is_void) then
					(l_l.item.url).item.url_free
				end
				if (not (l_l.item.local_name).is_void) then
					free (create {CE_VOID_POINTER}.make_cast (l_l.item.local_name))
				end
				free (create {CE_VOID_POINTER}.make_cast (l_l))
				l_l := l_next
			end
		end

	getproxy (a_u: CE_POINTER [PD_URL]): CE_POINTER [INTEGER_8]
		local
			l_proxy: CE_POINTER [INTEGER_8]
			l_rewritten_url: CE_POINTER [INTEGER_8]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_proxy := create {CE_INTEGER_8_POINTER}.make_shared (default_pointer)
					if not (create {P_OPTIONS_STATIC}).opt.item.item.use_proxy then
						Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						if (no_proxy_match (a_u.item.host, (create {P_OPTIONS_STATIC}).opt.item.item.no_proxy)) then
							Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						inspect (a_u.item.scheme).to_integer_32 -- switch 1 head
						when 0 then l_pos := -1 + 2
						when 1 then l_pos := -1 + 3
						when 2 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 2 then -- switch 1, case 0
						if (not ((create {P_OPTIONS_STATIC}).opt.item.item.http_proxy).is_void) then
							l_proxy := (create {P_OPTIONS_STATIC}).opt.item.item.http_proxy
						else
							l_proxy := (getenv (ce_string ("http_proxy")))
						end
						l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 1
						if (not ((create {P_OPTIONS_STATIC}).opt.item.item.ftp_proxy).is_void) then
							l_proxy := (create {P_OPTIONS_STATIC}).opt.item.item.ftp_proxy
						else
							l_proxy := (getenv (ce_string ("ftp_proxy")))
						end
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, case 2
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
						if l_proxy.is_void then
							Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
							l_return := True
						elseif not (l_proxy[0]).to_boolean then
							Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
							l_return := True
						end
				when 6 then -- if done
					if not l_return then
						l_rewritten_url := (create {P_URL_STATIC}).rewrite_shorthand_url (l_proxy)
						if (not l_rewritten_url.is_void) then
							strncpy (rewritten_storage.item, l_rewritten_url, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (1_024).to_natural_32)).do_nothing
							rewritten_storage.item[(({PLATFORM}.Integer_8_bytes.to_natural_32 * (1_024).to_natural_32) - {NATURAL_32} 1).to_integer_32] := (('%/0/').code).to_integer_8
							l_proxy := rewritten_storage.item
						end
						Result := l_proxy
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	url_uses_proxy (a_u: CE_POINTER [PD_URL]): BOOLEAN
		local
			l_ret: BOOLEAN
			l_return: BOOLEAN
		do
			if a_u.is_void then
				Result := (0).to_boolean
				l_return := True
			end
			if not l_return then
				l_ret := (not (getproxy (a_u)).is_void)
				Result := l_ret
			end
		end

	no_proxy_match (a_host: CE_POINTER [INTEGER_8]; a_no_proxy: CE_POINTER [CE_POINTER [INTEGER_8]]): BOOLEAN
		do
			if a_no_proxy.is_void then
				Result := (0).to_boolean
			else
				Result := ((create {P_HOST_STATIC}).sufmatch (a_no_proxy, a_host))
			end
		end

	set_local_file (a_file: CE_POINTER [CE_POINTER [INTEGER_8]]; a_default_file: CE_POINTER [INTEGER_8])
		do
			if (not ((create {P_OPTIONS_STATIC}).opt.item.item.output_document).is_void) then
				if output_stream_regular.item then
					a_file.item := (create {P_OPTIONS_STATIC}).opt.item.item.output_document
				end
			else
				a_file.item := a_default_file
			end
		end

feature {ANY} -- routine addresses

	line_terminator_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent line_terminator)
		end

end
