class
	PD_BAR_PROGRESS

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end
	S_ASSERT_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_LIBINTL_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end
	S_WCHAR_STATIC
		redefine
			copy,
			is_equal
		end
	S_LOCALE_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_INTEGER_8_POINTER} buffer.make
			create hist.make
			Precursor
		end

feature {ANY} -- fields

	initial_length: INTEGER_64 assign set_initial_length

	total_length: INTEGER_64 assign set_total_length

	count: INTEGER_64 assign set_count

	last_screen_update: REAL_64 assign set_last_screen_update

	width: INTEGER_32 assign set_width

	buffer: CE_POINTER [INTEGER_8] assign set_buffer

	tick: INTEGER_32 assign set_tick

	hist: PD_BAR_PROGRESS_HIST assign set_hist

	recent_start: REAL_64 assign set_recent_start

	recent_bytes: INTEGER_64 assign set_recent_bytes

	stalled: BOOLEAN assign set_stalled

	last_eta_time: REAL_64 assign set_last_eta_time

	last_eta_value: INTEGER_32 assign set_last_eta_value

feature {ANY} -- field setters

	set_initial_length (a_initial_length: INTEGER_64)
		do
			initial_length := a_initial_length
			update_memory_field ("initial_length")
		end

	set_total_length (a_total_length: INTEGER_64)
		do
			total_length := a_total_length
			update_memory_field ("total_length")
		end

	set_count (a_count: INTEGER_64)
		do
			count := a_count
			update_memory_field ("count")
		end

	set_last_screen_update (a_last_screen_update: REAL_64)
		do
			last_screen_update := a_last_screen_update
			update_memory_field ("last_screen_update")
		end

	set_width (a_width: INTEGER_32)
		do
			width := a_width
			update_memory_field ("width")
		end

	set_buffer (a_buffer: CE_POINTER [INTEGER_8])
		do
			buffer := a_buffer
			update_memory_field ("buffer")
		end

	set_tick (a_tick: INTEGER_32)
		do
			tick := a_tick
			update_memory_field ("tick")
		end

	set_hist (a_hist: PD_BAR_PROGRESS_HIST)
		do
			hist := a_hist
			update_memory_field ("hist")
		end

	set_recent_start (a_recent_start: REAL_64)
		do
			recent_start := a_recent_start
			update_memory_field ("recent_start")
		end

	set_recent_bytes (a_recent_bytes: INTEGER_64)
		do
			recent_bytes := a_recent_bytes
			update_memory_field ("recent_bytes")
		end

	set_stalled (a_stalled: BOOLEAN)
		do
			stalled := a_stalled
			update_memory_field ("stalled")
		end

	set_last_eta_time (a_last_eta_time: REAL_64)
		do
			last_eta_time := a_last_eta_time
			update_memory_field ("last_eta_time")
		end

	set_last_eta_value (a_last_eta_value: INTEGER_32)
		do
			last_eta_value := a_last_eta_value
			update_memory_field ("last_eta_value")
		end

feature {PD_BAR_PROGRESS} -- private attributes

	eta_str: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := ce_string ("  eta %%s%/0/")
		end

	eta_trans: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	bytes_cols_diff: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	short_units: CE_STATIC [CE_ARRAY [CE_POINTER [INTEGER_8]]]
		once ("PROCESS")
			Result.item := create {CE_POINTER_ARRAY [INTEGER_8]}.make_initialized (<<
					ce_string ("B/s"),
					ce_string ("K/s"),
					ce_string ("M/s"),
					ce_string ("G/s")
				>>)
		end

	cached_sep: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	cached_grouping: CE_STATIC [CE_POINTER [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_POINTER}.make
		end

	initialized: CE_STATIC [BOOLEAN]
		once ("PROCESS")
		end

	outbuf: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_8_ARRAY}.make (<<48>>)
		end

feature {ANY} -- public routines

	update_speed_ring (a_howmuch: INTEGER_64; a_dltime: REAL_64)
		local
			l_hist: CE_POINTER [PD_BAR_PROGRESS_HIST]
			l_recent_age: REAL_64
			l_return: BOOLEAN
		do
			l_hist := create {CE_CLASS_POINTER [PD_BAR_PROGRESS_HIST]}.make_shared (c_pointer + (create {PD_BAR_PROGRESS}.make).offset_of ("hist"))
			l_recent_age := a_dltime - recent_start
			Current.recent_bytes := recent_bytes + a_howmuch
			update_class_fields -- Current might have been changed; synchronize
			if l_recent_age < 0.15 then
				l_return := True
			end
			if not l_return and then (a_howmuch = {INTEGER_64} 0) then
				if l_recent_age >= (5).to_double then
					Current.stalled := (1).to_boolean
					update_class_fields -- Current might have been changed; synchronize
					memset (create {CE_VOID_POINTER}.make_cast (l_hist), ('%/0/').code, (create {PD_BAR_PROGRESS_HIST}.make).structure_size.to_natural_32).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					Current.recent_bytes := (0).to_integer_64
					update_class_fields -- Current might have been changed; synchronize
				end
				l_return := True
			end
			if not l_return then
				if stalled then
					Current.stalled := (0).to_boolean
					update_class_fields -- Current might have been changed; synchronize
					l_recent_age := (1).to_double
				end
				l_hist.item.total_time := l_hist.item.total_time - (l_hist.item.times[l_hist.item.pos])
				update_class_fields -- Current might have been changed; synchronize
				l_hist.item.total_bytes := l_hist.item.total_bytes - (l_hist.item.bytes[l_hist.item.pos])
				update_class_fields -- Current might have been changed; synchronize
				l_hist.item.times[l_hist.item.pos] := l_recent_age
				update_class_fields -- Current might have been changed; synchronize
				l_hist.item.bytes[l_hist.item.pos] := recent_bytes
				update_class_fields -- Current might have been changed; synchronize
				l_hist.item.total_time := l_hist.item.total_time + l_recent_age
				update_class_fields -- Current might have been changed; synchronize
				l_hist.item.total_bytes := l_hist.item.total_bytes + recent_bytes
				update_class_fields -- Current might have been changed; synchronize
				Current.recent_start := a_dltime
				update_class_fields -- Current might have been changed; synchronize
				Current.recent_bytes := (0).to_integer_64
				update_class_fields -- Current might have been changed; synchronize
				l_hist.item.pos := (l_hist.item.pos) + 1
				update_class_fields -- Current might have been changed; synchronize
				if l_hist.item.pos = 20 then
					l_hist.item.pos := 0
					update_class_fields -- Current might have been changed; synchronize
				end
			end
		end

	create_image (a_dl_total_time: REAL_64; a_done: BOOLEAN)
		local
			l_p___0: CE_POINTER [INTEGER_8]
			l_size: INTEGER_64
			l_size_grouped: CE_POINTER [INTEGER_8]
			l_tmp: CE_POINTER [INTEGER_8]
			l_size_grouped_len: INTEGER_32
			l_tmp___0: INTEGER_32
			l_size_grouped_diff: INTEGER_32
			l_tmp___1: NATURAL_32
			l_size_grouped_pad: INTEGER_32
			l_hist: CE_POINTER [PD_BAR_PROGRESS_HIST]
			l_dlbytes_size: INTEGER_32
			l_tmp___2: INTEGER_32
			l_progress_size: INTEGER_32
			l_bytes_cols_diff___0: CE_ARRAY [INTEGER_32]
			l_percentage: INTEGER_32
			l_insz: INTEGER_32
			l_dlsz: INTEGER_32
			l_begin: CE_POINTER [INTEGER_8]
			l_i: INTEGER_32
			l_tmp___3: CE_POINTER [INTEGER_8]
			l_tmp___4: CE_POINTER [INTEGER_8]
			l_tmp___5: CE_POINTER [INTEGER_8]
			l_tmp___6: CE_POINTER [INTEGER_8]
			l_tmp___7: CE_POINTER [INTEGER_8]
			l_tmp___8: CE_POINTER [INTEGER_8]
			l_ind: INTEGER_32
			l_i___0: INTEGER_32
			l_pos2: INTEGER_32
			l_tmp___9: CE_POINTER [INTEGER_8]
			l_tmp___10: CE_POINTER [INTEGER_8]
			l_tmp___11: CE_POINTER [INTEGER_8]
			l_tmp___12: CE_POINTER [INTEGER_8]
			l_tmp___13: CE_POINTER [INTEGER_8]
			l_tmp___14: CE_POINTER [INTEGER_8]
			l_tmp___15: CE_POINTER [INTEGER_8]
			l_units: CE_ARRAY [INTEGER_32]
			l_dlquant: INTEGER_64
			l_dltime: REAL_64
			l_dlspeed: REAL_64
			l_tmp___16: REAL_64
			l_tmp___17: INTEGER_32
			l_tmp___18: INTEGER_32
			l_eta: INTEGER_32
			l_bytes_remaining: INTEGER_64
			l_eta_: REAL_64
			l_tmp___19: CE_POINTER [INTEGER_8]
			l_tmp___20: CE_POINTER [INTEGER_8]
			l_nbytes: INTEGER_32
			l_ncols: INTEGER_32
			l_tmp___21: CE_POINTER [INTEGER_8]
			l_tmp___22: NATURAL_32
			l_tmp___23: CE_POINTER [INTEGER_8]
			l_tmp___24: CE_POINTER [INTEGER_8]
			l_tmp___25: CE_POINTER [INTEGER_8]
			l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_bytes_cols_diff___0.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_units.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_p___0 := buffer
					l_size := initial_length + count
					l_tmp := with_thousand_seps (l_size)
					update_class_fields -- Current might have been changed; synchronize
					l_size_grouped := l_tmp
					l_tmp___0 := count_cols (l_size_grouped)
					update_class_fields -- Current might have been changed; synchronize
					l_size_grouped_len := l_tmp___0
					l_tmp___1 := strlen (l_size_grouped)
					update_class_fields -- Current might have been changed; synchronize
					l_size_grouped_diff := (l_tmp___1 - l_size_grouped_len.to_natural_32).to_integer_32
					l_hist := create {CE_CLASS_POINTER [PD_BAR_PROGRESS_HIST]}.make_shared (c_pointer + (create {PD_BAR_PROGRESS}.make).offset_of ("hist"))
					if l_size_grouped_len >= 11 then
						l_tmp___2 := l_size_grouped_len
					else
						l_tmp___2 := 11
					end
					l_dlbytes_size := 1 + l_tmp___2
					l_progress_size := width - (((6 + l_dlbytes_size) + 8) + 14)
					l_bytes_cols_diff___0.item := 0
					update_class_fields -- Current might have been changed; synchronize
					if l_progress_size < 5 then
						l_progress_size := 0
					end
					if total_length > {INTEGER_64} 0 then
						l_percentage := ((100.0 * l_size.to_double) / total_length.to_double).truncated_to_integer
						if not (l_percentage <= 100) then
							assert_fail (ce_string ("percentage <= 100"), ce_string ("progress.c"), {NATURAL_32} 897, ce_string ("create_image"))
							update_class_fields -- Current might have been changed; synchronize
						end
						if l_percentage < 100 then
							sprintf ([l_p___0, ce_string ("%%2d%%%% "), l_percentage]).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						else
							strcpy (l_p___0, ce_string ("100%%")).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						l_p___0 := l_p___0 + (4)
					else
						from
						until
							(l_break)
						loop
							l_break := False
							memcpy (create {CE_VOID_POINTER}.make_cast (l_p___0), create {CE_VOID_POINTER}.make_cast (ce_string ("    ")), (("    ").count + 1).to_natural_32 - {NATURAL_32} 1).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_p___0 := l_p___0 + ((("    ").count + 1).to_natural_32 - {NATURAL_32} 1).to_integer_32
							l_break := True
						end
						l_break := False
					end
					if not (l_progress_size.to_boolean) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if total_length > {INTEGER_64} 0 then
							l_insz := ((initial_length.to_double / total_length.to_double) * l_progress_size.to_double).truncated_to_integer
							l_dlsz := ((l_size.to_double / total_length.to_double) * l_progress_size.to_double).truncated_to_integer
							if not (l_dlsz <= l_progress_size) then
								assert_fail (ce_string ("dlsz <= progress_size"), ce_string ("progress.c"), {NATURAL_32} 920, ce_string ("create_image"))
								update_class_fields -- Current might have been changed; synchronize
							end
							if not (l_insz <= l_dlsz) then
								assert_fail (ce_string ("insz <= dlsz"), ce_string ("progress.c"), {NATURAL_32} 921, ce_string ("create_image"))
								update_class_fields -- Current might have been changed; synchronize
							end
							l_tmp___3 := l_p___0
							l_p___0 := l_p___0 + 1
							l_tmp___3[0] := (('[').code).to_integer_8
							update_class_fields -- Current might have been changed; synchronize
							l_begin := l_p___0
							l_i := 0
							from
							until
								not (l_i < l_insz)
							loop
								l_tmp___4 := l_p___0
								l_p___0 := l_p___0 + 1
								l_tmp___4[0] := (('+').code).to_integer_8
								update_class_fields -- Current might have been changed; synchronize
								l_i := l_i + 1
							end
							l_dlsz := l_dlsz - l_insz
							if l_dlsz > 0 then
								l_i := 0
								from
								until
									not (l_i < l_dlsz - 1)
								loop
									l_tmp___5 := l_p___0
									l_p___0 := l_p___0 + 1
									l_tmp___5[0] := (('=').code).to_integer_8
									update_class_fields -- Current might have been changed; synchronize
									l_i := l_i + 1
								end
								l_tmp___6 := l_p___0
								l_p___0 := l_p___0 + 1
								l_tmp___6[0] := (('>').code).to_integer_8
								update_class_fields -- Current might have been changed; synchronize
							end
							from
							until
								not (l_p___0.distance (l_begin) < l_progress_size)
							loop
								l_tmp___7 := l_p___0
								l_p___0 := l_p___0 + 1
								l_tmp___7[0] := ((' ').code).to_integer_8
								update_class_fields -- Current might have been changed; synchronize
							end
							l_tmp___8 := l_p___0
							l_p___0 := l_p___0 + 1
							l_tmp___8[0] := ((']').code).to_integer_8
							update_class_fields -- Current might have been changed; synchronize
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						if l_progress_size.to_boolean then
							l_ind := tick \\ (l_progress_size * 2 - 6)
							if l_ind < l_progress_size - 2 then
								l_pos2 := l_ind + 1
							else
								l_pos2 := l_progress_size - ((l_ind - l_progress_size) + 5)
							end
							l_tmp___9 := l_p___0
							l_p___0 := l_p___0 + 1
							l_tmp___9[0] := (('[').code).to_integer_8
							update_class_fields -- Current might have been changed; synchronize
							l_i___0 := 0
							from
							until
								not (l_i___0 < l_progress_size)
							loop
								if l_i___0 = l_pos2 - 1 then
									l_tmp___10 := l_p___0
									l_p___0 := l_p___0 + 1
									l_tmp___10[0] := (('<').code).to_integer_8
									update_class_fields -- Current might have been changed; synchronize
								elseif l_i___0 = l_pos2 then
									l_tmp___11 := l_p___0
									l_p___0 := l_p___0 + 1
									l_tmp___11[0] := (('=').code).to_integer_8
									update_class_fields -- Current might have been changed; synchronize
								elseif l_i___0 = l_pos2 + 1 then
									l_tmp___12 := l_p___0
									l_p___0 := l_p___0 + 1
									l_tmp___12[0] := (('>').code).to_integer_8
									update_class_fields -- Current might have been changed; synchronize
								else
									l_tmp___13 := l_p___0
									l_p___0 := l_p___0 + 1
									l_tmp___13[0] := ((' ').code).to_integer_8
									update_class_fields -- Current might have been changed; synchronize
								end
								l_i___0 := l_i___0 + 1
							end
							l_tmp___14 := l_p___0
							l_p___0 := l_p___0 + 1
							l_tmp___14[0] := ((']').code).to_integer_8
							update_class_fields -- Current might have been changed; synchronize
							Current.tick := tick + 1
							update_class_fields -- Current might have been changed; synchronize
						end
				when 4 then
					sprintf ([l_p___0, ce_string (" %%s"), l_size_grouped]).do_nothing
					update_class_fields -- Current might have been changed; synchronize
					l_p___0 := strchr (l_p___0, ('%/0/').code)
					update_class_fields -- Current might have been changed; synchronize
					l_size_grouped_pad := 11 - l_size_grouped_len
					from
					until
						not (l_size_grouped_pad > 0)
					loop
						l_tmp___15 := l_p___0
						l_p___0 := l_p___0 + 1
						l_tmp___15[0] := ((' ').code).to_integer_8
						update_class_fields -- Current might have been changed; synchronize
						l_size_grouped_pad := l_size_grouped_pad - 1
					end
					if not (l_hist.item.total_time > (0).to_double) then
						l_pos := -1 + 7 -- skip if block
					end
				when 5 then -- if:
						if (l_hist.item.total_bytes).to_boolean then
							l_units.item := 0
							update_class_fields -- Current might have been changed; synchronize
							l_dlquant := l_hist.item.total_bytes + recent_bytes
							l_dltime := l_hist.item.total_time + (a_dl_total_time - recent_start)
							l_tmp___16 := (create {P_RETR_STATIC}).calc_rate (l_dlquant, l_dltime, l_units)
							update_class_fields -- Current might have been changed; synchronize
							l_dlspeed := l_tmp___16
							if l_dlspeed >= 99.95 then
								l_tmp___18 := 0
							else
								if l_dlspeed >= 9.995 then
									l_tmp___17 := 1
								else
									l_tmp___17 := 2
								end
								l_tmp___18 := l_tmp___17
							end
							sprintf ([l_p___0, ce_string (" %%4.*f%%s"), l_tmp___18, l_dlspeed, short_units.item[l_units.item]]).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_p___0 := strchr (l_p___0, ('%/0/').code)
							update_class_fields -- Current might have been changed; synchronize
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L___0:
						from
						until
							(l_break)
						loop
							l_break := False
							memcpy (create {CE_VOID_POINTER}.make_cast (l_p___0), create {CE_VOID_POINTER}.make_cast (ce_string (" --.-K/s")), ((" --.-K/s").count + 1).to_natural_32 - {NATURAL_32} 1).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_p___0 := l_p___0 + (((" --.-K/s").count + 1).to_natural_32 - {NATURAL_32} 1).to_integer_32
							l_break := True
						end
						l_break := False
				when 8 then
					if a_done then
						l_pos := -1 + 26 -- skip if block
					end
				when 9 then -- if:
						if not (total_length > {INTEGER_64} 0) then
							l_pos := -1 + 23 -- skip if block
						end
				when 10 then -- if:
						if not (count > {INTEGER_64} 0) then
							l_pos := -1 + 21 -- skip if block
						end
				when 11 then -- if:
						if not (a_dl_total_time > (3).to_double) then
							l_pos := -1 + 19 -- skip if block
						end
				when 12 then -- if:
						if total_length = l_size then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if last_eta_value /= 0 then
							if a_dl_total_time - last_eta_time < 0.99 then
								l_eta := last_eta_value
							else
								l_pos := -1 + 15 -- goto _L___2
							end
						else
							l_pos := -1 + 15 -- goto _L___2
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 15 then -- else: | _L___2:
						l_bytes_remaining := total_length - l_size
						l_eta_ := (a_dl_total_time * l_bytes_remaining.to_double) / count.to_double
						if l_eta_ >= (2_147_483_646).to_double then
							l_pos := -1 + 24 -- goto skip_eta
						end
				when 16 then
						l_eta := (l_eta_ + 0.5).truncated_to_integer
						Current.last_eta_value := l_eta
						update_class_fields -- Current might have been changed; synchronize
						Current.last_eta_time := a_dl_total_time
						update_class_fields -- Current might have been changed; synchronize
				when 17 then
						l_tmp___19 := (create {P_PROGRESS_STATIC}).eta_to_human_short (l_eta, (0).to_boolean)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___20 := get_eta (l_bytes_cols_diff___0)
						update_class_fields -- Current might have been changed; synchronize
						sprintf ([l_p___0, l_tmp___20, l_tmp___19]).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_p___0 := strchr (l_p___0, ('%/0/').code)
						update_class_fields -- Current might have been changed; synchronize
				when 18 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:
						l_pos := -1 + 23 -- goto _L___4
				when 20 then -- if done | if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						l_pos := -1 + 23 -- goto _L___4
				when 22 then -- if done | if block done, skip else block:
						l_pos := -1 + 25
				when 23 then -- else: | _L___4:
						if not (total_length > {INTEGER_64} 0) then
							l_pos := -1 + 25 -- skip if block
						end
				when 24 then -- if: | skip_eta:
						from
						until
							(l_break)
						loop
							l_break := False
							memcpy (create {CE_VOID_POINTER}.make_cast (l_p___0), create {CE_VOID_POINTER}.make_cast (ce_string ("             ")), (("             ").count + 1).to_natural_32 - {NATURAL_32} 1).do_nothing
							update_class_fields -- Current might have been changed; synchronize
							l_p___0 := l_p___0 + ((("             ").count + 1).to_natural_32 - {NATURAL_32} 1).to_integer_32
							l_break := True
						end
						l_break := False
				when 25 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 27
				when 26 then -- else:
						l_tmp___21 := gettext (ce_string ("   in "))
						update_class_fields -- Current might have been changed; synchronize
						strcpy (l_p___0, l_tmp___21).do_nothing
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___22 := strlen (l_p___0)
						update_class_fields -- Current might have been changed; synchronize
						l_nbytes := l_tmp___22.to_integer_32
						l_ncols := count_cols (l_p___0)
						update_class_fields -- Current might have been changed; synchronize
						l_bytes_cols_diff___0.item := l_nbytes - l_ncols
						update_class_fields -- Current might have been changed; synchronize
						l_p___0 := l_p___0 + l_nbytes
						if a_dl_total_time >= (10).to_double then
							l_tmp___23 := (create {P_PROGRESS_STATIC}).eta_to_human_short ((a_dl_total_time + 0.5).truncated_to_integer, (0).to_boolean)
							update_class_fields -- Current might have been changed; synchronize
							strcpy (l_p___0, l_tmp___23).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						else
							l_tmp___24 := (create {P_UTILS_STATIC}).print_decimal (a_dl_total_time)
							update_class_fields -- Current might have been changed; synchronize
							sprintf ([l_p___0, ce_string ("%%ss"), l_tmp___24]).do_nothing
							update_class_fields -- Current might have been changed; synchronize
						end
						l_p___0 := strchr (l_p___0, ('%/0/').code)
						update_class_fields -- Current might have been changed; synchronize
				when 27 then
					from
					until
						not (((l_p___0.distance (buffer)) - l_bytes_cols_diff___0.item) - l_size_grouped_diff < width)
					loop
						l_tmp___25 := l_p___0
						l_p___0 := l_p___0 + 1
						l_tmp___25[0] := ((' ').code).to_integer_8
						update_class_fields -- Current might have been changed; synchronize
					end
					l_p___0[0] := (('%/0/').code).to_integer_8
					update_class_fields -- Current might have been changed; synchronize
				when 28 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {PD_BAR_PROGRESS} -- private routines

	get_eta (a_bcd: CE_POINTER [INTEGER_32]): CE_POINTER [INTEGER_8]
		local
			l_nbytes: INTEGER_32
			l_ncols: INTEGER_32
		do
			if eta_trans.item.is_void then
				eta_trans.item := (gettext (eta_str.item))
				l_nbytes := (strlen (eta_trans.item)).to_integer_32
				l_ncols := count_cols (eta_trans.item)
				bytes_cols_diff.item := l_nbytes - l_ncols
			end
			if not a_bcd.is_void then
				a_bcd.item := bytes_cols_diff.item
			end
			Result := eta_trans.item
		end

	count_cols (a_mbs: CE_POINTER [INTEGER_8]): INTEGER_32
		require
			not a_mbs.is_void
		local
			l_mbs: CE_POINTER [INTEGER_8]
			l_wc: CE_ARRAY [INTEGER_32]
			l_bytes: INTEGER_32
			l_remaining: INTEGER_32
			l_cols: INTEGER_32
			l_wccols: INTEGER_32
			l_tmp___0: INTEGER_32
			l_return: BOOLEAN
		do
			create {CE_INTEGER_32_ARRAY} l_wc.make (<<1>>)

			l_mbs := a_mbs
			l_remaining := (strlen (l_mbs)).to_integer_32
			l_cols := 0
			from
			until
				not (not (l_return) and then ((l_mbs[0]).to_integer_32 /= 0))
			loop
				l_bytes := mbtowc (l_wc, l_mbs, l_remaining.to_natural_32)
				if l_bytes = 0 then
					assert_fail (ce_string ("bytes != 0"), ce_string ("progress.c"), {NATURAL_32} 783, ce_string ("count_cols"))
				end
				if l_bytes = -1 then
					Result := (l_cols + l_remaining)
					l_return := True
				end
				if not l_return then
					l_mbs := l_mbs + l_bytes
					l_remaining := l_remaining - l_bytes
					l_wccols := wcwidth (l_wc.item)
					if l_wccols = -1 then
						l_tmp___0 := 1
					else
						l_tmp___0 := l_wccols
					end
					l_cols := l_cols + l_tmp___0
				end
			end
			if not l_return then
				Result := l_cols
			end
		end

	with_thousand_seps (a_n: INTEGER_64): CE_POINTER [INTEGER_8]
		local
			l_n: INTEGER_64
			l_p___0: CE_POINTER [INTEGER_8]
			l_grouping: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_sep: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_seplen: INTEGER_32
			l_i: INTEGER_32
			l_groupsize: INTEGER_32
			l_atgroup: CE_POINTER [INTEGER_8]
			l_negative: BOOLEAN
			l_tmp___0: CE_POINTER [INTEGER_8]
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_break: BOOLEAN
		do
			create {CE_POINTER_ARRAY [INTEGER_8]} l_grouping.make (<<1>>)
			create {CE_POINTER_ARRAY [INTEGER_8]} l_sep.make (<<1>>)

			l_n := a_n
			l_p___0 := outbuf.item + ({PLATFORM}.Integer_8_bytes.to_natural_32 * (48).to_natural_32).to_integer_32
			l_i := 0
			l_negative := (l_n < {INTEGER_64} 0)
			get_grouping_data (l_sep, l_grouping)
			l_seplen := (strlen (l_sep.item)).to_integer_32
			l_atgroup := l_grouping.item
			l_tmp___0 := l_atgroup
			l_atgroup := l_atgroup + 1
			l_groupsize := (l_tmp___0[0]).to_integer_32
			if l_negative then
				l_n := -l_n
			end
			l_p___0 := l_p___0 - 1
			l_p___0[0] := (('%/0/').code).to_integer_8
			from
			until
				(l_break)
			loop
				l_break := False
				l_p___0 := l_p___0 - 1
				l_p___0[0] := (l_n \\ {INTEGER_64} 10 + {INTEGER_64} 48).to_integer_8
				l_n := l_n // ({INTEGER_64} 10)
				if l_n = {INTEGER_64} 0 then
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
					if l_i = l_groupsize then
						if l_seplen = 1 then
							l_p___0 := l_p___0 - 1
							l_p___0[0] := l_sep.item[0]
						else
							l_p___0 := l_p___0 - l_seplen
							memcpy (create {CE_VOID_POINTER}.make_cast (l_p___0), create {CE_VOID_POINTER}.make_cast (l_sep.item), l_seplen.to_natural_32).do_nothing
						end
						l_i := 0
						if (l_atgroup[0]).to_boolean then
							l_tmp___1 := l_atgroup
							l_atgroup := l_atgroup + 1
							l_groupsize := (l_tmp___1[0]).to_integer_32
						end
					end
				end
			end
			l_break := False
			if l_negative then
				l_p___0 := l_p___0 - 1
				l_p___0[0] := (('-').code).to_integer_8
			end
			Result := l_p___0
		end

	get_grouping_data (a_sep, a_grouping: CE_POINTER [CE_POINTER [INTEGER_8]])
		require
			not a_sep.is_void
			not a_grouping.is_void
		local
			l_lconv: CE_POINTER [SD_LCONV]
		do
			if not initialized.item then
				l_lconv := (localeconv)
				cached_sep.item := l_lconv.item.thousands_sep
				cached_grouping.item := l_lconv.item.grouping
				if not (cached_sep.item[0]).to_boolean then
					if ((l_lconv.item.decimal_point).item).to_integer_32 /= 44 then
						cached_sep.item := ce_string (",")
					else
						cached_sep.item := ce_string (".")
					end
					cached_grouping.item := ce_string ("%/3/")
				end
				initialized.item := (1).to_boolean
			end
			a_sep.item := cached_sep.item
			a_grouping.item := cached_grouping.item
		end

feature {ANY}

	copy (a_other: like Current)
		do
			initial_length := a_other.initial_length
			total_length := a_other.total_length
			count := a_other.count
			last_screen_update := a_other.last_screen_update
			width := a_other.width
			buffer := a_other.buffer
			tick := a_other.tick
			hist := a_other.hist.twin
			recent_start := a_other.recent_start
			recent_bytes := a_other.recent_bytes
			stalled := a_other.stalled
			last_eta_time := a_other.last_eta_time
			last_eta_value := a_other.last_eta_value
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				initial_length = a_other.initial_length and
				total_length = a_other.total_length and
				count = a_other.count and
				last_screen_update = a_other.last_screen_update and
				width = a_other.width and
				buffer.is_equal (a_other.buffer) and
				tick = a_other.tick and
				hist.is_equal (a_other.hist) and
				recent_start = a_other.recent_start and
				recent_bytes = a_other.recent_bytes and
				stalled = a_other.stalled and
				last_eta_time = a_other.last_eta_time and
				last_eta_value = a_other.last_eta_value
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("initial_length"),
					create {CE_STRUCTURE_FIELD}.make ("total_length"),
					create {CE_STRUCTURE_FIELD}.make ("count"),
					create {CE_STRUCTURE_FIELD}.make ("last_screen_update"),
					create {CE_STRUCTURE_FIELD}.make ("width"),
					create {CE_STRUCTURE_FIELD}.make ("buffer"),
					create {CE_STRUCTURE_FIELD}.make ("tick"),
					create {CE_STRUCTURE_FIELD}.make ("hist"),
					create {CE_STRUCTURE_FIELD}.make ("recent_start"),
					create {CE_STRUCTURE_FIELD}.make ("recent_bytes"),
					create {CE_STRUCTURE_FIELD}.make ("stalled"),
					create {CE_STRUCTURE_FIELD}.make ("last_eta_time"),
					create {CE_STRUCTURE_FIELD}.make ("last_eta_value")
				>>)
			init_structure_info
		end

end
