class
	PD_DOT_PROGRESS

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {ANY} -- fields

	initial_length: INTEGER_64 assign set_initial_length

	total_length: INTEGER_64 assign set_total_length

	accumulated: INTEGER_32 assign set_accumulated

	rows: INTEGER_32 assign set_rows

	dots: INTEGER_32 assign set_dots

	last_timer_value: REAL_64 assign set_last_timer_value

feature {ANY} -- field setters

	set_initial_length (a_initial_length: INTEGER_64)
		do
			initial_length := a_initial_length
			update_memory_field ("initial_length")
		end

	set_total_length (a_total_length: INTEGER_64)
		do
			total_length := a_total_length
			update_memory_field ("total_length")
		end

	set_accumulated (a_accumulated: INTEGER_32)
		do
			accumulated := a_accumulated
			update_memory_field ("accumulated")
		end

	set_rows (a_rows: INTEGER_32)
		do
			rows := a_rows
			update_memory_field ("rows")
		end

	set_dots (a_dots: INTEGER_32)
		do
			dots := a_dots
			update_memory_field ("dots")
		end

	set_last_timer_value (a_last_timer_value: REAL_64)
		do
			last_timer_value := a_last_timer_value
			update_memory_field ("last_timer_value")
		end

feature {PD_DOT_PROGRESS} -- private attributes

	names: CE_STATIC [CE_ARRAY [INTEGER_8]]
		once ("PROCESS")
			Result.item := ce_string (" KMG")
		end

feature {ANY} -- public routines

	print_row_stats (a_dltime: REAL_64; a_last___0: BOOLEAN)
		local
			l_ROW_BYTES: INTEGER_64
			l_bytes_displayed: INTEGER_64
			l_percentage: INTEGER_32
			l_units: CE_ARRAY [INTEGER_32]
			l_rate: REAL_64
			l_bytes_this_row: INTEGER_64
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_bytes_remaining: INTEGER_64
			l_bytes_sofar: INTEGER_64
			l_eta: REAL_64
			l_tmp___1: CE_POINTER [INTEGER_8]
			l_tmp___2: CE_POINTER [INTEGER_8]
			l_tmp___3: CE_POINTER [INTEGER_8]
		do
			create {CE_INTEGER_32_ARRAY} l_units.make (<<1>>)

			l_ROW_BYTES := (create {P_OPTIONS_STATIC}).opt.item.item.dot_bytes * ((create {P_OPTIONS_STATIC}).opt.item.item.dots_in_line).to_integer_64
			l_bytes_displayed := rows.to_integer_64 * l_ROW_BYTES + dots.to_integer_64 * (create {P_OPTIONS_STATIC}).opt.item.item.dot_bytes
			if a_last___0 then
				l_bytes_displayed := l_bytes_displayed + accumulated.to_integer_64
			end
			if total_length.to_boolean then
				l_percentage := ((100.0 * l_bytes_displayed.to_double) / total_length.to_double).truncated_to_integer;
				(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, ce_string ("%%3d%%%%"), l_percentage])
				update_class_fields -- Current might have been changed; synchronize
			end
			if not a_last___0 then
				l_bytes_this_row := l_ROW_BYTES
			else
				l_bytes_this_row := dots.to_integer_64 * (create {P_OPTIONS_STATIC}).opt.item.item.dot_bytes + accumulated.to_integer_64
			end
			if rows.to_integer_64 = initial_length // l_ROW_BYTES then
				l_bytes_this_row := l_bytes_this_row - (initial_length \\ l_ROW_BYTES)
			end
			l_rate := (create {P_RETR_STATIC}).calc_rate (l_bytes_this_row, a_dltime - last_timer_value, l_units)
			update_class_fields -- Current might have been changed; synchronize
			if l_rate >= 99.95 then
				l_tmp___0 := 0
			else
				if l_rate >= 9.995 then
					l_tmp := 1
				else
					l_tmp := 2
				end
				l_tmp___0 := l_tmp
			end
			(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, ce_string (" %%4.*f%%c"), l_tmp___0, l_rate, (names.item[l_units.item]).to_integer_32])
			update_class_fields -- Current might have been changed; synchronize
			Current.last_timer_value := a_dltime
			update_class_fields -- Current might have been changed; synchronize
			if not a_last___0 then
				if total_length.to_boolean then
					l_bytes_remaining := total_length - l_bytes_displayed
					l_bytes_sofar := l_bytes_displayed - initial_length
					l_eta := (a_dltime * l_bytes_remaining.to_double) / l_bytes_sofar.to_double
					if l_eta < (2_147_483_646).to_double then
						l_tmp___1 := (create {P_PROGRESS_STATIC}).eta_to_human_short ((l_eta + 0.5).truncated_to_integer, (1).to_boolean)
						update_class_fields; -- Current might have been changed; synchronize
						(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, ce_string (" %%s"), l_tmp___1])
						update_class_fields -- Current might have been changed; synchronize
					end
				end
			elseif a_dltime >= (10).to_double then
				l_tmp___2 := (create {P_PROGRESS_STATIC}).eta_to_human_short ((a_dltime + 0.5).truncated_to_integer, (1).to_boolean)
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, ce_string ("=%%s"), l_tmp___2])
				update_class_fields -- Current might have been changed; synchronize
			else
				l_tmp___3 := (create {P_UTILS_STATIC}).print_decimal (a_dltime)
				update_class_fields; -- Current might have been changed; synchronize
				(create {P_LOG_STATIC}).logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, ce_string ("=%%ss"), l_tmp___3])
				update_class_fields -- Current might have been changed; synchronize
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			initial_length := a_other.initial_length
			total_length := a_other.total_length
			accumulated := a_other.accumulated
			rows := a_other.rows
			dots := a_other.dots
			last_timer_value := a_other.last_timer_value
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				initial_length = a_other.initial_length and
				total_length = a_other.total_length and
				accumulated = a_other.accumulated and
				rows = a_other.rows and
				dots = a_other.dots and
				last_timer_value = a_other.last_timer_value
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("initial_length"),
					create {CE_STRUCTURE_FIELD}.make ("total_length"),
					create {CE_STRUCTURE_FIELD}.make ("accumulated"),
					create {CE_STRUCTURE_FIELD}.make ("rows"),
					create {CE_STRUCTURE_FIELD}.make ("dots"),
					create {CE_STRUCTURE_FIELD}.make ("last_timer_value")
				>>)
			init_structure_info
		end

end
