class
	PD_PTIMER

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_TIME_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create start.make
			Precursor
		end

feature {ANY} -- fields

	start: SD_TIMESPEC assign set_start

	elapsed_last: REAL_64 assign set_elapsed_last

	elapsed_pre_start: REAL_64 assign set_elapsed_pre_start

feature {ANY} -- field setters

	set_start (a_start: SD_TIMESPEC)
		do
			start := a_start
			update_memory_field ("start")
		end

	set_elapsed_last (a_elapsed_last: REAL_64)
		do
			elapsed_last := a_elapsed_last
			update_memory_field ("elapsed_last")
		end

	set_elapsed_pre_start (a_elapsed_pre_start: REAL_64)
		do
			elapsed_pre_start := a_elapsed_pre_start
			update_memory_field ("elapsed_pre_start")
		end

feature {ANY} -- public routines

	ptimer_reset
		do
			posix_measure (create {CE_CLASS_POINTER [SD_TIMESPEC]}.make_shared (c_pointer + (create {PD_PTIMER}.make).offset_of ("start")))
			update_class_fields -- Current might have been changed; synchronize
			Current.elapsed_last := (0).to_double
			update_class_fields -- Current might have been changed; synchronize
			Current.elapsed_pre_start := (0).to_double
			update_class_fields -- Current might have been changed; synchronize
		end

	ptimer_measure: REAL_64
		local
			l_now: CE_ARRAY [SD_TIMESPEC]
			l_elapsed: REAL_64
			l_tmp: REAL_64
		do
			create {CE_CLASS_ARRAY [SD_TIMESPEC]} l_now.make (<<1>>)

			posix_measure (l_now)
			update_class_fields -- Current might have been changed; synchronize
			l_tmp := posix_diff (l_now, create {CE_CLASS_POINTER [SD_TIMESPEC]}.make_shared (c_pointer + (create {PD_PTIMER}.make).offset_of ("start")))
			update_class_fields -- Current might have been changed; synchronize
			l_elapsed := elapsed_pre_start + l_tmp
			if l_elapsed < elapsed_last then
				Current.start := (l_now.item).twin
				update_class_fields -- Current might have been changed; synchronize
				Current.elapsed_pre_start := elapsed_last
				update_class_fields -- Current might have been changed; synchronize
				l_elapsed := elapsed_last
			end
			Current.elapsed_last := l_elapsed
			update_class_fields -- Current might have been changed; synchronize
			Result := l_elapsed
		end

feature {PD_PTIMER} -- private routines

	posix_measure (a_pst: CE_POINTER [SD_TIMESPEC])
		do
			clock_gettime ((create {P_PTIMER_STATIC}).posix_clock_id.item, a_pst).do_nothing
		end

	posix_diff (a_pst1, a_pst2: CE_POINTER [SD_TIMESPEC]): REAL_64
		require
			not a_pst1.is_void
			not a_pst2.is_void
		do
			Result := ((a_pst1.item.tv_sec - a_pst2.item.tv_sec).to_double + (a_pst1.item.tv_nsec - a_pst2.item.tv_nsec).to_double / 1.e9)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			start := a_other.start.twin
			elapsed_last := a_other.elapsed_last
			elapsed_pre_start := a_other.elapsed_pre_start
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				start.is_equal (a_other.start) and
				elapsed_last = a_other.elapsed_last and
				elapsed_pre_start = a_other.elapsed_pre_start
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("start"),
					create {CE_STRUCTURE_FIELD}.make ("elapsed_last"),
					create {CE_STRUCTURE_FIELD}.make ("elapsed_pre_start")
				>>)
			init_structure_info
		end

end
