class
	P_SELECT_STATIC

inherit
	CE_UTILITIES
	S_SYS_TIME_STATIC
	S_UNISTD_STATIC
	S_BITS_ERRNO_STATIC
	S_SIGNAL_STATIC
	S_SYS_WAIT_STATIC
	S_SYS_STAT_STATIC
	S_STDLIB_STATIC
	S_STRING_STATIC
	S_FCNTL_STATIC
	S_SYS_SELECT_STATIC
	LINKS_ASSEMBLER

feature {ANY} -- public attributes

	threads: CE_STATIC [CE_ARRAY [PD_THREAD]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_THREAD]}.make (<<1_024>>)
		end

	w_read: CE_STATIC [CE_ARRAY [SD_FD_SET]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_FD_SET]}.make (<<1>>)
		end

	w_write: CE_STATIC [CE_ARRAY [SD_FD_SET]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_FD_SET]}.make (<<1>>)
		end

	w_error: CE_STATIC [CE_ARRAY [SD_FD_SET]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_FD_SET]}.make (<<1>>)
		end

	x_read: CE_STATIC [CE_ARRAY [SD_FD_SET]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_FD_SET]}.make (<<1>>)
		end

	x_write: CE_STATIC [CE_ARRAY [SD_FD_SET]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_FD_SET]}.make (<<1>>)
		end

	x_error: CE_STATIC [CE_ARRAY [SD_FD_SET]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_FD_SET]}.make (<<1>>)
		end

	w_max: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	timer_id: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	timers: CE_STATIC [CE_ARRAY [PD_LIST_HEAD]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_LIST_HEAD]}.make (<<1>>)
			Result.item[0] := create {PD_LIST_HEAD}.make_initialized ([create {CE_VOID_POINTER}.make_cast (timers.item), create {CE_VOID_POINTER}.make_cast (timers.item)])
		end

	bottom_halves: CE_STATIC [CE_ARRAY [PD_LIST_HEAD]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_LIST_HEAD]}.make (<<1>>)
			Result.item[0] := create {PD_LIST_HEAD}.make_initialized ([create {CE_VOID_POINTER}.make_cast (bottom_halves.item), create {CE_VOID_POINTER}.make_cast (bottom_halves.item)])
		end

	last_time: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	signal_mask: CE_STATIC [CE_ARRAY [INTEGER_32]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_32_ARRAY}.make (<<32>>)
		end

	signal_handlers: CE_STATIC [CE_ARRAY [PD_SIGNAL_HANDLER]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [PD_SIGNAL_HANDLER]}.make (<<32>>)
		end

	signal_pipe: CE_STATIC [CE_ARRAY [INTEGER_32]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_32_ARRAY}.make (<<2>>)
		end

	sa_zero: CE_STATIC [CE_ARRAY [SD_SIGACTION]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_SIGACTION]}.make (<<1>>)
		end

	terminate: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

feature {ANY} -- public routines

	get_time: INTEGER_32
		local
			l_tv: CE_ARRAY [SD_TIMEVAL]
		do
			create {CE_CLASS_ARRAY [SD_TIMEVAL]} l_tv.make (<<1>>)

			gettimeofday (l_tv, create {CE_CLASS_POINTER [SD_TIMEZONE]}.make_shared (default_pointer)).do_nothing
			Result := (l_tv.item.tv_sec * 1_000 + l_tv.item.tv_usec // 1_000)
		end

	register_bottom_half (a_fn: CE_ROUTINE; a_data: CE_POINTER [ANY]): INTEGER_32
		local
			l_bh: CE_POINTER [PD_BOTTOM_HALF]
			l_return, l_break: BOOLEAN
		do
			l_bh := create {CE_CLASS_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item.item.next)
			from
			until
				not (not (l_return) and then (not l_bh.is_equal (bottom_halves.item)))
			loop
				if (l_bh.item.fn).is_equal (a_fn) then
					if (l_bh.item.data).is_equal (a_data) then
						Result := 0
						l_return := True
					end
				end
				if not l_return then
					l_bh := l_bh.item.next
				end
			end
			if not l_return then
				l_bh := create {CE_CLASS_POINTER [PD_BOTTOM_HALF]}.make_cast (((create {P_ERROR_STATIC}).debug_mem_alloc (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c")), 73, (create {PD_BOTTOM_HALF}.make).structure_size.to_natural_32)))
				if l_bh.is_void then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				l_bh.item.fn := a_fn
				l_bh.item.data := a_data
				from
				until
					(l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (bottom_halves.item))
					l_bh.item.next := (create {CE_CLASS_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item)).item.next
					l_bh.item.prev := create {CE_CLASS_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item);
					(create {CE_CLASS_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item)).item.next := l_bh;
					(l_bh.item.next).item.prev := l_bh;
					(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (bottom_halves.item))
					l_break := True
				end
				l_break := False
				Result := 0
			end
		end

	check_bottom_halves
		local
			l_bh: CE_POINTER [PD_BOTTOM_HALF]
			l_fn: CE_ROUTINE
			l_data: CE_POINTER [ANY]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start: | rep:
					if (bottom_halves.item.item.next).is_equal (bottom_halves.item) then
						l_return := True
					end
					if not l_return then
						l_bh := create {CE_CLASS_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item.item.prev)
						l_fn := l_bh.item.fn
						l_data := l_bh.item.data;
						(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_bh));
						(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_bh.item.next)).item.prev := create {CE_VOID_POINTER}.make_cast (l_bh.item.prev);
						(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_bh.item.prev)).item.next := create {CE_VOID_POINTER}.make_cast (l_bh.item.next);
						(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_bh));
						(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c")), 91, create {CE_VOID_POINTER}.make_cast (l_bh))
						l_fn.call ([l_data])
						l_pos := -1 + 0 -- goto rep
					end
				when 1 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	check_timers
		local
			l_interval: INTEGER_32
			l_t: CE_POINTER [PD_TIMER]
			l_tt: CE_POINTER [PD_TIMER]
			l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_interval := (get_time) - last_time.item
					l_t := create {CE_CLASS_POINTER [PD_TIMER]}.make_cast (timers.item.item.next)
					from
					until
						l_t.is_equal (timers.item)
					loop
						l_t.item.interval := l_t.item.interval - l_interval
						l_t := l_t.item.next
					end
				when 1 then -- ch:
					l_t := create {CE_CLASS_POINTER [PD_TIMER]}.make_cast (timers.item.item.next)
				when 2 then -- loop condition:
					if not (not (l_break) and then (not l_t.is_equal (timers.item))) then
						l_pos := -1 + 5 -- skip loop
					end
				when 3 then -- loop body:
						l_break := False
						if l_t.item.interval <= 0 then
							l_tt := l_t;
							(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_tt));
							(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_tt.item.next)).item.prev := create {CE_VOID_POINTER}.make_cast (l_tt.item.prev);
							(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_tt.item.prev)).item.next := create {CE_VOID_POINTER}.make_cast (l_tt.item.next);
							(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_tt));
							(l_tt.item.func).call ([l_tt.item.data]);
							(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c")), 110, create {CE_VOID_POINTER}.make_cast (l_tt))
							if not ((bottom_halves.item.item.next).is_equal (bottom_halves.item)) then
								check_bottom_halves
							end
							l_pos := -1 + 1 -- goto ch
						else
							l_break := True
						end
				when 4 then
						if not (l_break) then
							l_t := l_t.item.next
						end
						l_pos := -1 + 2 -- back to loop condition
				when 5 then -- loop done
					l_break := False
					last_time.item := last_time.item + l_interval
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	install_timer (a_t: INTEGER_32; a_func: CE_ROUTINE; a_data: CE_POINTER [ANY]): INTEGER_32
		local
			l_tm: CE_POINTER [PD_TIMER]
			l_tt: CE_POINTER [PD_TIMER]
			l_tmp___0: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tm := create {CE_CLASS_POINTER [PD_TIMER]}.make_cast (((create {P_ERROR_STATIC}).debug_mem_alloc (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c")), 120, (create {PD_TIMER}.make).structure_size.to_natural_32)))
			if l_tm.is_void then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				l_tm.item.interval := a_t
				l_tm.item.func := a_func
				l_tm.item.data := a_data
				l_tmp___0 := timer_id.item
				timer_id.item := timer_id.item + 1
				l_tm.item.id := l_tmp___0
				l_tt := create {CE_CLASS_POINTER [PD_TIMER]}.make_cast (timers.item.item.next)
				from
				until
					not (not (l_break) and then (not l_tt.is_equal (timers.item)))
				loop
					l_break := False
					if l_tt.item.interval >= a_t then
						l_break := True
					end
					if not (l_break) then
						l_tt := l_tt.item.next
					end
				end
				l_break := False
				from
				until
					(l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_tt.item.prev))
					l_tm.item.next := (l_tt.item.prev).item.next
					l_tm.item.prev := l_tt.item.prev;
					(l_tt.item.prev).item.next := l_tm;
					(l_tm.item.next).item.prev := l_tm;
					(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_tt.item.prev))
					l_break := True
				end
				l_break := False
				Result := (l_tm.item.id)
			end
		end

	kill_timer (a_id: INTEGER_32)
		local
			l_tm: CE_POINTER [PD_TIMER]
			l_k: INTEGER_32
			l_tt: CE_POINTER [PD_TIMER]
		do
			l_k := 0
			l_tm := create {CE_CLASS_POINTER [PD_TIMER]}.make_cast (timers.item.item.next)
			from
			until
				l_tm.is_equal (timers.item)
			loop
				if l_tm.item.id = a_id then
					l_tt := l_tm;
					(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_tm));
					(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_tm.item.next)).item.prev := create {CE_VOID_POINTER}.make_cast (l_tm.item.prev);
					(create {CE_CLASS_POINTER [PD_LIST_HEAD]}.make_cast (l_tm.item.prev)).item.next := create {CE_VOID_POINTER}.make_cast (l_tm.item.next);
					(create {P_ERROR_STATIC}).do_not_optimize_here (create {CE_VOID_POINTER}.make_cast (l_tm))
					l_tm := l_tm.item.prev;
					(create {P_ERROR_STATIC}).debug_mem_free (create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c")), 138, create {CE_VOID_POINTER}.make_cast (l_tt))
					l_k := l_k + 1
				end
				l_tm := l_tm.item.next
			end
			if not l_k.to_boolean then
				from
				until
					False
				loop
					(create {P_LINKS_STATIC}).errfile.item := create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c"));
					(create {P_LINKS_STATIC}).errline.item := 141;
					(create {P_ERROR_STATIC}).int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("trying to kill nonexisting timer"))])
				end
			end
			if l_k >= 2 then
				from
				until
					False
				loop
					(create {P_LINKS_STATIC}).errfile.item := create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c"));
					(create {P_LINKS_STATIC}).errline.item := 142;
					(create {P_ERROR_STATIC}).int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("more timers with same id"))])
				end
			end
		end

	set_handlers (a_fd: INTEGER_32; a_read_func, a_write_func, a_error_func: CE_ROUTINE; a_data: CE_POINTER [ANY])
		local
			l_fd: CE_ARRAY [INTEGER_32]
			l_i: CE_ARRAY [INTEGER_32]
			l___result: CE_ARRAY [INTEGER_8]
			l___result___0: CE_ARRAY [INTEGER_8]
			l___result___1: CE_ARRAY [INTEGER_8]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_INTEGER_32_ARRAY} l_fd.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_i.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l___result.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l___result___0.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l___result___1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_fd.item := a_fd
					if not (l_fd.item < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (l_fd.item >= 1_024) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							(create {P_LINKS_STATIC}).errfile.item := create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c"));
							(create {P_LINKS_STATIC}).errline.item := 164;
							(create {P_ERROR_STATIC}).int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("set_handlers: handle %%d >= FD_SETSIZE %%d")), l_fd.item, 1_024])
						end
						l_return := True
				when 5 then -- if done
					if l_return then
						l_pos := -1 + 16 -- skip if block
					end
				when 6 then -- if:
						threads.item[l_fd.item].read_func := a_read_func
						threads.item[l_fd.item].write_func := a_write_func
						threads.item[l_fd.item].error_func := a_error_func
						threads.item[l_fd.item].data := a_data
						if (not a_read_func.is_void) then
							asm6 (w_read.item, l_fd)
						else
							asm7 (w_read.item, l_fd)
							asm8 (x_read.item, l_fd)
						end
						if (not a_write_func.is_void) then
							asm9 (w_write.item, l_fd)
						else
							asm10 (w_write.item, l_fd)
							asm11 (x_write.item, l_fd)
						end
						if (not a_error_func.is_void) then
							asm12 (w_error.item, l_fd)
						else
							asm13 (w_error.item, l_fd)
							asm14 (x_error.item, l_fd)
						end
						if a_read_func.is_void then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 13 -- goto _L___0
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else:
						if a_write_func.is_void then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L___0
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else:
						if a_error_func.is_void then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L___0:
						if l_fd.item >= w_max.item then
							w_max.item := l_fd.item + 1
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else:
						if l_fd.item = w_max.item - 1 then
							l_i.item := l_fd.item - 1
							from
							until
								not (not (l_break) and then (l_i.item >= 0))
							loop
								l_break := False
								asm15 (l___result, l_i, w_read.item)
								if (l___result.item).to_boolean then
									l_break := True
								else
									asm16 (l___result___0, l_i, w_write.item)
									if (l___result___0.item).to_boolean then
										l_break := True
									else
										asm17 (l___result___1, l_i, w_error.item)
										if (l___result___1.item).to_boolean then
											l_break := True
										end
									end
								end
								if not (l_break) then
									l_i.item := l_i.item - 1
								end
							end
							l_break := False
							w_max.item := l_i.item + 1
						end
				when 16 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	signal_break (a_data: CE_POINTER [ANY])
		local
			l_c: CE_ARRAY [INTEGER_8]
			l_break: BOOLEAN
		do
			create {CE_INTEGER_8_ARRAY} l_c.make (<<1>>)

			from
			until
				(l_break)
			loop
				l_break := False
				if not ((create {P_LINKS_STATIC}).can_read (signal_pipe.item[0])).to_boolean then
					l_break := True
				end
				if not (l_break) then
					read (signal_pipe.item[0], create {CE_VOID_POINTER}.make_cast (l_c), (1).to_natural_32).do_nothing
				end
			end
			l_break := False
		end

	got_signal (a_sig: INTEGER_32)
		local
			l_sv_errno: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sv_errno := (errno_location).item
					if a_sig >= 32 then
						l_pos := -1 + 4 -- goto ret
					elseif a_sig < 0 then
						l_pos := -1 + 4 -- goto ret
					end
				when 1 then
					if (signal_handlers.item[a_sig].fn).is_void then
						l_pos := -1 + 4 -- goto ret
					end
				when 2 then
					if (signal_handlers.item[a_sig].critical).to_boolean then
						(signal_handlers.item[a_sig].fn).call ([signal_handlers.item[a_sig].data])
						l_pos := -1 + 4 -- goto ret
					end
				when 3 then
					signal_mask.item[a_sig] := 1
				when 4 then -- ret:
					if ((create {P_LINKS_STATIC}).can_write (signal_pipe.item[1])).to_boolean then
						write (signal_pipe.item[1], create {CE_VOID_POINTER}.make_cast (ce_string ("x")), (1).to_natural_32).do_nothing
					end
					(errno_location).item := l_sv_errno
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	install_signal_handler (a_sig: INTEGER_32; a_fn: CE_ROUTINE; a_data: CE_POINTER [ANY]; a_critical: INTEGER_32)
		local
			l_sa: CE_ARRAY [SD_SIGACTION]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [SD_SIGACTION]} l_sa.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sa.item := (sa_zero.item.item).twin
					if not (a_sig >= 32) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_sig < 0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							(create {P_LINKS_STATIC}).errfile.item := create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c"));
							(create {P_LINKS_STATIC}).errline.item := 240;
							(create {P_ERROR_STATIC}).int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bad signal number: %%d")), a_sig])
						end
						l_return := True
				when 5 then -- if done
					if not l_return then
						if a_fn.is_void then
							l_sa.item.sigaction_handler.sa_handler := create {CE_ROUTINE}.make_shared (default_pointer + 1)
						else
							l_sa.item.sigaction_handler.sa_handler := got_signal_address
						end
						sigfillset (create {CE_CLASS_POINTER [SD_SIGSET_T]}.make_shared (l_sa.item.c_pointer + (create {SD_SIGACTION}.make).offset_of ("sa_mask"))).do_nothing
						l_sa.item.sa_flags := 268_435_456
						if a_fn.is_void then
							sigaction (a_sig, l_sa, create {CE_CLASS_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
						end
						signal_handlers.item[a_sig].fn := a_fn
						signal_handlers.item[a_sig].data := a_data
						signal_handlers.item[a_sig].critical := a_critical
						if (not a_fn.is_void) then
							sigaction (a_sig, l_sa, create {CE_CLASS_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
						end
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	interruptible_signal (a_sig, a_in: INTEGER_32)
		local
			l_sa: CE_ARRAY [SD_SIGACTION]
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [SD_SIGACTION]} l_sa.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sa.item := (sa_zero.item.item).twin
					if not (a_sig >= 32) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_sig < 0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							(create {P_LINKS_STATIC}).errfile.item := create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("select.c"));
							(create {P_LINKS_STATIC}).errline.item := 258;
							(create {P_ERROR_STATIC}).int_error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("bad signal number: %%d")), a_sig])
						end
						l_return := True
				when 5 then -- if done
					if not l_return and then ((signal_handlers.item[a_sig].fn).is_void) then
						l_return := True
					end
					if not l_return then
						l_sa.item.sigaction_handler.sa_handler := got_signal_address
						sigfillset (create {CE_CLASS_POINTER [SD_SIGSET_T]}.make_shared (l_sa.item.c_pointer + (create {SD_SIGACTION}.make).offset_of ("sa_mask"))).do_nothing
						if not a_in.to_boolean then
							l_sa.item.sa_flags := 268_435_456
						end
						sigaction (a_sig, l_sa, create {CE_CLASS_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	check_signals: INTEGER_32
		local
			l_i: INTEGER_32
			l_r: INTEGER_32
		do
			l_r := 0
			l_i := 0
			from
			until
				not (l_i < 32)
			loop
				if (signal_mask.item[l_i]).to_boolean then
					signal_mask.item[l_i] := 0
					if (not (signal_handlers.item[l_i].fn).is_void) then
						(signal_handlers.item[l_i].fn).call ([signal_handlers.item[l_i].data])
					end
					if not ((bottom_halves.item.item.next).is_equal (bottom_halves.item)) then
						check_bottom_halves
					end
					l_r := 1
				end
				l_i := l_i + 1
			end
			Result := l_r
		end

	sigchld (a_p: CE_POINTER [ANY])
		local
			l_break: BOOLEAN
		do
			from
			until
				(l_break)
			loop
				l_break := False
				if not ((waitpid (-1, create {CE_INTEGER_32_POINTER}.make_shared (default_pointer), 1)) > 0) then
					l_break := True
				end
			end
			l_break := False
		end

	set_sigcld
		do
			install_signal_handler (17, sigchld_address, create {CE_VOID_POINTER}.make_shared (default_pointer), 1)
		end

	select_loop (a_init___1: CE_ROUTINE)
		require
			not a_init___1.is_void
		local
			l_st: CE_ARRAY [SD_STAT]
			l___d0: CE_ARRAY [INTEGER_32]
			l___d1: CE_ARRAY [INTEGER_32]
			l___d0___0: CE_ARRAY [INTEGER_32]
			l___d1___0: CE_ARRAY [INTEGER_32]
			l___d0___1: CE_ARRAY [INTEGER_32]
			l___d1___1: CE_ARRAY [INTEGER_32]
			l_n: INTEGER_32
			l_i: CE_ARRAY [INTEGER_32]
			l_tv: CE_ARRAY [SD_TIMEVAL]
			l_tm: CE_POINTER [SD_TIMEVAL]
			l_tt: INTEGER_32
			l_k: INTEGER_32
			l___result: CE_ARRAY [INTEGER_8]
			l___result___0: CE_ARRAY [INTEGER_8]
			l___result___1: CE_ARRAY [INTEGER_8]
			l_return, l_break, l_continue: BOOLEAN
		do
			create {CE_CLASS_ARRAY [SD_STAT]} l_st.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l___d0.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l___d1.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l___d0___0.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l___d1___0.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l___d0___1.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l___d1___1.make (<<1>>)
			create {CE_INTEGER_32_ARRAY} l_i.make (<<1>>)
			create {CE_CLASS_ARRAY [SD_TIMEVAL]} l_tv.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l___result.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l___result___0.make (<<1>>)
			create {CE_INTEGER_8_ARRAY} l___result___1.make (<<1>>)

			if (stat (ce_string ("."), l_st)).to_boolean then
				if (not (getenv (ce_string ("HOME"))).is_void) then
					chdir ((getenv (ce_string ("HOME")))).do_nothing
				end
			end
			memset (create {CE_VOID_POINTER}.make_cast (sa_zero.item), 0, (create {SD_SIGACTION}.make).structure_size.to_natural_32).do_nothing
			memset (create {CE_VOID_POINTER}.make_cast (signal_mask.item), 0, ({PLATFORM}.Integer_32_bytes.to_natural_32 * (32).to_natural_32)).do_nothing
			memset (create {CE_VOID_POINTER}.make_cast (signal_handlers.item), 0, ((create {PD_SIGNAL_HANDLER}.make).structure_size.to_natural_32 * (32).to_natural_32)).do_nothing
			from
			until
				(l_break)
			loop
				l_break := False
				asm18 (l___d0, l___d1, w_read.item)
				l_break := True
			end
			l_break := False
			from
			until
				(l_break)
			loop
				l_break := False
				asm19 (l___d0___0, l___d1___0, w_write.item)
				l_break := True
			end
			l_break := False
			from
			until
				(l_break)
			loop
				l_break := False
				asm20 (l___d0___1, l___d1___1, w_error.item)
				l_break := True
			end
			l_break := False
			w_max.item := 0
			last_time.item := get_time
			signal (13, create {CE_ROUTINE}.make_shared (default_pointer + 1)).do_nothing
			if ((create {P_OS_DEP_STATIC}).c2_pipe (signal_pipe.item)).to_boolean then
				(create {P_ERROR_STATIC}).error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: can%'t create pipe for signal handling"))]);
				(create {P_KBD_STATIC}).fatal_tty_exit
				exit (4)
			end
			fcntl ([signal_pipe.item[0], 4, 2_048]).do_nothing
			fcntl ([signal_pipe.item[1], 4, 2_048]).do_nothing
			set_handlers (signal_pipe.item[0], signal_break_address, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_ROUTINE}.make_shared (default_pointer), create {CE_VOID_POINTER}.make_shared (default_pointer))
			a_init___1.call ([])
			if not ((bottom_halves.item.item.next).is_equal (bottom_halves.item)) then
				check_bottom_halves
			end
			from
			until
				not (not (l_return or l_break) and then (not terminate.item.to_boolean))
			loop
				l_break := False
				l_continue := False
				l_tm := create {CE_CLASS_POINTER [SD_TIMEVAL]}.make_shared (default_pointer)
				check_signals.do_nothing
				check_timers;
				(create {P_TERMINAL_STATIC}).redraw_all_terminals
				if not ((timers.item.item.next).is_equal (timers.item)) then
					l_tt := (create {CE_CLASS_POINTER [PD_TIMER]}.make_cast (timers.item.item.next)).item.interval + 1
					if l_tt < 0 then
						l_tt := (0).to_integer_32
					end
					l_tv.item.tv_sec := l_tt // 1_000
					l_tv.item.tv_usec := (l_tt \\ 1_000) * 1_000
					l_tm := l_tv
				end
				memcpy (create {CE_VOID_POINTER}.make_cast (x_read.item), create {CE_VOID_POINTER}.make_cast (w_read.item), (create {SD_FD_SET}.make).structure_size.to_natural_32).do_nothing
				memcpy (create {CE_VOID_POINTER}.make_cast (x_write.item), create {CE_VOID_POINTER}.make_cast (w_write.item), (create {SD_FD_SET}.make).structure_size.to_natural_32).do_nothing
				memcpy (create {CE_VOID_POINTER}.make_cast (x_error.item), create {CE_VOID_POINTER}.make_cast (w_error.item), (create {SD_FD_SET}.make).structure_size.to_natural_32).do_nothing
				if terminate.item.to_boolean then
					l_break := True
				end
				if not (l_return or l_break or l_continue) and then (not w_max.item.to_boolean) then
					if (timers.item.item.next).is_equal (timers.item) then
						l_break := True
					end
				end
				if not (l_return or l_break or l_continue) then
					if (check_signals).to_boolean then
						l_continue := True
					end
				end
				if not (l_return or l_break or l_continue) then
					l_n := select2 (w_max.item, x_read.item, x_write.item, x_error.item, l_tm)
					if l_n < 0 then
						if (errno_location).item /= 4 then
							(create {P_ERROR_STATIC}).error ([create {CE_NATURAL_8_POINTER}.make_cast (ce_string ("ERROR: select failed: %%d")), (errno_location).item])
						end
						l_continue := True
					end
				end
				if not (l_return or l_break or l_continue) then
					check_signals.do_nothing
					check_timers
					l_i.item := -1
					from
					until
						(l_break)
					loop
						l_break := False
						if l_n > 0 then
							l_i.item := l_i.item + 1
							if not (l_i.item < w_max.item) then
								l_break := True
							end
						else
							l_break := True
						end
						if not (l_break) then
							l_k := 0
							asm21 (l___result, l_i, x_read.item)
							if (l___result.item).to_boolean then
								if (not (threads.item[l_i.item].read_func).is_void) then
									(threads.item[l_i.item].read_func).call ([threads.item[l_i.item].data])
									if not ((bottom_halves.item.item.next).is_equal (bottom_halves.item)) then
										check_bottom_halves
									end
								end
								l_k := 1
							end
							asm22 (l___result___0, l_i, x_write.item)
							if (l___result___0.item).to_boolean then
								if (not (threads.item[l_i.item].write_func).is_void) then
									(threads.item[l_i.item].write_func).call ([threads.item[l_i.item].data])
									if not ((bottom_halves.item.item.next).is_equal (bottom_halves.item)) then
										check_bottom_halves
									end
								end
								l_k := 1
							end
							asm23 (l___result___1, l_i, x_error.item)
							if (l___result___1.item).to_boolean then
								if (not (threads.item[l_i.item].error_func).is_void) then
									(threads.item[l_i.item].error_func).call ([threads.item[l_i.item].data])
									if not ((bottom_halves.item.item.next).is_equal (bottom_halves.item)) then
										check_bottom_halves
									end
								end
								l_k := 1
							end
							l_n := l_n - l_k
						end
					end
					l_break := False
				end
			end
			l_break := False
			l_continue := False
		end

feature {ANY} -- routine addresses

	signal_break_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent signal_break)
		end

	got_signal_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent got_signal)
		end

	sigchld_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent sigchld)
		end

end
