class
	SD_GIOCHANNEL

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal,
			bitfield_helper
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			make_memory -- has bitfields; back with memory for correct semantics
		end

feature {ANY} -- fields

	ref_count: INTEGER_32 assign set_ref_count

	funcs: CE_POINTER [SD_GIOFUNCS] assign set_funcs

	encoding: CE_POINTER [INTEGER_8] assign set_encoding

	read_cd: CE_POINTER [ANY] assign set_read_cd

	write_cd: CE_POINTER [ANY] assign set_write_cd

	line_term: CE_POINTER [INTEGER_8] assign set_line_term

	line_term_len: NATURAL_32 assign set_line_term_len

	buf_size: NATURAL_32 assign set_buf_size

	read_buf: CE_POINTER [SD_GSTRING] assign set_read_buf

	encoded_read_buf: CE_POINTER [SD_GSTRING] assign set_encoded_read_buf

	write_buf: CE_POINTER [SD_GSTRING] assign set_write_buf

	partial_write_buf: CE_ARRAY [INTEGER_8]

	use_buffer: NATURAL_32 assign set_use_buffer

	do_encode: NATURAL_32 assign set_do_encode

	close_on_unref: NATURAL_32 assign set_close_on_unref

	is_readable: NATURAL_32 assign set_is_readable

	is_writeable: NATURAL_32 assign set_is_writeable

	is_seekable: NATURAL_32 assign set_is_seekable

	reserved1: CE_POINTER [ANY] assign set_reserved1

	reserved2: CE_POINTER [ANY] assign set_reserved2

feature {ANY} -- field setters

	set_ref_count (a_ref_count: INTEGER_32)
		do
			ref_count := a_ref_count
			update_memory_field ("ref_count")
		end

	set_funcs (a_funcs: CE_POINTER [SD_GIOFUNCS])
		do
			funcs := a_funcs
			update_memory_field ("funcs")
		end

	set_encoding (a_encoding: CE_POINTER [INTEGER_8])
		do
			encoding := a_encoding
			update_memory_field ("encoding")
		end

	set_read_cd (a_read_cd: CE_POINTER [ANY])
		do
			read_cd := a_read_cd
			update_memory_field ("read_cd")
		end

	set_write_cd (a_write_cd: CE_POINTER [ANY])
		do
			write_cd := a_write_cd
			update_memory_field ("write_cd")
		end

	set_line_term (a_line_term: CE_POINTER [INTEGER_8])
		do
			line_term := a_line_term
			update_memory_field ("line_term")
		end

	set_line_term_len (a_line_term_len: NATURAL_32)
		do
			line_term_len := a_line_term_len
			update_memory_field ("line_term_len")
		end

	set_buf_size (a_buf_size: NATURAL_32)
		do
			buf_size := a_buf_size
			update_memory_field ("buf_size")
		end

	set_read_buf (a_read_buf: CE_POINTER [SD_GSTRING])
		do
			read_buf := a_read_buf
			update_memory_field ("read_buf")
		end

	set_encoded_read_buf (a_encoded_read_buf: CE_POINTER [SD_GSTRING])
		do
			encoded_read_buf := a_encoded_read_buf
			update_memory_field ("encoded_read_buf")
		end

	set_write_buf (a_write_buf: CE_POINTER [SD_GSTRING])
		do
			write_buf := a_write_buf
			update_memory_field ("write_buf")
		end

	set_use_buffer (a_use_buffer: NATURAL_32)
		do
			use_buffer := a_use_buffer
			update_memory_field ("use_buffer")
		end

	set_do_encode (a_do_encode: NATURAL_32)
		do
			do_encode := a_do_encode
			update_memory_field ("do_encode")
		end

	set_close_on_unref (a_close_on_unref: NATURAL_32)
		do
			close_on_unref := a_close_on_unref
			update_memory_field ("close_on_unref")
		end

	set_is_readable (a_is_readable: NATURAL_32)
		do
			is_readable := a_is_readable
			update_memory_field ("is_readable")
		end

	set_is_writeable (a_is_writeable: NATURAL_32)
		do
			is_writeable := a_is_writeable
			update_memory_field ("is_writeable")
		end

	set_is_seekable (a_is_seekable: NATURAL_32)
		do
			is_seekable := a_is_seekable
			update_memory_field ("is_seekable")
		end

	set_reserved1 (a_reserved1: CE_POINTER [ANY])
		do
			reserved1 := a_reserved1
			update_memory_field ("reserved1")
		end

	set_reserved2 (a_reserved2: CE_POINTER [ANY])
		do
			reserved2 := a_reserved2
			update_memory_field ("reserved2")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			ref_count := a_other.ref_count
			funcs := a_other.funcs
			encoding := a_other.encoding
			read_cd := a_other.read_cd
			write_cd := a_other.write_cd
			line_term := a_other.line_term
			line_term_len := a_other.line_term_len
			buf_size := a_other.buf_size
			read_buf := a_other.read_buf
			encoded_read_buf := a_other.encoded_read_buf
			write_buf := a_other.write_buf
			partial_write_buf := a_other.partial_write_buf.twin
			use_buffer := a_other.use_buffer
			do_encode := a_other.do_encode
			close_on_unref := a_other.close_on_unref
			is_readable := a_other.is_readable
			is_writeable := a_other.is_writeable
			is_seekable := a_other.is_seekable
			reserved1 := a_other.reserved1
			reserved2 := a_other.reserved2
			create_memory_copy -- has bitfields; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				ref_count = a_other.ref_count and
				funcs.is_equal (a_other.funcs) and
				encoding.is_equal (a_other.encoding) and
				read_cd.is_equal (a_other.read_cd) and
				write_cd.is_equal (a_other.write_cd) and
				line_term.is_equal (a_other.line_term) and
				line_term_len = a_other.line_term_len and
				buf_size = a_other.buf_size and
				read_buf.is_equal (a_other.read_buf) and
				encoded_read_buf.is_equal (a_other.encoded_read_buf) and
				write_buf.is_equal (a_other.write_buf) and
				partial_write_buf.is_equal (a_other.partial_write_buf) and
				use_buffer = a_other.use_buffer and
				do_encode = a_other.do_encode and
				close_on_unref = a_other.close_on_unref and
				is_readable = a_other.is_readable and
				is_writeable = a_other.is_writeable and
				is_seekable = a_other.is_seekable and
				reserved1.is_equal (a_other.reserved1) and
				reserved2.is_equal (a_other.reserved2)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("ref_count"),
					create {CE_STRUCTURE_FIELD}.make ("funcs"),
					create {CE_STRUCTURE_FIELD}.make ("encoding"),
					create {CE_STRUCTURE_FIELD}.make ("read_cd"),
					create {CE_STRUCTURE_FIELD}.make ("write_cd"),
					create {CE_STRUCTURE_FIELD}.make ("line_term"),
					create {CE_STRUCTURE_FIELD}.make ("line_term_len"),
					create {CE_STRUCTURE_FIELD}.make ("buf_size"),
					create {CE_STRUCTURE_FIELD}.make ("read_buf"),
					create {CE_STRUCTURE_FIELD}.make ("encoded_read_buf"),
					create {CE_STRUCTURE_FIELD}.make ("write_buf"),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("partial_write_buf", <<6>>),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("use_buffer"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("do_encode"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("close_on_unref"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("is_readable"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("is_writeable"),
					create {CE_STRUCTURE_FIELD}.make_bitfield ("is_seekable"),
					create {CE_STRUCTURE_FIELD}.make ("reserved1"),
					create {CE_STRUCTURE_FIELD}.make ("reserved2")
				>>)
			init_structure_info
		end

	bitfield_helper (a_operation: INTEGER_32; a_field_name, a_struct_pointer: POINTER; a_value: NATURAL_64): NATURAL_64
		external
			"C inline use <glib.h>"
		alias
			"[
				char *field_name = (char *)$a_field_name;
				if($a_operation == 1) {
					if(field_name == (char *)0) return sizeof(struct _GIOChannel);
					if(strcmp(field_name, "reserved1") == 0) return offsetof(struct _GIOChannel, reserved1);
				} else {
					struct _GIOChannel *p = (struct _GIOChannel *)$a_struct_pointer;
					if($a_operation == 2) {
						     if(strcmp(field_name, "use_buffer") == 0) p->use_buffer = (unsigned int)$a_value;
						else if(strcmp(field_name, "do_encode") == 0) p->do_encode = (unsigned int)$a_value;
						else if(strcmp(field_name, "close_on_unref") == 0) p->close_on_unref = (unsigned int)$a_value;
						else if(strcmp(field_name, "is_readable") == 0) p->is_readable = (unsigned int)$a_value;
						else if(strcmp(field_name, "is_writeable") == 0) p->is_writeable = (unsigned int)$a_value;
						else if(strcmp(field_name, "is_seekable") == 0) p->is_seekable = (unsigned int)$a_value;
					} else if($a_operation == 3) {
						if(strcmp(field_name, "use_buffer") == 0) return p->use_buffer;
						if(strcmp(field_name, "do_encode") == 0) return p->do_encode;
						if(strcmp(field_name, "close_on_unref") == 0) return p->close_on_unref;
						if(strcmp(field_name, "is_readable") == 0) return p->is_readable;
						if(strcmp(field_name, "is_writeable") == 0) return p->is_writeable;
						if(strcmp(field_name, "is_seekable") == 0) return p->is_seekable;
					}
				}
			]"
		end

end
