class
	PD_MY_TM

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {ANY} -- fields

	tm_sec: INTEGER_32 assign set_tm_sec

	tm_min: INTEGER_32 assign set_tm_min

	tm_hour: INTEGER_32 assign set_tm_hour

	tm_mday: INTEGER_32 assign set_tm_mday

	tm_mon: INTEGER_32 assign set_tm_mon

	tm_year: INTEGER_32 assign set_tm_year

feature {ANY} -- field setters

	set_tm_sec (a_tm_sec: INTEGER_32)
		do
			tm_sec := a_tm_sec
			update_memory_field ("tm_sec")
		end

	set_tm_min (a_tm_min: INTEGER_32)
		do
			tm_min := a_tm_min
			update_memory_field ("tm_min")
		end

	set_tm_hour (a_tm_hour: INTEGER_32)
		do
			tm_hour := a_tm_hour
			update_memory_field ("tm_hour")
		end

	set_tm_mday (a_tm_mday: INTEGER_32)
		do
			tm_mday := a_tm_mday
			update_memory_field ("tm_mday")
		end

	set_tm_mon (a_tm_mon: INTEGER_32)
		do
			tm_mon := a_tm_mon
			update_memory_field ("tm_mon")
		end

	set_tm_year (a_tm_year: INTEGER_32)
		do
			tm_year := a_tm_year
			update_memory_field ("tm_year")
		end

feature {PD_MY_TM} -- private attributes

	month_days_cumulative: CE_STATIC [CE_ARRAY [INTEGER_32]]
		once ("PROCESS")
			Result.item := create {CE_INTEGER_32_ARRAY}.make_initialized (<<
					0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
				>>)
		end

feature {ANY} -- public routines

	my_timegm: INTEGER_32
		local
			l_month: INTEGER_32
			l_year: INTEGER_32
			l_leap_days: INTEGER_32
			l_return: BOOLEAN
		do
			if tm_year < 70 then
				Result := (-1).to_integer_32
				l_return := True
			end
			if not l_return then
				l_year := tm_year + 1_900
				l_month := tm_mon
				if l_month < 0 then
					l_year := l_year + ((11 - l_month) // 12)
					l_month := 11 - (11 - l_month) \\ 12
				elseif l_month >= 12 then
					l_year := l_year - (l_month // 12)
					l_month := l_month \\ (12)
				end
				l_leap_days := l_year - (tm_mon <= 1).to_integer
				l_leap_days := ((((l_leap_days // 4 - l_leap_days // 100) + l_leap_days // 400) - 492) + 19) - 4
				Result := ((((((((l_year - 1_970).to_integer_32 * 365 + l_leap_days.to_integer_32) + (month_days_cumulative.item[l_month]).to_integer_32) + tm_mday.to_integer_32) - 1) * 24 + tm_hour.to_integer_32) * 60 + tm_min.to_integer_32) * 60 + tm_sec.to_integer_32)
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			tm_sec := a_other.tm_sec
			tm_min := a_other.tm_min
			tm_hour := a_other.tm_hour
			tm_mday := a_other.tm_mday
			tm_mon := a_other.tm_mon
			tm_year := a_other.tm_year
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				tm_sec = a_other.tm_sec and
				tm_min = a_other.tm_min and
				tm_hour = a_other.tm_hour and
				tm_mday = a_other.tm_mday and
				tm_mon = a_other.tm_mon and
				tm_year = a_other.tm_year
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("tm_sec"),
					create {CE_STRUCTURE_FIELD}.make ("tm_min"),
					create {CE_STRUCTURE_FIELD}.make ("tm_hour"),
					create {CE_STRUCTURE_FIELD}.make ("tm_mday"),
					create {CE_STRUCTURE_FIELD}.make ("tm_mon"),
					create {CE_STRUCTURE_FIELD}.make ("tm_year")
				>>)
			init_structure_info
		end

end
