class
	P_BESSEL_SEQUENCE_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	rk_step (a_nu, a_x, a_dx: REAL_64; a_Jp, a_J: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_Jp.is_void
			not a_J.is_void
		local
			l_p_0: REAL_64
			l_u_0: REAL_64
			l_p_1: REAL_64
			l_u_1: REAL_64
			l_p_2: REAL_64
			l_u_2: REAL_64
			l_p_3: REAL_64
			l_u_3: REAL_64
			l_p_4: REAL_64
			l_u_4: REAL_64
		do
			l_p_0 := a_Jp.item
			l_u_0 := a_J.item
			l_p_1 := a_dx * (-l_p_0 / a_x + ((a_nu * a_nu) / (a_x * a_x) - 1.0) * l_u_0)
			l_u_1 := a_dx * l_p_0
			l_p_2 := a_dx * (-(l_p_0 + 0.5 * l_p_1) / (a_x + 0.5 * a_dx) + ((a_nu * a_nu) / ((a_x + 0.5 * a_dx) * (a_x + 0.5 * a_dx)) - 1.0) * (l_u_0 + 0.5 * l_u_1))
			l_u_2 := a_dx * (l_p_0 + 0.5 * l_p_1)
			l_p_3 := a_dx * (-(l_p_0 + 0.5 * l_p_2) / (a_x + 0.5 * a_dx) + ((a_nu * a_nu) / ((a_x + 0.5 * a_dx) * (a_x + 0.5 * a_dx)) - 1.0) * (l_u_0 + 0.5 * l_u_2))
			l_u_3 := a_dx * (l_p_0 + 0.5 * l_p_2)
			l_p_4 := a_dx * (-(l_p_0 + l_p_3) / (a_x + a_dx) + ((a_nu * a_nu) / ((a_x + a_dx) * (a_x + a_dx)) - 1.0) * (l_u_0 + l_u_3))
			l_u_4 := a_dx * (l_p_0 + l_p_3)
			a_Jp.item := (((l_p_0 + l_p_1 / 6.0) + l_p_2 / 3.0) + l_p_3 / 3.0) + l_p_4 / 6.0
			a_J.item := (((l_u_0 + l_u_1 / 6.0) + l_u_2 / 3.0) + l_u_3 / 3.0) + l_u_4 / 6.0
			Result := 0
		end

	gsl_sf_bessel_sequence_Jnu_e (a_nu: REAL_64; a_mode, a_size: NATURAL_32; a_v: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_goal: NATURAL_32
			l_dx_array: CE_ARRAY [REAL_64]
			l_dx_nominal: REAL_64
			l_cnu: INTEGER_32
			l_nu13: REAL_64
			l_smalls: CE_ARRAY [REAL_64]
			l_x_small: REAL_64
			l_tmp___2: REAL_64
			l_J0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_J1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Jp: CE_ARRAY [REAL_64]
			l_J: CE_ARRAY [REAL_64]
			l_x: REAL_64
			l_i: NATURAL_32
			l_dv: REAL_64
			l_Nd: INTEGER_32
			l_dx: REAL_64
			l_xj: REAL_64
			l_j2: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_dx_array.make (<<3>>)
			create {CE_REAL_64_ARRAY} l_smalls.make (<<11>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_J0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_J1.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_Jp.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_J.make (<<1>>)

			if a_nu < 0.0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("bessel_sequence.c"), 62, 1)
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_size = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("error"), ce_string ("bessel_sequence.c"), 65, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_goal := ((create {P_GSL_GSL_MODE_STATIC}).GSL_MODE_PREC (a_mode))
				l_dx_array[0] := 0.001
				l_dx_array[1] := 0.03
				l_dx_array[2] := 0.1
				l_dx_nominal := l_dx_array[l_goal.to_integer_32]
				l_cnu := (ceil (a_nu)).truncated_to_integer
				l_nu13 := (pow (a_nu, 1.0 / 3.0))
				l_smalls[0] := 0.01
				l_smalls[1] := 0.02
				l_smalls[2] := 0.4
				l_smalls[3] := 0.7
				l_smalls[4] := 1.3
				l_smalls[5] := 2.0
				l_smalls[6] := 2.5
				l_smalls[7] := 3.2
				l_smalls[8] := 3.5
				l_smalls[9] := 4.5
				l_smalls[10] := 6.0
				if a_nu >= 10.0 then
					l_tmp___2 := a_nu - l_nu13
				else
					l_tmp___2 := l_smalls[l_cnu]
				end
				l_x_small := l_tmp___2
				l_i := (0).to_natural_32
				l_x := a_v[0]
				l_J0.item.gsl_sf_bessel_Jnu_e (a_nu, l_x).do_nothing
				a_v[0] := l_J0.item.val
				l_i := l_i + 1
				if l_x = 0.0 then
					if a_v[1] <= l_x then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("error"), ce_string ("bessel_sequence.c"), 94, 5)
							Result := 5
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_x := a_v[1]
						l_J0.item.gsl_sf_bessel_Jnu_e (a_nu, l_x).do_nothing
						a_v[1] := l_J0.item.val
						l_i := l_i + 1
					end
				end
				if not l_return then
					from
					until
						(l_return or l_break)
					loop
						l_break := False
						if a_v[l_i.to_integer_32] < l_x_small then
							if not (l_i < a_size) then
								l_break := True
							end
						else
							l_break := True
						end
						if not (l_return or l_break) then
							if a_v[l_i.to_integer_32] <= l_x then
								from
								until
									(l_return or l_break)
								loop
									l_break := False;
									(create {P_ERROR_STATIC}).gsl_error (ce_string ("error"), ce_string ("bessel_sequence.c"), 109, 5)
									Result := 5
									l_return := True
									if not l_return then
										l_break := True
									end
								end
								if not l_return then
									l_break := False
								end
							end
							if not l_return then
								l_x := a_v[l_i.to_integer_32]
								l_J0.item.gsl_sf_bessel_Jnu_e (a_nu, l_x).do_nothing
								a_v[l_i.to_integer_32] := l_J0.item.val
								l_i := l_i + 1
							end
						end
					end
				end
				if not l_return then
					l_break := False
					l_J1.item.gsl_sf_bessel_Jnu_e (a_nu + 1.0, l_x).do_nothing
					l_J.item := l_J0.item.val
					l_Jp.item := -l_J1.item.val + (a_nu / l_x) * l_J0.item.val
					from
					until
						not (not (l_return) and then (l_i < a_size))
					loop
						l_dv := a_v[l_i.to_integer_32] - l_x
						l_Nd := (ceil (l_dv / l_dx_nominal)).truncated_to_integer
						l_dx := l_dv / l_Nd.to_double
						if a_v[l_i.to_integer_32] <= l_x then
							from
							until
								(l_return or l_break)
							loop
								l_break := False;
								(create {P_ERROR_STATIC}).gsl_error (ce_string ("error"), ce_string ("bessel_sequence.c"), 137, 5)
								Result := 5
								l_return := True
								if not l_return then
									l_break := True
								end
							end
							if not l_return then
								l_break := False
							end
						end
						if not l_return then
							l_j2 := 0
							l_xj := l_x
							from
							until
								not (l_j2 < l_Nd)
							loop
								rk_step (a_nu, l_xj, l_dx, l_Jp, l_J).do_nothing
								l_j2 := l_j2 + 1
								l_xj := l_xj + l_dx
							end
							l_x := a_v[l_i.to_integer_32]
							a_v[l_i.to_integer_32] := l_J.item
							l_i := l_i + 1
						end
					end
				end
				if not l_return then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				Result := 0
			end
		end

end
