class
	P_COULOMB_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	C0sq (a_eta: REAL_64): REAL_64
		local
			l_twopieta: REAL_64
			l_scale: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_scale.make (<<1>>)

			l_twopieta := (2.0 * 3.14159265358979323846) * a_eta
			if (fabs (a_eta)) < 2.2204460492503131e-16 then
				Result := (1.0)
			elseif l_twopieta > 7.0978271289338397e+02 then
				Result := (0.0)
			else
				l_scale.item.gsl_sf_expm1_e (l_twopieta).do_nothing
				Result := (l_twopieta / l_scale.item.val)
			end
		end

	gsl_sf_coulomb_CL_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta: REAL_64; a_cl: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_cl.is_void
		local
			l_k: INTEGER_32
			l_cl_0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_L: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_cl_0.make (<<1>>)

			l_cl_0.item.gsl_sf_coulomb_CL_e (a_lam_min, a_eta).do_nothing
			a_cl[0] := l_cl_0.item.val
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_L := a_lam_min + l_k.to_double
				a_cl[l_k] := (a_cl[(l_k - 1)] * (hypot (l_L, a_eta))) / (l_L * (2.0 * l_L + 1.0))
				l_k := l_k + 1
			end
			Result := 0
		end

	gsl_sf_coulomb_wave_F_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta, a_x: REAL_64; a_fc_array, a_F_exp: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_F_exp.is_void
		local
			l_k: INTEGER_32
			l_f_0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_x_inv: REAL_64
			l_lam_max: REAL_64
			l_F___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Fp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_G: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Gp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_G_exp: CE_ARRAY [REAL_64]
			l_stat_FG: INTEGER_32
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam: REAL_64
			l_k___0: INTEGER_32
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_f_0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_F___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_Fp.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_G.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_Gp.make (<<1>>)
			create {CE_REAL_64_ARRAY} l_G_exp.make (<<1>>)

			if a_x = 0.0 then
				a_F_exp.item := 0.0
				l_k := 0
				from
				until
					not (l_k <= a_kmax___0)
				loop
					a_fc_array[l_k] := 0.0
					l_k := l_k + 1
				end
				if a_lam_min = 0.0 then
					l_f_0.item.CLeta (0.0, a_eta).do_nothing
					a_fc_array[0] := l_f_0.item.val
				end
				Result := 0
			else
				l_x_inv := 1.0 / a_x
				l_lam_max := a_lam_min + a_kmax___0.to_double
				l_stat_FG := (l_F___0.item.gsl_sf_coulomb_wave_FG_e (a_eta, a_x, l_lam_max, 0, l_Fp, l_G, l_Gp, a_F_exp, l_G_exp))
				l_fcl := l_F___0.item.val
				l_fpl := l_Fp.item.val
				l_lam := l_lam_max
				a_fc_array[a_kmax___0] := l_F___0.item.val
				l_k___0 := a_kmax___0 - 1
				from
				until
					not (l_k___0 >= 0)
				loop
					l_el := a_eta / l_lam
					l_rl := (hypot (1.0, l_el))
					l_sl := l_el + l_lam * l_x_inv
					l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
					a_fc_array[l_k___0] := l_fc_lm1
					l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
					l_fcl := l_fc_lm1
					l_lam := l_lam - (1.0)
					l_k___0 := l_k___0 - 1
				end
				Result := l_stat_FG
			end
		end

	gsl_sf_coulomb_wave_FG_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta, a_x: REAL_64; a_fc_array, a_gc_array, a_F_exp, a_G_exp: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_fc_array.is_void
			not a_gc_array.is_void
			not a_F_exp.is_void
			not a_G_exp.is_void
		local
			l_x_inv: REAL_64
			l_lam_max: REAL_64
			l_F___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Fp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_G: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Gp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_FG: INTEGER_32
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam: REAL_64
			l_k: INTEGER_32
			l_gcl: REAL_64
			l_gpl: REAL_64
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
			l_el___0: REAL_64
			l_rl___0: REAL_64
			l_sl___0: REAL_64
			l_gcl1: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_F___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_Fp.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_G.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_Gp.make (<<1>>)

			l_x_inv := 1.0 / a_x
			l_lam_max := a_lam_min + a_kmax___0.to_double
			l_stat_FG := (l_F___0.item.gsl_sf_coulomb_wave_FG_e (a_eta, a_x, l_lam_max, a_kmax___0, l_Fp, l_G, l_Gp, a_F_exp, a_G_exp))
			l_fcl := l_F___0.item.val
			l_fpl := l_Fp.item.val
			l_lam := l_lam_max
			a_fc_array[a_kmax___0] := l_F___0.item.val
			l_k := a_kmax___0 - 1
			from
			until
				not (l_k >= 0)
			loop
				l_el := a_eta / l_lam
				l_rl := (hypot (1.0, l_el))
				l_sl := l_el + l_lam * l_x_inv
				l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
				a_fc_array[l_k] := l_fc_lm1
				l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
				l_fcl := l_fc_lm1
				l_lam := l_lam - (1.0)
				l_k := l_k - 1
			end
			l_gcl := l_G.item.val
			l_gpl := l_Gp.item.val
			l_lam := a_lam_min + 1.0
			a_gc_array[0] := l_G.item.val
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_el___0 := a_eta / l_lam
				l_rl___0 := (hypot (1.0, l_el___0))
				l_sl___0 := l_el___0 + l_lam * l_x_inv
				l_gcl1 := (l_sl___0 * l_gcl - l_gpl) / l_rl___0
				a_gc_array[l_k] := l_gcl1
				l_gpl := l_rl___0 * l_gcl - l_sl___0 * l_gcl1
				l_gcl := l_gcl1
				l_lam := l_lam + (1.0)
				l_k := l_k + 1
			end
			Result := l_stat_FG
		end

	gsl_sf_coulomb_wave_FGp_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta, a_x: REAL_64; a_fc_array, a_fcp_array, a_gc_array, a_gcp_array, a_F_exp, a_G_exp: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_fc_array.is_void
			not a_fcp_array.is_void
			not a_gc_array.is_void
			not a_gcp_array.is_void
			not a_F_exp.is_void
			not a_G_exp.is_void
		local
			l_x_inv: REAL_64
			l_lam_max: REAL_64
			l_F___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Fp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_G: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Gp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_FG: INTEGER_32
			l_fcl: REAL_64
			l_fpl: REAL_64
			l_lam: REAL_64
			l_k: INTEGER_32
			l_gcl: REAL_64
			l_gpl: REAL_64
			l_el: REAL_64
			l_rl: REAL_64
			l_sl: REAL_64
			l_fc_lm1: REAL_64
			l_el___0: REAL_64
			l_rl___0: REAL_64
			l_sl___0: REAL_64
			l_gcl1: REAL_64
		do
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_F___0.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_Fp.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_G.make (<<1>>)
			create {CE_CLASS_ARRAY [PD_GSL_SF_RESULT_STRUCT]} l_Gp.make (<<1>>)

			l_x_inv := 1.0 / a_x
			l_lam_max := a_lam_min + a_kmax___0.to_double
			l_stat_FG := (l_F___0.item.gsl_sf_coulomb_wave_FG_e (a_eta, a_x, l_lam_max, a_kmax___0, l_Fp, l_G, l_Gp, a_F_exp, a_G_exp))
			l_fcl := l_F___0.item.val
			l_fpl := l_Fp.item.val
			l_lam := l_lam_max
			a_fc_array[a_kmax___0] := l_F___0.item.val
			a_fcp_array[a_kmax___0] := l_Fp.item.val
			l_k := a_kmax___0 - 1
			from
			until
				not (l_k >= 0)
			loop
				l_el := a_eta / l_lam
				l_rl := (hypot (1.0, l_el))
				l_sl := l_el + l_lam * l_x_inv
				l_fc_lm1 := (l_fcl * l_sl + l_fpl) / l_rl
				a_fc_array[l_k] := l_fc_lm1
				l_fpl := l_fc_lm1 * l_sl - l_fcl * l_rl
				a_fcp_array[l_k] := l_fpl
				l_fcl := l_fc_lm1
				l_lam := l_lam - (1.0)
				l_k := l_k - 1
			end
			l_gcl := l_G.item.val
			l_gpl := l_Gp.item.val
			l_lam := a_lam_min + 1.0
			a_gc_array[0] := l_G.item.val
			a_gcp_array[0] := l_Gp.item.val
			l_k := 1
			from
			until
				not (l_k <= a_kmax___0)
			loop
				l_el___0 := a_eta / l_lam
				l_rl___0 := (hypot (1.0, l_el___0))
				l_sl___0 := l_el___0 + l_lam * l_x_inv
				l_gcl1 := (l_sl___0 * l_gcl - l_gpl) / l_rl___0
				a_gc_array[l_k] := l_gcl1
				l_gpl := l_rl___0 * l_gcl - l_sl___0 * l_gcl1
				a_gcp_array[l_k] := l_gpl
				l_gcl := l_gcl1
				l_lam := l_lam + (1.0)
				l_k := l_k + 1
			end
			Result := l_stat_FG
		end

	gsl_sf_coulomb_wave_sphF_array (a_lam_min: REAL_64; a_kmax___0: INTEGER_32; a_eta, a_x: REAL_64; a_fc_array, a_F_exp: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_k: INTEGER_32
			l_k___0: INTEGER_32
			l_stat_F: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_lam_min < -0.5) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("domain error"), ce_string ("coulomb.c"), 1_387, 1)
							Result := 1
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_x < 10.0 / 1.7976931348623157e+308 then
							l_k := 0
							from
							until
								not (l_k <= a_kmax___0)
							loop
								a_fc_array[l_k] := 0.0
								l_k := l_k + 1
							end
							if a_lam_min = 0.0 then
								a_fc_array[0] := sqrt ((C0sq (a_eta)))
							end
							a_F_exp.item := 0.0
							if a_x = 0.0 then
								Result := 0
								l_return := True
							else
								from
								until
									(l_return or l_break)
								loop
									l_break := False;
									(create {P_ERROR_STATIC}).gsl_error (ce_string ("underflow"), ce_string ("coulomb.c"), 1_401, 15)
									Result := 15
									l_return := True
									if not l_return then
										l_break := True
									end
								end
								if not l_return then
									l_break := False
								end
							end
						else
							l_stat_F := (gsl_sf_coulomb_wave_F_array (a_lam_min, a_kmax___0, a_eta, a_x, a_fc_array, a_F_exp))
							l_k___0 := 0
							from
							until
								not (l_k___0 <= a_kmax___0)
							loop
								a_fc_array[l_k___0] := a_fc_array[l_k___0] / a_x
								l_k___0 := l_k___0 + 1
							end
							Result := l_stat_F
							l_return := True
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

end
