class
	P_HYPERGEOMETRIC_STATIC

inherit
	CE_UTILITIES
	S_GCC_BUILTIN_STATIC

feature {ANY} -- public routines

	lower_tail (a_k, a_n1, a_n2, a_t: NATURAL_32): REAL_64
		local
			l_relerr: REAL_64
			l_i: INTEGER_32
			l_s: REAL_64
			l_P___2: REAL_64
			l_factor: REAL_64
			l_break: BOOLEAN
		do
			l_i := a_k.to_integer_32
			l_s := (create {P_HYPERG_STATIC}).gsl_ran_hypergeometric_pdf (l_i.to_natural_32, a_n1, a_n2, a_t)
			l_P___2 := l_s
			from
			until
				not (not (l_break) and then (l_i > 0))
			loop
				l_break := False
				l_factor := (l_i.to_double / ((a_n1 - l_i.to_natural_32).to_real_64 + 1.0)) * (((a_n2 + l_i.to_natural_32) - a_t).to_real_64 / ((a_t - l_i.to_natural_32).to_real_64 + 1.0))
				l_s := l_s * l_factor
				l_P___2 := l_P___2 + l_s
				l_relerr := l_s / l_P___2
				if l_relerr < 2.2204460492503131e-16 then
					l_break := True
				end
				if not (l_break) then
					l_i := l_i - 1
				end
			end
			l_break := False
			Result := l_P___2
		end

	upper_tail (a_k, a_n1, a_n2, a_t: NATURAL_32): REAL_64
		local
			l_relerr: REAL_64
			l_i: NATURAL_32
			l_s: REAL_64
			l_Q___1: REAL_64
			l_factor: REAL_64
			l_break: BOOLEAN
		do
			l_i := a_k + {NATURAL_32} 1
			l_s := (create {P_HYPERG_STATIC}).gsl_ran_hypergeometric_pdf (l_i, a_n1, a_n2, a_t)
			l_Q___1 := l_s
			from
			until
				not (not (l_break) and then (l_i < a_t))
			loop
				l_break := False
				l_factor := ((a_n1 - l_i).to_real_64 / (l_i.to_real_64 + 1.0)) * ((a_t - l_i).to_real_64 / (((a_n2 + l_i).to_real_64 + 1.0) - a_t.to_real_64))
				l_s := l_s * l_factor
				l_Q___1 := l_Q___1 + l_s
				l_relerr := l_s / l_Q___1
				if l_relerr < 2.2204460492503131e-16 then
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			Result := l_Q___1
		end

	gsl_cdf_hypergeometric_P (a_k, a_n1, a_n2, a_t: NATURAL_32): REAL_64
		local
			l_P___2: REAL_64
			l_midpoint: REAL_64
			l_return, l_break: BOOLEAN
		do
			if a_t > a_n1 + a_n2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("t larger than population size"), ce_string ("hypergeometric.c"), 119, 1)
					Result := (builtin_nanf (ce_string (""))).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_k >= a_n1 then
				l_P___2 := 1.0
			elseif a_k >= a_t then
				l_P___2 := 1.0
			elseif a_k.to_real_64 < 0.0 then
				l_P___2 := 0.0
			else
				l_midpoint := (a_t.to_real_64 * a_n1.to_real_64) / (a_n1.to_real_64 + a_n2.to_real_64)
				if a_k.to_real_64 >= l_midpoint then
					l_P___2 := (1).to_double - (upper_tail (a_k, a_n1, a_n2, a_t))
				else
					l_P___2 := lower_tail (a_k, a_n1, a_n2, a_t)
				end
			end
			if not l_return then
				Result := l_P___2
			end
		end

	gsl_cdf_hypergeometric_Q (a_k, a_n1, a_n2, a_t: NATURAL_32): REAL_64
		local
			l_Q___1: REAL_64
			l_midpoint: REAL_64
			l_return, l_break: BOOLEAN
		do
			if a_t > a_n1 + a_n2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("t larger than population size"), ce_string ("hypergeometric.c"), 158, 1)
					Result := (builtin_nanf (ce_string (""))).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_k >= a_n1 then
				l_Q___1 := 0.0
			elseif a_k >= a_t then
				l_Q___1 := 0.0
			elseif a_k.to_real_64 < 0.0 then
				l_Q___1 := 1.0
			else
				l_midpoint := (a_t.to_real_64 * a_n1.to_real_64) / (a_n1.to_real_64 + a_n2.to_real_64)
				if a_k.to_real_64 < l_midpoint then
					l_Q___1 := (1).to_double - (lower_tail (a_k, a_n1, a_n2, a_t))
				else
					l_Q___1 := upper_tail (a_k, a_n1, a_n2, a_t)
				end
			end
			if not l_return then
				Result := l_Q___1
			end
		end

end
