class
	P_VARIANCE_SOURCE_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	compute_long_double_variance (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := (a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_long_double_tss (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := (a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_long_double_variance_with_fixed_mean (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_long_double_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_long_double_sd_with_fixed_mean (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_long_double_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_long_double_variance_m (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_long_double_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_long_double_sd_m (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_long_double_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_long_double_variance (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_double_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_long_double_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_long_double_sd (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_double_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_long_double_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_long_double_tss_m (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_long_double_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_long_double_tss (a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_double_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_long_double_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_variance (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 ((a_data[(l_i * a_stride).to_integer_32] - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_tss (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 ((a_data[(l_i * a_stride).to_integer_32] - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_variance_with_fixed_mean (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_sd_with_fixed_mean (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_variance_m (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_sd_m (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_variance (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_sd (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_tss_m (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_tss (a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_float_variance (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_float_tss (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_float_variance_with_fixed_mean (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_float_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_float_sd_with_fixed_mean (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_float_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_float_variance_m (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_float_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_float_sd_m (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_float_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_float_variance (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_float_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_float_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_float_sd (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_float_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_float_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_float_tss_m (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_float_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_float_tss (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_float_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_float_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_ulong_variance (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_ulong_tss (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_ulong_variance_with_fixed_mean (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_ulong_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_ulong_sd_with_fixed_mean (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_ulong_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_ulong_variance_m (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_ulong_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_ulong_sd_m (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_ulong_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_ulong_variance (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ulong_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_ulong_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_ulong_sd (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ulong_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_ulong_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_ulong_tss_m (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_ulong_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_ulong_tss (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ulong_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_ulong_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_long_variance (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_long_tss (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_long_variance_with_fixed_mean (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_long_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_long_sd_with_fixed_mean (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_long_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_long_variance_m (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_long_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_long_sd_m (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_long_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_long_variance (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_long_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_long_sd (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_long_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_long_tss_m (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_long_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_long_tss (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_long_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_long_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_uint_variance (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_uint_tss (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_uint_variance_with_fixed_mean (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_uint_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_uint_sd_with_fixed_mean (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_uint_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_uint_variance_m (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_uint_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_uint_sd_m (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_uint_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_uint_variance (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uint_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_uint_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_uint_sd (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uint_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_uint_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_uint_tss_m (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_uint_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_uint_tss (a_data: CE_POINTER [NATURAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uint_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_uint_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_int_variance (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_int_tss (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_int_variance_with_fixed_mean (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_int_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_int_sd_with_fixed_mean (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_int_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_int_variance_m (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_int_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_int_sd_m (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_int_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_int_variance (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_int_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_int_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_int_sd (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_int_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_int_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_int_tss_m (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_int_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_int_tss (a_data: CE_POINTER [INTEGER_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_int_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_int_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_ushort_variance (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_ushort_tss (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_ushort_variance_with_fixed_mean (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_ushort_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_ushort_sd_with_fixed_mean (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_ushort_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_ushort_variance_m (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_ushort_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_ushort_sd_m (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_ushort_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_ushort_variance (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ushort_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_ushort_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_ushort_sd (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ushort_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_ushort_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_ushort_tss_m (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_ushort_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_ushort_tss (a_data: CE_POINTER [NATURAL_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_ushort_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_ushort_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_short_variance (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_short_tss (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_short_variance_with_fixed_mean (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_short_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_short_sd_with_fixed_mean (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_short_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_short_variance_m (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_short_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_short_sd_m (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_short_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_short_variance (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_short_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_short_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_short_sd (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_short_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_short_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_short_tss_m (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_short_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_short_tss (a_data: CE_POINTER [INTEGER_16]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_short_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_short_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_uchar_variance (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_uchar_tss (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_real_64 - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_uchar_variance_with_fixed_mean (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_uchar_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_uchar_sd_with_fixed_mean (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_uchar_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_uchar_variance_m (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_uchar_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_uchar_sd_m (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_uchar_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_uchar_variance (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uchar_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_uchar_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_uchar_sd (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uchar_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_uchar_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_uchar_tss_m (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_uchar_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_uchar_tss (a_data: CE_POINTER [NATURAL_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_uchar_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_uchar_tss_m (a_data, a_stride, a_n, l_mean))
		end

	compute_char_variance (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_variance := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_variance := l_variance + ((l_delta___0 * l_delta___0 - l_variance) / real_96_from_natural_32 ((l_i + {NATURAL_32} 1)))
				l_i := l_i + 1
			end
			Result := l_variance.truncated_to_real_64
		end

	compute_char_tss (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_96
			l_i: NATURAL_32
			l_delta___0: REAL_96
		do
			l_tss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_mean))
				l_tss := l_tss + (l_delta___0 * l_delta___0)
				l_i := l_i + 1
			end
			Result := l_tss.truncated_to_real_64
		end

	gsl_stats_char_variance_with_fixed_mean (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_char_variance (a_data, a_stride, a_n, a_mean))
			Result := l_variance
		end

	gsl_stats_char_sd_with_fixed_mean (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_char_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance))
			Result := l_sd
		end

	gsl_stats_char_variance_m (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
		do
			l_variance := (compute_char_variance (a_data, a_stride, a_n, a_mean))
			Result := (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64))
		end

	gsl_stats_char_sd_m (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_sd: REAL_64
		do
			l_variance := (compute_char_variance (a_data, a_stride, a_n, a_mean))
			l_sd := (sqrt (l_variance * (a_n.to_real_64 / (a_n - {NATURAL_32} 1).to_real_64)))
			Result := l_sd
		end

	gsl_stats_char_variance (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_char_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_char_variance_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_char_sd (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_char_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_char_sd_m (a_data, a_stride, a_n, l_mean))
		end

	gsl_stats_char_tss_m (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32; a_mean: REAL_64): REAL_64
		local
			l_tss: REAL_64
		do
			l_tss := (compute_char_tss (a_data, a_stride, a_n, a_mean))
			Result := l_tss
		end

	gsl_stats_char_tss (a_data: CE_POINTER [INTEGER_8]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_mean: REAL_64
		do
			l_mean := ((create {P_MEAN_SOURCE_STATIC}).gsl_stats_char_mean (a_data, a_stride, a_n))
			Result := (gsl_stats_char_tss_m (a_data, a_stride, a_n, l_mean))
		end

end
