class
	P_VEGAS_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC
	S_STDLIB_STATIC

feature {ANY} -- public routines

	gsl_monte_vegas_integrate (a_f: CE_POINTER [PD_GSL_MONTE_FUNCTION_STRUCT]; a_xl, a_xu: CE_POINTER [REAL_64]; a_dim, a_calls: NATURAL_32; a_r___3: CE_POINTER [PD_GSL_RNG]; a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_result, a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_state.is_void
		local
			l_calls: NATURAL_32
			l_cum_int: REAL_64
			l_cum_sig: REAL_64
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_it: NATURAL_32
			l_bins: NATURAL_32
			l_boxes: NATURAL_32
			l_box_per_bin: INTEGER_32
			l_tmp___1: NATURAL_32
			l_tot_boxes: REAL_64
			l_intgrl: REAL_64
			l_intgrl_sq: REAL_64
			l_tss: REAL_64
			l_wgt: REAL_64
			l_var: REAL_64
			l_sig: REAL_64
			l_calls_per_box: NATURAL_32
			l_jacbin: REAL_64
			l_x: CE_POINTER [REAL_64]
			l_bin: CE_POINTER [INTEGER_32]
			l_m___4: REAL_64
			l_q___3: REAL_64
			l_f_sq_sum: REAL_64
			l_fval: REAL_64
			l_bin_vol: CE_ARRAY [REAL_64]
			l_d: REAL_64
			l_f_sq: REAL_64
			l_sum_wgts: REAL_64
			l_wtd_int_sum: REAL_64
			l_m___5: REAL_64
			l_tmp___6: REAL_64
			l_q___4: REAL_64
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_bin_vol.make (<<1>>)

			l_calls := a_calls
			if a_dim /= a_state.item.dim then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("number of dimensions must match allocated size"), ce_string ("vegas.c"), 121, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_dim))
				loop
					if a_xu[l_i.to_integer_32] <= a_xl[l_i.to_integer_32] then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("xu must be greater than xl"), ce_string ("vegas.c"), 128, 4)
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return and then (a_xu[l_i.to_integer_32] - a_xl[l_i.to_integer_32] > 1.7976931348623157e+308) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("Range of integration is too large, please rescale"), ce_string ("vegas.c"), 134, 4)
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				if a_state.item.stage = 0 then
					a_state.item.init_grid (a_xl, a_xu, a_dim)
					if a_state.item.verbose >= 0 then
						a_state.item.print_lim (a_xl, a_xu, a_dim.to_natural_32)
					end
				end
				if a_state.item.stage <= 1 then
					a_state.item.wtd_int_sum := (0).to_double
					a_state.item.sum_wgts := (0).to_double
					a_state.item.chi_sum := (0).to_double
					a_state.item.it_num := {NATURAL_32} 1
					a_state.item.samples := {NATURAL_32} 0
					a_state.item.chisq := (0).to_double
				end
				if a_state.item.stage <= 2 then
					l_bins := a_state.item.bins_max
					l_boxes := {NATURAL_32} 1
					if a_state.item.mode /= 0 then
						l_boxes := (floor ((pow (l_calls.to_real_64 / 2.0, 1.0 / a_dim.to_real_64)))).truncated_to_integer_64.to_natural_32
						a_state.item.mode := 1
						if {NATURAL_32} 2 * l_boxes >= a_state.item.bins_max then
							if l_boxes // a_state.item.bins_max > {NATURAL_32} 1 then
								l_tmp___1 := l_boxes // a_state.item.bins_max
							else
								l_tmp___1 := {NATURAL_32} 1
							end
							l_box_per_bin := l_tmp___1.to_integer_32
							if l_boxes // l_box_per_bin.to_natural_32 < a_state.item.bins_max then
								l_bins := l_boxes // l_box_per_bin.to_natural_32
							else
								l_bins := a_state.item.bins_max
							end
							l_boxes := l_box_per_bin.to_natural_32 * l_bins
							a_state.item.mode := -1
						end
					end
					l_tot_boxes := ((create {P_POW_INT_STATIC}).gsl_pow_int (l_boxes.to_real_64, a_dim.to_integer_32))
					if l_calls.to_real_64 / l_tot_boxes > (2).to_double then
						a_state.item.calls_per_box := (l_calls.to_real_64 / l_tot_boxes).truncated_to_integer_64.to_natural_32
					else
						a_state.item.calls_per_box := {NATURAL_32} 2
					end
					l_calls := ((a_state.item.calls_per_box).to_real_64 * l_tot_boxes).truncated_to_integer_64.to_natural_32
					a_state.item.jac := (a_state.item.vol * (pow (l_bins.to_real_64, a_dim.to_real_64))) / l_calls.to_real_64
					a_state.item.boxes := l_boxes
					if l_bins /= a_state.item.bins then
						a_state.item.resize_grid (l_bins)
						if a_state.item.verbose > 1 then
							a_state.item.print_grid (a_dim.to_natural_32)
						end
					end
					if a_state.item.verbose >= 0 then
						a_state.item.print_head (a_dim.to_natural_32, l_calls.to_natural_32, a_state.item.it_num, a_state.item.bins, a_state.item.boxes)
					end
				end
				a_state.item.it_start := a_state.item.it_num
				l_cum_int := 0.0
				l_cum_sig := 0.0
				l_it := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_it < a_state.item.iterations))
				loop
					l_intgrl := 0.0
					l_intgrl_sq := 0.0
					l_tss := 0.0
					l_calls_per_box := a_state.item.calls_per_box
					l_jacbin := a_state.item.jac
					l_x := a_state.item.x
					l_bin := a_state.item.bin
					a_state.item.it_num := a_state.item.it_start + l_it
					a_state.item.reset_grid_values
					a_state.item.init_box_coord (a_state.item.box)
					from
					until
						(l_return or l_break)
					loop
						l_break := False
						l_m___4 := (0).to_double
						l_q___3 := (0).to_double
						l_f_sq_sum := 0.0
						l_k := (0).to_natural_32
						from
						until
							not (l_k < l_calls_per_box)
						loop
							a_state.item.random_point (l_x, l_bin, l_bin_vol, a_state.item.box, a_xl, a_xu, a_r___3)
							l_fval := ((l_jacbin * l_bin_vol.item) * ((a_f.item.f).real_64_item ([l_x, a_f.item.dim, a_f.item.params])))
							l_d := (l_fval - l_m___4)
							l_m___4 := l_m___4 + (l_d / (l_k.to_real_64 + 1.0))
							l_q___3 := l_q___3 + ((l_d * l_d) * (l_k.to_real_64 / (l_k.to_real_64 + 1.0)))
							if a_state.item.mode /= -1 then
								l_f_sq := (l_fval * l_fval)
								a_state.item.accumulate_distribution (l_bin, l_f_sq)
							end
							l_k := l_k + 1
						end
						l_intgrl := l_intgrl + (l_m___4 * l_calls_per_box.to_real_64)
						l_f_sq_sum := (l_q___3 * l_calls_per_box.to_real_64)
						l_tss := l_tss + l_f_sq_sum
						if a_state.item.mode = -1 then
							a_state.item.accumulate_distribution (l_bin, l_f_sq_sum)
						end
						if not (a_state.item.change_box_coord (a_state.item.box)).to_boolean then
							l_break := True
						end
					end
					l_break := False
					l_var := l_tss / (l_calls_per_box.to_real_64 - 1.0)
					if l_var > (0).to_double then
						l_wgt := 1.0 / l_var
					elseif a_state.item.sum_wgts > (0).to_double then
						l_wgt := a_state.item.sum_wgts / (a_state.item.samples).to_real_64
					else
						l_wgt := 0.0
					end
					l_intgrl_sq := l_intgrl * l_intgrl
					l_sig := sqrt (l_var)
					a_state.item.result2 := l_intgrl
					a_state.item.sigma := l_sig
					if l_wgt > 0.0 then
						l_sum_wgts := a_state.item.sum_wgts
						l_wtd_int_sum := a_state.item.wtd_int_sum
						if l_sum_wgts > (0).to_double then
							l_tmp___6 := l_wtd_int_sum / l_sum_wgts
						else
							l_tmp___6 := (0).to_double
						end
						l_m___5 := l_tmp___6
						l_q___4 := l_intgrl - l_m___5
						a_state.item.samples := (a_state.item.samples) + 1
						a_state.item.sum_wgts := a_state.item.sum_wgts + l_wgt
						a_state.item.wtd_int_sum := a_state.item.wtd_int_sum + (l_intgrl * l_wgt)
						a_state.item.chi_sum := a_state.item.chi_sum + (l_intgrl_sq * l_wgt)
						l_cum_int := a_state.item.wtd_int_sum / a_state.item.sum_wgts
						l_cum_sig := sqrt ((1).to_double / a_state.item.sum_wgts)
						if a_state.item.samples = {NATURAL_32} 1 then
							a_state.item.chisq := (0).to_double
						else
							a_state.item.chisq := a_state.item.chisq * ((a_state.item.samples).to_real_64 - 2.0)
							a_state.item.chisq := a_state.item.chisq + (((l_wgt / ((1).to_double + l_wgt / l_sum_wgts)) * l_q___4) * l_q___4)
							a_state.item.chisq := a_state.item.chisq / ((a_state.item.samples).to_real_64 - 1.0)
						end
					else
						l_cum_int := l_cum_int + ((l_intgrl - l_cum_int) / (l_it.to_real_64 + 1.0))
						l_cum_sig := 0.0
					end
					if a_state.item.verbose >= 0 then
						a_state.item.print_res (a_state.item.it_num, l_intgrl, l_sig, l_cum_int, l_cum_sig, a_state.item.chisq)
						if l_it + {NATURAL_32} 1 = a_state.item.iterations then
							if a_state.item.verbose > 0 then
								a_state.item.print_grid (a_dim.to_natural_32)
							end
						end
					end
					if a_state.item.verbose > 1 then
						a_state.item.print_dist (a_dim.to_natural_32)
					end
					a_state.item.refine_grid
					if a_state.item.verbose > 1 then
						a_state.item.print_grid (a_dim.to_natural_32)
					end
					l_it := l_it + 1
				end
				a_state.item.stage := 1
				a_result.item := l_cum_int
				a_abserr.item := l_cum_sig
				Result := 0
			end
		end

	gsl_monte_vegas_alloc (a_dim: NATURAL_32): CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]
		local
			l_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]
			l_return, l_break: BOOLEAN
		do
			l_s := create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_cast ((malloc ((create {PD_GSL_MONTE_VEGAS_STATE}.make).structure_size.to_natural_32)))
			if l_s.is_void then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vegas state struct"), ce_string ("vegas.c"), 390, 8)
					Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.item.delx := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_dim * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.delx).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for delx"), ce_string ("vegas.c"), 398, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.d := create {CE_REAL_64_POINTER}.make_cast ((malloc (({NATURAL_32} 50 * a_dim) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.d).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for d"), ce_string ("vegas.c"), 407, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.xi := create {CE_REAL_64_POINTER}.make_cast ((malloc (({NATURAL_32} 51 * a_dim) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.xi).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.d))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for xi"), ce_string ("vegas.c"), 417, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.xin := create {CE_REAL_64_POINTER}.make_cast ((malloc ({NATURAL_32} 51 * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.xin).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xi))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.d))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for xin"), ce_string ("vegas.c"), 428, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.weight := create {CE_REAL_64_POINTER}.make_cast ((malloc ({NATURAL_32} 50 * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.weight).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xin))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xi))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.d))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for xin"), ce_string ("vegas.c"), 440, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.box := create {CE_INTEGER_32_POINTER}.make_cast ((malloc (a_dim * {PLATFORM}.Integer_32_bytes.to_natural_32)))
				if (l_s.item.box).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.weight))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xin))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xi))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.d))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for box"), ce_string ("vegas.c"), 453, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.bin := create {CE_INTEGER_32_POINTER}.make_cast ((malloc (a_dim * {PLATFORM}.Integer_32_bytes.to_natural_32)))
				if (l_s.item.bin).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.box))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.weight))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xin))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xi))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.d))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for bin"), ce_string ("vegas.c"), 467, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.x := create {CE_REAL_64_POINTER}.make_cast ((malloc (a_dim * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.x).is_void then
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.bin))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.box))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.weight))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xin))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.xi))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.d))
					free (create {CE_VOID_POINTER}.make_cast (l_s.item.delx))
					free (create {CE_VOID_POINTER}.make_cast (l_s))
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for x"), ce_string ("vegas.c"), 482, 8)
						Result := (create {CE_CLASS_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_s.item.dim := a_dim
				l_s.item.bins_max := (50).to_natural_32
				l_s.item.gsl_monte_vegas_init.do_nothing
				Result := l_s
			end
		end

	gsl_monte_vegas_free (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE])
		local
			l_return: BOOLEAN
		do
			if a_s.is_void then
				l_return := True
			end
			if not l_return then
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.x))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.delx))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.d))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.xi))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.xin))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.weight))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.box))
				free (create {CE_VOID_POINTER}.make_cast (a_s.item.bin))
				free (create {CE_VOID_POINTER}.make_cast (a_s))
			end
		end

end
