class
	P_WVARIANCE_SOURCE_STATIC

inherit
	CE_UTILITIES
	S_MATH_STATIC

feature {ANY} -- public routines

	compute_long_double_wvariance (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_96
			l_W: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_96
			l_delta___0: REAL_96
		do
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > real_96_from_integer_32 (0) then
					l_delta___0 := (a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_wmean))
					l_W := l_W + l_wi
					l_wvariance := l_wvariance + ((l_delta___0 * l_delta___0 - l_wvariance) * (l_wi / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wvariance.truncated_to_real_64
		end

	compute_long_double_wtss (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wtss: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_96
			l_delta___0: REAL_96
		do
			l_wtss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > real_96_from_integer_32 (0) then
					l_delta___0 := (a_data[(l_i * a_stride).to_integer_32] - real_96_from_real_64 (a_wmean))
					l_wtss := l_wtss + ((l_wi * l_delta___0) * l_delta___0)
				end
				l_i := l_i + 1
			end
			Result := l_wtss.truncated_to_real_64
		end

	compute_long_double_factor (a_w: CE_POINTER [REAL_96]; a_wstride, a_n: NATURAL_32): REAL_64
		local
			l_a___5: REAL_96
			l_b___0: REAL_96
			l_factor: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_96
		do
			l_a___5 := real_96_from_integer_32 (0)
			l_b___0 := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > real_96_from_integer_32 (0) then
					l_a___5 := l_a___5 + l_wi
					l_b___0 := l_b___0 + (l_wi * l_wi)
				end
				l_i := l_i + 1
			end
			l_factor := (l_a___5 * l_a___5) / (l_a___5 * l_a___5 - l_b___0)
			Result := l_factor.truncated_to_real_64
		end

	gsl_stats_long_double_wvariance_with_fixed_mean (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_64
		do
			l_wvariance := (compute_long_double_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			Result := l_wvariance
		end

	gsl_stats_long_double_wsd_with_fixed_mean (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_64
			l_wsd: REAL_64
		do
			l_wvariance := (compute_long_double_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_wsd := (sqrt (l_wvariance))
			Result := l_wsd
		end

	gsl_stats_long_double_wvariance_m (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_scale: REAL_64
		do
			l_variance := (compute_long_double_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_scale := (compute_long_double_factor (a_w, a_wstride, a_n))
			Result := (l_scale * l_variance)
		end

	gsl_stats_long_double_wsd_m (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_scale: REAL_64
			l_wsd: REAL_64
		do
			l_variance := (compute_long_double_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_scale := (compute_long_double_factor (a_w, a_wstride, a_n))
			l_wsd := (sqrt (l_scale * l_variance))
			Result := l_wsd
		end

	gsl_stats_long_double_wsd (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_long_double_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_long_double_wsd_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	gsl_stats_long_double_wvariance (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_long_double_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_long_double_wvariance_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	gsl_stats_long_double_wtss_m (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wtss: REAL_64
		do
			l_wtss := (compute_long_double_wtss (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			Result := l_wtss
		end

	gsl_stats_long_double_wtss (a_w: CE_POINTER [REAL_96]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_96]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_long_double_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_long_double_wtss_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	compute_wvariance (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_96
			l_W: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_64
			l_delta___0: REAL_96
		do
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_double then
					l_delta___0 := real_96_from_real_64 ((a_data[(l_i * a_stride).to_integer_32] - a_wmean))
					l_W := l_W + real_96_from_real_64 (l_wi)
					l_wvariance := l_wvariance + ((l_delta___0 * l_delta___0 - l_wvariance) * (real_96_from_real_64 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wvariance.truncated_to_real_64
		end

	compute_wtss (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wtss: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_64
			l_delta___0: REAL_96
		do
			l_wtss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_double then
					l_delta___0 := real_96_from_real_64 ((a_data[(l_i * a_stride).to_integer_32] - a_wmean))
					l_wtss := l_wtss + ((real_96_from_real_64 (l_wi) * l_delta___0) * l_delta___0)
				end
				l_i := l_i + 1
			end
			Result := l_wtss.truncated_to_real_64
		end

	compute_factor (a_w: CE_POINTER [REAL_64]; a_wstride, a_n: NATURAL_32): REAL_64
		local
			l_a___5: REAL_96
			l_b___0: REAL_96
			l_factor: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_64
		do
			l_a___5 := real_96_from_integer_32 (0)
			l_b___0 := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_double then
					l_a___5 := l_a___5 + real_96_from_real_64 (l_wi)
					l_b___0 := l_b___0 + real_96_from_real_64 ((l_wi * l_wi))
				end
				l_i := l_i + 1
			end
			l_factor := (l_a___5 * l_a___5) / (l_a___5 * l_a___5 - l_b___0)
			Result := l_factor.truncated_to_real_64
		end

	gsl_stats_wvariance_with_fixed_mean (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_64
		do
			l_wvariance := (compute_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			Result := l_wvariance
		end

	gsl_stats_wsd_with_fixed_mean (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_64
			l_wsd: REAL_64
		do
			l_wvariance := (compute_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_wsd := (sqrt (l_wvariance))
			Result := l_wsd
		end

	gsl_stats_wvariance_m (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_scale: REAL_64
		do
			l_variance := (compute_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_scale := (compute_factor (a_w, a_wstride, a_n))
			Result := (l_scale * l_variance)
		end

	gsl_stats_wsd_m (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_scale: REAL_64
			l_wsd: REAL_64
		do
			l_variance := (compute_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_scale := (compute_factor (a_w, a_wstride, a_n))
			l_wsd := (sqrt (l_scale * l_variance))
			Result := l_wsd
		end

	gsl_stats_wsd (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_wsd_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	gsl_stats_wvariance (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_wvariance_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	gsl_stats_wtss_m (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wtss: REAL_64
		do
			l_wtss := (compute_wtss (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			Result := l_wtss
		end

	gsl_stats_wtss (a_w: CE_POINTER [REAL_64]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_wtss_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	compute_float_wvariance (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_96
			l_W: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_32
			l_delta___0: REAL_96
		do
			l_wvariance := real_96_from_integer_32 (0)
			l_W := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_real then
					l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_wmean))
					l_W := l_W + real_96_from_real_32 (l_wi)
					l_wvariance := l_wvariance + ((l_delta___0 * l_delta___0 - l_wvariance) * (real_96_from_real_32 (l_wi) / l_W))
				end
				l_i := l_i + 1
			end
			Result := l_wvariance.truncated_to_real_64
		end

	compute_float_wtss (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wtss: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_32
			l_delta___0: REAL_96
		do
			l_wtss := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_real then
					l_delta___0 := real_96_from_real_64 (((a_data[(l_i * a_stride).to_integer_32]).to_double - a_wmean))
					l_wtss := l_wtss + ((real_96_from_real_32 (l_wi) * l_delta___0) * l_delta___0)
				end
				l_i := l_i + 1
			end
			Result := l_wtss.truncated_to_real_64
		end

	compute_float_factor (a_w: CE_POINTER [REAL_32]; a_wstride, a_n: NATURAL_32): REAL_64
		local
			l_a___5: REAL_96
			l_b___0: REAL_96
			l_factor: REAL_96
			l_i: NATURAL_32
			l_wi: REAL_32
		do
			l_a___5 := real_96_from_integer_32 (0)
			l_b___0 := real_96_from_integer_32 (0)
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				l_wi := a_w[(l_i * a_wstride).to_integer_32]
				if l_wi > (0).to_real then
					l_a___5 := l_a___5 + real_96_from_real_32 (l_wi)
					l_b___0 := l_b___0 + real_96_from_real_32 ((l_wi * l_wi))
				end
				l_i := l_i + 1
			end
			l_factor := (l_a___5 * l_a___5) / (l_a___5 * l_a___5 - l_b___0)
			Result := l_factor.truncated_to_real_64
		end

	gsl_stats_float_wvariance_with_fixed_mean (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_64
		do
			l_wvariance := (compute_float_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			Result := l_wvariance
		end

	gsl_stats_float_wsd_with_fixed_mean (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wvariance: REAL_64
			l_wsd: REAL_64
		do
			l_wvariance := (compute_float_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_wsd := (sqrt (l_wvariance))
			Result := l_wsd
		end

	gsl_stats_float_wvariance_m (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_scale: REAL_64
		do
			l_variance := (compute_float_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_scale := (compute_float_factor (a_w, a_wstride, a_n))
			Result := (l_scale * l_variance)
		end

	gsl_stats_float_wsd_m (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_variance: REAL_64
			l_scale: REAL_64
			l_wsd: REAL_64
		do
			l_variance := (compute_float_wvariance (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			l_scale := (compute_float_factor (a_w, a_wstride, a_n))
			l_wsd := (sqrt (l_scale * l_variance))
			Result := l_wsd
		end

	gsl_stats_float_wsd (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_float_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_float_wsd_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	gsl_stats_float_wvariance (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_float_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_float_wvariance_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

	gsl_stats_float_wtss_m (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_wmean: REAL_64): REAL_64
		local
			l_wtss: REAL_64
		do
			l_wtss := (compute_float_wtss (a_w, a_wstride, a_data, a_stride, a_n, a_wmean))
			Result := l_wtss
		end

	gsl_stats_float_wtss (a_w: CE_POINTER [REAL_32]; a_wstride: NATURAL_32; a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): REAL_64
		local
			l_wmean: REAL_64
		do
			l_wmean := ((create {P_WMEAN_SOURCE_STATIC}).gsl_stats_float_wmean (a_w, a_wstride, a_data, a_stride, a_n))
			Result := (gsl_stats_float_wtss_m (a_w, a_wstride, a_data, a_stride, a_n, l_wmean))
		end

end
