class
	PD_GSL_BSPLINE_WORKSPACE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} knots.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} deltal.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} deltar.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} B.make
		end

feature {ANY} -- fields

	k: NATURAL_32 assign set_k

	km1: NATURAL_32 assign set_km1

	l: NATURAL_32 assign set_l

	nbreak: NATURAL_32 assign set_nbreak

	n: NATURAL_32 assign set_n

	knots: CE_POINTER [PD_GSL_VECTOR] assign set_knots

	deltal: CE_POINTER [PD_GSL_VECTOR] assign set_deltal

	deltar: CE_POINTER [PD_GSL_VECTOR] assign set_deltar

	B: CE_POINTER [PD_GSL_VECTOR] assign set_B

feature {ANY} -- field setters

	set_k (a_k: NATURAL_32)
		do
			k := a_k
			update_memory_field ("k")
		end

	set_km1 (a_km1: NATURAL_32)
		do
			km1 := a_km1
			update_memory_field ("km1")
		end

	set_l (a_l: NATURAL_32)
		do
			l := a_l
			update_memory_field ("l")
		end

	set_nbreak (a_nbreak: NATURAL_32)
		do
			nbreak := a_nbreak
			update_memory_field ("nbreak")
		end

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_knots (a_knots: CE_POINTER [PD_GSL_VECTOR])
		do
			knots := a_knots
			update_memory_field ("knots")
		end

	set_deltal (a_deltal: CE_POINTER [PD_GSL_VECTOR])
		do
			deltal := a_deltal
			update_memory_field ("deltal")
		end

	set_deltar (a_deltar: CE_POINTER [PD_GSL_VECTOR])
		do
			deltar := a_deltar
			update_memory_field ("deltar")
		end

	set_B (a_B: CE_POINTER [PD_GSL_VECTOR])
		do
			B := a_B
			update_memory_field ("B")
		end

feature {ANY} -- public routines

	gsl_bspline_ncoeffs: NATURAL_32
		do
			Result := n
		end

	gsl_bspline_order: NATURAL_32
		do
			Result := k
		end

	gsl_bspline_nbreak: NATURAL_32
		do
			Result := nbreak
		end

	gsl_bspline_breakpoint (a_i: NATURAL_32): REAL_64
		local
			l_j: NATURAL_32
			l_tmp: REAL_64
		do
			l_j := (a_i + k) - {NATURAL_32} 1
			l_tmp := knots.item.gsl_vector_get (l_j)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_bspline_greville_abscissa (a_i: NATURAL_32): REAL_64
		local
			l_stride: NATURAL_32
			l_km1: NATURAL_32
			l_data: CE_POINTER [REAL_64]
			l_tmp: REAL_64
		do
			l_stride := knots.item.stride
			l_km1 := km1
			l_data := knots.item.data + ((a_i + {NATURAL_32} 1) * l_stride).to_integer_32
			if l_km1 = {NATURAL_32} 0 then
				l_km1 := (2).to_natural_32
				l_data := l_data - l_stride.to_integer_32
			end
			l_tmp := (create {P_MEAN_SOURCE_STATIC}).gsl_stats_mean (l_data, l_stride, l_km1)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_bspline_knots_uniform (a_a___5, a_b___0: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_delta___0: REAL_64
			l_x: REAL_64
		do
			l_delta___0 := (a_b___0 - a_a___5) / l.to_real_64
			l_i := (0).to_natural_32
			from
			until
				not (l_i < k)
			loop
				knots.item.gsl_vector_set (l_i, a_a___5)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_x := a_a___5 + l_delta___0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l - {NATURAL_32} 1)
			loop
				knots.item.gsl_vector_set (k + l_i, l_x)
				update_class_fields -- Current might have been changed; synchronize
				l_x := l_x + l_delta___0
				l_i := l_i + 1
			end
			l_i := n
			from
			until
				not (l_i < n + k)
			loop
				knots.item.gsl_vector_set (l_i, a_b___0)
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	bspline_find_interval (a_x: REAL_64; a_flag: CE_POINTER [INTEGER_32]): NATURAL_32
		local
			l_i: NATURAL_32
			l_tmp: REAL_64
			l_ti: REAL_64
			l_tmp___0: REAL_64
			l_tip1: REAL_64
			l_tmp___1: REAL_64
			l_tmp___2: REAL_64
			l_return, l_break: BOOLEAN
		do
			l_tmp := knots.item.gsl_vector_get ((0).to_natural_32)
			update_class_fields -- Current might have been changed; synchronize
			if a_x < l_tmp then
				a_flag.item := -1
				update_class_fields -- Current might have been changed; synchronize
				Result := (0).to_natural_32
				l_return := True
			end
			if not l_return then
				l_i := k - {NATURAL_32} 1
				from
				until
					not (not (l_return or l_break) and then (l_i < (k + l) - {NATURAL_32} 1))
				loop
					l_break := False
					l_tmp___0 := knots.item.gsl_vector_get (l_i)
					update_class_fields -- Current might have been changed; synchronize
					l_ti := l_tmp___0
					l_tmp___1 := knots.item.gsl_vector_get (l_i + {NATURAL_32} 1)
					update_class_fields -- Current might have been changed; synchronize
					l_tip1 := l_tmp___1
					if l_tip1 < l_ti then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("knots vector is not increasing"), ce_string ("bspline.c"), 699, 4)
							update_class_fields -- Current might have been changed; synchronize
							Result := (4).to_natural_32
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						if l_ti <= a_x then
							if a_x < l_tip1 then
								l_break := True
							end
						end
						if not (l_return or l_break) and then (l_ti < a_x) then
							if a_x = l_tip1 then
								l_tmp___2 := knots.item.gsl_vector_get ((k + l) - {NATURAL_32} 1)
								update_class_fields -- Current might have been changed; synchronize
								if l_tip1 = l_tmp___2 then
									l_break := True
								end
							end
						end
						if not (l_return or l_break) then
							l_i := l_i + 1
						end
					end
				end
			end
			if not l_return then
				l_break := False
				if l_i = (k + l) - {NATURAL_32} 1 then
					a_flag.item := 1
					update_class_fields -- Current might have been changed; synchronize
				else
					a_flag.item := 0
					update_class_fields -- Current might have been changed; synchronize
				end
				Result := l_i
			end
		end

	bspline_process_interval_for_eval (a_x: REAL_64; a_i: CE_POINTER [NATURAL_32]; a_flag: INTEGER_32): INTEGER_32
		local
			l_tmp: REAL_64
			l_tmp___0: REAL_64
			l_tmp___1: REAL_64
			l_return, l_break: BOOLEAN
		do
			if a_flag = -1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("x outside of knot interval"), ce_string ("bspline.c"), 731, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_flag = 1 then
				l_tmp := knots.item.gsl_vector_get (a_i.item)
				update_class_fields -- Current might have been changed; synchronize
				if a_x <= l_tmp + 2.2204460492503131e-16 then
					a_i.item := (a_i.item) - 1
					update_class_fields -- Current might have been changed; synchronize
				else
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("x outside of knot interval"), ce_string ("bspline.c"), 741, 4)
						update_class_fields -- Current might have been changed; synchronize
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_tmp___0 := knots.item.gsl_vector_get (a_i.item)
				update_class_fields -- Current might have been changed; synchronize
				l_tmp___1 := knots.item.gsl_vector_get (a_i.item + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp___0 = l_tmp___1 then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("knot(i) = knot(i+1) will result in division by zero"), ce_string ("bspline.c"), 748, 4)
						update_class_fields -- Current might have been changed; synchronize
						Result := 4
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			k := a_other.k
			km1 := a_other.km1
			l := a_other.l
			nbreak := a_other.nbreak
			n := a_other.n
			knots := a_other.knots
			deltal := a_other.deltal
			deltar := a_other.deltar
			B := a_other.B
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				k = a_other.k and
				km1 = a_other.km1 and
				l = a_other.l and
				nbreak = a_other.nbreak and
				n = a_other.n and
				knots.is_equal (a_other.knots) and
				deltal.is_equal (a_other.deltal) and
				deltar.is_equal (a_other.deltar) and
				B.is_equal (a_other.B)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("k"),
					create {CE_STRUCTURE_FIELD}.make ("km1"),
					create {CE_STRUCTURE_FIELD}.make ("l"),
					create {CE_STRUCTURE_FIELD}.make ("nbreak"),
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("knots"),
					create {CE_STRUCTURE_FIELD}.make ("deltal"),
					create {CE_STRUCTURE_FIELD}.make ("deltar"),
					create {CE_STRUCTURE_FIELD}.make ("B")
				>>)
			init_structure_info
		end

end
