class
	PD_GSL_EIGEN_NONSYMM_WORKSPACE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} diag.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} tau.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} Z.make
			create {CE_CLASS_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE]} francis_workspace_p.make
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	diag: CE_POINTER [PD_GSL_VECTOR] assign set_diag

	tau: CE_POINTER [PD_GSL_VECTOR] assign set_tau

	Z: CE_POINTER [PD_GSL_MATRIX] assign set_Z

	do_balance: INTEGER_32 assign set_do_balance

	n_evals: NATURAL_32 assign set_n_evals

	francis_workspace_p: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE] assign set_francis_workspace_p

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_diag (a_diag: CE_POINTER [PD_GSL_VECTOR])
		do
			diag := a_diag
			update_memory_field ("diag")
		end

	set_tau (a_tau: CE_POINTER [PD_GSL_VECTOR])
		do
			tau := a_tau
			update_memory_field ("tau")
		end

	set_Z (a_Z: CE_POINTER [PD_GSL_MATRIX])
		do
			Z := a_Z
			update_memory_field ("Z")
		end

	set_do_balance (a_do_balance: INTEGER_32)
		do
			do_balance := a_do_balance
			update_memory_field ("do_balance")
		end

	set_n_evals (a_n_evals: NATURAL_32)
		do
			n_evals := a_n_evals
			update_memory_field ("n_evals")
		end

	set_francis_workspace_p (a_francis_workspace_p: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE])
		do
			francis_workspace_p := a_francis_workspace_p
			update_memory_field ("francis_workspace_p")
		end

feature {ANY} -- public routines

	gsl_eigen_nonsymm_params (a_compute_t, a_balance: INTEGER_32)
		do
			gsl_eigen_francis_T (a_compute_t, francis_workspace_p)
			update_class_fields -- Current might have been changed; synchronize
			Current.do_balance := a_balance
			update_class_fields -- Current might have been changed; synchronize
		end

feature {PD_GSL_EIGEN_NONSYMM_WORKSPACE} -- private routines

	gsl_eigen_francis_T (a_compute_t: INTEGER_32; a_w: CE_POINTER [PD_GSL_EIGEN_FRANCIS_WORKSPACE])
		require
			not a_w.is_void
		do
			a_w.item.compute_t := a_compute_t
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			diag := a_other.diag
			tau := a_other.tau
			Z := a_other.Z
			do_balance := a_other.do_balance
			n_evals := a_other.n_evals
			francis_workspace_p := a_other.francis_workspace_p
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				diag.is_equal (a_other.diag) and
				tau.is_equal (a_other.tau) and
				Z.is_equal (a_other.Z) and
				do_balance = a_other.do_balance and
				n_evals = a_other.n_evals and
				francis_workspace_p.is_equal (a_other.francis_workspace_p)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("diag"),
					create {CE_STRUCTURE_FIELD}.make ("tau"),
					create {CE_STRUCTURE_FIELD}.make ("Z"),
					create {CE_STRUCTURE_FIELD}.make ("do_balance"),
					create {CE_STRUCTURE_FIELD}.make ("n_evals"),
					create {CE_STRUCTURE_FIELD}.make ("francis_workspace_p")
				>>)
			init_structure_info
		end

end
