class
	PD_GSL_FFT_HALFCOMPLEX_WAVETABLE_FLOAT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_NATURAL_32_ARRAY} factor.make (<<64>>)
			create {CE_POINTER_ARRAY [PD_GSL_COMPLEX_FLOAT]} twiddle.make (<<64>>)
			create {CE_CLASS_POINTER [PD_GSL_COMPLEX_FLOAT]} trig.make
			Precursor
		end

feature {ANY} -- fields

	n: NATURAL_32 assign set_n

	nf: NATURAL_32 assign set_nf

	factor: CE_ARRAY [NATURAL_32]

	twiddle: CE_ARRAY [CE_POINTER [PD_GSL_COMPLEX_FLOAT]]

	trig: CE_POINTER [PD_GSL_COMPLEX_FLOAT] assign set_trig

feature {ANY} -- field setters

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_nf (a_nf: NATURAL_32)
		do
			nf := a_nf
			update_memory_field ("nf")
		end

	set_trig (a_trig: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		do
			trig := a_trig
			update_memory_field ("trig")
		end

feature {ANY} -- public routines

	gsl_fft_halfcomplex_float_transform (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_REAL_WORKSPACE_FLOAT]): INTEGER_32
		require
			not a_work.is_void
		local
			l_scratch: CE_POINTER [REAL_32]
			l_in: CE_POINTER [REAL_32]
			l_out: CE_POINTER [REAL_32]
			l_istride: NATURAL_32
			l_ostride: NATURAL_32
			l_factor: NATURAL_32
			l_product: NATURAL_32
			l_q___3: NATURAL_32
			l_i: NATURAL_32
			l_nf: NATURAL_32
			l_state: INTEGER_32
			l_product_1: INTEGER_32
			l_tskip: INTEGER_32
			l_twiddle1: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle2: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle3: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_twiddle4: CE_POINTER [PD_GSL_COMPLEX_FLOAT]
			l_return, l_break: BOOLEAN
		do
			l_scratch := a_work.item.scratch
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("length n must be positive integer"), ce_string ("hc_main.c"), 88, 1)
					update_class_fields -- Current might have been changed; synchronize
					Result := 1
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_n = {NATURAL_32} 1) then
				Result := 0
				l_return := True
			end
			if not l_return and then (a_n /= n) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("wavetable does not match length of data"), ce_string ("hc_main.c"), 98, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_n /= a_work.item.n) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("workspace does not match length of data"), ce_string ("hc_main.c"), 103, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_nf := nf
				l_product := (1).to_natural_32
				l_state := 0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_nf)
				loop
					l_factor := factor[l_i.to_integer_32]
					l_product_1 := l_product.to_integer_32
					l_product := l_product * l_factor
					l_q___3 := a_n // l_product
					l_tskip := ((l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2 - {NATURAL_32} 1).to_integer_32
					if l_state = 0 then
						l_in := a_data
						l_istride := a_stride
						l_out := l_scratch
						l_ostride := (1).to_natural_32
						l_state := 1
					else
						l_in := l_scratch
						l_istride := (1).to_natural_32
						l_out := a_data
						l_ostride := a_stride
						l_state := 0
					end
					if l_factor = {NATURAL_32} 2 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						fft_halfcomplex_float_pass_2 (l_in, l_istride, l_out, l_ostride, l_product, a_n, l_twiddle1)
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 3 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_tskip
						fft_halfcomplex_float_pass_3 (l_in, l_istride, l_out, l_ostride, l_product, a_n, l_twiddle1, l_twiddle2)
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 4 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_tskip
						l_twiddle3 := l_twiddle2 + l_tskip
						fft_halfcomplex_float_pass_4 (l_in, l_istride, l_out, l_ostride, l_product, a_n, l_twiddle1, l_twiddle2, l_twiddle3)
						update_class_fields -- Current might have been changed; synchronize
					elseif l_factor = {NATURAL_32} 5 then
						l_twiddle1 := twiddle[l_i.to_integer_32]
						l_twiddle2 := l_twiddle1 + l_tskip
						l_twiddle3 := l_twiddle2 + l_tskip
						l_twiddle4 := l_twiddle3 + l_tskip
						fft_halfcomplex_float_pass_5 (l_in, l_istride, l_out, l_ostride, l_product, a_n, l_twiddle1, l_twiddle2, l_twiddle3, l_twiddle4)
						update_class_fields -- Current might have been changed; synchronize
					else
						l_twiddle1 := twiddle[l_i.to_integer_32]
						fft_halfcomplex_float_pass_n (l_in, l_istride, l_out, l_ostride, l_factor, l_product, a_n, l_twiddle1)
						update_class_fields -- Current might have been changed; synchronize
					end
					l_i := l_i + 1
				end
				if l_state = 1 then
					l_i := (0).to_natural_32
					from
					until
						not (l_i < a_n)
					loop
						a_data[(a_stride * l_i).to_integer_32] := l_scratch[l_i.to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_i := l_i + 1
					end
				end
				Result := 0
			end
		end

	gsl_fft_halfcomplex_float_backward (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_REAL_WORKSPACE_FLOAT]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_FFT_HALFCOMPLEX_WAVETABLE_FLOAT]}.make_shared (c_pointer))).item.gsl_fft_halfcomplex_float_transform (a_data, a_stride, a_n, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_fft_halfcomplex_float_inverse (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32; a_work: CE_POINTER [PD_GSL_FFT_REAL_WORKSPACE_FLOAT]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_norm___0: REAL_64
			l_i: NATURAL_32
			l_return: BOOLEAN
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_FFT_HALFCOMPLEX_WAVETABLE_FLOAT]}.make_shared (c_pointer))).item.gsl_fft_halfcomplex_float_transform (a_data, a_stride, a_n, a_work)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				Result := l_status
				l_return := True
			end
			if not l_return then
				l_norm___0 := 1.0 / a_n.to_real_64
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_n)
				loop
					a_data[(a_stride * l_i).to_integer_32] := ((a_data[(a_stride * l_i).to_integer_32]).to_double * l_norm___0).truncated_to_real
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := l_status
			end
		end

feature {PD_GSL_FFT_HALFCOMPLEX_WAVETABLE_FLOAT} -- private routines

	fft_halfcomplex_float_pass_2 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride, a_product, a_n: NATURAL_32; a_twiddle: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_jump: NATURAL_32
			l_factor: NATURAL_32
			l_q___3: NATURAL_32
			l_m___4: NATURAL_32
			l_product_1: NATURAL_32
			l_r0: REAL_32
			l_r1___1: REAL_32
			l_s0: REAL_32
			l_s1: REAL_32
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_from0: NATURAL_32
			l_from1: NATURAL_32
			l_z0_real: REAL_32
			l_z0_imag: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_to0: NATURAL_32
			l_to1: NATURAL_32
			l_from0___0: NATURAL_32
			l_to0___0: NATURAL_32
			l_to1___0: NATURAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (2).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_q___3
			l_k1 := (0).to_natural_32
			from
			until
				not (l_k1 < l_product_1)
			loop
				l_r0 := a_in[(a_istride * (({NATURAL_32} 2 * l_k1) * l_q___3)).to_integer_32]
				l_r1___1 := a_in[(a_istride * ((({NATURAL_32} 2 * l_k1) * l_q___3 + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1)).to_integer_32]
				l_s0 := l_r0 + l_r1___1
				l_s1 := l_r0 - l_r1___1
				a_out[(a_ostride * (l_q___3 * l_k1)).to_integer_32] := l_s0
				a_out[(a_ostride * (l_q___3 * l_k1 + l_m___4)).to_integer_32] := l_s1
				l_k1 := l_k1 + 1
			end
			if l_q___3 = {NATURAL_32} 1 then
				l_return := True
			end
			if not l_return then
				l_k := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < (l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2))
				loop
					l_w_real := (a_twiddle + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w_imag := (a_twiddle + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_k1 := (0).to_natural_32
					from
					until
						not (l_k1 < l_product_1)
					loop
						l_from0 := (({NATURAL_32} 2 * l_k1) * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_from1 := ((({NATURAL_32} 2 * l_k1) * l_q___3 - {NATURAL_32} 2 * l_k) + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
						l_z0_real := a_in[(a_istride * l_from0).to_integer_32]
						l_z0_imag := a_in[(a_istride * (l_from0 + {NATURAL_32} 1)).to_integer_32]
						l_z1_real := a_in[(a_istride * l_from1).to_integer_32]
						l_z1_imag := a_in[(a_istride * (l_from1 + {NATURAL_32} 1)).to_integer_32]
						l_x0_real := l_z0_real + l_z1_real
						l_x0_imag := l_z0_imag - l_z1_imag
						l_x1_real := l_z0_real - l_z1_real
						l_x1_imag := l_z0_imag + l_z1_imag
						l_to0 := (l_k1 * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_to1 := l_to0 + l_m___4
						a_out[(a_ostride * l_to0).to_integer_32] := l_x0_real
						a_out[(a_ostride * (l_to0 + {NATURAL_32} 1)).to_integer_32] := l_x0_imag
						a_out[(a_ostride * l_to1).to_integer_32] := l_w_real * l_x1_real - l_w_imag * l_x1_imag
						a_out[(a_ostride * (l_to1 + {NATURAL_32} 1)).to_integer_32] := l_w_imag * l_x1_real + l_w_real * l_x1_imag
						l_k1 := l_k1 + 1
					end
					l_k := l_k + 1
				end
				if l_q___3 \\ {NATURAL_32} 2 = {NATURAL_32} 1 then
					l_return := True
				end
			end
			if not l_return then
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_product_1)
				loop
					l_from0___0 := (({NATURAL_32} 2 * l_k1) * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_to0___0 := (l_k1 * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_to1___0 := l_to0___0 + l_m___4
					a_out[(a_ostride * l_to0___0).to_integer_32] := ((2).to_real * a_in[(a_istride * l_from0___0).to_integer_32])
					a_out[(a_ostride * l_to1___0).to_integer_32] := ((-2).to_real * a_in[(a_istride * (l_from0___0 + {NATURAL_32} 1)).to_integer_32])
					l_k1 := l_k1 + 1
				end
			end
		end

	fft_halfcomplex_float_pass_3 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride, a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_jump: NATURAL_32
			l_factor: NATURAL_32
			l_q___3: NATURAL_32
			l_m___4: NATURAL_32
			l_product_1: NATURAL_32
			l_tau: REAL_32
			l_from0: NATURAL_32
			l_from1: NATURAL_32
			l_z0_real: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_t1_real: REAL_32
			l_t2_real: REAL_32
			l_t3_imag: REAL_32
			l_to0: NATURAL_32
			l_to1: NATURAL_32
			l_to2: NATURAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_from0___0: NATURAL_32
			l_from1___0: NATURAL_32
			l_from2: NATURAL_32
			l_z0_real___0: REAL_32
			l_z0_imag: REAL_32
			l_z1_real___0: REAL_32
			l_z1_imag___0: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_t1_real___0: REAL_32
			l_t1_imag: REAL_32
			l_t2_real___0: REAL_32
			l_t2_imag: REAL_32
			l_t3_real: REAL_32
			l_t3_imag___0: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_to0___0: NATURAL_32
			l_to1___0: NATURAL_32
			l_to2___0: NATURAL_32
			l_from0___1: NATURAL_32
			l_from1___1: NATURAL_32
			l_z0_real___1: REAL_32
			l_z0_imag___0: REAL_32
			l_z1_real___1: REAL_32
			l_t1_real___1: REAL_32
			l_t2_real___1: REAL_32
			l_x0_real___0: REAL_32
			l_x1_real___0: REAL_32
			l_x2_real___0: REAL_32
			l_to0___1: NATURAL_32
			l_to1___1: NATURAL_32
			l_to2___1: NATURAL_32
			l_return: BOOLEAN
		do
			l_tau := ((sqrt (3.0)) / 2.0).truncated_to_real
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (3).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_q___3
			l_k1 := (0).to_natural_32
			from
			until
				not (l_k1 < l_product_1)
			loop
				l_from0 := ({NATURAL_32} 3 * l_k1) * l_q___3
				l_from1 := (l_from0 + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
				l_z0_real := a_in[(a_istride * l_from0).to_integer_32]
				l_z1_real := a_in[(a_istride * l_from1).to_integer_32]
				l_z1_imag := a_in[(a_istride * (l_from1 + {NATURAL_32} 1)).to_integer_32]
				l_t1_real := (2).to_real * l_z1_real
				l_t2_real := l_z0_real - l_z1_real
				l_t3_imag := ((2).to_real * l_tau) * l_z1_imag
				l_to0 := l_q___3 * l_k1
				l_to1 := l_to0 + l_m___4
				l_to2 := l_to1 + l_m___4
				a_out[(a_ostride * l_to0).to_integer_32] := l_z0_real + l_t1_real
				a_out[(a_ostride * l_to1).to_integer_32] := l_t2_real - l_t3_imag
				a_out[(a_ostride * l_to2).to_integer_32] := l_t2_real + l_t3_imag
				l_k1 := l_k1 + 1
			end
			if l_q___3 = {NATURAL_32} 1 then
				l_return := True
			end
			if not l_return then
				l_k := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < (l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2))
				loop
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_k1 := (0).to_natural_32
					from
					until
						not (l_k1 < l_product_1)
					loop
						l_from0___0 := (({NATURAL_32} 3 * l_k1) * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_from1___0 := l_from0___0 + {NATURAL_32} 2 * l_q___3
						l_from2 := ((({NATURAL_32} 3 * l_k1) * l_q___3 - {NATURAL_32} 2 * l_k) + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
						l_z0_real___0 := a_in[(a_istride * l_from0___0).to_integer_32]
						l_z0_imag := a_in[(a_istride * (l_from0___0 + {NATURAL_32} 1)).to_integer_32]
						l_z1_real___0 := a_in[(a_istride * l_from1___0).to_integer_32]
						l_z1_imag___0 := a_in[(a_istride * (l_from1___0 + {NATURAL_32} 1)).to_integer_32]
						l_z2_real := a_in[(a_istride * l_from2).to_integer_32]
						l_z2_imag := (-a_in[(a_istride * (l_from2 + {NATURAL_32} 1)).to_integer_32])
						l_t1_real___0 := l_z1_real___0 + l_z2_real
						l_t1_imag := l_z1_imag___0 + l_z2_imag
						l_t2_real___0 := (l_z0_real___0.to_double - l_t1_real___0.to_double / 2.0).truncated_to_real
						l_t2_imag := (l_z0_imag.to_double - l_t1_imag.to_double / 2.0).truncated_to_real
						l_t3_real := l_tau * (l_z1_real___0 - l_z2_real)
						l_t3_imag___0 := l_tau * (l_z1_imag___0 - l_z2_imag)
						l_x0_real := l_z0_real___0 + l_t1_real___0
						l_x0_imag := l_z0_imag + l_t1_imag
						l_x1_real := l_t2_real___0 - l_t3_imag___0
						l_x1_imag := l_t2_imag + l_t3_real
						l_x2_real := l_t2_real___0 + l_t3_imag___0
						l_x2_imag := l_t2_imag - l_t3_real
						l_to0___0 := (l_k1 * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_to1___0 := l_to0___0 + l_m___4
						l_to2___0 := l_to1___0 + l_m___4
						a_out[(a_ostride * l_to0___0).to_integer_32] := l_x0_real
						a_out[(a_ostride * (l_to0___0 + {NATURAL_32} 1)).to_integer_32] := l_x0_imag
						a_out[(a_ostride * l_to1___0).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
						a_out[(a_ostride * (l_to1___0 + {NATURAL_32} 1)).to_integer_32] := l_w1_imag * l_x1_real + l_w1_real * l_x1_imag
						a_out[(a_ostride * l_to2___0).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
						a_out[(a_ostride * (l_to2___0 + {NATURAL_32} 1)).to_integer_32] := l_w2_imag * l_x2_real + l_w2_real * l_x2_imag
						l_k1 := l_k1 + 1
					end
					l_k := l_k + 1
				end
				if l_q___3 \\ {NATURAL_32} 2 = {NATURAL_32} 1 then
					l_return := True
				end
			end
			if not l_return then
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_product_1)
				loop
					l_from0___1 := (({NATURAL_32} 3 * l_k1) * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_from1___1 := l_from0___1 + {NATURAL_32} 2 * l_q___3
					l_z0_real___1 := a_in[(a_istride * l_from0___1).to_integer_32]
					l_z0_imag___0 := a_in[(a_istride * (l_from0___1 + {NATURAL_32} 1)).to_integer_32]
					l_z1_real___1 := a_in[(a_istride * l_from1___1).to_integer_32]
					l_t1_real___1 := l_z0_real___1 - l_z1_real___1
					l_t2_real___1 := ((2).to_real * l_tau) * l_z0_imag___0
					l_x0_real___0 := (2).to_real * l_z0_real___1 + l_z1_real___1
					l_x1_real___0 := l_t1_real___1 - l_t2_real___1
					l_x2_real___0 := -l_t1_real___1 - l_t2_real___1
					l_to0___1 := (l_k1 * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_to1___1 := l_to0___1 + l_m___4
					l_to2___1 := l_to1___1 + l_m___4
					a_out[(a_ostride * l_to0___1).to_integer_32] := l_x0_real___0
					a_out[(a_ostride * l_to1___1).to_integer_32] := l_x1_real___0
					a_out[(a_ostride * l_to2___1).to_integer_32] := l_x2_real___0
					l_k1 := l_k1 + 1
				end
			end
		end

	fft_halfcomplex_float_pass_4 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride, a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2, a_twiddle3: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_jump: NATURAL_32
			l_factor: NATURAL_32
			l_q___3: NATURAL_32
			l_m___4: NATURAL_32
			l_product_1: NATURAL_32
			l_from0: NATURAL_32
			l_from1: NATURAL_32
			l_from2: NATURAL_32
			l_z0_real: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_t1_real: REAL_32
			l_t2_real: REAL_32
			l_t3_real: REAL_32
			l_t4_imag: REAL_32
			l_to0: NATURAL_32
			l_to1: NATURAL_32
			l_to2: NATURAL_32
			l_to3: NATURAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_w3_real: REAL_32
			l_w3_imag: REAL_32
			l_from0___0: NATURAL_32
			l_from1___0: NATURAL_32
			l_from2___0: NATURAL_32
			l_from3: NATURAL_32
			l_z0_real___0: REAL_32
			l_z0_imag: REAL_32
			l_z1_real___0: REAL_32
			l_z1_imag___0: REAL_32
			l_z2_real___0: REAL_32
			l_z2_imag: REAL_32
			l_z3_real: REAL_32
			l_z3_imag: REAL_32
			l_t1_real___0: REAL_32
			l_t1_imag: REAL_32
			l_t2_real___0: REAL_32
			l_t2_imag: REAL_32
			l_t3_real___0: REAL_32
			l_t3_imag: REAL_32
			l_t4_real: REAL_32
			l_t4_imag___0: REAL_32
			l_x0_real: REAL_32
			l_x0_imag: REAL_32
			l_x1_real: REAL_32
			l_x1_imag: REAL_32
			l_x2_real: REAL_32
			l_x2_imag: REAL_32
			l_x3_real: REAL_32
			l_x3_imag: REAL_32
			l_to0___0: NATURAL_32
			l_to1___0: NATURAL_32
			l_to2___0: NATURAL_32
			l_to3___0: NATURAL_32
			l_from0___1: NATURAL_32
			l_from1___1: NATURAL_32
			l_z0_real___1: REAL_32
			l_z0_imag___0: REAL_32
			l_z1_real___1: REAL_32
			l_z1_imag___1: REAL_32
			l_t1_real___1: REAL_32
			l_t2_real___1: REAL_32
			l_x0_real___0: REAL_32
			l_x1_real___0: REAL_32
			l_x2_real___0: REAL_32
			l_x3_real___0: REAL_32
			l_to0___1: NATURAL_32
			l_to1___1: NATURAL_32
			l_to2___1: NATURAL_32
			l_to3___1: NATURAL_32
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (4).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_q___3
			l_k1 := (0).to_natural_32
			from
			until
				not (l_k1 < l_product_1)
			loop
				l_from0 := ({NATURAL_32} 4 * l_k1) * l_q___3
				l_from1 := (l_from0 + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
				l_from2 := l_from1 + {NATURAL_32} 2 * l_q___3
				l_z0_real := a_in[(a_istride * l_from0).to_integer_32]
				l_z1_real := a_in[(a_istride * l_from1).to_integer_32]
				l_z1_imag := a_in[(a_istride * (l_from1 + {NATURAL_32} 1)).to_integer_32]
				l_z2_real := a_in[(a_istride * l_from2).to_integer_32]
				l_t1_real := l_z0_real + l_z2_real
				l_t2_real := (2).to_real * l_z1_real
				l_t3_real := l_z0_real - l_z2_real
				l_t4_imag := (2).to_real * l_z1_imag
				l_to0 := l_q___3 * l_k1
				l_to1 := l_to0 + l_m___4
				l_to2 := l_to1 + l_m___4
				l_to3 := l_to2 + l_m___4
				a_out[(a_ostride * l_to0).to_integer_32] := l_t1_real + l_t2_real
				a_out[(a_ostride * l_to1).to_integer_32] := l_t3_real - l_t4_imag
				a_out[(a_ostride * l_to2).to_integer_32] := l_t1_real - l_t2_real
				a_out[(a_ostride * l_to3).to_integer_32] := l_t3_real + l_t4_imag
				l_k1 := l_k1 + 1
			end
			if l_q___3 = {NATURAL_32} 1 then
				l_return := True
			end
			if not l_return then
				l_k := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < (l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2))
				loop
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_k1 := (0).to_natural_32
					from
					until
						not (l_k1 < l_product_1)
					loop
						l_from0___0 := (({NATURAL_32} 4 * l_k1) * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_from1___0 := l_from0___0 + {NATURAL_32} 2 * l_q___3
						l_from2___0 := ((({NATURAL_32} 4 * l_k1) * l_q___3 - {NATURAL_32} 2 * l_k) + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
						l_from3 := l_from2___0 + {NATURAL_32} 2 * l_q___3
						l_z0_real___0 := a_in[(a_istride * l_from0___0).to_integer_32]
						l_z0_imag := a_in[(a_istride * (l_from0___0 + {NATURAL_32} 1)).to_integer_32]
						l_z1_real___0 := a_in[(a_istride * l_from1___0).to_integer_32]
						l_z1_imag___0 := a_in[(a_istride * (l_from1___0 + {NATURAL_32} 1)).to_integer_32]
						l_z2_real___0 := a_in[(a_istride * l_from3).to_integer_32]
						l_z2_imag := (-a_in[(a_istride * (l_from3 + {NATURAL_32} 1)).to_integer_32])
						l_z3_real := a_in[(a_istride * l_from2___0).to_integer_32]
						l_z3_imag := (-a_in[(a_istride * (l_from2___0 + {NATURAL_32} 1)).to_integer_32])
						l_t1_real___0 := l_z0_real___0 + l_z2_real___0
						l_t1_imag := l_z0_imag + l_z2_imag
						l_t2_real___0 := l_z1_real___0 + l_z3_real
						l_t2_imag := l_z1_imag___0 + l_z3_imag
						l_t3_real___0 := l_z0_real___0 - l_z2_real___0
						l_t3_imag := l_z0_imag - l_z2_imag
						l_t4_real := l_z1_real___0 - l_z3_real
						l_t4_imag___0 := l_z1_imag___0 - l_z3_imag
						l_x0_real := l_t1_real___0 + l_t2_real___0
						l_x0_imag := l_t1_imag + l_t2_imag
						l_x1_real := l_t3_real___0 - l_t4_imag___0
						l_x1_imag := l_t3_imag + l_t4_real
						l_x2_real := l_t1_real___0 - l_t2_real___0
						l_x2_imag := l_t1_imag - l_t2_imag
						l_x3_real := l_t3_real___0 + l_t4_imag___0
						l_x3_imag := l_t3_imag - l_t4_real
						l_to0___0 := (l_k1 * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_to1___0 := l_to0___0 + l_m___4
						l_to2___0 := l_to1___0 + l_m___4
						l_to3___0 := l_to2___0 + l_m___4
						a_out[(a_ostride * l_to0___0).to_integer_32] := l_x0_real
						a_out[(a_ostride * (l_to0___0 + {NATURAL_32} 1)).to_integer_32] := l_x0_imag
						a_out[(a_ostride * l_to1___0).to_integer_32] := l_w1_real * l_x1_real - l_w1_imag * l_x1_imag
						a_out[(a_ostride * (l_to1___0 + {NATURAL_32} 1)).to_integer_32] := l_w1_imag * l_x1_real + l_w1_real * l_x1_imag
						a_out[(a_ostride * l_to2___0).to_integer_32] := l_w2_real * l_x2_real - l_w2_imag * l_x2_imag
						a_out[(a_ostride * (l_to2___0 + {NATURAL_32} 1)).to_integer_32] := l_w2_imag * l_x2_real + l_w2_real * l_x2_imag
						a_out[(a_ostride * l_to3___0).to_integer_32] := l_w3_real * l_x3_real - l_w3_imag * l_x3_imag
						a_out[(a_ostride * (l_to3___0 + {NATURAL_32} 1)).to_integer_32] := l_w3_real * l_x3_imag + l_w3_imag * l_x3_real
						l_k1 := l_k1 + 1
					end
					l_k := l_k + 1
				end
				if l_q___3 \\ {NATURAL_32} 2 = {NATURAL_32} 1 then
					l_return := True
				end
			end
			if not l_return then
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_product_1)
				loop
					l_from0___1 := (({NATURAL_32} 4 * l_k1) * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_from1___1 := l_from0___1 + {NATURAL_32} 2 * l_q___3
					l_z0_real___1 := a_in[(a_istride * l_from0___1).to_integer_32]
					l_z0_imag___0 := a_in[(a_istride * (l_from0___1 + {NATURAL_32} 1)).to_integer_32]
					l_z1_real___1 := a_in[(a_istride * l_from1___1).to_integer_32]
					l_z1_imag___1 := a_in[(a_istride * (l_from1___1 + {NATURAL_32} 1)).to_integer_32]
					l_t1_real___1 := ((sqrt (2.0)) * (l_z0_imag___0 + l_z1_imag___1).to_double).truncated_to_real
					l_t2_real___1 := ((sqrt (2.0)) * (l_z0_real___1 - l_z1_real___1).to_double).truncated_to_real
					l_x0_real___0 := (2).to_real * (l_z0_real___1 + l_z1_real___1)
					l_x1_real___0 := l_t2_real___1 - l_t1_real___1
					l_x2_real___0 := (2).to_real * (l_z1_imag___1 - l_z0_imag___0)
					l_x3_real___0 := -(l_t2_real___1 + l_t1_real___1)
					l_to0___1 := (l_k1 * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_to1___1 := l_to0___1 + l_m___4
					l_to2___1 := l_to1___1 + l_m___4
					l_to3___1 := l_to2___1 + l_m___4
					a_out[(a_ostride * l_to0___1).to_integer_32] := l_x0_real___0
					a_out[(a_ostride * l_to1___1).to_integer_32] := l_x1_real___0
					a_out[(a_ostride * l_to2___1).to_integer_32] := l_x2_real___0
					a_out[(a_ostride * l_to3___1).to_integer_32] := l_x3_real___0
					l_k1 := l_k1 + 1
				end
			end
		end

	fft_halfcomplex_float_pass_5 (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride, a_product, a_n: NATURAL_32; a_twiddle1, a_twiddle2, a_twiddle3, a_twiddle4: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_jump: NATURAL_32
			l_factor: NATURAL_32
			l_q___3: NATURAL_32
			l_m___4: NATURAL_32
			l_product_1: NATURAL_32
			l_sina: REAL_32
			l_sinb: REAL_32
			l_from0: NATURAL_32
			l_from1: NATURAL_32
			l_from2: NATURAL_32
			l_z0_real: REAL_32
			l_z1_real: REAL_32
			l_z1_imag: REAL_32
			l_z2_real: REAL_32
			l_z2_imag: REAL_32
			l_t1_real: REAL_32
			l_t2_real: REAL_32
			l_t3_real: REAL_32
			l_t4_real: REAL_32
			l_t5_real: REAL_32
			l_t6_imag: REAL_32
			l_t7_imag: REAL_32
			l_x0_real: REAL_32
			l_x1_real: REAL_32
			l_x2_real: REAL_32
			l_x3_real: REAL_32
			l_x4_real: REAL_32
			l_to0: NATURAL_32
			l_to1: NATURAL_32
			l_to2: NATURAL_32
			l_to3: NATURAL_32
			l_to4: NATURAL_32
			l_w1_real: REAL_32
			l_w1_imag: REAL_32
			l_w2_real: REAL_32
			l_w2_imag: REAL_32
			l_w3_real: REAL_32
			l_w3_imag: REAL_32
			l_w4_real: REAL_32
			l_w4_imag: REAL_32
			l_from0___0: NATURAL_32
			l_from1___0: NATURAL_32
			l_from2___0: NATURAL_32
			l_from3: NATURAL_32
			l_from4: NATURAL_32
			l_z0_real___0: REAL_32
			l_z0_imag: REAL_32
			l_z1_real___0: REAL_32
			l_z1_imag___0: REAL_32
			l_z2_real___0: REAL_32
			l_z2_imag___0: REAL_32
			l_z3_real: REAL_32
			l_z3_imag: REAL_32
			l_z4_real: REAL_32
			l_z4_imag: REAL_32
			l_t1_real___0: REAL_32
			l_t1_imag: REAL_32
			l_t2_real___0: REAL_32
			l_t2_imag: REAL_32
			l_t3_real___0: REAL_32
			l_t3_imag: REAL_32
			l_t4_real___0: REAL_32
			l_t4_imag: REAL_32
			l_t5_real___0: REAL_32
			l_t5_imag: REAL_32
			l_t6_real: REAL_32
			l_t6_imag___0: REAL_32
			l_t7_real: REAL_32
			l_t7_imag___0: REAL_32
			l_t8_real: REAL_32
			l_t8_imag: REAL_32
			l_t9_real: REAL_32
			l_t9_imag: REAL_32
			l_t10_real: REAL_32
			l_t10_imag: REAL_32
			l_t11_real: REAL_32
			l_t11_imag: REAL_32
			l_x0_real___0: REAL_32
			l_x0_imag: REAL_32
			l_x1_real___0: REAL_32
			l_x1_imag: REAL_32
			l_x2_real___0: REAL_32
			l_x2_imag: REAL_32
			l_x3_real___0: REAL_32
			l_x3_imag: REAL_32
			l_x4_real___0: REAL_32
			l_x4_imag: REAL_32
			l_to0___0: NATURAL_32
			l_to1___0: NATURAL_32
			l_to2___0: NATURAL_32
			l_to3___0: NATURAL_32
			l_to4___0: NATURAL_32
			l_from0___1: NATURAL_32
			l_from1___1: NATURAL_32
			l_from2___1: NATURAL_32
			l_z0_real___1: REAL_32
			l_z0_imag___0: REAL_32
			l_z1_real___1: REAL_32
			l_z1_imag___1: REAL_32
			l_z2_real___1: REAL_32
			l_t1_real___1: REAL_32
			l_t2_real___1: REAL_32
			l_t3_real___1: REAL_32
			l_t4_real___1: REAL_32
			l_t5_real___1: REAL_32
			l_t6_real___0: REAL_32
			l_t7_real___0: REAL_32
			l_x0_real___1: REAL_32
			l_x1_real___1: REAL_32
			l_x2_real___1: REAL_32
			l_x3_real___1: REAL_32
			l_x4_real___1: REAL_32
			l_to0___1: NATURAL_32
			l_to1___1: NATURAL_32
			l_to2___1: NATURAL_32
			l_to3___1: NATURAL_32
			l_to4___1: NATURAL_32
			l_return: BOOLEAN
		do
			l_sina := (sin ((2.0 * 3.14159265358979323846) / 5.0)).truncated_to_real
			l_sinb := (sin ((2.0 * 3.14159265358979323846) / 10.0)).truncated_to_real
			l_i := (0).to_natural_32
			l_j := (0).to_natural_32
			l_factor := (5).to_natural_32
			l_m___4 := a_n // l_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // l_factor
			l_jump := (l_factor - {NATURAL_32} 1) * l_q___3
			l_k1 := (0).to_natural_32
			from
			until
				not (l_k1 < l_product_1)
			loop
				l_from0 := ({NATURAL_32} 5 * l_k1) * l_q___3
				l_from1 := (l_from0 + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
				l_from2 := l_from1 + {NATURAL_32} 2 * l_q___3
				l_z0_real := a_in[(a_istride * l_from0).to_integer_32]
				l_z1_real := a_in[(a_istride * l_from1).to_integer_32]
				l_z1_imag := a_in[(a_istride * (l_from1 + {NATURAL_32} 1)).to_integer_32]
				l_z2_real := a_in[(a_istride * l_from2).to_integer_32]
				l_z2_imag := a_in[(a_istride * (l_from2 + {NATURAL_32} 1)).to_integer_32]
				l_t1_real := (2).to_real * (l_z1_real + l_z2_real)
				l_t2_real := (((2).to_double * ((sqrt (5.0)) / 4.0)) * (l_z1_real - l_z2_real).to_double).truncated_to_real
				l_t3_real := (l_z0_real.to_double - l_t1_real.to_double / 4.0).truncated_to_real
				l_t4_real := l_t2_real + l_t3_real
				l_t5_real := -l_t2_real + l_t3_real
				l_t6_imag := (2).to_real * (l_sina * l_z1_imag + l_sinb * l_z2_imag)
				l_t7_imag := (2).to_real * (l_sinb * l_z1_imag - l_sina * l_z2_imag)
				l_x0_real := l_z0_real + l_t1_real
				l_x1_real := l_t4_real - l_t6_imag
				l_x2_real := l_t5_real - l_t7_imag
				l_x3_real := l_t5_real + l_t7_imag
				l_x4_real := l_t4_real + l_t6_imag
				l_to0 := l_q___3 * l_k1
				l_to1 := l_to0 + l_m___4
				l_to2 := l_to1 + l_m___4
				l_to3 := l_to2 + l_m___4
				l_to4 := l_to3 + l_m___4
				a_out[(a_ostride * l_to0).to_integer_32] := l_x0_real
				a_out[(a_ostride * l_to1).to_integer_32] := l_x1_real
				a_out[(a_ostride * l_to2).to_integer_32] := l_x2_real
				a_out[(a_ostride * l_to3).to_integer_32] := l_x3_real
				a_out[(a_ostride * l_to4).to_integer_32] := l_x4_real
				l_k1 := l_k1 + 1
			end
			if l_q___3 = {NATURAL_32} 1 then
				l_return := True
			end
			if not l_return then
				l_k := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < (l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2))
				loop
					l_w1_real := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w1_imag := (a_twiddle1 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w2_real := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w2_imag := (a_twiddle2 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w3_real := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w3_imag := (a_twiddle3 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_w4_real := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[0]
					l_w4_imag := (a_twiddle4 + (l_k - {NATURAL_32} 1).to_integer_32).item.dat[1]
					l_k1 := (0).to_natural_32
					from
					until
						not (l_k1 < l_product_1)
					loop
						l_from0___0 := (({NATURAL_32} 5 * l_k1) * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_from1___0 := l_from0___0 + {NATURAL_32} 2 * l_q___3
						l_from2___0 := l_from1___0 + {NATURAL_32} 2 * l_q___3
						l_from3 := ((({NATURAL_32} 5 * l_k1) * l_q___3 - {NATURAL_32} 2 * l_k) + {NATURAL_32} 2 * l_q___3) - {NATURAL_32} 1
						l_from4 := l_from3 + {NATURAL_32} 2 * l_q___3
						l_z0_real___0 := a_in[(a_istride * l_from0___0).to_integer_32]
						l_z0_imag := a_in[(a_istride * (l_from0___0 + {NATURAL_32} 1)).to_integer_32]
						l_z1_real___0 := a_in[(a_istride * l_from1___0).to_integer_32]
						l_z1_imag___0 := a_in[(a_istride * (l_from1___0 + {NATURAL_32} 1)).to_integer_32]
						l_z2_real___0 := a_in[(a_istride * l_from2___0).to_integer_32]
						l_z2_imag___0 := a_in[(a_istride * (l_from2___0 + {NATURAL_32} 1)).to_integer_32]
						l_z3_real := a_in[(a_istride * l_from4).to_integer_32]
						l_z3_imag := (-a_in[(a_istride * (l_from4 + {NATURAL_32} 1)).to_integer_32])
						l_z4_real := a_in[(a_istride * l_from3).to_integer_32]
						l_z4_imag := (-a_in[(a_istride * (l_from3 + {NATURAL_32} 1)).to_integer_32])
						l_t1_real___0 := l_z1_real___0 + l_z4_real
						l_t1_imag := l_z1_imag___0 + l_z4_imag
						l_t2_real___0 := l_z2_real___0 + l_z3_real
						l_t2_imag := l_z2_imag___0 + l_z3_imag
						l_t3_real___0 := l_z1_real___0 - l_z4_real
						l_t3_imag := l_z1_imag___0 - l_z4_imag
						l_t4_real___0 := l_z2_real___0 - l_z3_real
						l_t4_imag := l_z2_imag___0 - l_z3_imag
						l_t5_real___0 := l_t1_real___0 + l_t2_real___0
						l_t5_imag := l_t1_imag + l_t2_imag
						l_t6_real := (((sqrt (5.0)) / 4.0) * (l_t1_real___0 - l_t2_real___0).to_double).truncated_to_real
						l_t6_imag___0 := (((sqrt (5.0)) / 4.0) * (l_t1_imag - l_t2_imag).to_double).truncated_to_real
						l_t7_real := (l_z0_real___0.to_double - l_t5_real___0.to_double / 4.0).truncated_to_real
						l_t7_imag___0 := (l_z0_imag.to_double - l_t5_imag.to_double / 4.0).truncated_to_real
						l_t8_real := l_t7_real + l_t6_real
						l_t8_imag := l_t7_imag___0 + l_t6_imag___0
						l_t9_real := l_t7_real - l_t6_real
						l_t9_imag := l_t7_imag___0 - l_t6_imag___0
						l_t10_real := l_sina * l_t3_real___0 + l_sinb * l_t4_real___0
						l_t10_imag := l_sina * l_t3_imag + l_sinb * l_t4_imag
						l_t11_real := l_sinb * l_t3_real___0 - l_sina * l_t4_real___0
						l_t11_imag := l_sinb * l_t3_imag - l_sina * l_t4_imag
						l_x0_real___0 := l_z0_real___0 + l_t5_real___0
						l_x0_imag := l_z0_imag + l_t5_imag
						l_x1_real___0 := l_t8_real - l_t10_imag
						l_x1_imag := l_t8_imag + l_t10_real
						l_x2_real___0 := l_t9_real - l_t11_imag
						l_x2_imag := l_t9_imag + l_t11_real
						l_x3_real___0 := l_t9_real + l_t11_imag
						l_x3_imag := l_t9_imag - l_t11_real
						l_x4_real___0 := l_t8_real + l_t10_imag
						l_x4_imag := l_t8_imag - l_t10_real
						l_to0___0 := (l_k1 * l_q___3 + {NATURAL_32} 2 * l_k) - {NATURAL_32} 1
						l_to1___0 := l_to0___0 + l_m___4
						l_to2___0 := l_to1___0 + l_m___4
						l_to3___0 := l_to2___0 + l_m___4
						l_to4___0 := l_to3___0 + l_m___4
						a_out[(a_ostride * l_to0___0).to_integer_32] := l_x0_real___0
						a_out[(a_ostride * (l_to0___0 + {NATURAL_32} 1)).to_integer_32] := l_x0_imag
						a_out[(a_ostride * l_to1___0).to_integer_32] := l_w1_real * l_x1_real___0 - l_w1_imag * l_x1_imag
						a_out[(a_ostride * (l_to1___0 + {NATURAL_32} 1)).to_integer_32] := l_w1_real * l_x1_imag + l_w1_imag * l_x1_real___0
						a_out[(a_ostride * l_to2___0).to_integer_32] := l_w2_real * l_x2_real___0 - l_w2_imag * l_x2_imag
						a_out[(a_ostride * (l_to2___0 + {NATURAL_32} 1)).to_integer_32] := l_w2_real * l_x2_imag + l_w2_imag * l_x2_real___0
						a_out[(a_ostride * l_to3___0).to_integer_32] := l_w3_real * l_x3_real___0 - l_w3_imag * l_x3_imag
						a_out[(a_ostride * (l_to3___0 + {NATURAL_32} 1)).to_integer_32] := l_w3_real * l_x3_imag + l_w3_imag * l_x3_real___0
						a_out[(a_ostride * l_to4___0).to_integer_32] := l_w4_real * l_x4_real___0 - l_w4_imag * l_x4_imag
						a_out[(a_ostride * (l_to4___0 + {NATURAL_32} 1)).to_integer_32] := l_w4_real * l_x4_imag + l_w4_imag * l_x4_real___0
						l_k1 := l_k1 + 1
					end
					l_k := l_k + 1
				end
				if l_q___3 \\ {NATURAL_32} 2 = {NATURAL_32} 1 then
					l_return := True
				end
			end
			if not l_return then
				l_k1 := (0).to_natural_32
				from
				until
					not (l_k1 < l_product_1)
				loop
					l_from0___1 := (({NATURAL_32} 5 * l_k1) * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_from1___1 := l_from0___1 + {NATURAL_32} 2 * l_q___3
					l_from2___1 := l_from1___1 + {NATURAL_32} 2 * l_q___3
					l_z0_real___1 := ((2).to_real * a_in[(a_istride * l_from0___1).to_integer_32])
					l_z0_imag___0 := ((2).to_real * a_in[(a_istride * (l_from0___1 + {NATURAL_32} 1)).to_integer_32])
					l_z1_real___1 := ((2).to_real * a_in[(a_istride * l_from1___1).to_integer_32])
					l_z1_imag___1 := ((2).to_real * a_in[(a_istride * (l_from1___1 + {NATURAL_32} 1)).to_integer_32])
					l_z2_real___1 := a_in[(a_istride * l_from2___1).to_integer_32]
					l_t1_real___1 := l_z0_real___1 + l_z1_real___1
					l_t2_real___1 := (l_t1_real___1.to_double / 4.0 - l_z2_real___1.to_double).truncated_to_real
					l_t3_real___1 := (((sqrt (5.0)) / 4.0) * (l_z0_real___1 - l_z1_real___1).to_double).truncated_to_real
					l_t4_real___1 := l_sinb * l_z0_imag___0 + l_sina * l_z1_imag___1
					l_t5_real___1 := l_sina * l_z0_imag___0 - l_sinb * l_z1_imag___1
					l_t6_real___0 := l_t3_real___1 + l_t2_real___1
					l_t7_real___0 := l_t3_real___1 - l_t2_real___1
					l_x0_real___1 := l_t1_real___1 + l_z2_real___1
					l_x1_real___1 := l_t6_real___0 - l_t4_real___1
					l_x2_real___1 := l_t7_real___0 - l_t5_real___1
					l_x3_real___1 := -l_t7_real___0 - l_t5_real___1
					l_x4_real___1 := -l_t6_real___0 - l_t4_real___1
					l_to0___1 := (l_k1 * l_q___3 + l_q___3) - {NATURAL_32} 1
					l_to1___1 := l_to0___1 + l_m___4
					l_to2___1 := l_to1___1 + l_m___4
					l_to3___1 := l_to2___1 + l_m___4
					l_to4___1 := l_to3___1 + l_m___4
					a_out[(a_ostride * l_to0___1).to_integer_32] := l_x0_real___1
					a_out[(a_ostride * l_to1___1).to_integer_32] := l_x1_real___1
					a_out[(a_ostride * l_to2___1).to_integer_32] := l_x2_real___1
					a_out[(a_ostride * l_to3___1).to_integer_32] := l_x3_real___1
					a_out[(a_ostride * l_to4___1).to_integer_32] := l_x4_real___1
					l_k1 := l_k1 + 1
				end
			end
		end

	fft_halfcomplex_float_pass_n (a_in: CE_POINTER [REAL_32]; a_istride: NATURAL_32; a_out: CE_POINTER [REAL_32]; a_ostride, a_factor, a_product, a_n: NATURAL_32; a_twiddle: CE_POINTER [PD_GSL_COMPLEX_FLOAT])
		local
			l_k: NATURAL_32
			l_k1: NATURAL_32
			l_m___4: NATURAL_32
			l_q___3: NATURAL_32
			l_product_1: NATURAL_32
			l_e1: NATURAL_32
			l_e2: NATURAL_32
			l_d_theta: REAL_64
			l_cos_d_theta: REAL_32
			l_sin_d_theta: REAL_32
			l_dw_real: REAL_32
			l_dw_imag: REAL_32
			l_sum_real: REAL_32
			l_w_real: REAL_32
			l_w_imag: REAL_32
			l_tmp_real: REAL_32
			l_tmp_imag: REAL_32
			l_z_real: REAL_32
			l_z_imag: REAL_32
			l_tmp_real___0: REAL_32
			l_tmp_imag___0: REAL_32
			l_from_idx: NATURAL_32
			l_from_idx___0: NATURAL_32
			l_from_idx___1: NATURAL_32
			l_to_idx: NATURAL_32
			l_dw_real___0: REAL_32
			l_dw_imag___0: REAL_32
			l_z_real___0: REAL_32
			l_z_imag___0: REAL_32
			l_sum_real___0: REAL_32
			l_sum_imag: REAL_32
			l_w_real___0: REAL_32
			l_w_imag___0: REAL_32
			l_t_real: REAL_32
			l_t_imag: REAL_32
			l_tmp_real___1: REAL_32
			l_tmp_imag___1: REAL_32
			l_from0: NATURAL_32
			l_from0___0: NATURAL_32
			l_tskip: NATURAL_32
			l_to0: NATURAL_32
			l_tw_arg: REAL_64
			l_cos_tw_arg: REAL_32
			l_sin_tw_arg: REAL_32
			l_dw_real___1: REAL_32
			l_dw_imag___1: REAL_32
			l_tw_real: REAL_32
			l_tw_imag: REAL_32
			l_w_real___1: REAL_32
			l_w_imag___1: REAL_32
			l_z_real___1: REAL_32
			l_z_imag___1: REAL_32
			l_sum_real___1: REAL_32
			l_tmp_real___2: REAL_32
			l_tmp_imag___2: REAL_32
			l_t_real___0: REAL_32
			l_t_imag___0: REAL_32
			l_tmp_real___3: REAL_32
			l_tmp_imag___3: REAL_32
			l_from0___1: NATURAL_32
			l_from0___2: NATURAL_32
			l_to0___0: NATURAL_32
			l_return: BOOLEAN
		do
			l_m___4 := a_n // a_factor
			l_q___3 := a_n // a_product
			l_product_1 := a_product // a_factor
			l_d_theta := (2.0 * 3.14159265358979323846) / a_factor.to_real_64
			l_cos_d_theta := (cos (l_d_theta)).truncated_to_real
			l_sin_d_theta := (sin (l_d_theta)).truncated_to_real
			l_k1 := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_k1 < l_product_1))
			loop
				l_dw_real := (1.0).truncated_to_real
				l_dw_imag := (0.0).truncated_to_real
				l_e1 := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_e1 < a_factor))
				loop
					l_sum_real := (0.0).truncated_to_real
					l_w_real := (1.0).truncated_to_real
					l_w_imag := (0.0).truncated_to_real
					if l_e1 > {NATURAL_32} 0 then
						l_tmp_real := l_dw_real * l_cos_d_theta - l_dw_imag * l_sin_d_theta
						l_tmp_imag := l_dw_real * l_sin_d_theta + l_dw_imag * l_cos_d_theta
						l_dw_real := l_tmp_real
						l_dw_imag := l_tmp_imag
					end
					l_e2 := (0).to_natural_32
					from
					until
						not (l_e2 <= a_factor - l_e2)
					loop
						if l_e2 > {NATURAL_32} 0 then
							l_tmp_real___0 := l_dw_real * l_w_real - l_dw_imag * l_w_imag
							l_tmp_imag___0 := l_dw_real * l_w_imag + l_dw_imag * l_w_real
							l_w_real := l_tmp_real___0
							l_w_imag := l_tmp_imag___0
						end
						if l_e2 = {NATURAL_32} 0 then
							l_from_idx := (a_factor * l_k1) * l_q___3
							l_z_real := a_in[(a_istride * l_from_idx).to_integer_32]
							l_z_imag := (0.0).truncated_to_real
							l_sum_real := l_sum_real + (l_w_real * l_z_real - l_w_imag * l_z_imag)
						elseif l_e2 = a_factor - l_e2 then
							l_from_idx___0 := ((a_factor * l_q___3) * l_k1 + ({NATURAL_32} 2 * l_e2) * l_q___3) - {NATURAL_32} 1
							l_z_real := a_in[(a_istride * l_from_idx___0).to_integer_32]
							l_z_imag := (0.0).truncated_to_real
							l_sum_real := l_sum_real + (l_w_real * l_z_real)
						else
							l_from_idx___1 := ((a_factor * l_q___3) * l_k1 + ({NATURAL_32} 2 * l_e2) * l_q___3) - {NATURAL_32} 1
							l_z_real := a_in[(a_istride * l_from_idx___1).to_integer_32]
							l_z_imag := a_in[(a_istride * (l_from_idx___1 + {NATURAL_32} 1)).to_integer_32]
							l_sum_real := l_sum_real + ((2).to_real * (l_w_real * l_z_real - l_w_imag * l_z_imag))
						end
						l_e2 := l_e2 + 1
					end
					l_to_idx := l_q___3 * l_k1 + l_e1 * l_m___4
					a_out[(a_ostride * l_to_idx).to_integer_32] := l_sum_real
					l_e1 := l_e1 + 1
				end
				l_k1 := l_k1 + 1
			end
			if l_q___3 = {NATURAL_32} 1 then
				l_return := True
			end
			if not l_return then
				l_k := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < (l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2))
				loop
					l_k1 := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k1 < l_product_1))
					loop
						l_dw_real___0 := (1.0).truncated_to_real
						l_dw_imag___0 := (0.0).truncated_to_real
						l_e1 := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_e1 < a_factor))
						loop
							l_sum_real___0 := (0.0).truncated_to_real
							l_sum_imag := (0.0).truncated_to_real
							l_w_real___0 := (1.0).truncated_to_real
							l_w_imag___0 := (0.0).truncated_to_real
							if l_e1 > {NATURAL_32} 0 then
								l_t_real := l_dw_real___0 * l_cos_d_theta - l_dw_imag___0 * l_sin_d_theta
								l_t_imag := l_dw_real___0 * l_sin_d_theta + l_dw_imag___0 * l_cos_d_theta
								l_dw_real___0 := l_t_real
								l_dw_imag___0 := l_t_imag
							end
							l_e2 := (0).to_natural_32
							from
							until
								not (l_e2 < a_factor)
							loop
								if l_e2 > {NATURAL_32} 0 then
									l_tmp_real___1 := l_dw_real___0 * l_w_real___0 - l_dw_imag___0 * l_w_imag___0
									l_tmp_imag___1 := l_dw_real___0 * l_w_imag___0 + l_dw_imag___0 * l_w_real___0
									l_w_real___0 := l_tmp_real___1
									l_w_imag___0 := l_tmp_imag___1
								end
								if l_e2 < a_factor - l_e2 then
									l_from0 := (((a_factor * l_k1) * l_q___3 + {NATURAL_32} 2 * l_k) + ({NATURAL_32} 2 * l_e2) * l_q___3) - {NATURAL_32} 1
									l_z_real___0 := a_in[(a_istride * l_from0).to_integer_32]
									l_z_imag___0 := a_in[(a_istride * (l_from0 + {NATURAL_32} 1)).to_integer_32]
								else
									l_from0___0 := (((a_factor * l_k1) * l_q___3 - {NATURAL_32} 2 * l_k) + ({NATURAL_32} 2 * (a_factor - l_e2)) * l_q___3) - {NATURAL_32} 1
									l_z_real___0 := a_in[(a_istride * l_from0___0).to_integer_32]
									l_z_imag___0 := (-a_in[(a_istride * (l_from0___0 + {NATURAL_32} 1)).to_integer_32])
								end
								l_sum_real___0 := l_sum_real___0 + (l_w_real___0 * l_z_real___0 - l_w_imag___0 * l_z_imag___0)
								l_sum_imag := l_sum_imag + (l_w_real___0 * l_z_imag___0 + l_w_imag___0 * l_z_real___0)
								l_e2 := l_e2 + 1
							end
							if l_k = {NATURAL_32} 0 then
								l_w_real___0 := (1.0).truncated_to_real
								l_w_imag___0 := (0.0).truncated_to_real
							elseif l_e1 = {NATURAL_32} 0 then
								l_w_real___0 := (1.0).truncated_to_real
								l_w_imag___0 := (0.0).truncated_to_real
							else
								l_tskip := (l_q___3 + {NATURAL_32} 1) // {NATURAL_32} 2 - {NATURAL_32} 1
								l_w_real___0 := (a_twiddle + ((l_k - {NATURAL_32} 1) + l_tskip * (l_e1 - {NATURAL_32} 1)).to_integer_32).item.dat[0]
								l_w_imag___0 := (a_twiddle + ((l_k - {NATURAL_32} 1) + l_tskip * (l_e1 - {NATURAL_32} 1)).to_integer_32).item.dat[1]
							end
							l_to0 := ((l_k1 * l_q___3 + {NATURAL_32} 2 * l_k) + l_e1 * l_m___4) - {NATURAL_32} 1
							a_out[(a_ostride * l_to0).to_integer_32] := l_w_real___0 * l_sum_real___0 - l_w_imag___0 * l_sum_imag
							a_out[(a_ostride * (l_to0 + {NATURAL_32} 1)).to_integer_32] := l_w_real___0 * l_sum_imag + l_w_imag___0 * l_sum_real___0
							l_e1 := l_e1 + 1
						end
						l_k1 := l_k1 + 1
					end
					l_k := l_k + 1
				end
				if l_q___3 \\ {NATURAL_32} 2 = {NATURAL_32} 1 then
					l_return := True
				end
			end
			if not l_return then
				l_tw_arg := 3.14159265358979323846 / a_factor.to_real_64
				l_cos_tw_arg := (cos (l_tw_arg)).truncated_to_real
				l_sin_tw_arg := (sin (l_tw_arg)).truncated_to_real
				l_k1 := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k1 < l_product_1))
				loop
					l_dw_real___1 := (1.0).truncated_to_real
					l_dw_imag___1 := (0.0).truncated_to_real
					l_tw_real := (1.0).truncated_to_real
					l_tw_imag := (0.0).truncated_to_real
					l_e1 := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_e1 < a_factor))
					loop
						l_sum_real___1 := (0.0).truncated_to_real
						if l_e1 > {NATURAL_32} 0 then
							l_tmp_real___2 := l_tw_real * l_cos_tw_arg - l_tw_imag * l_sin_tw_arg
							l_tmp_imag___2 := l_tw_real * l_sin_tw_arg + l_tw_imag * l_cos_tw_arg
							l_tw_real := l_tmp_real___2
							l_tw_imag := l_tmp_imag___2
						end
						l_w_real___1 := l_tw_real
						l_w_imag___1 := l_tw_imag
						if l_e1 > {NATURAL_32} 0 then
							l_t_real___0 := l_dw_real___1 * l_cos_d_theta - l_dw_imag___1 * l_sin_d_theta
							l_t_imag___0 := l_dw_real___1 * l_sin_d_theta + l_dw_imag___1 * l_cos_d_theta
							l_dw_real___1 := l_t_real___0
							l_dw_imag___1 := l_t_imag___0
						end
						l_e2 := (0).to_natural_32
						from
						until
							not (l_e2 <= (a_factor - l_e2) - {NATURAL_32} 1)
						loop
							if l_e2 > {NATURAL_32} 0 then
								l_tmp_real___3 := l_dw_real___1 * l_w_real___1 - l_dw_imag___1 * l_w_imag___1
								l_tmp_imag___3 := l_dw_real___1 * l_w_imag___1 + l_dw_imag___1 * l_w_real___1
								l_w_real___1 := l_tmp_real___3
								l_w_imag___1 := l_tmp_imag___3
							end
							if l_e2 = (a_factor - l_e2) - {NATURAL_32} 1 then
								l_from0___1 := (((a_factor * l_k1) * l_q___3 + l_q___3) + ({NATURAL_32} 2 * l_e2) * l_q___3) - {NATURAL_32} 1
								l_z_real___1 := a_in[(a_istride * l_from0___1).to_integer_32]
								l_z_imag___1 := (0.0).truncated_to_real
								l_sum_real___1 := l_sum_real___1 + (l_w_real___1 * l_z_real___1 - l_w_imag___1 * l_z_imag___1)
							else
								l_from0___2 := (((a_factor * l_k1) * l_q___3 + l_q___3) + ({NATURAL_32} 2 * l_e2) * l_q___3) - {NATURAL_32} 1
								l_z_real___1 := a_in[(a_istride * l_from0___2).to_integer_32]
								l_z_imag___1 := a_in[(a_istride * (l_from0___2 + {NATURAL_32} 1)).to_integer_32]
								l_sum_real___1 := l_sum_real___1 + ((2).to_real * (l_w_real___1 * l_z_real___1 - l_w_imag___1 * l_z_imag___1))
							end
							l_e2 := l_e2 + 1
						end
						l_to0___0 := ((l_k1 * l_q___3 + l_q___3) + l_e1 * l_m___4) - {NATURAL_32} 1
						a_out[(a_ostride * l_to0___0).to_integer_32] := l_sum_real___1
						l_e1 := l_e1 + 1
					end
					l_k1 := l_k1 + 1
				end
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			n := a_other.n
			nf := a_other.nf
			factor := a_other.factor.twin
			twiddle := a_other.twiddle.twin
			trig := a_other.trig
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				n = a_other.n and
				nf = a_other.nf and
				factor.is_equal (a_other.factor) and
				twiddle.is_equal (a_other.twiddle) and
				trig.is_equal (a_other.trig)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("nf"),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("factor", <<64>>),
					create {CE_STRUCTURE_FIELD}.make_arrayfield ("twiddle", <<64>>),
					create {CE_STRUCTURE_FIELD}.make ("trig")
				>>)
			init_structure_info
		end

end
