class
	PD_GSL_INTEGRATION_QAWO_TABLE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_64_POINTER} chebmo.make
		end

feature {ANY} -- fields

	n: NATURAL_32 assign set_n

	omega: REAL_64 assign set_omega

	L: REAL_64 assign set_L

	par: REAL_64 assign set_par

	sine: NATURAL_32 assign set_sine

	chebmo: CE_POINTER [REAL_64] assign set_chebmo

feature {ANY} -- field setters

	set_n (a_n: NATURAL_32)
		do
			n := a_n
			update_memory_field ("n")
		end

	set_omega (a_omega: REAL_64)
		do
			omega := a_omega
			update_memory_field ("omega")
		end

	set_L (a_L: REAL_64)
		do
			L := a_L
			update_memory_field ("L")
		end

	set_par (a_par: REAL_64)
		do
			par := a_par
			update_memory_field ("par")
		end

	set_sine (a_sine: NATURAL_32)
		do
			sine := a_sine
			update_memory_field ("sine")
		end

	set_chebmo (a_chebmo: CE_POINTER [REAL_64])
		do
			chebmo := a_chebmo
			update_memory_field ("chebmo")
		end

feature {ANY} -- public routines

	gsl_integration_qawo_table_set (a_omega, a_L: REAL_64; a_sine: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_scale: REAL_64
		do
			Current.omega := a_omega
			update_class_fields -- Current might have been changed; synchronize
			Current.sine := a_sine
			update_class_fields -- Current might have been changed; synchronize
			Current.L := a_L
			update_class_fields -- Current might have been changed; synchronize
			Current.par := (0.5 * a_omega) * a_L
			update_class_fields -- Current might have been changed; synchronize
			l_scale := 1.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < n)
			loop
				(create {P_QMOMOF_STATIC}).compute_moments (par * l_scale, chebmo + ({NATURAL_32} 25 * l_i).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_scale := l_scale * (0.5)
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_integration_qawo_table_set_length (a_L: REAL_64): INTEGER_32
		local
			l_i: NATURAL_32
			l_scale: REAL_64
			l_return: BOOLEAN
		do
			if a_L = L then
				Result := 0
				l_return := True
			end
			if not l_return then
				Current.L := a_L
				update_class_fields -- Current might have been changed; synchronize
				Current.par := (0.5 * omega) * a_L
				update_class_fields -- Current might have been changed; synchronize
				l_scale := 1.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < n)
				loop
					(create {P_QMOMOF_STATIC}).compute_moments (par * l_scale, chebmo + ({NATURAL_32} 25 * l_i).to_integer_32)
					update_class_fields -- Current might have been changed; synchronize
					l_scale := l_scale * (0.5)
					l_i := l_i + 1
				end
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			n := a_other.n
			omega := a_other.omega
			L := a_other.L
			par := a_other.par
			sine := a_other.sine
			chebmo := a_other.chebmo
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				n = a_other.n and
				omega = a_other.omega and
				L = a_other.L and
				par = a_other.par and
				sine = a_other.sine and
				chebmo.is_equal (a_other.chebmo)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("n"),
					create {CE_STRUCTURE_FIELD}.make ("omega"),
					create {CE_STRUCTURE_FIELD}.make ("L"),
					create {CE_STRUCTURE_FIELD}.make ("par"),
					create {CE_STRUCTURE_FIELD}.make ("sine"),
					create {CE_STRUCTURE_FIELD}.make ("chebmo")
				>>)
			init_structure_info
		end

end
