class
	PD_GSL_MULTIFIT_FDFSOLVER

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_MULTIFIT_FDFSOLVER_TYPE]} type.make
			create {CE_CLASS_POINTER [PD_GSL_MULTIFIT_FUNCTION_FDF_STRUCT]} fdf.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} x.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} f.make
			create {CE_CLASS_POINTER [PD_GSL_MATRIX]} J.make
			create {CE_CLASS_POINTER [PD_GSL_VECTOR]} dx.make
			create {CE_VOID_POINTER} state.make
		end

feature {ANY} -- fields

	type: CE_POINTER [PD_GSL_MULTIFIT_FDFSOLVER_TYPE] assign set_type

	fdf: CE_POINTER [PD_GSL_MULTIFIT_FUNCTION_FDF_STRUCT] assign set_fdf

	x: CE_POINTER [PD_GSL_VECTOR] assign set_x

	f: CE_POINTER [PD_GSL_VECTOR] assign set_f

	J: CE_POINTER [PD_GSL_MATRIX] assign set_J

	dx: CE_POINTER [PD_GSL_VECTOR] assign set_dx

	state: CE_POINTER [ANY] assign set_state

feature {ANY} -- field setters

	set_type (a_type: CE_POINTER [PD_GSL_MULTIFIT_FDFSOLVER_TYPE])
		do
			type := a_type
			update_memory_field ("type")
		end

	set_fdf (a_fdf: CE_POINTER [PD_GSL_MULTIFIT_FUNCTION_FDF_STRUCT])
		do
			fdf := a_fdf
			update_memory_field ("fdf")
		end

	set_x (a_x: CE_POINTER [PD_GSL_VECTOR])
		do
			x := a_x
			update_memory_field ("x")
		end

	set_f (a_f: CE_POINTER [PD_GSL_VECTOR])
		do
			f := a_f
			update_memory_field ("f")
		end

	set_J (a_J: CE_POINTER [PD_GSL_MATRIX])
		do
			J := a_J
			update_memory_field ("J")
		end

	set_dx (a_dx: CE_POINTER [PD_GSL_VECTOR])
		do
			dx := a_dx
			update_memory_field ("dx")
		end

	set_state (a_state: CE_POINTER [ANY])
		do
			state := a_state
			update_memory_field ("state")
		end

feature {ANY} -- public routines

	gsl_multifit_fdfsolver_set (a_f: CE_POINTER [PD_GSL_MULTIFIT_FUNCTION_FDF_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		require
			not a_f.is_void
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if f.item.size /= a_f.item.n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("function size does not match solver"), ce_string ("fdfsolver.c"), 127, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (x.item.size /= a_x.item.size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length does not match solver"), ce_string ("fdfsolver.c"), 132, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.fdf := a_f
				update_class_fields -- Current might have been changed; synchronize
				x.item.gsl_vector_memcpy (a_x).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_tmp := (type.item.set).integer_32_item ([state, fdf, x, f, J, dx])
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
			end
		end

	gsl_multifit_fdfsolver_iterate: INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := (type.item.iterate).integer_32_item ([state, fdf, x, f, J, dx])
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_multifit_fdfsolver_name: CE_POINTER [INTEGER_8]
		do
			Result := (type.item.name)
		end

	gsl_multifit_fdfsolver_position: CE_POINTER [PD_GSL_VECTOR]
		do
			Result := x
		end

feature {ANY}

	copy (a_other: like Current)
		do
			type := a_other.type
			fdf := a_other.fdf
			x := a_other.x
			f := a_other.f
			J := a_other.J
			dx := a_other.dx
			state := a_other.state
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				type.is_equal (a_other.type) and
				fdf.is_equal (a_other.fdf) and
				x.is_equal (a_other.x) and
				f.is_equal (a_other.f) and
				J.is_equal (a_other.J) and
				dx.is_equal (a_other.dx) and
				state.is_equal (a_other.state)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("type"),
					create {CE_STRUCTURE_FIELD}.make ("fdf"),
					create {CE_STRUCTURE_FIELD}.make ("x"),
					create {CE_STRUCTURE_FIELD}.make ("f"),
					create {CE_STRUCTURE_FIELD}.make ("J"),
					create {CE_STRUCTURE_FIELD}.make ("dx"),
					create {CE_STRUCTURE_FIELD}.make ("state")
				>>)
			init_structure_info
		end

end
