class
	PD_GSL_NTUPLE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STDIO_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [SD_IO_FILE]} file.make
			create {CE_VOID_POINTER} ntuple_data.make
			Precursor
		end

feature {ANY} -- fields

	file: CE_POINTER [SD_IO_FILE] assign set_file

	ntuple_data: CE_POINTER [ANY] assign set_ntuple_data

	size: NATURAL_32 assign set_size

feature {ANY} -- field setters

	set_file (a_file: CE_POINTER [SD_IO_FILE])
		do
			file := a_file
			update_memory_field ("file")
		end

	set_ntuple_data (a_ntuple_data: CE_POINTER [ANY])
		do
			ntuple_data := a_ntuple_data
			update_memory_field ("ntuple_data")
		end

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

feature {ANY} -- public routines

	gsl_ntuple_write: INTEGER_32
		local
			l_nwrite: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_nwrite := fwrite (ntuple_data, size, (1).to_natural_32, file)
			update_class_fields -- Current might have been changed; synchronize
			if l_nwrite /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to write ntuple entry to file"), ce_string ("ntuple.c"), 103, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_ntuple_bookdata: INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := ((create {CE_CLASS_POINTER [PD_GSL_NTUPLE]}.make_shared (c_pointer))).item.gsl_ntuple_write
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_ntuple_read: INTEGER_32
		local
			l_nread: NATURAL_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_nread := fread (ntuple_data, size, (1).to_natural_32, file)
			update_class_fields -- Current might have been changed; synchronize
			if l_nread = {NATURAL_32} 0 then
				l_tmp := feof (file)
				update_class_fields -- Current might have been changed; synchronize
				if l_tmp.to_boolean then
					Result := 32
					l_return := True
				end
			end
			if not l_return and then (l_nread /= {NATURAL_32} 1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to read ntuple entry from file"), ce_string ("ntuple.c"), 136, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_ntuple_close: INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			l_tmp := fclose (file)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			if l_status.to_boolean then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to close ntuple file"), ce_string ("ntuple.c"), 196, 5)
					update_class_fields -- Current might have been changed; synchronize
					Result := 5
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				free (create {CE_VOID_POINTER}.make_shared (c_pointer))
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			file := a_other.file
			ntuple_data := a_other.ntuple_data
			size := a_other.size
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				file.is_equal (a_other.file) and
				ntuple_data.is_equal (a_other.ntuple_data) and
				size = a_other.size
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("file"),
					create {CE_STRUCTURE_FIELD}.make ("ntuple_data"),
					create {CE_STRUCTURE_FIELD}.make ("size")
				>>)
			init_structure_info
		end

end
