class
	PD_GSL_ROOT_FSOLVER

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_ROOT_FSOLVER_TYPE]} type.make
			create {CE_CLASS_POINTER [PD_GSL_FUNCTION_STRUCT]} function.make
			create {CE_VOID_POINTER} state.make
		end

feature {ANY} -- fields

	type: CE_POINTER [PD_GSL_ROOT_FSOLVER_TYPE] assign set_type

	function: CE_POINTER [PD_GSL_FUNCTION_STRUCT] assign set_function

	root: REAL_64 assign set_root

	x_lower: REAL_64 assign set_x_lower

	x_upper: REAL_64 assign set_x_upper

	state: CE_POINTER [ANY] assign set_state

feature {ANY} -- field setters

	set_type (a_type: CE_POINTER [PD_GSL_ROOT_FSOLVER_TYPE])
		do
			type := a_type
			update_memory_field ("type")
		end

	set_function (a_function: CE_POINTER [PD_GSL_FUNCTION_STRUCT])
		do
			function := a_function
			update_memory_field ("function")
		end

	set_root (a_root: REAL_64)
		do
			root := a_root
			update_memory_field ("root")
		end

	set_x_lower (a_x_lower: REAL_64)
		do
			x_lower := a_x_lower
			update_memory_field ("x_lower")
		end

	set_x_upper (a_x_upper: REAL_64)
		do
			x_upper := a_x_upper
			update_memory_field ("x_upper")
		end

	set_state (a_state: CE_POINTER [ANY])
		do
			state := a_state
			update_memory_field ("state")
		end

feature {ANY} -- public routines

	gsl_root_fsolver_set (a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_x_lower, a_x_upper: REAL_64): INTEGER_32
		local
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if a_x_lower > a_x_upper then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid interval (lower > upper)"), ce_string ("fsolver.c"), 58, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Current.function := a_f
				update_class_fields -- Current might have been changed; synchronize
				Current.root := 0.5 * (a_x_lower + a_x_upper)
				update_class_fields -- Current might have been changed; synchronize
				Current.x_lower := a_x_lower
				update_class_fields -- Current might have been changed; synchronize
				Current.x_upper := a_x_upper
				update_class_fields -- Current might have been changed; synchronize
				l_tmp := (type.item.set).integer_32_item ([state, function, create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_ROOT_FSOLVER}.make).offset_of ("root")), a_x_lower, a_x_upper])
				update_class_fields -- Current might have been changed; synchronize
				Result := l_tmp
			end
		end

	gsl_root_fsolver_iterate: INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := (type.item.iterate).integer_32_item ([state, function, create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_ROOT_FSOLVER}.make).offset_of ("root")), create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_ROOT_FSOLVER}.make).offset_of ("x_lower")), create {CE_REAL_64_POINTER}.make_shared (c_pointer + (create {PD_GSL_ROOT_FSOLVER}.make).offset_of ("x_upper"))])
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_root_fsolver_name: CE_POINTER [INTEGER_8]
		do
			Result := (type.item.name)
		end

	gsl_root_fsolver_root: REAL_64
		do
			Result := root
		end

	gsl_root_fsolver_x_lower: REAL_64
		do
			Result := x_lower
		end

	gsl_root_fsolver_x_upper: REAL_64
		do
			Result := x_upper
		end

feature {ANY}

	copy (a_other: like Current)
		do
			type := a_other.type
			function := a_other.function
			root := a_other.root
			x_lower := a_other.x_lower
			x_upper := a_other.x_upper
			state := a_other.state
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				type.is_equal (a_other.type) and
				function.is_equal (a_other.function) and
				root = a_other.root and
				x_lower = a_other.x_lower and
				x_upper = a_other.x_upper and
				state.is_equal (a_other.state)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("type"),
					create {CE_STRUCTURE_FIELD}.make ("function"),
					create {CE_STRUCTURE_FIELD}.make ("root"),
					create {CE_STRUCTURE_FIELD}.make ("x_lower"),
					create {CE_STRUCTURE_FIELD}.make ("x_upper"),
					create {CE_STRUCTURE_FIELD}.make ("state")
				>>)
			init_structure_info
		end

end
