class
	PD_GSL_SPLINE

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_STRING_STATIC
		redefine
			copy,
			is_equal
		end
	S_GCC_BUILTIN_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_INTERP]} interp.make
			create {CE_REAL_64_POINTER} x.make
			create {CE_REAL_64_POINTER} y.make
			Precursor
		end

feature {ANY} -- fields

	interp: CE_POINTER [PD_GSL_INTERP] assign set_interp

	x: CE_POINTER [REAL_64] assign set_x

	y: CE_POINTER [REAL_64] assign set_y

	size: NATURAL_32 assign set_size

feature {ANY} -- field setters

	set_interp (a_interp: CE_POINTER [PD_GSL_INTERP])
		do
			interp := a_interp
			update_memory_field ("interp")
		end

	set_x (a_x: CE_POINTER [REAL_64])
		do
			x := a_x
			update_memory_field ("x")
		end

	set_y (a_y: CE_POINTER [REAL_64])
		do
			y := a_y
			update_memory_field ("y")
		end

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

feature {ANY} -- public routines

	gsl_spline_init (a_x_array, a_y_array: CE_POINTER [REAL_64]; a_size: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if a_size /= size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("data must match size of spline object"), ce_string ("spline.c"), 74, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				memcpy (create {CE_VOID_POINTER}.make_cast (x), create {CE_VOID_POINTER}.make_cast (a_x_array), a_size * {PLATFORM}.Real_64_bytes.to_natural_32).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				memcpy (create {CE_VOID_POINTER}.make_cast (y), create {CE_VOID_POINTER}.make_cast (a_y_array), a_size * {PLATFORM}.Real_64_bytes.to_natural_32).do_nothing
				update_class_fields -- Current might have been changed; synchronize
				l_tmp := gsl_interp_init (interp, a_x_array, a_y_array, a_size)
				update_class_fields -- Current might have been changed; synchronize
				l_status := l_tmp
				Result := l_status
			end
		end

	gsl_spline_name: CE_POINTER [INTEGER_8]
		local
			l_tmp: CE_POINTER [INTEGER_8]
		do
			l_tmp := gsl_interp_name (interp)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_min_size: NATURAL_32
		local
			l_tmp: NATURAL_32
		do
			l_tmp := gsl_interp_min_size (interp)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_e (a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_y: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_interp_eval_e (interp, x, y, a_x, a_a___5, a_y)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval (a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_tmp: REAL_64
		do
			l_tmp := gsl_interp_eval (interp, x, y, a_x, a_a___5)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_deriv_e (a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_dydx: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_interp_eval_deriv_e (interp, x, y, a_x, a_a___5, a_dydx)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_deriv (a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_tmp: REAL_64
		do
			l_tmp := gsl_interp_eval_deriv (interp, x, y, a_x, a_a___5)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_deriv2_e (a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_d2: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_interp_eval_deriv2_e (interp, x, y, a_x, a_a___5, a_d2)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_deriv2 (a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_tmp: REAL_64
		do
			l_tmp := gsl_interp_eval_deriv2 (interp, x, y, a_x, a_a___5)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_integ_e (a_a___5, a_b___0: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_interp_eval_integ_e (interp, x, y, a_a___5, a_b___0, a_acc, a_result)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_spline_eval_integ (a_a___5, a_b___0: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_tmp: REAL_64
		do
			l_tmp := gsl_interp_eval_integ (interp, x, y, a_a___5, a_b___0, a_acc)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

feature {PD_GSL_SPLINE} -- private routines

	gsl_interp_init (a_interp: CE_POINTER [PD_GSL_INTERP]; a_x_array, a_y_array: CE_POINTER [REAL_64]; a_size: NATURAL_32): INTEGER_32
		require
			not a_interp.is_void
		local
			l_i: NATURAL_32
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			if a_size /= a_interp.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("data must match size of interpolation object"), ce_string ("interp.c"), 76, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_size))
				loop
					if not (a_x_array[(l_i - {NATURAL_32} 1).to_integer_32] < a_x_array[l_i.to_integer_32]) then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("x values must be monotonically increasing"), ce_string ("interp.c"), 83, 4)
							Result := 4
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if not l_return then
						l_i := l_i + 1
					end
				end
			end
			if not l_return then
				a_interp.item.xmin := a_x_array[0]
				a_interp.item.xmax := a_x_array[(a_size - {NATURAL_32} 1).to_integer_32]
				l_status := (((a_interp.item.type).item.init).integer_32_item ([a_interp.item.state, a_x_array, a_y_array, a_size]))
				Result := l_status
			end
		end

	gsl_interp_name (a_interp: CE_POINTER [PD_GSL_INTERP]): CE_POINTER [INTEGER_8]
		require
			not a_interp.is_void
		do
			Result := ((a_interp.item.type).item.name)
		end

	gsl_interp_min_size (a_interp: CE_POINTER [PD_GSL_INTERP]): NATURAL_32
		require
			not a_interp.is_void
		do
			Result := ((a_interp.item.type).item.min_size)
		end

	gsl_interp_eval_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_y: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_interp.is_void
		local
			l_return: BOOLEAN
		do
			if a_x < a_interp.item.xmin then
				a_y.item := a_ya[0]
				Result := 1
				l_return := True
			elseif a_x > a_interp.item.xmax then
				a_y.item := a_ya[(a_interp.item.size - {NATURAL_32} 1).to_integer_32]
				Result := 1
				l_return := True
			end
			if not l_return then
				Result := (((a_interp.item.type).item.eval).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, a_y]))
			end
		end

	gsl_interp_eval (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		require
			not a_interp.is_void
		local
			l_y: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_y.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, l_y]))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 147, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_y.item)
			end
		end

	gsl_interp_eval_deriv_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_dydx: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_interp.is_void
		local
			l_return: BOOLEAN
		do
			if a_x < a_interp.item.xmin then
				a_dydx.item := 0.0
				Result := 1
				l_return := True
			elseif a_x > a_interp.item.xmax then
				a_dydx.item := 0.0
				Result := 1
				l_return := True
			end
			if not l_return then
				Result := (((a_interp.item.type).item.eval_deriv).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, a_dydx]))
			end
		end

	gsl_interp_eval_deriv (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		require
			not a_interp.is_void
		local
			l_dydx: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_dydx.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval_deriv).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, l_dydx]))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 181, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_dydx.item)
			end
		end

	gsl_interp_eval_deriv2_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_d2: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_interp.is_void
		local
			l_return: BOOLEAN
		do
			if a_x < a_interp.item.xmin then
				a_d2.item := 0.0
				Result := 1
				l_return := True
			elseif a_x > a_interp.item.xmax then
				a_d2.item := 0.0
				Result := 1
				l_return := True
			end
			if not l_return then
				Result := (((a_interp.item.type).item.eval_deriv2).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, a_d2]))
			end
		end

	gsl_interp_eval_deriv2 (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		require
			not a_interp.is_void
		local
			l_d2: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_d2.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval_deriv2).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, l_d2]))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 215, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_d2.item)
			end
		end

	gsl_interp_eval_integ_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_a___5, a_b___0: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if a_a___5 > a_b___0 then
				a_result.item := 0.0
				Result := 1
				l_return := True
			elseif a_a___5 < a_interp.item.xmin then
				a_result.item := 0.0
				Result := 1
				l_return := True
			elseif a_b___0 > a_interp.item.xmax then
				a_result.item := 0.0
				Result := 1
				l_return := True
			elseif a_a___5 = a_b___0 then
				a_result.item := 0.0
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := (((a_interp.item.type).item.eval_integ).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_acc, a_a___5, a_b___0, a_result]))
			end
		end

	gsl_interp_eval_integ (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa, a_ya: CE_POINTER [REAL_64]; a_a___5, a_b___0: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		require
			not a_interp.is_void
		local
			l_result: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			l_return, l_break: BOOLEAN
		do
			create {CE_REAL_64_ARRAY} l_result.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval_integ).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_acc, a_a___5, a_b___0, l_result]))
			if l_status /= 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 252, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := (l_result.item)
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			interp := a_other.interp
			x := a_other.x
			y := a_other.y
			size := a_other.size
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				interp.is_equal (a_other.interp) and
				x.is_equal (a_other.x) and
				y.is_equal (a_other.y) and
				size = a_other.size
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("interp"),
					create {CE_STRUCTURE_FIELD}.make ("x"),
					create {CE_STRUCTURE_FIELD}.make ("y"),
					create {CE_STRUCTURE_FIELD}.make ("size")
				>>)
			init_structure_info
		end

end
