class
	PD_GSL_VECTOR_FLOAT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_32_POINTER} data.make
			create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]} block.make
			Precursor
		end

feature {ANY} -- fields

	size: NATURAL_32 assign set_size

	stride: NATURAL_32 assign set_stride

	data: CE_POINTER [REAL_32] assign set_data

	block: CE_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT] assign set_block

	owner: INTEGER_32 assign set_owner

feature {ANY} -- field setters

	set_size (a_size: NATURAL_32)
		do
			size := a_size
			update_memory_field ("size")
		end

	set_stride (a_stride: NATURAL_32)
		do
			stride := a_stride
			update_memory_field ("stride")
		end

	set_data (a_data: CE_POINTER [REAL_32])
		do
			data := a_data
			update_memory_field ("data")
		end

	set_block (a_block: CE_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT])
		do
			block := a_block
			update_memory_field ("block")
		end

	set_owner (a_owner: INTEGER_32)
		do
			owner := a_owner
			update_memory_field ("owner")
		end

feature {ANY} -- public routines

	gsl_vector_float_get (a_i: NATURAL_32): REAL_32
		do
			Result := (data[(a_i * stride).to_integer_32])
		end

	gsl_vector_float_set (a_i: NATURAL_32; a_x: REAL_32)
		do
			data[(a_i * stride).to_integer_32] := a_x
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_vector_float_ptr (a_i: NATURAL_32): CE_POINTER [REAL_32]
		do
			Result := (data + (a_i * stride).to_integer_32)
		end

	gsl_vector_float_const_ptr (a_i: NATURAL_32): CE_POINTER [REAL_32]
		do
			Result := (data + (a_i * stride).to_integer_32)
		end

	gsl_blas_sdsdot (a_alpha: REAL_32; a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_result: CE_POINTER [REAL_32]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				a_result.item := cblas_sdsdot (size.to_integer_32, a_alpha, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 57, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_dsdot (a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_result: CE_POINTER [REAL_64]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				a_result.item := cblas_dsdot (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 74, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_sdot (a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_result: CE_POINTER [REAL_32]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				a_result.item := cblas_sdot (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 91, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_snrm2: REAL_32
		local
			l_tmp: REAL_32
		do
			l_tmp := cblas_snrm2 (size.to_integer_32, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_blas_sasum: REAL_32
		local
			l_tmp: REAL_32
		do
			l_tmp := cblas_sasum (size.to_integer_32, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_blas_isamax: NATURAL_32
		local
			l_tmp: NATURAL_32
		do
			l_tmp := cblas_isamax (size.to_integer_32, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_blas_sswap (a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_sswap (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 271, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_scopy (a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_scopy (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 334, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_saxpy (a_alpha: REAL_32; a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_saxpy (size.to_integer_32, a_alpha, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 398, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_srot (a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_c___1, a_s: REAL_32): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_srot (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_c___1, a_s)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 479, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_srotm (a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_P___2: CE_POINTER [REAL_32]): INTEGER_32
		require
			not a_Y.is_void
		local
			l_return, l_break: BOOLEAN
		do
			if size = a_Y.item.size then
				cblas_srotm (size.to_integer_32, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_P___2)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
				l_return := True
			else
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 529, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_blas_sscal (a_alpha: REAL_32)
		do
			cblas_sscal (size.to_integer_32, a_alpha, data, stride.to_integer_32)
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_blas_sger (a_alpha: REAL_32; a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_A___0: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_A___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_A___0.item.size1
					l_N := a_A___0.item.size2
					if size /= l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if a_Y.item.size = l_N then
							cblas_sger ({PC_CBLAS_ORDER}.CblasRowMajor, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, stride.to_integer_32, a_Y.item.data, (a_Y.item.stride).to_integer_32, a_A___0.item.data, (a_A___0.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 3 -- goto _L
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 992, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 4 then
					if not l_return then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_view_vector (a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_FLOAT_VIEW
		local
			l_view: PD_GSL_MATRIX_FLOAT_VIEW
			l_m___4: PD_GSL_MATRIX_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 102, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 107, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 112, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_n2 > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 117, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_view_vector_with_tda (a_n1, a_n2, a_tda: NATURAL_32): PD_GSL_MATRIX_FLOAT_VIEW
		local
			l_view: PD_GSL_MATRIX_FLOAT_VIEW
			l_m___4: PD_GSL_MATRIX_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 152, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 157, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 > a_tda then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must not exceed tda"), ce_string ("view_source.c"), 162, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_tda > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 167, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_tda
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_view_vector (a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_MATRIX_FLOAT_CONST_VIEW
			l_m___4: PD_GSL_MATRIX_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 102, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 107, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 112, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_n2 > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 117, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_view_vector_with_tda (a_n1, a_n2, a_tda: NATURAL_32): PD_GSL_MATRIX_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_MATRIX_FLOAT_CONST_VIEW
			l_m___4: PD_GSL_MATRIX_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_m___4.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("view_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("view_source.c"), 152, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif stride /= {NATURAL_32} 1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector must have unit stride"), ce_string ("view_source.c"), 157, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 > a_tda then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must not exceed tda"), ce_string ("view_source.c"), 162, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 * a_tda > size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix size exceeds size of original"), ce_string ("view_source.c"), 167, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_m___4.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.data := data
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.tda := a_tda
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_m___4.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_sort_vector_float_smallest (a_dest: CE_POINTER [REAL_32]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_float_smallest (a_dest, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_sort_vector_float_largest (a_dest: CE_POINTER [REAL_32]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_float_largest (a_dest, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_sort_vector_float
		do
			gsl_sort_float (data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_sort_vector_float_smallest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_float_smallest_index (a_p, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_sort_vector_float_largest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32): INTEGER_32
		local
			l_tmp: INTEGER_32
		do
			l_tmp := gsl_sort_float_largest_index (a_p, a_k, data, stride, size)
			update_class_fields -- Current might have been changed; synchronize
			Result := l_tmp
		end

	gsl_vector_float_isnull: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if (data[(l_stride * l_j + l_k).to_integer_32]).to_double /= 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_float_ispos: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if (data[(l_stride * l_j + l_k).to_integer_32]).to_double <= 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_float_isneg: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if (data[(l_stride * l_j + l_k).to_integer_32]).to_double >= 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_float_isnonneg: INTEGER_32
		local
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_n := size
			l_stride := stride
			l_j := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_j < l_n))
			loop
				l_k := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_k < {NATURAL_32} 1))
				loop
					if (data[(l_stride * l_j + l_k).to_integer_32]).to_double < 0.0 then
						Result := 0
						l_return := True
					end
					if not l_return then
						l_k := l_k + 1
					end
				end
				if not l_return then
					l_j := l_j + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_vector_float_max: REAL_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_max: REAL_32
			l_i: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x > l_max then
					l_max := l_x
				end
				if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x.to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_x
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_max
			end
		end

	gsl_vector_float_min: REAL_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_min: REAL_32
			l_i: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
				end
				if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x.to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_x
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_min
			end
		end

	gsl_vector_float_minmax (a_min_out, a_max_out: CE_POINTER [REAL_32])
		require
			not a_min_out.is_void
			not a_max_out.is_void
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_max: REAL_32
			l_min: REAL_32
			l_i: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_break: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < l_N))
			loop
				l_break := False
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
				end
				if l_x > l_max then
					l_max := l_x
				end
				if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x.to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					l_min := l_x
					l_max := l_x
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			a_min_out.item := l_min
			update_class_fields -- Current might have been changed; synchronize
			a_max_out.item := l_max
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_vector_float_max_index: NATURAL_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_max: REAL_32
			l_imax: NATURAL_32
			l_i: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_imax := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x > l_max then
					l_max := l_x
					l_imax := l_i
				end
				if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x.to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_i
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_imax
			end
		end

	gsl_vector_float_min_index: NATURAL_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_min: REAL_32
			l_imin: NATURAL_32
			l_i: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_imin := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_N))
			loop
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
					l_imin := l_i
				end
				if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x.to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					Result := l_i
					l_return := True
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_imin
			end
		end

	gsl_vector_float_minmax_index (a_imin_out, a_imax_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_imax_out.is_void
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_imin: NATURAL_32
			l_imax: NATURAL_32
			l_max: REAL_32
			l_min: REAL_32
			l_i: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_break: BOOLEAN
		do
			l_N := size
			l_stride := stride
			l_imin := (0).to_natural_32
			l_imax := (0).to_natural_32
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_break) and then (l_i < l_N))
			loop
				l_break := False
				l_x := data[(l_i * l_stride).to_integer_32]
				if l_x < l_min then
					l_min := l_x
					l_imin := l_i
				end
				if l_x > l_max then
					l_max := l_x
					l_imax := l_i
				end
				if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
					l_tmp := isnanf (l_x)
					update_class_fields -- Current might have been changed; synchronize
					l_tmp___3 := l_tmp
				else
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
						l_tmp___0 := isnan (l_x.to_double)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___0
					else
						l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___2 := l_tmp___1
					end
					l_tmp___3 := l_tmp___2
				end
				if l_tmp___3.to_boolean then
					l_imin := l_i
					l_imax := l_i
					l_break := True
				end
				if not (l_break) then
					l_i := l_i + 1
				end
			end
			l_break := False
			a_imin_out.item := l_imin
			update_class_fields -- Current might have been changed; synchronize
			a_imax_out.item := l_imax
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_vector_float_swap (a_w: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_w.is_void
		local
			l_d1: CE_POINTER [REAL_32]
			l_d2: CE_POINTER [REAL_32]
			l_size: NATURAL_32
			l_s1: NATURAL_32
			l_s2: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_d1 := data
			l_d2 := a_w.item.data
			l_size := size
			l_s1 := stride
			l_s2 := a_w.item.stride
			if size /= a_w.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector lengths must be equal"), ce_string ("swap_source.c"), 32, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_size))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_tmp := l_d1[(l_i * l_s1 + l_k).to_integer_32]
						l_d1[(l_i * l_s1 + l_k).to_integer_32] := l_d2[(l_i * l_s2 + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_d2[(l_i * l_s2 + l_k).to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_vector_float_swap_elements (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_data: CE_POINTER [REAL_32]
			l_size: NATURAL_32
			l_stride: NATURAL_32
			l_s: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_data := data
			l_size := size
			l_stride := stride
			if a_i >= l_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first index is out of range"), ce_string ("swap_source.c"), 57, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second index is out of range"), ce_string ("swap_source.c"), 62, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_s := l_stride
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_tmp := l_data[(a_j * l_s + l_k).to_integer_32]
						l_data[(a_j * l_s + l_k).to_integer_32] := l_data[(a_i * l_s + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_data[(a_i * l_s + l_k).to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
				end
				Result := 0
			end
		end

	gsl_vector_float_reverse: INTEGER_32
		local
			l_data: CE_POINTER [REAL_32]
			l_size: NATURAL_32
			l_stride: NATURAL_32
			l_s: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_32
			l_return: BOOLEAN
		do
			l_data := data
			l_size := size
			l_stride := stride
			l_s := l_stride
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size // {NATURAL_32} 2))
			loop
				l_j := (l_size - l_i) - {NATURAL_32} 1
				l_k := (0).to_natural_32
				from
				until
					not (l_k < {NATURAL_32} 1)
				loop
					l_tmp := l_data[(l_j * l_s + l_k).to_integer_32]
					l_data[(l_j * l_s + l_k).to_integer_32] := l_data[(l_i * l_s + l_k).to_integer_32]
					update_class_fields -- Current might have been changed; synchronize
					l_data[(l_i * l_s + l_k).to_integer_32] := l_tmp
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_vector_float_memcpy (a_src: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size: NATURAL_32
			l_dest_size: NATURAL_32
			l_src_stride: NATURAL_32
			l_dest_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_src_size := a_src.item.size
			l_dest_size := size
			if l_src_size /= l_dest_size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector lengths are not equal"), ce_string ("copy_source.c"), 29, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_src_stride := a_src.item.stride
				l_dest_stride := stride
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_src_size))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						data[(l_dest_stride * l_j + l_k).to_integer_32] := a_src.item.data[(l_src_stride * l_j + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_vector_float_add (a_b___0: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 27, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					data[(l_i * l_stride_a).to_integer_32] := data[(l_i * l_stride_a).to_integer_32] + (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_vector_float_sub (a_b___0: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 52, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					data[(l_i * l_stride_a).to_integer_32] := data[(l_i * l_stride_a).to_integer_32] - (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_vector_float_mul (a_b___0: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 77, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					data[(l_i * l_stride_a).to_integer_32] := data[(l_i * l_stride_a).to_integer_32] * (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_vector_float_div (a_b___0: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_N: NATURAL_32
			l_stride_a: NATURAL_32
			l_stride_b: NATURAL_32
			l_i: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_N := size
			if a_b___0.item.size /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vectors must have same length"), ce_string ("oper_source.c"), 102, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			else
				l_stride_a := stride
				l_stride_b := a_b___0.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_N)
				loop
					data[(l_i * l_stride_a).to_integer_32] := data[(l_i * l_stride_a).to_integer_32] / (a_b___0.item.data[(l_i * l_stride_b).to_integer_32])
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + 1
				end
				Result := 0
				l_return := True
			end
			if not l_return then
				Result := 0
			end
		end

	gsl_vector_float_scale (a_x: REAL_64): INTEGER_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_i: NATURAL_32
		do
			l_N := size
			l_stride := stride
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				data[(l_i * l_stride).to_integer_32] := ((data[(l_i * l_stride).to_integer_32]).to_double * a_x).truncated_to_real
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_vector_float_add_constant (a_x: REAL_64): INTEGER_32
		local
			l_N: NATURAL_32
			l_stride: NATURAL_32
			l_i: NATURAL_32
		do
			l_N := size
			l_stride := stride
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				data[(l_i * l_stride).to_integer_32] := ((data[(l_i * l_stride).to_integer_32]).to_double + a_x).truncated_to_real
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_vector_float_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_float_raw_fread (a_stream, data, size, stride)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_float_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_float_raw_fwrite (a_stream, data, size, stride)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_float_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_float_raw_fprintf (a_stream, data, size, stride, a_format)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_float_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_tmp: INTEGER_32
		do
			l_tmp := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_float_raw_fscanf (a_stream, data, size, stride)
			update_class_fields -- Current might have been changed; synchronize
			l_status := l_tmp
			Result := l_status
		end

	gsl_vector_float_subvector (a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_s: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 28, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 34, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := data + (stride * a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := stride
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_float_subvector_with_stride (a_offset, a_stride, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_s: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 59, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("subvector_source.c"), 65, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) * a_stride >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 71, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := data + (stride * a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := stride * a_stride
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_float_const_subvector (a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_s: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 28, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 34, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := (data + (stride * a_offset).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := stride
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_float_const_subvector_with_stride (a_offset, a_stride, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_s: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("subvector_source.c"), 59, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("subvector_source.c"), 65, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) * a_stride >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("view would extend past end of vector"), ce_string ("subvector_source.c"), 71, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := (data + (stride * a_offset).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_s.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_s.stride := (stride * a_stride)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_vector_float_alloc_from_vector (a_offset, a_n, a_stride: NATURAL_32): CE_POINTER [PD_GSL_VECTOR_FLOAT]
		local
			l_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("init_source.c"), 131, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_stride = {NATURAL_32} 0) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("stride must be positive integer"), ce_string ("init_source.c"), 136, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_offset + (a_n - {NATURAL_32} 1) * a_stride >= size) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector would extend past end of block"), ce_string ("init_source.c"), 141, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_VECTOR_FLOAT}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v := create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_cast (l_tmp)
				if l_v.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vector struct"), ce_string ("init_source.c"), 149, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_v.item.data := data + (stride * a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.stride := a_stride * stride
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				Result := l_v
			end
		end

	gsl_vector_float_set_all (a_x: REAL_32)
		local
			l_data: CE_POINTER [REAL_32]
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_i: NATURAL_32
		do
			l_data := data
			l_n := size
			l_stride := stride
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_data[(l_i * l_stride).to_integer_32] := a_x
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_vector_float_set_zero
		local
			l_data: CE_POINTER [REAL_32]
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_zero: REAL_32
			l_i: NATURAL_32
		do
			l_data := data
			l_n := size
			l_stride := stride
			l_zero := {REAL_32} 0.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_n)
			loop
				l_data[(l_i * l_stride).to_integer_32] := l_zero
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

	gsl_vector_float_set_basis (a_i: NATURAL_32): INTEGER_32
		local
			l_data: CE_POINTER [REAL_32]
			l_n: NATURAL_32
			l_stride: NATURAL_32
			l_zero: REAL_32
			l_one: REAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_data := data
			l_n := size
			l_stride := stride
			l_zero := {REAL_32} 0.0
			l_one := {REAL_32} 1.0
			if a_i >= l_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("index out of range"), ce_string ("init_source.c"), 219, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_k := (0).to_natural_32
				from
				until
					not (l_k < l_n)
				loop
					l_data[(l_k * l_stride).to_integer_32] := l_zero
					update_class_fields -- Current might have been changed; synchronize
					l_k := l_k + 1
				end
				l_data[(a_i * l_stride).to_integer_32] := l_one
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

feature {PD_GSL_VECTOR_FLOAT} -- private routines

	gsl_sort_float_smallest (a_dest: CE_POINTER [REAL_32]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_32
			l_i1: NATURAL_32
			l_xi: REAL_32
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subset_source.c"), 28, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_dest[0] := l_xbound
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi >= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi > a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_dest[l_i1.to_integer_32] := a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_dest[l_i1.to_integer_32] := l_xi
						l_xbound := a_dest[(l_j - {NATURAL_32} 1).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_sort_float_largest (a_dest: CE_POINTER [REAL_32]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_32
			l_i1: NATURAL_32
			l_xi: REAL_32
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subset_source.c"), 93, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_dest[0] := l_xbound
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi <= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi < a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_dest[l_i1.to_integer_32] := a_dest[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_dest[l_i1.to_integer_32] := l_xi
						l_xbound := a_dest[(l_j - {NATURAL_32} 1).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_sort_float (a_data: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32)
		local
			l_N: NATURAL_32
			l_k: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			if a_n = {NATURAL_32} 0 then
				l_return := True
			end
			if not l_return then
				l_N := a_n - {NATURAL_32} 1
				l_k := l_N // {NATURAL_32} 2
				l_k := l_k + 1
				from
				until
					(l_break)
				loop
					l_break := False
					l_k := l_k - 1
					my_float_downheap (a_data, a_stride, l_N, l_k)
					if not (l_k > {NATURAL_32} 0) then
						l_break := True
					end
				end
				l_break := False
				from
				until
					not (l_N > {NATURAL_32} 0)
				loop
					l_tmp := a_data[({NATURAL_32} 0).to_integer_32]
					a_data[({NATURAL_32} 0).to_integer_32] := a_data[(l_N * a_stride).to_integer_32]
					a_data[(l_N * a_stride).to_integer_32] := l_tmp
					l_N := l_N - 1
					my_float_downheap (a_data, a_stride, l_N, (0).to_natural_32)
				end
			end
		end

	my_float_downheap (a_data: CE_POINTER [REAL_32]; a_stride, a_N, a_k: NATURAL_32)
		require
			not a_data.is_void
		local
			l_k: NATURAL_32
			l_v: REAL_32
			l_j: NATURAL_32
			l_break: BOOLEAN
		do
			l_k := a_k
			l_v := a_data[(l_k * a_stride).to_integer_32]
			from
			until
				not (not (l_break) and then (l_k <= a_N // {NATURAL_32} 2))
			loop
				l_break := False
				l_j := {NATURAL_32} 2 * l_k
				if l_j < a_N then
					if a_data[(l_j * a_stride).to_integer_32] < a_data[((l_j + {NATURAL_32} 1) * a_stride).to_integer_32] then
						l_j := l_j + 1
					end
				end
				if not (l_v < a_data[(l_j * a_stride).to_integer_32]) then
					l_break := True
				end
				if not (l_break) then
					a_data[(l_k * a_stride).to_integer_32] := a_data[(l_j * a_stride).to_integer_32]
					l_k := l_j
				end
			end
			l_break := False
			a_data[(l_k * a_stride).to_integer_32] := l_v
		end

	gsl_sort_float_smallest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_32
			l_i1: NATURAL_32
			l_xi: REAL_32
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subsetind_source.c"), 28, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_p[0] := (0).to_natural_32
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi >= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi > a_src[(a_p[(l_i1 - {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_p[l_i1.to_integer_32] := a_p[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_p[l_i1.to_integer_32] := l_i
						l_xbound := a_src[(a_p[(l_j - {NATURAL_32} 1).to_integer_32] * a_stride).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	gsl_sort_float_largest_index (a_p: CE_POINTER [NATURAL_32]; a_k: NATURAL_32; a_src: CE_POINTER [REAL_32]; a_stride, a_n: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_xbound: REAL_32
			l_i1: NATURAL_32
			l_xi: REAL_32
			l_return, l_break, l_continue: BOOLEAN
		do
			if a_k > a_n then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subset length k exceeds vector length n"), ce_string ("subsetind_source.c"), 93, 4)
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_k = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				elseif a_n = {NATURAL_32} 0 then
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_j := (1).to_natural_32
				l_xbound := a_src[({NATURAL_32} 0).to_integer_32]
				a_p[0] := (0).to_natural_32
				l_i := (1).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_continue := False
					l_xi := a_src[(l_i * a_stride).to_integer_32]
					if l_j < a_k then
						l_j := l_j + 1
					elseif l_xi <= l_xbound then
						l_continue := True
					end
					if not (l_return or l_break or l_continue) then
						l_i1 := l_j - {NATURAL_32} 1
						from
						until
							not (not (l_break) and then (l_i1 > {NATURAL_32} 0))
						loop
							l_break := False
							if l_xi < a_src[(a_stride * a_p[(l_i1 - {NATURAL_32} 1).to_integer_32]).to_integer_32] then
								l_break := True
							end
							if not (l_return or l_break) then
								a_p[l_i1.to_integer_32] := a_p[(l_i1 - {NATURAL_32} 1).to_integer_32]
								l_i1 := l_i1 - 1
							end
						end
						l_break := False
						a_p[l_i1.to_integer_32] := l_i
						l_xbound := a_src[(a_stride * a_p[(l_j - {NATURAL_32} 1).to_integer_32]).to_integer_32]
					end
					if (not l_return) then
						l_i := l_i + 1
					end
				end
				l_continue := False
				Result := 0
			end
		end

	cblas_sasum (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32): REAL_32
		local
			l_r___3: REAL_32
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_return: BOOLEAN
		do
			l_r___3 := (0.0).truncated_to_real
			l_ix := 0
			if a_incX <= 0 then
				Result := (0).to_real
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					l_r___3 := (l_r___3.to_double + (fabs ((a_X[l_ix]).to_double))).truncated_to_real
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
				Result := l_r___3
			end
		end

	cblas_saxpy (a_N: INTEGER_32; a_alpha: REAL_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_m___4: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha.to_double = 0.0 then
						l_return := True
					end
					if l_return then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if a_incX /= 1 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_incY = 1 then
							l_m___4 := a_N \\ 4
							l_i := 0
							from
							until
								not (l_i < l_m___4)
							loop
								a_Y[l_i] := a_Y[l_i] + (a_alpha * a_X[l_i])
								l_i := l_i + 1
							end
							l_i := l_m___4
							from
							until
								not (l_i + 3 < a_N)
							loop
								a_Y[l_i] := a_Y[l_i] + (a_alpha * a_X[l_i])
								a_Y[(l_i + 1)] := a_Y[(l_i + 1)] + (a_alpha * a_X[(l_i + 1)])
								a_Y[(l_i + 2)] := a_Y[(l_i + 2)] + (a_alpha * a_X[(l_i + 2)])
								a_Y[(l_i + 3)] := a_Y[(l_i + 3)] + (a_alpha * a_X[(l_i + 3)])
								l_i := l_i + (4)
							end
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if a_incX > 0 then
							l_tmp := 0
						else
							l_tmp := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp
						if a_incY > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incY
						end
						l_iy := l_tmp___0
						l_i := 0
						from
						until
							not (l_i < a_N)
						loop
							a_Y[l_iy] := a_Y[l_iy] + (a_alpha * a_X[l_ix])
							l_ix := l_ix + a_incX
							l_iy := l_iy + a_incY
							l_i := l_i + 1
						end
				when 5 then -- if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_scopy (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				a_Y[l_iy] := a_X[l_ix]
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
		end

	cblas_sdot (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32): REAL_32
		local
			l_r___3: REAL_32
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			l_r___3 := (0.0).truncated_to_real
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_r___3 := l_r___3 + (a_X[l_ix] * a_Y[l_iy])
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
			Result := l_r___3
		end

	cblas_sdsdot (a_N: INTEGER_32; a_alpha: REAL_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32): REAL_32
		local
			l_r___3: REAL_64
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			l_r___3 := a_alpha.to_double
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_r___3 := l_r___3 + (a_X[l_ix] * a_Y[l_iy]).to_double
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
			Result := l_r___3.truncated_to_real
		end

	cblas_sger (a_order: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_tmp___0: REAL_32
			l_jy: INTEGER_32
			l_tmp___1: INTEGER_32
			l_jy___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: REAL_32
			l_ix___0: INTEGER_32
			l_tmp___4: INTEGER_32
			l_return: BOOLEAN
		do
			if a_order = {NATURAL_32} 101 then
				if a_incX > 0 then
					l_tmp := 0
				else
					l_tmp := (a_M - 1) * -a_incX
				end
				l_ix := l_tmp
				l_i := 0
				from
				until
					not (not (l_return) and then (l_i < a_M))
				loop
					l_tmp___0 := a_alpha * a_X[l_ix]
					if a_incY > 0 then
						l_tmp___1 := 0
					else
						l_tmp___1 := (a_N - 1) * -a_incY
					end
					l_jy := l_tmp___1
					l_j := 0
					from
					until
						not (l_j < a_N)
					loop
						a_A___0[(a_lda * l_i + l_j)] := a_A___0[(a_lda * l_i + l_j)] + (a_Y[l_jy] * l_tmp___0)
						l_jy := l_jy + a_incY
						l_j := l_j + 1
					end
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
			elseif a_order = {NATURAL_32} 102 then
				if a_incY > 0 then
					l_tmp___2 := 0
				else
					l_tmp___2 := (a_N - 1) * -a_incY
				end
				l_jy___0 := l_tmp___2
				l_j := 0
				from
				until
					not (not (l_return) and then (l_j < a_N))
				loop
					l_tmp___3 := a_alpha * a_Y[l_jy___0]
					if a_incX > 0 then
						l_tmp___4 := 0
					else
						l_tmp___4 := (a_M - 1) * -a_incX
					end
					l_ix___0 := l_tmp___4
					l_i := 0
					from
					until
						not (l_i < a_M)
					loop
						a_A___0[(l_i + a_lda * l_j)] := a_A___0[(l_i + a_lda * l_j)] + (a_X[l_ix___0] * l_tmp___3)
						l_ix___0 := l_ix___0 + a_incX
						l_i := l_i + 1
					end
					l_jy___0 := l_jy___0 + a_incY
					l_j := l_j + 1
				end
			else
				(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_ger.h"), ce_string ("unrecognized operation")])
			end
		end

	cblas_snrm2 (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32): REAL_32
		local
			l_scale: REAL_32
			l_ssq: REAL_32
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_x: REAL_32
			l_ax: REAL_32
			l_return: BOOLEAN
		do
			l_scale := (0.0).truncated_to_real
			l_ssq := (1.0).truncated_to_real
			l_ix := 0
			if a_N <= 0 then
				Result := (0).to_real
				l_return := True
			elseif a_incX <= 0 then
				Result := (0).to_real
				l_return := True
			elseif a_N = 1 then
				Result := (fabs ((a_X[0]).to_double)).truncated_to_real
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					l_x := a_X[l_ix]
					if l_x.to_double /= 0.0 then
						l_ax := (fabs (l_x.to_double)).truncated_to_real
						if l_scale < l_ax then
							l_ssq := (1.0 + ((l_ssq * (l_scale / l_ax)) * (l_scale / l_ax)).to_double).truncated_to_real
							l_scale := l_ax
						else
							l_ssq := l_ssq + ((l_ax / l_scale) * (l_ax / l_scale))
						end
					end
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
				Result := (l_scale.to_double * (sqrt (l_ssq.to_double))).truncated_to_real
			end
		end

	cblas_srot (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32; a_c___1, a_s: REAL_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_x: REAL_32
			l_y: REAL_32
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_x := a_X[l_ix]
				l_y := a_Y[l_iy]
				a_X[l_ix] := a_c___1 * l_x + a_s * l_y
				a_Y[l_iy] := -a_s * l_x + a_c___1 * l_y
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
		end

	cblas_srotm (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32; a_P___2: CE_POINTER [REAL_32])
		require
			not a_P___2.is_void
		local
			l_n: INTEGER_32
			l_i: INTEGER_32
			l_tmp: INTEGER_32
			l_j: INTEGER_32
			l_tmp___0: INTEGER_32
			l_h11: REAL_32
			l_h21: REAL_32
			l_h12: REAL_32
			l_h22: REAL_32
			l_w: REAL_32
			l_z: REAL_32
			l_return: BOOLEAN
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_i := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_j := l_tmp___0
			if (a_P___2[0]).to_double = -1.0 then
				l_h11 := a_P___2[1]
				l_h21 := a_P___2[2]
				l_h12 := a_P___2[3]
				l_h22 := a_P___2[4]
			elseif (a_P___2[0]).to_double = 0.0 then
				l_h11 := (1.0).truncated_to_real
				l_h21 := a_P___2[2]
				l_h12 := a_P___2[3]
				l_h22 := (1.0).truncated_to_real
			elseif (a_P___2[0]).to_double = 1.0 then
				l_h11 := a_P___2[1]
				l_h21 := (-1.0).truncated_to_real
				l_h12 := (1.0).truncated_to_real
				l_h22 := a_P___2[4]
			elseif (a_P___2[0]).to_double = -2.0 then
				l_return := True
			else
				(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_rotm.h"), ce_string ("unrecognized value of P[0]")])
				l_return := True
			end
			if not l_return then
				l_n := 0
				from
				until
					not (l_n < a_N)
				loop
					l_w := a_X[l_i]
					l_z := a_Y[l_j]
					a_X[l_i] := l_h11 * l_w + l_h12 * l_z
					a_Y[l_j] := l_h21 * l_w + l_h22 * l_z
					l_i := l_i + a_incX
					l_j := l_j + a_incY
					l_n := l_n + 1
				end
			end
		end

	cblas_sscal (a_N: INTEGER_32; a_alpha: REAL_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_return: BOOLEAN
		do
			l_ix := 0
			if a_incX <= 0 then
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					a_X[l_ix] := a_X[l_ix] * a_alpha
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
			end
		end

	cblas_sswap (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: REAL_32
		do
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_tmp___1 := a_X[l_ix]
				a_X[l_ix] := a_Y[l_iy]
				a_Y[l_iy] := l_tmp___1
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
		end

	cblas_dsdot (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32): REAL_64
		local
			l_r___3: REAL_64
			l_i: INTEGER_32
			l_ix: INTEGER_32
			l_tmp: INTEGER_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			l_r___3 := 0.0
			if a_incX > 0 then
				l_tmp := 0
			else
				l_tmp := (a_N - 1) * -a_incX
			end
			l_ix := l_tmp
			if a_incY > 0 then
				l_tmp___0 := 0
			else
				l_tmp___0 := (a_N - 1) * -a_incY
			end
			l_iy := l_tmp___0
			l_i := 0
			from
			until
				not (l_i < a_N)
			loop
				l_r___3 := l_r___3 + (a_X[l_ix] * a_Y[l_iy]).to_double
				l_ix := l_ix + a_incX
				l_iy := l_iy + a_incY
				l_i := l_i + 1
			end
			Result := l_r___3
		end

	cblas_isamax (a_N: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32): NATURAL_32
		local
			l_max: REAL_32
			l_ix: INTEGER_32
			l_i: INTEGER_32
			l_result: NATURAL_32
			l_return: BOOLEAN
		do
			l_max := (0.0).truncated_to_real
			l_ix := 0
			l_result := (0).to_natural_32
			if a_incX <= 0 then
				Result := (0).to_natural_32
				l_return := True
			end
			if not l_return then
				l_i := 0
				from
				until
					not (l_i < a_N)
				loop
					if (fabs ((a_X[l_ix]).to_double)) > l_max.to_double then
						l_max := (fabs ((a_X[l_ix]).to_double)).truncated_to_real
						l_result := l_i.to_natural_32
					end
					l_ix := l_ix + a_incX
					l_i := l_i + 1
				end
				Result := l_result
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size := a_other.size
			stride := a_other.stride
			data := a_other.data
			block := a_other.block
			owner := a_other.owner
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size = a_other.size and
				stride = a_other.stride and
				data.is_equal (a_other.data) and
				block.is_equal (a_other.block) and
				owner = a_other.owner
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size"),
					create {CE_STRUCTURE_FIELD}.make ("stride"),
					create {CE_STRUCTURE_FIELD}.make ("data"),
					create {CE_STRUCTURE_FIELD}.make ("block"),
					create {CE_STRUCTURE_FIELD}.make ("owner")
				>>)
			init_structure_info
		end

end
