class
	PD_GSL_WAVELET

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_CLASS_POINTER [PD_GSL_WAVELET_TYPE]} type.make
			create {CE_REAL_64_POINTER} h1.make
			create {CE_REAL_64_POINTER} g1.make
			create {CE_REAL_64_POINTER} h2.make
			create {CE_REAL_64_POINTER} g2.make
		end

feature {ANY} -- fields

	type: CE_POINTER [PD_GSL_WAVELET_TYPE] assign set_type

	h1: CE_POINTER [REAL_64] assign set_h1

	g1: CE_POINTER [REAL_64] assign set_g1

	h2: CE_POINTER [REAL_64] assign set_h2

	g2: CE_POINTER [REAL_64] assign set_g2

	nc: NATURAL_32 assign set_nc

	offset: NATURAL_32 assign set_offset

feature {ANY} -- field setters

	set_type (a_type: CE_POINTER [PD_GSL_WAVELET_TYPE])
		do
			type := a_type
			update_memory_field ("type")
		end

	set_h1 (a_h1: CE_POINTER [REAL_64])
		do
			h1 := a_h1
			update_memory_field ("h1")
		end

	set_g1 (a_g1: CE_POINTER [REAL_64])
		do
			g1 := a_g1
			update_memory_field ("g1")
		end

	set_h2 (a_h2: CE_POINTER [REAL_64])
		do
			h2 := a_h2
			update_memory_field ("h2")
		end

	set_g2 (a_g2: CE_POINTER [REAL_64])
		do
			g2 := a_g2
			update_memory_field ("g2")
		end

	set_nc (a_nc: NATURAL_32)
		do
			nc := a_nc
			update_memory_field ("nc")
		end

	set_offset (a_offset: NATURAL_32)
		do
			offset := a_offset
			update_memory_field ("offset")
		end

feature {ANY} -- public routines

	gsl_wavelet_name: CE_POINTER [INTEGER_8]
		do
			Result := (type.item.name)
		end

	dwt_step (a_a___5: CE_POINTER [REAL_64]; a_stride, a_n: NATURAL_32; a_dir: INTEGER_32; a_work: CE_POINTER [PD_GSL_WAVELET_WORKSPACE])
		require
			not a_work.is_void
		local
			l_ai: REAL_64
			l_ai1: REAL_64
			l_i: NATURAL_32
			l_ii: NATURAL_32
			l_jf: NATURAL_32
			l_k: NATURAL_32
			l_n1: NATURAL_32
			l_ni: NATURAL_32
			l_nh: NATURAL_32
			l_nmod: NATURAL_32
			l_h: REAL_64
			l_g: REAL_64
			l_return: BOOLEAN
		do
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_work.item.n)
			loop
				a_work.item.scratch[l_i.to_integer_32] := 0.0
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			l_nmod := (nc * a_n)
			l_nmod := l_nmod - offset
			l_n1 := a_n - {NATURAL_32} 1
			l_nh := (a_n).bit_shift_right (1)
			if a_dir = 1 then
				l_ii := (0).to_natural_32
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_h := (0).to_double
					l_g := (0).to_double
					l_ni := l_i + l_nmod
					l_k := (0).to_natural_32
					from
					until
						not (l_k < nc)
					loop
						l_jf := (l_n1).bit_and ((l_ni + l_k))
						l_h := l_h + (h1[l_k.to_integer_32] * a_a___5[(a_stride * l_jf).to_integer_32])
						l_g := l_g + (g1[l_k.to_integer_32] * a_a___5[(a_stride * l_jf).to_integer_32])
						l_k := l_k + 1
					end
					a_work.item.scratch[l_ii.to_integer_32] := a_work.item.scratch[l_ii.to_integer_32] + l_h
					update_class_fields -- Current might have been changed; synchronize
					a_work.item.scratch[(l_ii + l_nh).to_integer_32] := a_work.item.scratch[(l_ii + l_nh).to_integer_32] + l_g
					update_class_fields -- Current might have been changed; synchronize
					l_i := l_i + ({NATURAL_32} 2)
					l_ii := l_ii + 1
				end
			else
				l_ii := (0).to_natural_32
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < a_n))
				loop
					l_ai := a_a___5[(a_stride * l_ii).to_integer_32]
					l_ai1 := a_a___5[(a_stride * (l_ii + l_nh)).to_integer_32]
					l_ni := l_i + l_nmod
					l_k := (0).to_natural_32
					from
					until
						not (l_k < nc)
					loop
						l_jf := (l_n1).bit_and ((l_ni + l_k))
						a_work.item.scratch[l_jf.to_integer_32] := a_work.item.scratch[l_jf.to_integer_32] + (h2[l_k.to_integer_32] * l_ai + g2[l_k.to_integer_32] * l_ai1)
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_i := l_i + ({NATURAL_32} 2)
					l_ii := l_ii + 1
				end
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				a_a___5[(a_stride * l_i).to_integer_32] := a_work.item.scratch[l_i.to_integer_32]
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			type := a_other.type
			h1 := a_other.h1
			g1 := a_other.g1
			h2 := a_other.h2
			g2 := a_other.g2
			nc := a_other.nc
			offset := a_other.offset
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				type.is_equal (a_other.type) and
				h1.is_equal (a_other.h1) and
				g1.is_equal (a_other.g1) and
				h2.is_equal (a_other.h2) and
				g2.is_equal (a_other.g2) and
				nc = a_other.nc and
				offset = a_other.offset
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("type"),
					create {CE_STRUCTURE_FIELD}.make ("h1"),
					create {CE_STRUCTURE_FIELD}.make ("g1"),
					create {CE_STRUCTURE_FIELD}.make ("h2"),
					create {CE_STRUCTURE_FIELD}.make ("g2"),
					create {CE_STRUCTURE_FIELD}.make ("nc"),
					create {CE_STRUCTURE_FIELD}.make ("offset")
				>>)
			init_structure_info
		end

end
