class
	PD_GSL_MATRIX_FLOAT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end
	S_MATH_STATIC
		redefine
			copy,
			is_equal
		end
	S_STDLIB_STATIC
		redefine
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			create {CE_REAL_32_POINTER} data.make
			create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]} block.make
			Precursor
		end

feature {ANY} -- fields

	size1: NATURAL_32 assign set_size1

	size2: NATURAL_32 assign set_size2

	tda: NATURAL_32 assign set_tda

	data: CE_POINTER [REAL_32] assign set_data

	block: CE_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT] assign set_block

	owner: INTEGER_32 assign set_owner

feature {ANY} -- field setters

	set_size1 (a_size1: NATURAL_32)
		do
			size1 := a_size1
			update_memory_field ("size1")
		end

	set_size2 (a_size2: NATURAL_32)
		do
			size2 := a_size2
			update_memory_field ("size2")
		end

	set_tda (a_tda: NATURAL_32)
		do
			tda := a_tda
			update_memory_field ("tda")
		end

	set_data (a_data: CE_POINTER [REAL_32])
		do
			data := a_data
			update_memory_field ("data")
		end

	set_block (a_block: CE_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT])
		do
			block := a_block
			update_memory_field ("block")
		end

	set_owner (a_owner: INTEGER_32)
		do
			owner := a_owner
			update_memory_field ("owner")
		end

feature {ANY} -- public routines

	gsl_matrix_float_get (a_i, a_j: NATURAL_32): REAL_32
		do
			Result := (data[(a_i * tda + a_j).to_integer_32])
		end

	gsl_matrix_float_set (a_i, a_j: NATURAL_32; a_x: REAL_32)
		do
			data[(a_i * tda + a_j).to_integer_32] := a_x
			update_class_fields -- Current might have been changed; synchronize
		end

	gsl_matrix_float_ptr (a_i, a_j: NATURAL_32): CE_POINTER [REAL_32]
		do
			Result := (data + (a_i * tda + a_j).to_integer_32)
		end

	gsl_matrix_float_const_ptr (a_i, a_j: NATURAL_32): CE_POINTER [REAL_32]
		do
			Result := (data + (a_i * tda + a_j).to_integer_32)
		end

	gsl_blas_sgemv (a_TransA: NATURAL_32; a_alpha: REAL_32; a_X: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_beta: REAL_32; a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_TransA /= {NATURAL_32} 111 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_N = a_X.item.size then
							if l_M = a_Y.item.size then
								cblas_sgemv ({PC_CBLAS_ORDER}.CblasRowMajor, a_TransA, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32, a_beta, a_Y.item.data, (a_Y.item.stride).to_integer_32)
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							else
								l_pos := -1 + 3 -- goto _L___2
							end
						else
							l_pos := -1 + 3 -- goto _L___2
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else: | _L___2:
						if a_TransA /= {NATURAL_32} 112 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_M = a_X.item.size then
							if l_N = a_Y.item.size then
								cblas_sgemv ({PC_CBLAS_ORDER}.CblasRowMajor, a_TransA, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32, a_beta, a_Y.item.data, (a_Y.item.stride).to_integer_32)
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							else
								l_pos := -1 + 6 -- goto _L___0
							end
						else
							l_pos := -1 + 6 -- goto _L___0
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 614, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_ssymv (a_Uplo: NATURAL_32; a_alpha: REAL_32; a_X: CE_POINTER [PD_GSL_VECTOR_FLOAT]; a_beta: REAL_32; a_Y: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 754, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 3 then -- else:
						if l_N = a_X.item.size then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if l_N = a_Y.item.size then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 758, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						cblas_ssymv ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32, a_beta, a_Y.item.data, (a_Y.item.stride).to_integer_32)
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_strmv (a_Uplo, a_TransA, a_Diag: NATURAL_32; a_X: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 800, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_X.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 804, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				cblas_strmv ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_TransA, a_Diag, l_N.to_integer_32, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_blas_strsv (a_Uplo, a_TransA, a_Diag: NATURAL_32; a_X: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square"), ce_string ("blas.c"), 893, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= a_X.item.size then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 897, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				cblas_strsv ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_TransA, a_Diag, l_N.to_integer_32, data, tda.to_integer_32, a_X.item.data, (a_X.item.stride).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_blas_sgemm (a_TransA, a_TransB: NATURAL_32; a_alpha: REAL_32; a_B: CE_POINTER [PD_GSL_MATRIX_FLOAT]; a_beta: REAL_32; a_C: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_B.is_void
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_tmp: NATURAL_32
			l_NA: NATURAL_32
			l_tmp___0: NATURAL_32
			l_MB: NATURAL_32
			l_tmp___1: NATURAL_32
			l_NB: NATURAL_32
			l_tmp___2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_C.item.size1
					l_N := a_C.item.size2
					if a_TransA = {NATURAL_32} 111 then
						l_tmp := size1
					else
						l_tmp := size2
					end
					l_MA := l_tmp
					if a_TransA = {NATURAL_32} 111 then
						l_tmp___0 := size2
					else
						l_tmp___0 := size1
					end
					l_NA := l_tmp___0
					if a_TransB = {NATURAL_32} 111 then
						l_tmp___1 := a_B.item.size1
					else
						l_tmp___1 := a_B.item.size2
					end
					l_MB := l_tmp___1
					if a_TransB = {NATURAL_32} 111 then
						l_tmp___2 := a_B.item.size2
					else
						l_tmp___2 := a_B.item.size1
					end
					l_NB := l_tmp___2
					if l_M /= l_MA then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_N = l_NB then
							if l_NA = l_MB then
								cblas_sgemm ({PC_CBLAS_ORDER}.CblasRowMajor, a_TransA, a_TransB, l_M.to_integer_32, l_N.to_integer_32, l_NA.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
								update_class_fields -- Current might have been changed; synchronize
								Result := 0
								l_return := True
							else
								l_pos := -1 + 3 -- goto _L___0
							end
						else
							l_pos := -1 + 3 -- goto _L___0
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 4
				when 3 then -- else: | _L___0:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_328, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 4 then
					if not l_return then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_ssymm (a_Side, a_Uplo: NATURAL_32; a_alpha: REAL_32; a_B: CE_POINTER [PD_GSL_MATRIX_FLOAT]; a_beta: REAL_32; a_C: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_B.is_void
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_NA: NATURAL_32
			l_MB: NATURAL_32
			l_NB: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_C.item.size1
					l_N := a_C.item.size2
					l_MA := size1
					l_NA := size2
					l_MB := a_B.item.size1
					l_NB := a_B.item.size2
					if l_MA /= l_NA then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("blas.c"), 1_432, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if a_Side /= {NATURAL_32} 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_M = l_MA then
							if l_N = l_NB then
								if l_NA = l_MB then
									cblas_ssymm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
									update_class_fields -- Current might have been changed; synchronize
									Result := 0
									l_return := True
								else
									l_pos := -1 + 4 -- goto _L___0
								end
							else
								l_pos := -1 + 4 -- goto _L___0
							end
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else: | _L___0:
						if a_Side /= {NATURAL_32} 142 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_M = l_MB then
							if l_N = l_NA then
								if l_NB = l_MA then
									cblas_ssymm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
									update_class_fields -- Current might have been changed; synchronize
									Result := 0
									l_return := True
								else
									l_pos := -1 + 7 -- goto _L
								end
							else
								l_pos := -1 + 7 -- goto _L
							end
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_445, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_ssyrk (a_Uplo, a_Trans: NATURAL_32; a_alpha, a_beta: REAL_32; a_C: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_J: NATURAL_32
			l_tmp: NATURAL_32
			l_K: NATURAL_32
			l_tmp___0: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := a_C.item.size1
			l_N := a_C.item.size2
			if a_Trans = {NATURAL_32} 111 then
				l_tmp := size1
			else
				l_tmp := size2
			end
			l_J := l_tmp
			if a_Trans = {NATURAL_32} 111 then
				l_tmp___0 := size2
			else
				l_tmp___0 := size1
			end
			l_K := l_tmp___0
			if l_M /= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix C must be square"), ce_string ("blas.c"), 1_635, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif l_N /= l_J then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_639, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				cblas_ssyrk ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_Trans, l_N.to_integer_32, l_K.to_integer_32, a_alpha, data, tda.to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				Result := 0
			end
		end

	gsl_blas_ssyr2k (a_Uplo, a_Trans: NATURAL_32; a_alpha: REAL_32; a_B: CE_POINTER [PD_GSL_MATRIX_FLOAT]; a_beta: REAL_32; a_C: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_B.is_void
			not a_C.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_tmp: NATURAL_32
			l_NA: NATURAL_32
			l_tmp___0: NATURAL_32
			l_MB: NATURAL_32
			l_tmp___1: NATURAL_32
			l_NB: NATURAL_32
			l_tmp___2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_C.item.size1
					l_N := a_C.item.size2
					if a_Trans = {NATURAL_32} 111 then
						l_tmp := size1
					else
						l_tmp := size2
					end
					l_MA := l_tmp
					if a_Trans = {NATURAL_32} 111 then
						l_tmp___0 := size2
					else
						l_tmp___0 := size1
					end
					l_NA := l_tmp___0
					if a_Trans = {NATURAL_32} 111 then
						l_tmp___1 := a_B.item.size1
					else
						l_tmp___1 := a_B.item.size2
					end
					l_MB := l_tmp___1
					if a_Trans = {NATURAL_32} 111 then
						l_tmp___2 := a_B.item.size2
					else
						l_tmp___2 := a_B.item.size1
					end
					l_NB := l_tmp___2
					if l_M = l_N then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix C must be square"), ce_string ("blas.c"), 1_792, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 3 then -- else:
						if l_N = l_MA then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 6 then -- else:
						if l_N = l_MB then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 9 then -- else:
						if l_NA = l_NB then
							l_pos := -1 + 11 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_796, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 11 then -- if done | if done | if done
					if not l_return then
						cblas_ssyr2k ({PC_CBLAS_ORDER}.CblasRowMajor, a_Uplo, a_Trans, l_N.to_integer_32, l_NA.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32, a_beta, a_C.item.data, (a_C.item.tda).to_integer_32)
						update_class_fields -- Current might have been changed; synchronize
						Result := 0
					end
				when 12 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_strmm (a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_alpha: REAL_32; a_B: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_B.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_NA: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_B.item.size1
					l_N := a_B.item.size2
					l_MA := size1
					l_NA := size2
					if l_MA /= l_NA then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("blas.c"), 1_969, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if a_Side /= {NATURAL_32} 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_M = l_MA then
							cblas_strmm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else: | _L___0:
						if a_Side /= {NATURAL_32} 142 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_N = l_MA then
							cblas_strmm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 1_980, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_blas_strsm (a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_alpha: REAL_32; a_B: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_B.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_MA: NATURAL_32
			l_NA: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := a_B.item.size1
					l_N := a_B.item.size2
					l_MA := size1
					l_NA := size2
					if l_MA /= l_NA then
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix A must be square"), ce_string ("blas.c"), 2_088, 20)
							update_class_fields -- Current might have been changed; synchronize
							Result := 20
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
					end
					if l_return then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if a_Side /= {NATURAL_32} 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_M = l_MA then
							cblas_strsm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 4 -- goto _L___0
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else: | _L___0:
						if a_Side /= {NATURAL_32} 142 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_N = l_MA then
							cblas_strsm ({PC_CBLAS_ORDER}.CblasRowMajor, a_Side, a_Uplo, a_TransA, a_Diag, l_M.to_integer_32, l_N.to_integer_32, a_alpha, data, tda.to_integer_32, a_B.item.data, (a_B.item.tda).to_integer_32)
							update_class_fields -- Current might have been changed; synchronize
							Result := 0
							l_return := True
						else
							l_pos := -1 + 7 -- goto _L
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("invalid length"), ce_string ("blas.c"), 2_099, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 8 then -- if done | if done
					if not l_return then
						Result := 0
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_isnull: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if (data[((l_i * l_tda + l_j) + l_k).to_integer_32]).to_double /= 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_float_ispos: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if (data[((l_i * l_tda + l_j) + l_k).to_integer_32]).to_double <= 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_float_isneg: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if (data[((l_i * l_tda + l_j) + l_k).to_integer_32]).to_double >= 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_float_isnonneg: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_size1))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_size2))
				loop
					l_k := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_k < {NATURAL_32} 1))
					loop
						if (data[((l_i * l_tda + l_j) + l_k).to_integer_32]).to_double < 0.0 then
							Result := 0
							l_return := True
						end
						if not l_return then
							l_k := l_k + 1
						end
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := 1
			end
		end

	gsl_matrix_float_max: REAL_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_max: REAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x > l_max then
						l_max := l_x
					end
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x.to_double)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						Result := l_x
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_max
			end
		end

	gsl_matrix_float_min: REAL_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_min: REAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
					end
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x.to_double)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						Result := l_x
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				Result := l_min
			end
		end

	gsl_matrix_float_minmax (a_min_out, a_max_out: CE_POINTER [REAL_32])
		require
			not a_min_out.is_void
			not a_max_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_max: REAL_32
			l_min: REAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
					end
					if l_x > l_max then
						l_max := l_x
					end
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x.to_double)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_min_out.item := l_x
						update_class_fields -- Current might have been changed; synchronize
						a_max_out.item := l_x
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_min_out.item := l_min
				update_class_fields -- Current might have been changed; synchronize
				a_max_out.item := l_max
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_float_max_index (a_imax_out, a_jmax_out: CE_POINTER [NATURAL_32])
		require
			not a_imax_out.is_void
			not a_jmax_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_max: REAL_32
			l_imax: NATURAL_32
			l_jmax: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_imax := (0).to_natural_32
			l_jmax := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x > l_max then
						l_max := l_x
						l_imax := l_i
						l_jmax := l_j
					end
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x.to_double)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_imax_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmax_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_imax_out.item := l_imax
				update_class_fields -- Current might have been changed; synchronize
				a_jmax_out.item := l_jmax
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_float_min_index (a_imin_out, a_jmin_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_jmin_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_min: REAL_32
			l_imin: NATURAL_32
			l_jmin: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_imin := (0).to_natural_32
			l_jmin := (0).to_natural_32
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
						l_imin := l_i
						l_jmin := l_j
					end
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x.to_double)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_imin_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmin_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_imin_out.item := l_imin
				update_class_fields -- Current might have been changed; synchronize
				a_jmin_out.item := l_jmin
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_float_minmax_index (a_imin_out, a_jmin_out, a_imax_out, a_jmax_out: CE_POINTER [NATURAL_32])
		require
			not a_imin_out.is_void
			not a_jmin_out.is_void
			not a_imax_out.is_void
			not a_jmax_out.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_imin: NATURAL_32
			l_jmin: NATURAL_32
			l_imax: NATURAL_32
			l_jmax: NATURAL_32
			l_max: REAL_32
			l_min: REAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_x: REAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_tmp___3: INTEGER_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_imin := (0).to_natural_32
			l_jmin := (0).to_natural_32
			l_imax := (0).to_natural_32
			l_jmax := (0).to_natural_32
			l_max := data[({NATURAL_32} 0).to_integer_32]
			l_min := data[({NATURAL_32} 0).to_integer_32]
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_x := data[(l_i * l_tda + l_j).to_integer_32]
					if l_x < l_min then
						l_min := l_x
						l_imin := l_i
						l_jmin := l_j
					end
					if l_x > l_max then
						l_max := l_x
						l_imax := l_i
						l_jmax := l_j
					end
					if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_32_bytes.to_natural_32 then
						l_tmp := isnanf (l_x)
						update_class_fields -- Current might have been changed; synchronize
						l_tmp___3 := l_tmp
					else
						if {PLATFORM}.Real_32_bytes.to_natural_32 = {PLATFORM}.Real_64_bytes.to_natural_32 then
							l_tmp___0 := isnan (l_x.to_double)
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___0
						else
							l_tmp___1 := isnanl (real_96_from_real_32 (l_x))
							update_class_fields -- Current might have been changed; synchronize
							l_tmp___2 := l_tmp___1
						end
						l_tmp___3 := l_tmp___2
					end
					if l_tmp___3.to_boolean then
						a_imin_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmin_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						a_imax_out.item := l_i
						update_class_fields -- Current might have been changed; synchronize
						a_jmax_out.item := l_j
						update_class_fields -- Current might have been changed; synchronize
						l_return := True
					end
					if not l_return then
						l_j := l_j + 1
					end
				end
				if not l_return then
					l_i := l_i + 1
				end
			end
			if not l_return then
				a_imin_out.item := l_imin
				update_class_fields -- Current might have been changed; synchronize
				a_jmin_out.item := l_jmin
				update_class_fields -- Current might have been changed; synchronize
				a_imax_out.item := l_imax
				update_class_fields -- Current might have been changed; synchronize
				a_jmax_out.item := l_jmax
				update_class_fields -- Current might have been changed; synchronize
			end
		end

	gsl_matrix_float_row (a_i: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 27, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_i * tda).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_column (a_j: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 51, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + a_j.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_diagonal: PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := data
			update_class_fields -- Current might have been changed; synchronize
			if size1 < size2 then
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
			else
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
			end
			l_v.stride := tda + {NATURAL_32} 1
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := block
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector := l_v.twin
			update_class_fields -- Current might have been changed; synchronize
			Result := l_view.twin
		end

	gsl_matrix_float_subdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subdiagonal index is out of range"), ce_string ("rowcol_source.c"), 92, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_k * tda).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				if size1 - a_k < size2 then
					l_v.size := size1 - a_k
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := size2
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := tda + {NATURAL_32} 1
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_superdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 118, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + a_k.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				if size1 < size2 - a_k then
					l_v.size := size1
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := size2 - a_k
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := tda + {NATURAL_32} 1
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_subrow (a_i, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 142, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 151, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_i * tda + a_offset).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_subcolumn (a_j, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 175, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 180, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 184, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := data + (a_offset * tda + a_j).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_row (a_i: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 27, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_i * tda).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_column (a_j: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 51, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + a_j.to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_diagonal: PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_v.data := data
			update_class_fields -- Current might have been changed; synchronize
			if size1 < size2 then
				l_v.size := size1
				update_class_fields -- Current might have been changed; synchronize
			else
				l_v.size := size2
				update_class_fields -- Current might have been changed; synchronize
			end
			l_v.stride := (tda + {NATURAL_32} 1)
			update_class_fields -- Current might have been changed; synchronize
			l_v.block := block
			update_class_fields -- Current might have been changed; synchronize
			l_v.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector := l_v.twin
			update_class_fields -- Current might have been changed; synchronize
			Result := l_view.twin
		end

	gsl_matrix_float_const_subdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("subdiagonal index is out of range"), ce_string ("rowcol_source.c"), 92, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_k * tda).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				if size1 - a_k < size2 then
					l_v.size := (size1 - a_k)
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := size2
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := (tda + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_superdiagonal (a_k: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_k >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 118, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + a_k.to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				if size1 < size2 - a_k then
					l_v.size := size1
					update_class_fields -- Current might have been changed; synchronize
				else
					l_v.size := (size2 - a_k)
					update_class_fields -- Current might have been changed; synchronize
				end
				l_v.stride := (tda + {NATURAL_32} 1)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_subrow (a_i, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("rowcol_source.c"), 142, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 147, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 151, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_i * tda + a_offset).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_subcolumn (a_j, a_offset, a_n: NATURAL_32): PD_GSL_VECTOR_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_VECTOR_FLOAT_CONST_VIEW
			l_v: PD_GSL_VECTOR_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_v.make

			l_view.vector.size := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.stride := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.vector.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("rowcol_source.c"), 175, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("vector length n must be positive integer"), ce_string ("rowcol_source.c"), 180, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_offset + a_n > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimension n overflows matrix"), ce_string ("rowcol_source.c"), 184, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v.size := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_v.data := (data + (a_offset * tda + a_j).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v.size := a_n
				update_class_fields -- Current might have been changed; synchronize
				l_v.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_v.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.vector := l_v.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_swap_rows (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_row1: CE_POINTER [REAL_32]
			l_row2: CE_POINTER [REAL_32]
			l_k: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if a_i >= l_size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first row index is out of range"), ce_string ("swap_source.c"), 29, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second row index is out of range"), ce_string ("swap_source.c"), 34, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_row1 := data + (a_i * tda).to_integer_32
					l_row2 := data + (a_j * tda).to_integer_32
					l_k := (0).to_natural_32
					from
					until
						not (l_k < l_size2)
					loop
						l_tmp := l_row1[l_k.to_integer_32]
						l_row1[l_k.to_integer_32] := l_row2[l_k.to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_row2[l_k.to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
				end
				Result := 0
			end
		end

	gsl_matrix_float_swap_columns (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_col1: CE_POINTER [REAL_32]
			l_col2: CE_POINTER [REAL_32]
			l_p: NATURAL_32
			l_k: NATURAL_32
			l_n: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if a_i >= l_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first column index is out of range"), ce_string ("swap_source.c"), 64, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size2) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second column index is out of range"), ce_string ("swap_source.c"), 69, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				if a_i /= a_j then
					l_col1 := data + a_i.to_integer_32
					l_col2 := data + a_j.to_integer_32
					l_p := (0).to_natural_32
					from
					until
						not (not (l_return) and then (l_p < l_size1))
					loop
						l_n := l_p * tda
						l_k := (0).to_natural_32
						from
						until
							not (l_k < {NATURAL_32} 1)
						loop
							l_tmp := l_col1[(l_n + l_k).to_integer_32]
							l_col1[(l_n + l_k).to_integer_32] := l_col2[(l_n + l_k).to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							l_col2[(l_n + l_k).to_integer_32] := l_tmp
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_k + 1
						end
						l_p := l_p + 1
					end
				end
				Result := 0
			end
		end

	gsl_matrix_float_swap_rowcol (a_i, a_j: NATURAL_32): INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_row: CE_POINTER [REAL_32]
			l_col: CE_POINTER [REAL_32]
			l_p: NATURAL_32
			l_k: NATURAL_32
			l_r___3: NATURAL_32
			l_c___1: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if l_size1 /= l_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to swap row and column"), ce_string ("swap_source.c"), 106, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_i >= l_size1) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("swap_source.c"), 111, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_j >= l_size2) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("swap_source.c"), 116, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_row := data + (a_i * tda).to_integer_32
				l_col := data + a_j.to_integer_32
				l_p := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_p < l_size1))
				loop
					l_r___3 := l_p
					l_c___1 := l_p * tda
					l_k := (0).to_natural_32
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_tmp := l_col[(l_c___1 + l_k).to_integer_32]
						l_col[(l_c___1 + l_k).to_integer_32] := l_row[(l_r___3 + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_row[(l_r___3 + l_k).to_integer_32] := l_tmp
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_p := l_p + 1
				end
				Result := 0
			end
		end

	gsl_matrix_float_transpose: INTEGER_32
		local
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_e1: NATURAL_32
			l_e2: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
		do
			l_size1 := size1
			l_size2 := size2
			if l_size1 /= l_size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix must be square to take transpose"), ce_string ("swap_source.c"), 154, 20)
					update_class_fields -- Current might have been changed; synchronize
					Result := 20
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_size1))
				loop
					l_j := l_i + {NATURAL_32} 1
					from
					until
						not (not (l_return) and then (l_j < l_size2))
					loop
						l_k := (0).to_natural_32
						from
						until
							not (l_k < {NATURAL_32} 1)
						loop
							l_e1 := (l_i * tda + l_j) + l_k
							l_e2 := (l_j * tda + l_i) + l_k
							l_tmp := data[l_e1.to_integer_32]
							data[l_e1.to_integer_32] := data[l_e2.to_integer_32]
							update_class_fields -- Current might have been changed; synchronize
							data[l_e2.to_integer_32] := l_tmp
							update_class_fields -- Current might have been changed; synchronize
							l_k := l_k + 1
						end
						l_j := l_j + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_matrix_float_transpose_memcpy (a_src: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size1: NATURAL_32
			l_src_size2: NATURAL_32
			l_dest_size1: NATURAL_32
			l_dest_size2: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_e1: NATURAL_32
			l_e2: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_size1 := a_src.item.size1
					l_src_size2 := a_src.item.size2
					l_dest_size1 := size1
					l_dest_size2 := size2
					if l_dest_size2 = l_src_size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_dest_size1 = l_src_size2 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("dimensions of dest matrix must be transpose of src matrix"), ce_string ("swap_source.c"), 192, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_dest_size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (not (l_return) and then (l_j < l_dest_size2))
							loop
								l_k := (0).to_natural_32
								from
								until
									not (l_k < {NATURAL_32} 1)
								loop
									l_e1 := (l_i * tda + l_j) + l_k
									l_e2 := (l_j * a_src.item.tda + l_i) + l_k
									data[l_e1.to_integer_32] := a_src.item.data[l_e2.to_integer_32]
									update_class_fields -- Current might have been changed; synchronize
									l_k := l_k + 1
								end
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_memcpy (a_src: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size1: NATURAL_32
			l_src_size2: NATURAL_32
			l_dest_size1: NATURAL_32
			l_dest_size2: NATURAL_32
			l_src_tda: NATURAL_32
			l_dest_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_size1 := a_src.item.size1
					l_src_size2 := a_src.item.size2
					l_dest_size1 := size1
					l_dest_size2 := size2
					if l_src_size1 = l_dest_size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_src_size2 = l_dest_size2 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix sizes are different"), ce_string ("copy_source.c"), 31, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_src_tda := a_src.item.tda
						l_dest_tda := tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_src_size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_src_size2)
							loop
								data[(l_dest_tda * l_i + l_j).to_integer_32] := a_src.item.data[(l_src_tda * l_i + l_j).to_integer_32]
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_swap (a_src: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_src.is_void
		local
			l_src_size1: NATURAL_32
			l_src_size2: NATURAL_32
			l_dest_size1: NATURAL_32
			l_dest_size2: NATURAL_32
			l_src_tda: NATURAL_32
			l_dest_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_tmp: REAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_src_size1 := a_src.item.size1
					l_src_size2 := a_src.item.size2
					l_dest_size1 := size1
					l_dest_size2 := size2
					if l_src_size1 = l_dest_size1 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if l_src_size2 = l_dest_size2 then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix sizes are different"), ce_string ("copy_source.c"), 63, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if done
					if not l_return then
						l_src_tda := a_src.item.tda
						l_dest_tda := tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_src_size1))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_src_size2)
							loop
								l_tmp := a_src.item.data[(l_src_tda * l_i + l_j).to_integer_32]
								a_src.item.data[(l_src_tda * l_i + l_j).to_integer_32] := data[(l_dest_tda * l_i + l_j).to_integer_32]
								update_class_fields -- Current might have been changed; synchronize
								data[(l_dest_tda * l_i + l_j).to_integer_32] := l_tmp
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
					end
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_add (a_b___0: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 28, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] + (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_sub (a_b___0: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 57, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] - (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_mul_elements (a_b___0: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 86, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] * (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_div_elements (a_b___0: CE_POINTER [PD_GSL_MATRIX_FLOAT]): INTEGER_32
		require
			not a_b___0.is_void
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda_a: NATURAL_32
			l_tda_b: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_M := size1
					l_N := size2
					if a_b___0.item.size1 = l_M then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if a_b___0.item.size2 = l_N then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							(l_return or l_break)
						loop
							l_break := False;
							(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrices must have same dimensions"), ce_string ("oper_source.c"), 115, 19)
							update_class_fields -- Current might have been changed; synchronize
							Result := 19
							l_return := True
							if not l_return then
								l_break := True
							end
						end
						if not l_return then
							l_break := False
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_tda_a := tda
						l_tda_b := a_b___0.item.tda
						l_i := (0).to_natural_32
						from
						until
							not (not (l_return) and then (l_i < l_M))
						loop
							l_j := (0).to_natural_32
							from
							until
								not (l_j < l_N)
							loop
								data[(l_i * l_tda_a + l_j).to_integer_32] := data[(l_i * l_tda_a + l_j).to_integer_32] / (a_b___0.item.data[(l_i * l_tda_b + l_j).to_integer_32])
								update_class_fields -- Current might have been changed; synchronize
								l_j := l_j + 1
							end
							l_i := l_i + 1
						end
						Result := 0
						l_return := True
				when 7 then -- if done
					if not l_return then
						Result := 0
					end
				when 8 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	gsl_matrix_float_scale (a_x: REAL_64): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					data[(l_i * l_tda + l_j).to_integer_32] := ((data[(l_i * l_tda + l_j).to_integer_32]).to_double * a_x).truncated_to_real
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_matrix_float_add_constant (a_x: REAL_64): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_return: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_M))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_N)
				loop
					data[(l_i * l_tda + l_j).to_integer_32] := ((data[(l_i * l_tda + l_j).to_integer_32]).to_double + a_x).truncated_to_real
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_matrix_float_add_diagonal (a_x: REAL_64): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_loop_lim: NATURAL_32
			l_tmp: NATURAL_32
			l_i: NATURAL_32
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			if l_M < l_N then
				l_tmp := l_M
			else
				l_tmp := l_N
			end
			l_loop_lim := l_tmp
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_loop_lim)
			loop
				data[(l_i * l_tda + l_i).to_integer_32] := ((data[(l_i * l_tda + l_i).to_integer_32]).to_double + a_x).truncated_to_real
				update_class_fields -- Current might have been changed; synchronize
				l_i := l_i + 1
			end
			Result := 0
		end

	gsl_matrix_float_set_row (a_i: NATURAL_32; a_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_v_data: CE_POINTER [REAL_32]
			l_row_data: CE_POINTER [REAL_32]
			l_stride: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			if a_i >= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("getset_source.c"), 99, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_v.item.size /= l_N) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix row size and vector length are not equal"), ce_string ("getset_source.c"), 105, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v_data := a_v.item.data
				l_row_data := data + (a_i * l_tda).to_integer_32
				l_stride := a_v.item.stride
				l_j := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_j < l_N))
				loop
					l_k := {NATURAL_32} 0
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_row_data[(l_j + l_k).to_integer_32] := l_v_data[(l_stride * l_j + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_j := l_j + 1
				end
				Result := 0
			end
		end

	gsl_matrix_float_set_col (a_j: NATURAL_32; a_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]): INTEGER_32
		local
			l_M: NATURAL_32
			l_N: NATURAL_32
			l_tda: NATURAL_32
			l_v_data: CE_POINTER [REAL_32]
			l_column_data: CE_POINTER [REAL_32]
			l_stride: NATURAL_32
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			l_N := size2
			l_tda := tda
			if a_j >= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("getset_source.c"), 140, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := 4
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return and then (a_v.item.size /= l_M) then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix column size and vector length are not equal"), ce_string ("getset_source.c"), 146, 19)
					update_class_fields -- Current might have been changed; synchronize
					Result := 19
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_v_data := a_v.item.data
				l_column_data := data + a_j.to_integer_32
				l_stride := a_v.item.stride
				l_i := (0).to_natural_32
				from
				until
					not (not (l_return) and then (l_i < l_M))
				loop
					l_k := {NATURAL_32} 0
					from
					until
						not (l_k < {NATURAL_32} 1)
					loop
						l_column_data[(l_i * l_tda + l_k).to_integer_32] := l_v_data[(l_stride * l_i + l_k).to_integer_32]
						update_class_fields -- Current might have been changed; synchronize
						l_k := l_k + 1
					end
					l_i := l_i + 1
				end
				Result := 0
			end
		end

	gsl_vector_float_alloc_row_from_matrix (a_i: NATURAL_32): CE_POINTER [PD_GSL_VECTOR_FLOAT]
		local
			l_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]
			l_M: NATURAL_32
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			l_M := size1
			if a_i >= l_M then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("getset_source.c"), 181, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_VECTOR_FLOAT}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v := create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_cast (l_tmp)
				if l_v.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vector struct"), ce_string ("getset_source.c"), 189, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_v.item.data := data + (a_i * tda).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.size := size2
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.stride := (1).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_v
			end
		end

	gsl_vector_float_alloc_col_from_matrix (a_j: NATURAL_32): CE_POINTER [PD_GSL_VECTOR_FLOAT]
		local
			l_v: CE_POINTER [PD_GSL_VECTOR_FLOAT]
			l_N: NATURAL_32
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			l_N := size2
			if a_j >= l_N then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("getset_source.c"), 210, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_VECTOR_FLOAT}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_v := create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_cast (l_tmp)
				if l_v.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for vector struct"), ce_string ("getset_source.c"), 218, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_VECTOR_FLOAT]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_v.item.data := data + a_j.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.size := size1
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.stride := tda
				update_class_fields -- Current might have been changed; synchronize
				l_v.item.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				Result := l_v
			end
		end

	gsl_matrix_float_fread (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_float_raw_fread (a_stream, data, l_size1 * l_size2, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_float_raw_fread (a_stream, data + (l_i * l_tda).to_integer_32, l_size2, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_float_fwrite (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_float_raw_fwrite (a_stream, data, l_size1 * l_size2, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FWRITE_SOURCE_STATIC}).gsl_block_float_raw_fwrite (a_stream, (data + (l_i * l_tda).to_integer_32), l_size2, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_float_fprintf (a_stream: CE_POINTER [SD_IO_FILE]; a_format: CE_POINTER [INTEGER_8]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_float_raw_fprintf (a_stream, data, l_size1 * l_size2, (1).to_natural_32, a_format)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_float_raw_fprintf (a_stream, (data + (l_i * l_tda).to_integer_32), l_size2, (1).to_natural_32, a_format)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_float_fscanf (a_stream: CE_POINTER [SD_IO_FILE]): INTEGER_32
		local
			l_status: INTEGER_32
			l_size1: NATURAL_32
			l_size2: NATURAL_32
			l_tda: NATURAL_32
			l_i: NATURAL_32
			l_break: BOOLEAN
		do
			l_status := 0
			l_size1 := size1
			l_size2 := size2
			l_tda := tda
			if l_tda = l_size2 then
				l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_float_raw_fscanf (a_stream, data, l_size1 * l_size2, (1).to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
			else
				l_i := (0).to_natural_32
				from
				until
					not (not (l_break) and then (l_i < l_size1))
				loop
					l_break := False
					l_status := (create {P_FPRINTF_SOURCE_STATIC}).gsl_block_float_raw_fscanf (a_stream, data + (l_i * l_tda).to_integer_32, l_size2, (1).to_natural_32)
					update_class_fields -- Current might have been changed; synchronize
					if l_status.to_boolean then
						l_break := True
					end
					if not (l_break) then
						l_i := l_i + 1
					end
				end
				l_break := False
			end
			Result := l_status
		end

	gsl_matrix_float_submatrix (a_i, a_j, a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_FLOAT_VIEW
		local
			l_view: PD_GSL_MATRIX_FLOAT_VIEW
			l_s: PD_GSL_MATRIX_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("submatrix_source.c"), 29, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("submatrix_source.c"), 33, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension must be non-zero"), ce_string ("submatrix_source.c"), 37, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension must be non-zero"), ce_string ("submatrix_source.c"), 41, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_i + a_n1 > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension overflows matrix"), ce_string ("submatrix_source.c"), 45, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j + a_n2 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension overflows matrix"), ce_string ("submatrix_source.c"), 49, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := data + (a_i * tda + a_j).to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := tda
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_const_submatrix (a_i, a_j, a_n1, a_n2: NATURAL_32): PD_GSL_MATRIX_FLOAT_CONST_VIEW
		local
			l_view: PD_GSL_MATRIX_FLOAT_CONST_VIEW
			l_s: PD_GSL_MATRIX_FLOAT
			l_return, l_break: BOOLEAN
		do
			create l_view.make
			create l_s.make

			l_view.matrix.size1 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.size2 := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.tda := (0).to_natural_32
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
			update_class_fields -- Current might have been changed; synchronize
			l_view.matrix.owner := 0
			update_class_fields -- Current might have been changed; synchronize
			if a_i >= size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("row index is out of range"), ce_string ("submatrix_source.c"), 29, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j >= size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("column index is out of range"), ce_string ("submatrix_source.c"), 33, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension must be non-zero"), ce_string ("submatrix_source.c"), 37, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension must be non-zero"), ce_string ("submatrix_source.c"), 41, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_i + a_n1 > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("first dimension overflows matrix"), ce_string ("submatrix_source.c"), 45, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_j + a_n2 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("second dimension overflows matrix"), ce_string ("submatrix_source.c"), 49, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := l_view.twin
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_s.size1 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := (0).to_natural_32
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := create {CE_REAL_32_POINTER}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := create {CE_CLASS_POINTER [PD_GSL_BLOCK_FLOAT_STRUCT]}.make_shared (default_pointer)
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_s.data := (data + (a_i * tda + a_j).to_integer_32)
				update_class_fields -- Current might have been changed; synchronize
				l_s.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_s.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_s.tda := tda
				update_class_fields -- Current might have been changed; synchronize
				l_s.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_s.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				l_view.matrix := l_s.twin
				update_class_fields -- Current might have been changed; synchronize
				Result := l_view.twin
			end
		end

	gsl_matrix_float_alloc_from_matrix (a_k1, a_k2, a_n1, a_n2: NATURAL_32): CE_POINTER [PD_GSL_MATRIX_FLOAT]
		local
			l_m___4: CE_POINTER [PD_GSL_MATRIX_FLOAT]
			l_tmp: CE_POINTER [ANY]
			l_return, l_break: BOOLEAN
		do
			if a_n1 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n1 must be positive integer"), ce_string ("init_source.c"), 146, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_n2 = {NATURAL_32} 0 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("matrix dimension n2 must be positive integer"), ce_string ("init_source.c"), 151, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_k1 + a_n1 > size1 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("submatrix dimension 1 exceeds size of original"), ce_string ("init_source.c"), 156, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			elseif a_k2 + a_n2 > size2 then
				from
				until
					(l_return or l_break)
				loop
					l_break := False;
					(create {P_ERROR_STATIC}).gsl_error (ce_string ("submatrix dimension 2 exceeds size of original"), ce_string ("init_source.c"), 161, 4)
					update_class_fields -- Current might have been changed; synchronize
					Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX_FLOAT]}.make_shared (default_pointer))
					l_return := True
					if not l_return then
						l_break := True
					end
				end
				if not l_return then
					l_break := False
				end
			end
			if not l_return then
				l_tmp := malloc ((create {PD_GSL_MATRIX_FLOAT}.make).structure_size.to_natural_32)
				update_class_fields -- Current might have been changed; synchronize
				l_m___4 := create {CE_CLASS_POINTER [PD_GSL_MATRIX_FLOAT]}.make_cast (l_tmp)
				if l_m___4.is_void then
					from
					until
						(l_return or l_break)
					loop
						l_break := False;
						(create {P_ERROR_STATIC}).gsl_error (ce_string ("failed to allocate space for matrix struct"), ce_string ("init_source.c"), 169, 8)
						update_class_fields -- Current might have been changed; synchronize
						Result := (create {CE_CLASS_POINTER [PD_GSL_MATRIX_FLOAT]}.make_shared (default_pointer))
						l_return := True
						if not l_return then
							l_break := True
						end
					end
					if not l_return then
						l_break := False
					end
				end
			end
			if not l_return then
				l_m___4.item.data := (data + (a_k1 * tda).to_integer_32) + a_k2.to_integer_32
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.size1 := a_n1
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.size2 := a_n2
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.tda := tda
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.block := block
				update_class_fields -- Current might have been changed; synchronize
				l_m___4.item.owner := 0
				update_class_fields -- Current might have been changed; synchronize
				Result := l_m___4
			end
		end

	gsl_matrix_float_set_identity
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_data: CE_POINTER [REAL_32]
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_tda: NATURAL_32
			l_zero: REAL_32
			l_one: REAL_32
			l_return: BOOLEAN
		do
			l_data := data
			l_p := size1
			l_q___3 := size2
			l_tda := tda
			l_zero := {REAL_32} 0.0
			l_one := {REAL_32} 1.0
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_p))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_q___3)
				loop
					if l_i = l_j then
						l_data[(l_i * l_tda + l_j).to_integer_32] := l_one
						update_class_fields -- Current might have been changed; synchronize
					else
						l_data[(l_i * l_tda + l_j).to_integer_32] := l_zero
						update_class_fields -- Current might have been changed; synchronize
					end
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

	gsl_matrix_float_set_zero
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_data: CE_POINTER [REAL_32]
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_tda: NATURAL_32
			l_zero: REAL_32
			l_return: BOOLEAN
		do
			l_data := data
			l_p := size1
			l_q___3 := size2
			l_tda := tda
			l_zero := {REAL_32} 0.0
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_p))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_q___3)
				loop
					l_data[(l_i * l_tda + l_j).to_integer_32] := l_zero
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

	gsl_matrix_float_set_all (a_x: REAL_32)
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_data: CE_POINTER [REAL_32]
			l_p: NATURAL_32
			l_q___3: NATURAL_32
			l_tda: NATURAL_32
			l_return: BOOLEAN
		do
			l_data := data
			l_p := size1
			l_q___3 := size2
			l_tda := tda
			l_i := (0).to_natural_32
			from
			until
				not (not (l_return) and then (l_i < l_p))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_q___3)
				loop
					l_data[(l_i * l_tda + l_j).to_integer_32] := a_x
					update_class_fields -- Current might have been changed; synchronize
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

feature {PD_GSL_MATRIX_FLOAT} -- private routines

	cblas_sgemm (a_Order, a_TransA, a_TransB: NATURAL_32; a_M, a_N, a_K: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_32]; a_ldb: INTEGER_32; a_beta: REAL_32; a_C: CE_POINTER [REAL_32]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_ldf: INTEGER_32
			l_ldg: INTEGER_32
			l_TransF: INTEGER_32
			l_TransG: INTEGER_32
			l_F___0: CE_POINTER [REAL_32]
			l_G: CE_POINTER [REAL_32]
			l_temp: REAL_32
			l_temp___0: REAL_32
			l_temp___1: REAL_32
			l_temp___2: REAL_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha.to_double = 0.0 then
						if a_beta.to_double = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_n1 := a_M
							l_n2 := a_N
							l_F___0 := a_A___0
							l_ldf := a_lda
							if a_TransA = {NATURAL_32} 113 then
								l_TransF := 112
							else
								l_TransF := a_TransA.to_integer_32
							end
							l_G := a_B
							l_ldg := a_ldb
							if a_TransB = {NATURAL_32} 113 then
								l_TransG := 112
							else
								l_TransG := a_TransB.to_integer_32
							end
						else
							l_n1 := a_N
							l_n2 := a_M
							l_F___0 := a_B
							l_ldf := a_ldb
							if a_TransB = {NATURAL_32} 113 then
								l_TransF := 112
							else
								l_TransF := a_TransB.to_integer_32
							end
							l_G := a_A___0
							l_ldg := a_lda
							if a_TransA = {NATURAL_32} 113 then
								l_TransG := 112
							else
								l_TransG := a_TransA.to_integer_32
							end
						end
						if a_beta.to_double = 0.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := (0.0).truncated_to_real
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						elseif a_beta.to_double /= 1.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						end
						if a_alpha.to_double = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_TransF /= 111 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_TransG = 111 then
							l_k := 0
							from
							until
								not (not (l_return) and then (l_k < a_K))
							loop
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_temp := a_alpha * l_F___0[(l_ldf * l_i + l_k)]
									if l_temp.to_double /= 0.0 then
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (l_temp * l_G[(l_ldg * l_k + l_j)])
											l_j := l_j + 1
										end
									end
									l_i := l_i + 1
								end
								l_k := l_k + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_TransF /= 111 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_TransG = 112 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp___0 := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___0 := l_temp___0 + (l_F___0[(l_ldf * l_i + l_k)] * l_G[(l_ldg * l_j + l_k)])
										l_k := l_k + 1
									end
									a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (a_alpha * l_temp___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_TransF /= 112 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_TransG = 111 then
							l_k := 0
							from
							until
								not (not (l_return) and then (l_k < a_K))
							loop
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_temp___1 := a_alpha * l_F___0[(l_ldf * l_k + l_i)]
									if l_temp___1.to_double /= 0.0 then
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (l_temp___1 * l_G[(l_ldg * l_k + l_j)])
											l_j := l_j + 1
										end
									end
									l_i := l_i + 1
								end
								l_k := l_k + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_TransF = 112 then
							if l_TransG = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___2 := (0.0).truncated_to_real
										l_k := 0
										from
										until
											not (l_k < a_K)
										loop
											l_temp___2 := l_temp___2 + (l_F___0[(l_ldf * l_k + l_i)] * l_G[(l_ldg * l_j + l_k)])
											l_k := l_k + 1
										end
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] + (a_alpha * l_temp___2)
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemm_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_sgemv (a_order, a_TransA: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_beta: REAL_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_lenX: INTEGER_32
			l_lenY: INTEGER_32
			l_Trans: INTEGER_32
			l_tmp: NATURAL_32
			l_iy: INTEGER_32
			l_tmp___0: INTEGER_32
			l_iy___0: INTEGER_32
			l_tmp___1: INTEGER_32
			l_iy___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_temp: REAL_32
			l_ix: INTEGER_32
			l_tmp___3: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___4: INTEGER_32
			l_temp___0: REAL_32
			l_iy___2: INTEGER_32
			l_tmp___5: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_TransA /= {NATURAL_32} 113 then
						l_tmp := a_TransA
					else
						l_tmp := {NATURAL_32} 112
					end
					l_Trans := l_tmp.to_integer_32
					if a_M = 0 then
						l_return := True
					elseif a_N = 0 then
						l_return := True
					end
					if not l_return and then (a_alpha.to_double = 0.0) then
						if a_beta.to_double = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if l_Trans = 111 then
							l_lenX := a_N
							l_lenY := a_M
						else
							l_lenX := a_M
							l_lenY := a_N
						end
						if a_beta.to_double = 0.0 then
							if a_incY > 0 then
								l_tmp___0 := 0
							else
								l_tmp___0 := (l_lenY - 1) * -a_incY
							end
							l_iy := l_tmp___0
							l_i := 0
							from
							until
								not (l_i < l_lenY)
							loop
								a_Y[l_iy] := (0.0).truncated_to_real
								l_iy := l_iy + a_incY
								l_i := l_i + 1
							end
						elseif a_beta.to_double /= 1.0 then
							if a_incY > 0 then
								l_tmp___1 := 0
							else
								l_tmp___1 := (l_lenY - 1) * -a_incY
							end
							l_iy___0 := l_tmp___1
							l_i := 0
							from
							until
								not (l_i < l_lenY)
							loop
								a_Y[l_iy___0] := a_Y[l_iy___0] * a_beta
								l_iy___0 := l_iy___0 + a_incY
								l_i := l_i + 1
							end
						end
						if a_alpha.to_double = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_Trans = 111 then
							l_pos := -1 + 6 -- goto _L___1
						else
							l_pos := -1 + 4 -- goto _L___3
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 4 then -- else: | _L___3:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___1:
						if a_incY > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (l_lenY - 1) * -a_incY
						end
						l_iy___1 := l_tmp___2
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < l_lenY))
						loop
							l_temp := (0.0).truncated_to_real
							if a_incX > 0 then
								l_tmp___3 := 0
							else
								l_tmp___3 := (l_lenX - 1) * -a_incX
							end
							l_ix := l_tmp___3
							l_j := 0
							from
							until
								not (l_j < l_lenX)
							loop
								l_temp := l_temp + (a_X[l_ix] * a_A___0[(a_lda * l_i + l_j)])
								l_ix := l_ix + a_incX
								l_j := l_j + 1
							end
							a_Y[l_iy___1] := a_Y[l_iy___1] + (a_alpha * l_temp)
							l_iy___1 := l_iy___1 + a_incY
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 10 -- goto _L___2
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 10 then -- else: | _L___2:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if l_Trans = 112 then
							l_pos := -1 + 15 -- goto _L
						else
							l_pos := -1 + 13 -- goto _L___0
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 13 then -- else: | _L___0:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 19 -- skip if block
						end
				when 14 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 17 -- skip if block
						end
				when 15 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___4 := 0
						else
							l_tmp___4 := (l_lenX - 1) * -a_incX
						end
						l_ix___0 := l_tmp___4
						l_j := 0
						from
						until
							not (not (l_return) and then (l_j < l_lenX))
						loop
							l_temp___0 := a_alpha * a_X[l_ix___0]
							if l_temp___0.to_double /= 0.0 then
								if a_incY > 0 then
									l_tmp___5 := 0
								else
									l_tmp___5 := (l_lenY - 1) * -a_incY
								end
								l_iy___2 := l_tmp___5
								l_i := 0
								from
								until
									not (l_i < l_lenY)
								loop
									a_Y[l_iy___2] := a_Y[l_iy___2] + (l_temp___0 * a_A___0[(a_lda * l_j + l_i)])
									l_iy___2 := l_iy___2 + a_incY
									l_i := l_i + 1
								end
							end
							l_ix___0 := l_ix___0 + a_incX
							l_j := l_j + 1
						end
				when 16 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 17 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemv_r.h"), ce_string ("unrecognized operation")])
				when 18 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_gemv_r.h"), ce_string ("unrecognized operation")])
				when 20 then -- if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_ssymm (a_Order, a_Side, a_Uplo: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_32]; a_ldb: INTEGER_32; a_beta: REAL_32; a_C: CE_POINTER [REAL_32]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_uplo: INTEGER_32
			l_side: INTEGER_32
			l_temp1: REAL_32
			l_temp2: REAL_32
			l_Aik: REAL_32
			l_temp1___0: REAL_32
			l_temp2___0: REAL_32
			l_Aik___0: REAL_32
			l_temp1___1: REAL_32
			l_temp2___1: REAL_32
			l_Ajk: REAL_32
			l_temp1___2: REAL_32
			l_temp2___2: REAL_32
			l_Ajk___0: REAL_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha.to_double = 0.0 then
						if a_beta.to_double = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_n1 := a_M
							l_n2 := a_N
							l_uplo := a_Uplo.to_integer_32
							l_side := a_Side.to_integer_32
						else
							l_n1 := a_N
							l_n2 := a_M
							if a_Uplo = {NATURAL_32} 121 then
								l_uplo := 122
							else
								l_uplo := 121
							end
							if a_Side = {NATURAL_32} 141 then
								l_side := 142
							else
								l_side := 141
							end
						end
						if a_beta.to_double = 0.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := (0.0).truncated_to_real
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						elseif a_beta.to_double /= 1.0 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (l_j < l_n2)
								loop
									a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						end
						if a_alpha.to_double = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_side /= 141 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_uplo = 121 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp1 := a_alpha * a_B[(a_ldb * l_i + l_j)]
									l_temp2 := (0.0).truncated_to_real
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1 * a_A___0[(l_i * a_lda + l_i)])
									l_k := l_i + 1
									from
									until
										not (l_k < l_n1)
									loop
										l_Aik := a_A___0[(l_i * a_lda + l_k)]
										a_C[(l_k * a_ldc + l_j)] := a_C[(l_k * a_ldc + l_j)] + (l_Aik * l_temp1)
										l_temp2 := l_temp2 + (l_Aik * a_B[(a_ldb * l_k + l_j)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp2)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_side /= 141 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_uplo = 122 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp1___0 := a_alpha * a_B[(a_ldb * l_i + l_j)]
									l_temp2___0 := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < l_i)
									loop
										l_Aik___0 := a_A___0[(l_i * a_lda + l_k)]
										a_C[(l_k * a_ldc + l_j)] := a_C[(l_k * a_ldc + l_j)] + (l_Aik___0 * l_temp1___0)
										l_temp2___0 := l_temp2___0 + (l_Aik___0 * a_B[(a_ldb * l_k + l_j)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1___0 * a_A___0[(l_i * a_lda + l_i)] + a_alpha * l_temp2___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_side /= 142 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_uplo = 121 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < l_n1))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j < l_n2))
								loop
									l_temp1___1 := a_alpha * a_B[(a_ldb * l_i + l_j)]
									l_temp2___1 := (0.0).truncated_to_real
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1___1 * a_A___0[(l_j * a_lda + l_j)])
									l_k := l_j + 1
									from
									until
										not (l_k < l_n2)
									loop
										l_Ajk := a_A___0[(l_j * a_lda + l_k)]
										a_C[(l_i * a_ldc + l_k)] := a_C[(l_i * a_ldc + l_k)] + (l_temp1___1 * l_Ajk)
										l_temp2___1 := l_temp2___1 + (a_B[(a_ldb * l_i + l_k)] * l_Ajk)
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp2___1)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_side = 142 then
							if l_uplo = 122 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp1___2 := a_alpha * a_B[(a_ldb * l_i + l_j)]
										l_temp2___2 := (0.0).truncated_to_real
										l_k := 0
										from
										until
											not (l_k < l_j)
										loop
											l_Ajk___0 := a_A___0[(l_j * a_lda + l_k)]
											a_C[(l_i * a_ldc + l_k)] := a_C[(l_i * a_ldc + l_k)] + (l_temp1___2 * l_Ajk___0)
											l_temp2___2 := l_temp2___2 + (a_B[(a_ldb * l_i + l_k)] * l_Ajk___0)
											l_k := l_k + 1
										end
										a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (l_temp1___2 * a_A___0[(l_j * a_lda + l_j)] + a_alpha * l_temp2___2)
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symm_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_ssymv (a_order, a_Uplo: NATURAL_32; a_N: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32; a_beta: REAL_32; a_Y: CE_POINTER [REAL_32]; a_incY: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_iy: INTEGER_32
			l_tmp: INTEGER_32
			l_iy___0: INTEGER_32
			l_tmp___0: INTEGER_32
			l_ix: INTEGER_32
			l_tmp___1: INTEGER_32
			l_iy___1: INTEGER_32
			l_tmp___2: INTEGER_32
			l_temp1: REAL_32
			l_temp2: REAL_32
			l_j_min: INTEGER_32
			l_j_max: INTEGER_32
			l_jx: INTEGER_32
			l_tmp___3: INTEGER_32
			l_jy: INTEGER_32
			l_tmp___4: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___5: INTEGER_32
			l_iy___2: INTEGER_32
			l_tmp___6: INTEGER_32
			l_temp1___0: REAL_32
			l_temp2___0: REAL_32
			l_j_min___0: INTEGER_32
			l_j_max___0: INTEGER_32
			l_jx___0: INTEGER_32
			l_tmp___7: INTEGER_32
			l_jy___0: INTEGER_32
			l_tmp___8: INTEGER_32
			l_tmp___9: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha.to_double = 0.0 then
						if a_beta.to_double = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_beta.to_double = 0.0 then
							if a_incY > 0 then
								l_tmp := 0
							else
								l_tmp := (a_N - 1) * -a_incY
							end
							l_iy := l_tmp
							l_i := 0
							from
							until
								not (l_i < a_N)
							loop
								a_Y[l_iy] := (0.0).truncated_to_real
								l_iy := l_iy + a_incY
								l_i := l_i + 1
							end
						elseif a_beta.to_double /= 1.0 then
							if a_incY > 0 then
								l_tmp___0 := 0
							else
								l_tmp___0 := (a_N - 1) * -a_incY
							end
							l_iy___0 := l_tmp___0
							l_i := 0
							from
							until
								not (l_i < a_N)
							loop
								a_Y[l_iy___0] := a_Y[l_iy___0] * a_beta
								l_iy___0 := l_iy___0 + a_incY
								l_i := l_i + 1
							end
						end
						if a_alpha.to_double = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_Uplo = {NATURAL_32} 121 then
							l_pos := -1 + 6 -- goto _L___1
						else
							l_pos := -1 + 4 -- goto _L___3
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 4 then -- else: | _L___3:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___1:
						if a_incX > 0 then
							l_tmp___1 := 0
						else
							l_tmp___1 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___1
						if a_incY > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (a_N - 1) * -a_incY
						end
						l_iy___1 := l_tmp___2
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_temp1 := a_alpha * a_X[l_ix]
							l_temp2 := (0.0).truncated_to_real
							l_j_min := l_i + 1
							l_j_max := a_N
							if a_incX > 0 then
								l_tmp___3 := 0
							else
								l_tmp___3 := (a_N - 1) * -a_incX
							end
							l_jx := l_tmp___3 + l_j_min * a_incX
							if a_incY > 0 then
								l_tmp___4 := 0
							else
								l_tmp___4 := (a_N - 1) * -a_incY
							end
							l_jy := l_tmp___4 + l_j_min * a_incY
							a_Y[l_iy___1] := a_Y[l_iy___1] + (l_temp1 * a_A___0[(a_lda * l_i + l_i)])
							l_j := l_j_min
							from
							until
								not (l_j < l_j_max)
							loop
								a_Y[l_jy] := a_Y[l_jy] + (l_temp1 * a_A___0[(a_lda * l_i + l_j)])
								l_temp2 := l_temp2 + (a_X[l_jx] * a_A___0[(a_lda * l_i + l_j)])
								l_jx := l_jx + a_incX
								l_jy := l_jy + a_incY
								l_j := l_j + 1
							end
							a_Y[l_iy___1] := a_Y[l_iy___1] + (a_alpha * l_temp2)
							l_ix := l_ix + a_incX
							l_iy___1 := l_iy___1 + a_incY
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 10 -- goto _L___2
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 10 then -- else: | _L___2:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						if a_Uplo = {NATURAL_32} 122 then
							l_pos := -1 + 15 -- goto _L
						else
							l_pos := -1 + 13 -- goto _L___0
						end
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 20
				when 13 then -- else: | _L___0:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 19 -- skip if block
						end
				when 14 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 17 -- skip if block
						end
				when 15 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___5 := 0
						else
							l_tmp___5 := (a_N - 1) * -a_incX
						end
						l_ix___0 := l_tmp___5 + (a_N - 1) * a_incX
						if a_incY > 0 then
							l_tmp___6 := 0
						else
							l_tmp___6 := (a_N - 1) * -a_incY
						end
						l_iy___2 := l_tmp___6 + (a_N - 1) * a_incY
						l_i := a_N
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___9 := l_i
								l_i := l_i - 1
								if not l_tmp___9.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_temp1___0 := a_alpha * a_X[l_ix___0]
								l_temp2___0 := (0.0).truncated_to_real
								l_j_min___0 := 0
								l_j_max___0 := l_i
								if a_incX > 0 then
									l_tmp___7 := 0
								else
									l_tmp___7 := (a_N - 1) * -a_incX
								end
								l_jx___0 := l_tmp___7 + l_j_min___0 * a_incX
								if a_incY > 0 then
									l_tmp___8 := 0
								else
									l_tmp___8 := (a_N - 1) * -a_incY
								end
								l_jy___0 := l_tmp___8 + l_j_min___0 * a_incY
								a_Y[l_iy___2] := a_Y[l_iy___2] + (l_temp1___0 * a_A___0[(a_lda * l_i + l_i)])
								l_j := l_j_min___0
								from
								until
									not (l_j < l_j_max___0)
								loop
									a_Y[l_jy___0] := a_Y[l_jy___0] + (l_temp1___0 * a_A___0[(a_lda * l_i + l_j)])
									l_temp2___0 := l_temp2___0 + (a_X[l_jx___0] * a_A___0[(a_lda * l_i + l_j)])
									l_jx___0 := l_jx___0 + a_incX
									l_jy___0 := l_jy___0 + a_incY
									l_j := l_j + 1
								end
								a_Y[l_iy___2] := a_Y[l_iy___2] + (a_alpha * l_temp2___0)
								l_ix___0 := l_ix___0 - a_incX
								l_iy___2 := l_iy___2 - a_incY
							end
						end
						l_break := False
				when 16 then -- if block done, skip else block:
						l_pos := -1 + 18
				when 17 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symv.h"), ce_string ("unrecognized operation")])
				when 18 then -- if done | if block done, skip else block:
						l_pos := -1 + 20
				when 19 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_symv.h"), ce_string ("unrecognized operation")])
				when 20 then -- if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_ssyr2k (a_Order, a_Uplo, a_Trans: NATURAL_32; a_N, a_K: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_32]; a_ldb: INTEGER_32; a_beta: REAL_32; a_C: CE_POINTER [REAL_32]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_temp: REAL_32
			l_temp1: REAL_32
			l_temp2: REAL_32
			l_temp___0: REAL_32
			l_temp1___0: REAL_32
			l_temp2___0: REAL_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha.to_double = 0.0 then
						if a_beta.to_double = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_uplo := a_Uplo.to_integer_32
							if a_Trans = {NATURAL_32} 113 then
								l_trans := 112
							else
								l_trans := a_Trans.to_integer_32
							end
						else
							if a_Uplo = {NATURAL_32} 121 then
								l_uplo := 122
							else
								l_uplo := 121
							end
							if a_Trans = {NATURAL_32} 112 then
								l_trans := 111
							elseif a_Trans = {NATURAL_32} 113 then
								l_trans := 111
							else
								l_trans := 112
							end
						end
						if a_beta.to_double = 0.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := (0.0).truncated_to_real
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := (0.0).truncated_to_real
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						elseif a_beta.to_double /= 1.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						end
						if a_alpha.to_double = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_uplo /= 121 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := l_i
								from
								until
									not (not (l_return) and then (l_j < a_N))
								loop
									l_temp := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp := l_temp + (a_A___0[(l_i * a_lda + l_k)] * a_B[(l_j * a_ldb + l_k)] + a_B[(l_i * a_ldb + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_uplo /= 121 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_trans = 112 then
							l_k := 0
							from
							until
								not (not (l_return) and then (l_k < a_K))
							loop
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_temp1 := a_alpha * a_A___0[(l_k * a_lda + l_i)]
									l_temp2 := a_alpha * a_B[(l_k * a_ldb + l_i)]
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(l_i * a_lda + l_j)] := a_C[(l_i * a_lda + l_j)] + (l_temp1 * a_B[(l_k * a_ldb + l_j)] + l_temp2 * a_A___0[(l_k * a_lda + l_j)])
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
								l_k := l_k + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_uplo /= 122 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j <= l_i))
								loop
									l_temp___0 := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___0 := l_temp___0 + (a_A___0[(l_i * a_lda + l_k)] * a_B[(l_j * a_ldb + l_k)] + a_B[(l_i * a_ldb + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_uplo = 122 then
							if l_trans = 112 then
								l_k := 0
								from
								until
									not (not (l_return) and then (l_k < a_K))
								loop
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < a_N))
									loop
										l_temp1___0 := a_alpha * a_A___0[(l_k * a_lda + l_i)]
										l_temp2___0 := a_alpha * a_B[(l_k * a_ldb + l_i)]
										l_j := 0
										from
										until
											not (l_j <= l_i)
										loop
											a_C[(l_i * a_lda + l_j)] := a_C[(l_i * a_lda + l_j)] + (l_temp1___0 * a_B[(l_k * a_ldb + l_j)] + l_temp2___0 * a_A___0[(l_k * a_lda + l_j)])
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
									l_k := l_k + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syr2k_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syr2k_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_ssyrk (a_Order, a_Uplo, a_Trans: NATURAL_32; a_N, a_K: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_beta: REAL_32; a_C: CE_POINTER [REAL_32]; a_ldc: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_temp: REAL_32
			l_temp___0: REAL_32
			l_temp___1: REAL_32
			l_temp___2: REAL_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_alpha.to_double = 0.0 then
						if a_beta.to_double = 1.0 then
							l_return := True
						end
					end
					if not l_return then
						if a_Order = {NATURAL_32} 101 then
							l_uplo := a_Uplo.to_integer_32
							if a_Trans = {NATURAL_32} 113 then
								l_trans := 112
							else
								l_trans := a_Trans.to_integer_32
							end
						else
							if a_Uplo = {NATURAL_32} 121 then
								l_uplo := 122
							else
								l_uplo := 121
							end
							if a_Trans = {NATURAL_32} 112 then
								l_trans := 111
							elseif a_Trans = {NATURAL_32} 113 then
								l_trans := 111
							else
								l_trans := 112
							end
						end
						if a_beta.to_double = 0.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := (0.0).truncated_to_real
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := (0.0).truncated_to_real
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						elseif a_beta.to_double /= 1.0 then
							if l_uplo = 121 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := l_i
									from
									until
										not (l_j < a_N)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (l_j <= l_i)
									loop
										a_C[(a_ldc * l_i + l_j)] := a_C[(a_ldc * l_i + l_j)] * a_beta
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							end
						end
						if a_alpha.to_double = 0.0 then
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 11 -- skip if block
					end
				when 1 then -- if:
						if l_uplo /= 121 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := l_i
								from
								until
									not (not (l_return) and then (l_j < a_N))
								loop
									l_temp := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp := l_temp + (a_A___0[(l_i * a_lda + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 4 -- goto _L___1
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 4 then -- else: | _L___1:
						if l_uplo /= 121 then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						if l_trans = 112 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := l_i
								from
								until
									not (not (l_return) and then (l_j < a_N))
								loop
									l_temp___0 := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___0 := l_temp___0 + (a_A___0[(l_k * a_lda + l_i)] * a_A___0[(l_k * a_lda + l_j)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___0)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 7 -- goto _L___0
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else: | _L___0:
						if l_uplo /= 122 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_trans = 111 then
							l_i := 0
							from
							until
								not (not (l_return) and then (l_i < a_N))
							loop
								l_j := 0
								from
								until
									not (not (l_return) and then (l_j <= l_i))
								loop
									l_temp___1 := (0.0).truncated_to_real
									l_k := 0
									from
									until
										not (l_k < a_K)
									loop
										l_temp___1 := l_temp___1 + (a_A___0[(l_i * a_lda + l_k)] * a_A___0[(l_j * a_lda + l_k)])
										l_k := l_k + 1
									end
									a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___1)
									l_j := l_j + 1
								end
								l_i := l_i + 1
							end
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_uplo = 122 then
							if l_trans = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < a_N))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j <= l_i))
									loop
										l_temp___2 := (0.0).truncated_to_real
										l_k := 0
										from
										until
											not (l_k < a_K)
										loop
											l_temp___2 := l_temp___2 + (a_A___0[(l_k * a_lda + l_i)] * a_A___0[(l_k * a_lda + l_j)])
											l_k := l_k + 1
										end
										a_C[(l_i * a_ldc + l_j)] := a_C[(l_i * a_ldc + l_j)] + (a_alpha * l_temp___2)
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syrk_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_syrk_r.h"), ce_string ("unrecognized operation")])
						end
				when 11 then -- if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_strmm (a_Order, a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_32]; a_ldb: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_nonunit: INTEGER_32
			l_side: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_temp: REAL_32
			l_temp___0: REAL_32
			l_tmp: INTEGER_32
			l_temp___1: REAL_32
			l_tmp___0: INTEGER_32
			l_temp___2: REAL_32
			l_temp___3: REAL_32
			l_tmp___1: INTEGER_32
			l_temp___4: REAL_32
			l_temp___5: REAL_32
			l_temp___6: REAL_32
			l_tmp___2: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_Order = {NATURAL_32} 101 then
						l_n1 := a_M
						l_n2 := a_N
						l_side := a_Side.to_integer_32
						l_uplo := a_Uplo.to_integer_32
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					else
						l_n1 := a_N
						l_n2 := a_M
						if a_Side = {NATURAL_32} 141 then
							l_side := 142
						else
							l_side := 141
						end
						if a_Uplo = {NATURAL_32} 121 then
							l_uplo := 122
						else
							l_uplo := 121
						end
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					end
					if l_side /= 141 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp := (0.0).truncated_to_real
										if l_nonunit.to_boolean then
											l_temp := (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_i + 1
										from
										until
											not (l_k < l_n1)
										loop
											l_temp := l_temp + (a_A___0[(a_lda * l_i + l_k)] * a_B[(l_k * a_ldb + l_j)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 3 -- goto _L___12
							end
						else
							l_pos := -1 + 3 -- goto _L___12
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else: | _L___12:
						if l_side /= 141 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp := l_i
										l_i := l_i - 1
										if not l_tmp.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										l_j := 0
										from
										until
											not (not (l_return) and then (l_j < l_n2))
										loop
											l_temp___0 := (0.0).truncated_to_real
											l_k := 0
											from
											until
												not (l_k < l_i)
											loop
												l_temp___0 := l_temp___0 + (a_A___0[(a_lda * l_k + l_i)] * a_B[(l_k * a_ldb + l_j)])
												l_k := l_k + 1
											end
											if l_nonunit.to_boolean then
												l_temp___0 := l_temp___0 + (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
											else
												l_temp___0 := l_temp___0 + (a_B[(l_i * a_ldb + l_j)])
											end
											a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___0
											l_j := l_j + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 6 -- goto _L___10
							end
						else
							l_pos := -1 + 6 -- goto _L___10
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else: | _L___10:
						if l_side /= 141 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp___0 := l_i
										l_i := l_i - 1
										if not l_tmp___0.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										l_j := 0
										from
										until
											not (not (l_return) and then (l_j < l_n2))
										loop
											l_temp___1 := (0.0).truncated_to_real
											l_k := 0
											from
											until
												not (l_k < l_i)
											loop
												l_temp___1 := l_temp___1 + (a_A___0[(a_lda * l_i + l_k)] * a_B[(l_k * a_ldb + l_j)])
												l_k := l_k + 1
											end
											if l_nonunit.to_boolean then
												l_temp___1 := l_temp___1 + (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
											else
												l_temp___1 := l_temp___1 + (a_B[(l_i * a_ldb + l_j)])
											end
											a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___1
											l_j := l_j + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 9 -- goto _L___8
							end
						else
							l_pos := -1 + 9 -- goto _L___8
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else: | _L___8:
						if l_side /= 141 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if l_uplo = 122 then
							if l_trans = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___2 := (0.0).truncated_to_real
										if l_nonunit.to_boolean then
											l_temp___2 := (a_A___0[(l_i * a_lda + l_i)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp___2 := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_i + 1
										from
										until
											not (l_k < l_n1)
										loop
											l_temp___2 := l_temp___2 + (a_A___0[(a_lda * l_k + l_i)] * a_B[(l_k * a_ldb + l_j)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___2
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 12 -- goto _L___6
							end
						else
							l_pos := -1 + 12 -- goto _L___6
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else: | _L___6:
						if l_side /= 142 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := l_n2
									from
									until
										(l_return or l_break)
									loop
										l_break := False
										if l_j > 0 then
											l_tmp___1 := l_j
											l_j := l_j - 1
											if not l_tmp___1.to_boolean then
												l_break := True
											end
										else
											l_break := True
										end
										if not (l_return or l_break) then
											l_temp___3 := (0.0).truncated_to_real
											l_k := 0
											from
											until
												not (l_k < l_j)
											loop
												l_temp___3 := l_temp___3 + (a_A___0[(a_lda * l_k + l_j)] * a_B[(l_i * a_ldb + l_k)])
												l_k := l_k + 1
											end
											if l_nonunit.to_boolean then
												l_temp___3 := l_temp___3 + (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
											else
												l_temp___3 := l_temp___3 + (a_B[(l_i * a_ldb + l_j)])
											end
											a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___3
										end
									end
									l_break := False
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 15 -- goto _L___4
							end
						else
							l_pos := -1 + 15 -- goto _L___4
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else: | _L___4:
						if l_side /= 142 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___4 := (0.0).truncated_to_real
										if l_nonunit.to_boolean then
											l_temp___4 := (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp___4 := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_j + 1
										from
										until
											not (l_k < l_n2)
										loop
											l_temp___4 := l_temp___4 + (a_A___0[(a_lda * l_j + l_k)] * a_B[(l_i * a_ldb + l_k)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___4
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 18 -- goto _L___2
							end
						else
							l_pos := -1 + 18 -- goto _L___2
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else: | _L___2:
						if l_side /= 142 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										l_temp___5 := (0.0).truncated_to_real
										if l_nonunit.to_boolean then
											l_temp___5 := (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
										else
											l_temp___5 := a_B[(l_i * a_ldb + l_j)]
										end
										l_k := l_j + 1
										from
										until
											not (l_k < l_n2)
										loop
											l_temp___5 := l_temp___5 + (a_A___0[(a_lda * l_k + l_j)] * a_B[(l_i * a_ldb + l_k)])
											l_k := l_k + 1
										end
										a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___5
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 21 -- goto _L___0
							end
						else
							l_pos := -1 + 21 -- goto _L___0
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L___0:
						if l_side = 142 then
							if l_uplo = 122 then
								if l_trans = 112 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := l_n2
										from
										until
											(l_return or l_break)
										loop
											l_break := False
											if l_j > 0 then
												l_tmp___2 := l_j
												l_j := l_j - 1
												if not l_tmp___2.to_boolean then
													l_break := True
												end
											else
												l_break := True
											end
											if not (l_return or l_break) then
												l_temp___6 := (0.0).truncated_to_real
												l_k := 0
												from
												until
													not (l_k < l_j)
												loop
													l_temp___6 := l_temp___6 + (a_A___0[(a_lda * l_j + l_k)] * a_B[(l_i * a_ldb + l_k)])
													l_k := l_k + 1
												end
												if l_nonunit.to_boolean then
													l_temp___6 := l_temp___6 + (a_A___0[(l_j * a_lda + l_j)] * a_B[(l_i * a_ldb + l_j)])
												else
													l_temp___6 := l_temp___6 + (a_B[(l_i * a_ldb + l_j)])
												end
												a_B[(a_ldb * l_i + l_j)] := a_alpha * l_temp___6
											end
										end
										l_break := False
										l_i := l_i + 1
									end
								else
									(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmm_r.h"), ce_string ("unrecognized operation")])
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmm_r.h"), ce_string ("unrecognized operation")])
						end
				when 22 then -- if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_strmv (a_order, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_N: INTEGER_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_nonunit: INTEGER_32
			l_Trans: INTEGER_32
			l_tmp: NATURAL_32
			l_ix: INTEGER_32
			l_tmp___0: INTEGER_32
			l_temp: REAL_32
			l_j_min: INTEGER_32
			l_j_max: INTEGER_32
			l_jx: INTEGER_32
			l_tmp___1: INTEGER_32
			l_ix___0: INTEGER_32
			l_tmp___2: INTEGER_32
			l_temp___0: REAL_32
			l_j_min___0: INTEGER_32
			l_j_max___0: INTEGER_32
			l_jx___0: INTEGER_32
			l_tmp___3: INTEGER_32
			l_tmp___4: INTEGER_32
			l_ix___1: INTEGER_32
			l_tmp___5: INTEGER_32
			l_temp___1: REAL_32
			l_j_min___1: INTEGER_32
			l_j_max___1: INTEGER_32
			l_jx___1: INTEGER_32
			l_tmp___6: INTEGER_32
			l_tmp___7: INTEGER_32
			l_ix___2: INTEGER_32
			l_tmp___8: INTEGER_32
			l_temp___2: REAL_32
			l_j_min___2: INTEGER_32
			l_j_max___2: INTEGER_32
			l_jx___2: INTEGER_32
			l_tmp___9: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_TransA /= {NATURAL_32} 113 then
						l_tmp := a_TransA
					else
						l_tmp := {NATURAL_32} 112
					end
					l_Trans := l_tmp.to_integer_32
					if a_order /= {NATURAL_32} 101 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 6 -- goto _L___12
							else
								l_pos := -1 + 3 -- goto _L___16
							end
						else
							l_pos := -1 + 3 -- goto _L___16
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 3 then -- else: | _L___16:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 12 -- skip if block
						end
				when 4 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 10 -- skip if block
						end
				when 5 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 8 -- skip if block
						end
				when 6 then -- if: | _L___12:
						if a_incX > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___0
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_temp := (0.0).truncated_to_real
							l_j_min := l_i + 1
							l_j_max := a_N
							if a_incX > 0 then
								l_tmp___1 := 0
							else
								l_tmp___1 := (a_N - 1) * -a_incX
							end
							l_jx := l_tmp___1 + l_j_min * a_incX
							l_j := l_j_min
							from
							until
								not (l_j < l_j_max)
							loop
								l_temp := l_temp + (a_X[l_jx] * a_A___0[(a_lda * l_i + l_j)])
								l_jx := l_jx + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix] := l_temp + a_X[l_ix] * a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix] := a_X[l_ix] + l_temp
							end
							l_ix := l_ix + a_incX
							l_i := l_i + 1
						end
				when 7 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 8 then -- else:
						l_pos := -1 + 12 -- goto _L___14
				when 9 then -- if done | if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else:
						l_pos := -1 + 12 -- goto _L___14
				when 11 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 12 then -- else: | _L___14:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 18 -- goto _L___7
							else
								l_pos := -1 + 15 -- goto _L___11
							end
						else
							l_pos := -1 + 15 -- goto _L___11
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 15 then -- else: | _L___11:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 24 -- skip if block
						end
				when 16 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 22 -- skip if block
						end
				when 17 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 20 -- skip if block
						end
				when 18 then -- if: | _L___7:
						if a_incX > 0 then
							l_tmp___2 := 0
						else
							l_tmp___2 := (a_N - 1) * -a_incX
						end
						l_ix___0 := l_tmp___2 + (a_N - 1) * a_incX
						l_i := a_N
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___4 := l_i
								l_i := l_i - 1
								if not l_tmp___4.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_temp___0 := (0.0).truncated_to_real
								l_j_min___0 := 0
								l_j_max___0 := l_i
								if a_incX > 0 then
									l_tmp___3 := 0
								else
									l_tmp___3 := (a_N - 1) * -a_incX
								end
								l_jx___0 := l_tmp___3 + l_j_min___0 * a_incX
								l_j := l_j_min___0
								from
								until
									not (l_j < l_j_max___0)
								loop
									l_temp___0 := l_temp___0 + (a_X[l_jx___0] * a_A___0[(a_lda * l_i + l_j)])
									l_jx___0 := l_jx___0 + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix___0] := l_temp___0 + a_X[l_ix___0] * a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix___0] := a_X[l_ix___0] + l_temp___0
								end
								l_ix___0 := l_ix___0 - a_incX
							end
						end
						l_break := False
				when 19 then -- if block done, skip else block:
						l_pos := -1 + 21
				when 20 then -- else:
						l_pos := -1 + 24 -- goto _L___9
				when 21 then -- if done | if block done, skip else block:
						l_pos := -1 + 23
				when 22 then -- else:
						l_pos := -1 + 24 -- goto _L___9
				when 23 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 24 then -- else: | _L___9:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 27 -- skip if block
						end
				when 25 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 30 -- goto _L___2
							else
								l_pos := -1 + 27 -- goto _L___6
							end
						else
							l_pos := -1 + 27 -- goto _L___6
						end
				when 26 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 27 then -- else: | _L___6:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 36 -- skip if block
						end
				when 28 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 34 -- skip if block
						end
				when 29 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 32 -- skip if block
						end
				when 30 then -- if: | _L___2:
						if a_incX > 0 then
							l_tmp___5 := 0
						else
							l_tmp___5 := (a_N - 1) * -a_incX
						end
						l_ix___1 := l_tmp___5 + (a_N - 1) * a_incX
						l_i := a_N
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___7 := l_i
								l_i := l_i - 1
								if not l_tmp___7.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_temp___1 := (0.0).truncated_to_real
								l_j_min___1 := 0
								l_j_max___1 := l_i
								if a_incX > 0 then
									l_tmp___6 := 0
								else
									l_tmp___6 := (a_N - 1) * -a_incX
								end
								l_jx___1 := l_tmp___6 + l_j_min___1 * a_incX
								l_j := l_j_min___1
								from
								until
									not (l_j < l_j_max___1)
								loop
									l_temp___1 := l_temp___1 + (a_X[l_jx___1] * a_A___0[(a_lda * l_j + l_i)])
									l_jx___1 := l_jx___1 + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix___1] := l_temp___1 + a_X[l_ix___1] * a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix___1] := a_X[l_ix___1] + l_temp___1
								end
								l_ix___1 := l_ix___1 - a_incX
							end
						end
						l_break := False
				when 31 then -- if block done, skip else block:
						l_pos := -1 + 33
				when 32 then -- else:
						l_pos := -1 + 36 -- goto _L___4
				when 33 then -- if done | if block done, skip else block:
						l_pos := -1 + 35
				when 34 then -- else:
						l_pos := -1 + 36 -- goto _L___4
				when 35 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 36 then -- else: | _L___4:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 39 -- skip if block
						end
				when 37 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 42 -- goto _L
							else
								l_pos := -1 + 39 -- goto _L___1
							end
						else
							l_pos := -1 + 39 -- goto _L___1
						end
				when 38 then -- if block done, skip else block:
						l_pos := -1 + 49
				when 39 then -- else: | _L___1:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 48 -- skip if block
						end
				when 40 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 46 -- skip if block
						end
				when 41 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 44 -- skip if block
						end
				when 42 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___8 := 0
						else
							l_tmp___8 := (a_N - 1) * -a_incX
						end
						l_ix___2 := l_tmp___8
						l_i := 0
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_temp___2 := (0.0).truncated_to_real
							l_j_min___2 := l_i + 1
							l_j_max___2 := a_N
							if a_incX > 0 then
								l_tmp___9 := 0
							else
								l_tmp___9 := (a_N - 1) * -a_incX
							end
							l_jx___2 := l_tmp___9 + (l_i + 1) * a_incX
							l_j := l_j_min___2
							from
							until
								not (l_j < l_j_max___2)
							loop
								l_temp___2 := l_temp___2 + (a_X[l_jx___2] * a_A___0[(a_lda * l_j + l_i)])
								l_jx___2 := l_jx___2 + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix___2] := l_temp___2 + a_X[l_ix___2] * a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix___2] := a_X[l_ix___2] + l_temp___2
							end
							l_ix___2 := l_ix___2 + a_incX
							l_i := l_i + 1
						end
				when 43 then -- if block done, skip else block:
						l_pos := -1 + 45
				when 44 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmv_r.h"), ce_string ("unrecognized operation")])
				when 45 then -- if done | if block done, skip else block:
						l_pos := -1 + 47
				when 46 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmv_r.h"), ce_string ("unrecognized operation")])
				when 47 then -- if done | if block done, skip else block:
						l_pos := -1 + 49
				when 48 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trmv_r.h"), ce_string ("unrecognized operation")])
				when 49 then -- if done | if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_strsm (a_Order, a_Side, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_M, a_N: INTEGER_32; a_alpha: REAL_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_B: CE_POINTER [REAL_32]; a_ldb: INTEGER_32)
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_k: INTEGER_32
			l_n1: INTEGER_32
			l_n2: INTEGER_32
			l_nonunit: INTEGER_32
			l_side: INTEGER_32
			l_uplo: INTEGER_32
			l_trans: INTEGER_32
			l_Aii: REAL_32
			l_Aki: REAL_32
			l_tmp: INTEGER_32
			l_Aii___0: REAL_32
			l_Aik: REAL_32
			l_Aii___1: REAL_32
			l_Aki___0: REAL_32
			l_Aii___2: REAL_32
			l_Aik___0: REAL_32
			l_tmp___0: INTEGER_32
			l_Ajj: REAL_32
			l_Bij: REAL_32
			l_Ajj___0: REAL_32
			l_Bij___0: REAL_32
			l_tmp___1: INTEGER_32
			l_Ajj___1: REAL_32
			l_Bij___1: REAL_32
			l_tmp___2: INTEGER_32
			l_Ajj___2: REAL_32
			l_Bij___2: REAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_Order = {NATURAL_32} 101 then
						l_n1 := a_M
						l_n2 := a_N
						l_side := a_Side.to_integer_32
						l_uplo := a_Uplo.to_integer_32
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					else
						l_n1 := a_N
						l_n2 := a_M
						if a_Side = {NATURAL_32} 141 then
							l_side := 142
						else
							l_side := 141
						end
						if a_Uplo = {NATURAL_32} 121 then
							l_uplo := 122
						else
							l_uplo := 121
						end
						if a_TransA = {NATURAL_32} 113 then
							l_trans := 112
						else
							l_trans := a_TransA.to_integer_32
						end
					end
					if l_side /= 141 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp := l_i
										l_i := l_i - 1
										if not l_tmp.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										if l_nonunit.to_boolean then
											l_Aii := a_A___0[(a_lda * l_i + l_i)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii
												l_j := l_j + 1
											end
										end
										l_k := 0
										from
										until
											not (not (l_return) and then (l_k < l_i))
										loop
											l_Aki := a_A___0[(l_k * a_lda + l_i)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aki * a_B[(a_ldb * l_i + l_j)])
												l_j := l_j + 1
											end
											l_k := l_k + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 3 -- goto _L___12
							end
						else
							l_pos := -1 + 3 -- goto _L___12
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 3 then -- else: | _L___12:
						if l_side /= 141 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									if l_nonunit.to_boolean then
										l_Aii___0 := a_A___0[(a_lda * l_i + l_i)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii___0
											l_j := l_j + 1
										end
									end
									l_k := l_i + 1
									from
									until
										not (not (l_return) and then (l_k < l_n1))
									loop
										l_Aik := a_A___0[(l_i * a_lda + l_k)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aik * a_B[(a_ldb * l_i + l_j)])
											l_j := l_j + 1
										end
										l_k := l_k + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 6 -- goto _L___10
							end
						else
							l_pos := -1 + 6 -- goto _L___10
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 6 then -- else: | _L___10:
						if l_side /= 141 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									if l_nonunit.to_boolean then
										l_Aii___1 := a_A___0[(a_lda * l_i + l_i)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii___1
											l_j := l_j + 1
										end
									end
									l_k := l_i + 1
									from
									until
										not (not (l_return) and then (l_k < l_n1))
									loop
										l_Aki___0 := a_A___0[(l_k * a_lda + l_i)]
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aki___0 * a_B[(a_ldb * l_i + l_j)])
											l_j := l_j + 1
										end
										l_k := l_k + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 9 -- goto _L___8
							end
						else
							l_pos := -1 + 9 -- goto _L___8
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 9 then -- else: | _L___8:
						if l_side /= 141 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						if l_uplo = 122 then
							if l_trans = 112 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := l_n1
								from
								until
									(l_return or l_break)
								loop
									l_break := False
									if l_i > 0 then
										l_tmp___0 := l_i
										l_i := l_i - 1
										if not l_tmp___0.to_boolean then
											l_break := True
										end
									else
										l_break := True
									end
									if not (l_return or l_break) then
										if l_nonunit.to_boolean then
											l_Aii___2 := a_A___0[(a_lda * l_i + l_i)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Aii___2
												l_j := l_j + 1
											end
										end
										l_k := 0
										from
										until
											not (not (l_return) and then (l_k < l_i))
										loop
											l_Aik___0 := a_A___0[(l_i * a_lda + l_k)]
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_k + l_j)] := a_B[(a_ldb * l_k + l_j)] - (l_Aik___0 * a_B[(a_ldb * l_i + l_j)])
												l_j := l_j + 1
											end
											l_k := l_k + 1
										end
									end
								end
								l_break := False
							else
								l_pos := -1 + 12 -- goto _L___6
							end
						else
							l_pos := -1 + 12 -- goto _L___6
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 12 then -- else: | _L___6:
						if l_side /= 142 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						if l_uplo = 121 then
							if l_trans = 111 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := 0
									from
									until
										not (not (l_return) and then (l_j < l_n2))
									loop
										if l_nonunit.to_boolean then
											l_Ajj := a_A___0[(a_lda * l_j + l_j)]
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj
										end
										l_Bij := a_B[(a_ldb * l_i + l_j)]
										l_k := l_j + 1
										from
										until
											not (l_k < l_n2)
										loop
											a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_j * a_lda + l_k)] * l_Bij)
											l_k := l_k + 1
										end
										l_j := l_j + 1
									end
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 15 -- goto _L___4
							end
						else
							l_pos := -1 + 15 -- goto _L___4
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 15 then -- else: | _L___4:
						if l_side /= 142 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if:
						if l_uplo = 121 then
							if l_trans = 112 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := l_n2
									from
									until
										(l_return or l_break)
									loop
										l_break := False
										if l_j > 0 then
											l_tmp___1 := l_j
											l_j := l_j - 1
											if not l_tmp___1.to_boolean then
												l_break := True
											end
										else
											l_break := True
										end
										if not (l_return or l_break) then
											if l_nonunit.to_boolean then
												l_Ajj___0 := a_A___0[(a_lda * l_j + l_j)]
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj___0
											end
											l_Bij___0 := a_B[(a_ldb * l_i + l_j)]
											l_k := 0
											from
											until
												not (l_k < l_j)
											loop
												a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_k * a_lda + l_j)] * l_Bij___0)
												l_k := l_k + 1
											end
										end
									end
									l_break := False
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 18 -- goto _L___2
							end
						else
							l_pos := -1 + 18 -- goto _L___2
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 18 then -- else: | _L___2:
						if l_side /= 142 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if:
						if l_uplo = 122 then
							if l_trans = 111 then
								if a_alpha.to_double /= 1.0 then
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (l_j < l_n2)
										loop
											a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								end
								l_i := 0
								from
								until
									not (not (l_return) and then (l_i < l_n1))
								loop
									l_j := l_n2
									from
									until
										(l_return or l_break)
									loop
										l_break := False
										if l_j > 0 then
											l_tmp___2 := l_j
											l_j := l_j - 1
											if not l_tmp___2.to_boolean then
												l_break := True
											end
										else
											l_break := True
										end
										if not (l_return or l_break) then
											if l_nonunit.to_boolean then
												l_Ajj___1 := a_A___0[(a_lda * l_j + l_j)]
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj___1
											end
											l_Bij___1 := a_B[(a_ldb * l_i + l_j)]
											l_k := 0
											from
											until
												not (l_k < l_j)
											loop
												a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_j * a_lda + l_k)] * l_Bij___1)
												l_k := l_k + 1
											end
										end
									end
									l_break := False
									l_i := l_i + 1
								end
							else
								l_pos := -1 + 21 -- goto _L___0
							end
						else
							l_pos := -1 + 21 -- goto _L___0
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else: | _L___0:
						if l_side = 142 then
							if l_uplo = 122 then
								if l_trans = 112 then
									if a_alpha.to_double /= 1.0 then
										l_i := 0
										from
										until
											not (not (l_return) and then (l_i < l_n1))
										loop
											l_j := 0
											from
											until
												not (l_j < l_n2)
											loop
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] * a_alpha
												l_j := l_j + 1
											end
											l_i := l_i + 1
										end
									end
									l_i := 0
									from
									until
										not (not (l_return) and then (l_i < l_n1))
									loop
										l_j := 0
										from
										until
											not (not (l_return) and then (l_j < l_n2))
										loop
											if l_nonunit.to_boolean then
												l_Ajj___2 := a_A___0[(a_lda * l_j + l_j)]
												a_B[(a_ldb * l_i + l_j)] := a_B[(a_ldb * l_i + l_j)] / l_Ajj___2
											end
											l_Bij___2 := a_B[(a_ldb * l_i + l_j)]
											l_k := l_j + 1
											from
											until
												not (l_k < l_n2)
											loop
												a_B[(a_ldb * l_i + l_k)] := a_B[(a_ldb * l_i + l_k)] - (a_A___0[(l_k * a_lda + l_j)] * l_Bij___2)
												l_k := l_k + 1
											end
											l_j := l_j + 1
										end
										l_i := l_i + 1
									end
								else
									(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsm_r.h"), ce_string ("unrecognized operation")])
								end
							else
								(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsm_r.h"), ce_string ("unrecognized operation")])
							end
						else
							(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsm_r.h"), ce_string ("unrecognized operation")])
						end
				when 22 then -- if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	cblas_strsv (a_order, a_Uplo, a_TransA, a_Diag: NATURAL_32; a_N: INTEGER_32; a_A___0: CE_POINTER [REAL_32]; a_lda: INTEGER_32; a_X: CE_POINTER [REAL_32]; a_incX: INTEGER_32)
		local
			l_nonunit: INTEGER_32
			l_ix: INTEGER_32
			l_jx: INTEGER_32
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_Trans: INTEGER_32
			l_tmp: NATURAL_32
			l_tmp___0: INTEGER_32
			l_tmp___1: REAL_32
			l_Aij: REAL_32
			l_tmp___2: INTEGER_32
			l_tmp___3: REAL_32
			l_Aij___0: REAL_32
			l_tmp___4: REAL_32
			l_Aji: REAL_32
			l_tmp___5: INTEGER_32
			l_tmp___6: REAL_32
			l_Aji___0: REAL_32
			l_tmp___7: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_nonunit := (a_Diag = {NATURAL_32} 131).to_integer
					if a_TransA /= {NATURAL_32} 113 then
						l_tmp := a_TransA
					else
						l_tmp := {NATURAL_32} 112
					end
					l_Trans := l_tmp.to_integer_32
					if a_N = 0 then
						l_return := True
					end
					if l_return then
						l_pos := -1 + 50 -- skip if block
					end
				when 1 then -- if:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 7 -- goto _L___12
							else
								l_pos := -1 + 4 -- goto _L___16
							end
						else
							l_pos := -1 + 4 -- goto _L___16
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 4 then -- else: | _L___16:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 13 -- skip if block
						end
				when 5 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 11 -- skip if block
						end
				when 6 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L___12:
						if a_incX > 0 then
							l_tmp___0 := 0
						else
							l_tmp___0 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___0 + a_incX * (a_N - 1)
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[(a_lda * (a_N - 1) + (a_N - 1))])
						end
						l_ix := l_ix - a_incX
						l_i := a_N - 1
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___2 := l_i
								l_i := l_i - 1
								if not l_tmp___2.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_tmp___1 := a_X[l_ix]
								l_jx := l_ix + a_incX
								l_j := l_i + 1
								from
								until
									not (l_j < a_N)
								loop
									l_Aij := a_A___0[(a_lda * l_i + l_j)]
									l_tmp___1 := l_tmp___1 - (l_Aij * a_X[l_jx])
									l_jx := l_jx + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix] := l_tmp___1 / a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix] := l_tmp___1
								end
								l_ix := l_ix - a_incX
							end
						end
						l_break := False
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_pos := -1 + 13 -- goto _L___14
				when 10 then -- if done | if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_pos := -1 + 13 -- goto _L___14
				when 12 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 13 then -- else: | _L___14:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						if l_Trans = 111 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 19 -- goto _L___7
							else
								l_pos := -1 + 16 -- goto _L___11
							end
						else
							l_pos := -1 + 16 -- goto _L___11
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 16 then -- else: | _L___11:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 25 -- skip if block
						end
				when 17 then -- if:
						if l_Trans /= 112 then
							l_pos := -1 + 23 -- skip if block
						end
				when 18 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if: | _L___7:
						if a_incX > 0 then
							l_ix := 0
						else
							l_ix := (a_N - 1) * -a_incX
						end
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[0])
						end
						l_ix := l_ix + a_incX
						l_i := 1
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_tmp___3 := a_X[l_ix]
							if a_incX > 0 then
								l_jx := 0
							else
								l_jx := (a_N - 1) * -a_incX
							end
							l_j := 0
							from
							until
								not (l_j < l_i)
							loop
								l_Aij___0 := a_A___0[(a_lda * l_i + l_j)]
								l_tmp___3 := l_tmp___3 - (l_Aij___0 * a_X[l_jx])
								l_jx := l_jx + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix] := l_tmp___3 / a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix] := l_tmp___3
							end
							l_ix := l_ix + a_incX
							l_i := l_i + 1
						end
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						l_pos := -1 + 25 -- goto _L___9
				when 22 then -- if done | if block done, skip else block:
						l_pos := -1 + 24
				when 23 then -- else:
						l_pos := -1 + 25 -- goto _L___9
				when 24 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 25 then -- else: | _L___9:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 28 -- skip if block
						end
				when 26 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 121 then
								l_pos := -1 + 31 -- goto _L___2
							else
								l_pos := -1 + 28 -- goto _L___6
							end
						else
							l_pos := -1 + 28 -- goto _L___6
						end
				when 27 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 28 then -- else: | _L___6:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 37 -- skip if block
						end
				when 29 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 35 -- skip if block
						end
				when 30 then -- if:
						if a_Uplo /= {NATURAL_32} 122 then
							l_pos := -1 + 33 -- skip if block
						end
				when 31 then -- if: | _L___2:
						if a_incX > 0 then
							l_ix := 0
						else
							l_ix := (a_N - 1) * -a_incX
						end
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[0])
						end
						l_ix := l_ix + a_incX
						l_i := 1
						from
						until
							not (not (l_return) and then (l_i < a_N))
						loop
							l_tmp___4 := a_X[l_ix]
							if a_incX > 0 then
								l_jx := 0
							else
								l_jx := (a_N - 1) * -a_incX
							end
							l_j := 0
							from
							until
								not (l_j < l_i)
							loop
								l_Aji := a_A___0[(a_lda * l_j + l_i)]
								l_tmp___4 := l_tmp___4 - (l_Aji * a_X[l_jx])
								l_jx := l_jx + a_incX
								l_j := l_j + 1
							end
							if l_nonunit.to_boolean then
								a_X[l_ix] := l_tmp___4 / a_A___0[(a_lda * l_i + l_i)]
							else
								a_X[l_ix] := l_tmp___4
							end
							l_ix := l_ix + a_incX
							l_i := l_i + 1
						end
				when 32 then -- if block done, skip else block:
						l_pos := -1 + 34
				when 33 then -- else:
						l_pos := -1 + 37 -- goto _L___4
				when 34 then -- if done | if block done, skip else block:
						l_pos := -1 + 36
				when 35 then -- else:
						l_pos := -1 + 37 -- goto _L___4
				when 36 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 37 then -- else: | _L___4:
						if a_order /= {NATURAL_32} 101 then
							l_pos := -1 + 40 -- skip if block
						end
				when 38 then -- if:
						if l_Trans = 112 then
							if a_Uplo = {NATURAL_32} 122 then
								l_pos := -1 + 43 -- goto _L
							else
								l_pos := -1 + 40 -- goto _L___1
							end
						else
							l_pos := -1 + 40 -- goto _L___1
						end
				when 39 then -- if block done, skip else block:
						l_pos := -1 + 50
				when 40 then -- else: | _L___1:
						if a_order /= {NATURAL_32} 102 then
							l_pos := -1 + 49 -- skip if block
						end
				when 41 then -- if:
						if l_Trans /= 111 then
							l_pos := -1 + 47 -- skip if block
						end
				when 42 then -- if:
						if a_Uplo /= {NATURAL_32} 121 then
							l_pos := -1 + 45 -- skip if block
						end
				when 43 then -- if: | _L:
						if a_incX > 0 then
							l_tmp___5 := 0
						else
							l_tmp___5 := (a_N - 1) * -a_incX
						end
						l_ix := l_tmp___5 + (a_N - 1) * a_incX
						if l_nonunit.to_boolean then
							a_X[l_ix] := a_X[l_ix] / (a_A___0[(a_lda * (a_N - 1) + (a_N - 1))])
						end
						l_ix := l_ix - a_incX
						l_i := a_N - 1
						from
						until
							(l_return or l_break)
						loop
							l_break := False
							if l_i > 0 then
								l_tmp___7 := l_i
								l_i := l_i - 1
								if not l_tmp___7.to_boolean then
									l_break := True
								end
							else
								l_break := True
							end
							if not (l_return or l_break) then
								l_tmp___6 := a_X[l_ix]
								l_jx := l_ix + a_incX
								l_j := l_i + 1
								from
								until
									not (l_j < a_N)
								loop
									l_Aji___0 := a_A___0[(a_lda * l_j + l_i)]
									l_tmp___6 := l_tmp___6 - (l_Aji___0 * a_X[l_jx])
									l_jx := l_jx + a_incX
									l_j := l_j + 1
								end
								if l_nonunit.to_boolean then
									a_X[l_ix] := l_tmp___6 / a_A___0[(a_lda * l_i + l_i)]
								else
									a_X[l_ix] := l_tmp___6
								end
								l_ix := l_ix - a_incX
							end
						end
						l_break := False
				when 44 then -- if block done, skip else block:
						l_pos := -1 + 46
				when 45 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsv_r.h"), ce_string ("unrecognized operation")])
				when 46 then -- if done | if block done, skip else block:
						l_pos := -1 + 48
				when 47 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsv_r.h"), ce_string ("unrecognized operation")])
				when 48 then -- if done | if block done, skip else block:
						l_pos := -1 + 50
				when 49 then -- else:;
						(create {P_XERBLA_STATIC}).cblas_xerbla ([0, ce_string ("source_trsv_r.h"), ce_string ("unrecognized operation")])
				when 50 then -- if done | if done | if done | if done | if done | if done | if done | if done
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

feature {ANY}

	copy (a_other: like Current)
		do
			size1 := a_other.size1
			size2 := a_other.size2
			tda := a_other.tda
			data := a_other.data
			block := a_other.block
			owner := a_other.owner
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				size1 = a_other.size1 and
				size2 = a_other.size2 and
				tda = a_other.tda and
				data.is_equal (a_other.data) and
				block.is_equal (a_other.block) and
				owner = a_other.owner
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("size1"),
					create {CE_STRUCTURE_FIELD}.make ("size2"),
					create {CE_STRUCTURE_FIELD}.make ("tda"),
					create {CE_STRUCTURE_FIELD}.make ("data"),
					create {CE_STRUCTURE_FIELD}.make ("block"),
					create {CE_STRUCTURE_FIELD}.make ("owner")
				>>)
			init_structure_info
		end

end
